
 UNIT PUnCRead;

 INTERFACE

 Uses
   UnCoTVar ; // for types to read catalogues and var NumBrStarsInHeap

 Var
   RFile            : File of RecStarCatType ; { TYCHO catalog }
   WFile            : File of LongInt ; { to define number in TYCHO catalog }
   BrStarFile       : File of RecStarAddType ; { bright stars only }
   StarRecT         : RecStarCatType ; { to read TYCHO catalog record }
   BrStarRec        : RecStarAddType ; { record to read BrFile }

 Var
   ExistBrFile,
   ExistTRCFile,
   WhiteColorOnly,
   BrightStarsOnly  : Boolean ;

 Procedure FromSphericToIdealCoordinates
           ( Alpha , Delta  : Extended ;      // in degree
             AlphaC , DeltaC  : Extended ;    // in degree
             Var  Ksi , Eta  : Extended ) ;   // in radian

 Procedure FromIdealToSphericCoordinates
           ( Ksi , Eta  : Extended ;            // in radian
             AlphaC , DeltaC  : Extended ;      // in degree
             Var  Alpha , Delta  : Extended ) ; // in degree

 Procedure ToGetStarFromHeap ( Num  : Word ) ;

 Procedure ToPutStarToHeap ( Num  : Word ) ;

 Procedure ToDeCodeStarRecT ( NumBrStars , NumInRegion  : LongInt ) ;

 Procedure ToDeCodeStarRecB ;

 Procedure ToReadStarsFromT ( NumBrStars  :  LongInt ;
                              Var  NumCurStars  : Word ) ;

 function ToGetBrightStars  : LongInt ;

 IMPLEMENTATION

 Uses
   UnConTyp , // simple constants
   UnGloVar , // for XinDegC YinDegC VAllMin Max VMagMin Max
   UnForFun ; // simple functions and procedures

 { to change sperical coordinates
       Alpha , Delta  of the star to ideal ( standart )
       with the centre in point  AlphaC , DeltaC :
   r = ( x , y , z ) ,  r1 = ( x1 , y1 , z1 )
   r1 = Rx ( Pi/2 - DeltaC ) * Rz ( AlphaC + Pi/2 ) * r
   Ksi = x1 / z1 , Eta = y1 / z1 ,
   algorihm was presented by doctor Konstantin V. Kuimov }

 Procedure FromSphericToIdealCoordinates
           ( Alpha , Delta  : Extended ;    // in degree
             AlphaC , DeltaC  : Extended ;    // in degree
             Var  Ksi , Eta  : Extended ) ; // in radian
 Var
   a,d,a0,d0,
   sa,ca,sd,cd,
   sa0,ca0,sd0,cd0,
   x,y,z,x1,y1,z1                : Extended ;
 Begin // centre position AlphaC DeltaC must be known
   a:=GraRad*Alpha; d:=GraRad*Delta;  // const form UnConTyp
   a0:=GraRad*AlphaC; d0:=GraRad*DeltaC; // AlphaC DeltaC from UnGloVar
   sa:=Sin(a);   ca:=Cos(a);
   sd:=Sin(d);   cd:=Cos(d);
   sa0:=Sin(a0); ca0:=Cos(a0);
   sd0:=Sin(d0); cd0:=Cos(d0);
   x:=ca*cd; y:=sa*cd; z:=sd;
   x1:=-sa0*x+ca0*y;
   y1:=-ca0*sd0*x-sa0*sd0*y+cd0*z;
   z1:=+ca0*cd0*x+sa0*cd0*y+sd0*z;
   Ksi:=x1/z1; // in radian
   Eta:=y1/z1; // in radian
 End;

 { to change ideal coordinates of the star  Ksi , Eta
      with the center in point  AlphaC , DeltaC
      to spheric coordinates  Alpha , Delta :
   r1 = ( Ksi , Eta , 1 ) ; r = ( x , y , z )
   r = Rz ( - ( Pi/2 + AlphaC ) ) * Rx ( - ( Pi/2 - DeltaC ) ) ,
   algorihm was presented by doctor Konstantin V. Kuimov }

 Procedure FromIdealToSphericCoordinates
           ( Ksi , Eta  : Extended ;            // in radian
             AlphaC , DeltaC  : Extended ;    // in degree
             Var  Alpha , Delta  : Extended ) ; // in degree
 Var
   a,d,a0,d0,q,
   sa0,ca0,sd0,cd0  : Extended ;
   x,y,z,x1,y1,z1   : Extended ;
 Begin // centre position AlphaC DeltaC must be known
   x1:=Ksi; y1:=Eta; z1:=1;
   a0:=GraRad*AlphaC; d0:=GraRad*DeltaC; // AlphaC DeltaC from UnGloVar
   sa0:=Sin(a0); ca0:=Cos(a0);
   sd0:=Sin(d0); cd0:=Cos(d0);
   x:=-sa0*x1-ca0*sd0*y1+ca0*cd0*z1;
   y:=+ca0*x1-sa0*sd0*y1+sa0*cd0*z1;
   z:=+cd0*y1+sd0*z1;
   q:=Sqrt(x*x+y*y);
   ArcTanTwoArg(y,x,a);  If a < 0  Then  a:=a+Pi22;
   ArcTanTwoArg(z,q,d);
   Alpha:=RadGra*a; // in degree
   Delta:=RadGra*d; // in degree
 End;

 Procedure ToGetStarFromHeap ( Num  : Word ) ;
 Begin
   StarOneCur:=StarAllHeap^[num];
 End;

 Procedure ToPutStarToHeap ( Num  : Word ) ;
 Begin
   StarAllHeap^[Num]:=StarOneCur;
 End;

 procedure ToFindOtherNumbers ( NumBrStars  : LongInt ) ;
 var
   ns  : Word ;
   da  : Extended ;
   dd  : Extended ;
   bo  : Boolean ;
 begin
   StarOneCur.HipNumber:=0; // unknown default
   StarOneCur.FK5Number:=0; // unknown default
   if  StarOneCur.Vt < 7.5
     then
       begin
         ns:=0;
         bo:=False;
         while  ( not bo ) and ( ns < NumBrStars )  do
           begin
             ns:=ns+1;
             da:=Abs(StarAllHeap^[ns].Alpha-StarOneCur.Alpha);
             dd:=Abs(StarAllHeap^[ns].Delta-StarOneCur.Delta);
             bo:= ( da < 1.0e-3 ) and ( dd < 1.0e-3 );
           end;
         if  bo
           then
             begin
               StarOneCur.HipNumber:=StarAllHeap^[ns].HipNumber;
               StarOneCur.FK5Number:=StarAllHeap^[ns].FK5Number;
               StarAllHeap^[ns].TycNumber:=StarOneCur.TycNumber;
               StarAllHeap^[ns].TrcNumber:=StarOneCur.TrcNumber;
               StarAllHeap^[ns].Bt:=StarOneCur.Bt;
               StarAllHeap^[ns].Vt:=StarOneCur.Vt;
             end;
       end;
 end;

 Procedure ToDeCodeStarRecT ( NumBrStars , NumInRegion  : LongInt ) ;
 Begin
   With  StarRecT  Do
     Begin
       if  Pos('TYC2',StrNameTychoCat) > 0
         then
           begin
             StarOneCur.TycNumber:=NumInRegion; // Tycho2 star
             StarOneCur.TrcNumber:=0;           // not TRC1 star
           end
         else
           begin
             StarOneCur.TycNumber:=0;           // not Tycho2 star
             StarOneCur.TrcNumber:=NumInRegion; // TRC1 star
           end;
       StarOneCur.HipNumber:=0; // unknown
       StarOneCur.FK5Number:=0; // unknown
       StarOneCur.Alpha:=ra/3600.00000000000e3;     // in degree
       StarOneCur.Delta:=sd/3600.00000000000e3-90;  // in degree
       StarOneCur.AlphaMotion:=1.0e-4*pma; { in arcsec/year }
       StarOneCur.DeltaMotion:=1.0e-4*pmd; { in arcsec/year }
       StarOneCur.Bt:=1.0e-3*bt;
       StarOneCur.Vt:=1.0e-3*vt;
     End;
   ToFindOtherNumbers(NumBrStars); // this may be Hipparcos and FK5 star
 End;

 Procedure ToDeCodeStarRecB ;
 Begin
   With  BrStarRec  Do
     Begin
       StarOneCur.TycNumber:=0; // no numbers from Tycho2 catalogue
       StarOneCur.TrcNumber:=0; // no numbers from TRC1 catalogue
       StarOneCur.HipNumber:=id; // may be Hipparcos star
       StarOneCur.FK5Number:=FK5num; // may be star from FK5
       StarOneCur.Alpha:=ra/3600.00000000000e3; // right ascension in degree
       StarOneCur.Delta:=sd/3600.0000000000e3-90; // declination in degree
       StarOneCur.AlphaMotion:=1.0e-5*pma; { in arcsec/year }
       StarOneCur.DeltaMotion:=1.0e-5*pmd; { in arcsec/year }
       StarOneCur.Bt:=1.0e-3*bt; // magnitude band B
       StarOneCur.Vt:=1.0e-3*vt; // magnitude band Visual
     End;
 End;

 { to read the stars from TYCHO file to Heap  StarHeap
   AlphaC DeltaC X Y inDegC is global variables defined in UnGloVar }

 Procedure ToReadStarsFromT ( NumBrStars  :  LongInt ;
                              Var  NumCurStars  : Word ) ;
 Var
   Index,IndA,NumCS   : Byte ;
   MinOneA,MaxOneA,
   MinTwoA,MaxTwoA,
   MinD,MaxD,
   n,NAlpha,NDelta    : Word ;
   Ksi,Eta,DifA,DifD  : Extended ;
   APlus,AMinus,
   DPlus,DMinus,
   CosDelta           : Extended ;
   StarV              : Single ;
   FK5Star            : Word ;
   FK5Constellation   : ShortString ;
   ChOne,ChTwo        : Char ;
   NumOfStarInReg     : Word ;
   NumRegion,
   NumInRegion,
   NumInRegOne,
   NumInRegTwo        : LongInt ;
 Begin
   MinD:=0; MaxD:=0;
   MinTwoA:=0; MaxTwoA:=0;
   CosDelta:=Cos(GraRad*DeltaC);
   If  CosDelta < 1.0e-4  Then
     Begin
       AMinus:=0; APlus:=360;
     End
                          Else
     Begin
       AMinus:=AlphaC-2*XinDegC/CosDelta; // XinDegC is defined in UnGloVar
       APlus:=AlphaC+2*XinDegC/CosDelta;
       If  ( ( APlus - AMinus ) > 60 )  Then
         Begin
           AMinus:=0; APlus:=360;
         End;
     End;
   If  AMinus < 0
     Then
       Begin   { two interval on Alpha }
         IndA:=2;
         MinOneA:=Trunc(360+AMinus);  MaxOneA:=359;
         MinTwoA:=0;  MaxTwoA:=Trunc(APlus);
       End
     Else
       If  APlus > 360
         Then
           Begin   { two interval on Alpha }
             IndA:=2;
             MinOneA:=Trunc(AMinus);  MaxOneA:=359;
             MinTwoA:=0;  MaxTwoA:=Trunc(APlus-360);
           End
         Else
           Begin  { one interval on Alpha }
             IndA:=1;
             MinOneA:=Trunc(AMinus); MaxOneA:=Trunc(APlus);
             If  MaxOneA = 360  Then  MaxOneA:=359;
           End;
   DMinus:=90+DeltaC-2*YinDegC; // YinDegC form UnGloVar
   DPlus:=90+DeltaC+2*YinDegC;
   If  ( ( DMinus < 0 ) OR (DPlus > 180 ) )
     Then
       Begin
         If  DMinus < 0
           Then
             Begin
               MinD:=0; MaxD:=Trunc(DPlus+1.5);
             End;
         If  DPlus > 180
           Then
             Begin
               MinD:=Trunc(DMinus-1.5); MaxD:=179;
             End;
         IndA:=1;
         MinOneA:=0; MaxOneA:=359;
       End
     Else
       Begin
         MinD:=Trunc(DMinus); MaxD:=Trunc(DPlus);
         If  MaxD = 180  Then  MaxD:=179;
       End;
   { dimension of the region on Alpha and delta is known }
   NumCurStars:=NumBrStars;
   n:=NumBrStars; // all bright stars are in memory
   For NDelta:=MinD To MaxD Do
     Begin
       NAlpha:=MinOneA;
       NumRegion:=360*NDelta+NAlpha;
       Seek(WFile,NumRegion);
       Read(WFile,NumInRegOne);
       NAlpha:=MaxOneA;
       NumRegion:=360*NDelta+NAlpha;
       Seek(WFile,NumRegion+1);
       Read(WFile,NumInRegTwo);
       NumOfStarInReg:=NumInRegTwo-NumInRegOne;
       If  NumOfStarInReg > 0  Then
         For NumInRegion:=NumInRegOne To NumInRegTwo-1  Do
           Begin
             Seek(RFile,NumInRegion);
             Read(RFile,StarRecT);
             ToDeCodeStarRecT(NumBrStars,NumInRegion);
             With  StarOneCur  Do
               FromSphericToIdealCoordinates(Alpha,Delta,
                                             AlphaC,DeltaC,Ksi,Eta);
             DifA:=RadGra*Ksi;
             DifD:=RadGra*Eta;
             If  ( ( Abs(DifA) < 4*XinDegC ) // XinDegC YinDegC from UnGloVar
                AND ( Abs(DifD) < 4*YinDegC ) )  Then
               Begin
                 StarV:=StarOneCur.Vt;
                 If  StarV < VAllMin  Then  VAllMin:=StarV;
                 If  StarV > VAllMax  Then  VAllMax:=StarV;
                 If  ( ( VMagMin <= StarV ) AND ( StarV <= VMagMax ) )
                   Then  { to select stars within magnitude }
                     Begin
                       n:=n+1;
                       If  n > MaxStarInHeap  Then  n:=MaxStarInHeap;
                       ToPutStarToHeap(n);
                     End;
               End;
           End;
     End;
   If  IndA = 2  Then
     For NDelta:=MinD To MaxD Do
       Begin
         NAlpha:=MinTwoA;
         NumRegion:=360*NDelta+NAlpha;
         Seek(WFile,NumRegion);
         Read(WFile,NumInRegOne);
         NAlpha:=MaxTwoA;
         NumRegion:=360*NDelta+NAlpha;
         Seek(WFile,NumRegion+1);
         Read(WFile,NumInRegTwo);
         NumOfStarInReg:=NumInRegTwo-NumInRegOne;
         If  NumOfStarInReg > 0  Then
           For NumInRegion:=NumInRegOne To NumInRegTwo-1  Do
             Begin
               Seek(RFile,NumInRegion);
               Read(RFile,StarRecT);
               ToDeCodeStarRecT(NumBrStars,NumInRegion);
               With  StarOneCur  Do
                 FromSphericToIdealCoordinates(Alpha,Delta,
                                               AlphaC,DeltaC,Ksi,Eta);
               DifA:=RadGra*Ksi;
               DifD:=RadGra*Eta;
               If  ( ( Abs(DifA) < 4*XinDegC )
                   AND ( Abs(DifD) < 4*YinDegC ) )  Then
                 Begin
                   StarV:=StarOneCur.Vt;
                   If  StarV < VAllMin  Then  VAllMin:=StarV;
                   If  StarV > VAllMax  Then  VAllMax:=StarV;
                   If  ( ( VMagMin <= StarV ) AND ( StarV <= VMagMax ) )
                     Then  { to select stars within magnitude }
                       Begin
                         n:=n+1;
                         If  n > MaxStarInHeap  Then  n:=MaxStarInHeap;
                         ToPutStarToHeap(n);
                       End;
                 End;
             End;
       End;
   NumCurStars:=n;
 End;

 { to read the stars from simple bright stars file to Heap  StarHeap }

 function ToGetBrightStars  : LongInt ;
 Var
   n  : Word ;
 Begin
   n:=0;
   Seek(BrStarFile,0);
   While  not EOF(BrStarFile)  Do
     Begin
       Inc(n);
       Read(BrStarFile,BrStarRec);
       ToDeCodeStarRecB ;
       If  n > MaxStarInHeap  Then  n:=MaxStarInHeap;
       ToPutStarToHeap(n);
     End;
   ToGetBrightStars:=n;
 End;

 END.
