
 UNIT PUnDE200 ; { for popular ephemeris Jet Propulsion Laboratory }

 INTERFACE

 Uses
   UnConTyp , //
   UnGloVar ; // for type TAllSystemRec

 Const
   MassRatio    = 0.012300034 ;
   CoefMassRatio= MassRatio/(1+MassRatio) ;

 Const
   NumberOfPlanets = 11 ;
   RadiusOfPlanet  : Array [1..NumberOfPlanets] of Extended  =
                   (      2440.0 ,
                          6052.0 ,
                          6378.0 ,
                          3394.0 ,
                         71400.0 ,
                         60000.0 ,
                         25900.0 ,
                         24800.0 ,
                          2000.0 ,
                          1737.0 ,
                        690990.0 ) ;

 Const
   NumCoefOrd : array[1..NumberOfPlanets] of smallint =
            ( 2, 146, 182, 272, 302, 329, 353, 377, 395, 413, 701 );
   NumbofCoef : array[1..NumberOfPlanets] of smallint =
            (12,  12,  15,  10,   9,   8,   8,   6,   6,  12,  15 );
   NumbofDay  : array[1..NumberOfPlanets] of smallint =
            ( 8,  32,  16,  32,  32,  32,  32,  32,  32,   4,  32 );
   NumbofCoor : array[1..NumberOfPlanets] of smallint =
            ( 3,   3,   3,   3,   3,   3,   3,   3,   3,   3,   3 );
   NMercury  = 1;
   NVenus    = 2;
   NEarth    = 3;
   NMars     = 4;
   NJupiter  = 5;
   NSaturn   = 6;
   NUranus   = 7;
   NNeptune  = 8;
   NPluto    = 9;
   NMoon     = 10;
   NSun      = 11;

   PlanetNameL   : Array [1..NumberOfPlanets] of String[12] =
                   ( 'Mercury' , 'Venus' , 'Earth' , 'Mars' ,
                     'Jupiter' , 'Saturn' , 'Uranus' , 'Neptune' ,
                                 'Pluto' , ' Moon' , 'Sun' );

   PlanetNameR    : Array [1..NumberOfPlanets] of String[12] =
                   ( '' , '' , '' , '' ,
                       '' , '' ,  '' , '' ,
                                  '' ,  '' , '' ) ;

 Type
   TypeCoef745       = Array [1..745] of Double ;

 { to read ephemeris DE200 }
 Var
   CoeFile                      : File of TypeCoef745;
   CoefEph                      : TypeCoef745;
   CoeFileName                  : ShortString;
   TinitEph,TlastEph            : Extended ;
   NofRec,irecmax               : Integer;
   BooOpenCoeFile,
   BooExistEph,BooExistPos      : Boolean ;

 Var
   PosTopSun  : TVect3 ;
   PosDim     : Array [1..NumberOfPlanets] of TVect3 ;
   MagVOneP   : Single ; 
   PhasOneP   : Single ;
   PosSystem  : Array [1..NumberOfPlanets] of TAllSystemRec ;

 Procedure ClcPosVel(NPlanet : Byte ;
                     TinTDB  : Extended ;
                     Var  Pos , Vel : TVect3 ) ;

 Procedure ToGetCoeFileName ; { all variables are external }

 Procedure ToReaDEphemeris ( TinTDB  : Extended ) ;

 Procedure ClcEphEarth( TinTDB : Extended ;
                        Var  PosEarth , VelEarth : TVect3 ) ;

 IMPLEMENTATION

 Uses
   UnFilIni ;

 { ClcPosVel is a procedure
   to obtain the position of the Planet
      reffered to the barycentre from DE200/LE200 Ephemeris }

 Procedure ClcPosVel(NPlanet : Byte ;
                     TinTDB  : Extended ;
                     Var  Pos , Vel : TVect3 ) ;
 Var
   PolQuanT,PolQuanU : array[0..15] of Extended;
   CoefQuant : array[1..3,0..15] of Extended;
   Tn,Tk,Tnn,Tkk,t,r,w,rw,x,xw,xr,xv : Extended;
   i,i1,i2,j,k,l,l0,m,n,np : integer;
 Begin
   ToReadEphemeris(TinTDB);
   If  NOT BooExistPos
     Then
       Exit ;
   k:=NumCoefOrd[NPlanet]; l:=NumbofCoef[NPlanet];
   l0:=l-1; m:=NumbofDay[NPlanet]; w:=m; rw:=2/w;
   t:=TinTDB; Tn:=CoefEph[1]; Tk:=Tn+32;
   r:=(t-Tn)/m; n:=Trunc(r-1.0e-10); np:=k+3*l*n-1;
   Tnn:=Tn+m*n; Tkk:=Tnn+m; x:=-1+(t-Tnn)*rw; xw:=2*x;
   for i:=1 to 3 do   for j:=0 to l0 do   begin
       np:=np+1; CoefQuant[i,j]:=CoefEph[np]; end;
   PolQuanT[0]:=1; PolQuanT[1]:=x; PolQuanU[0]:=1; PolQuanU[1]:=xw;
   for i:=2 to l do Begin  i1:=i-1; i2:=i-2;
       PolQuanT[i]:=xw*PolQuanT[i1]-PolQuanT[i2];
       PolQuanU[i]:=xw*PolQuanU[i1]-PolQuanU[i2]; End;
   for i:=1 to 3 do  Begin
       xr:=CoefQuant[i,0]; xv:=0.0;
       for j:=1 to l0 do  Begin
           xr:=xr+CoefQuant[i,j]*PolQuanT[j];
           xv:=xv+j*CoefQuant[i,j]*PolQuanU[j-1];  End;
       Pos[i]:=xr; Vel[i]:=rw*xv;  End;
 End;

 { ToGetCoeFileName is a procedure
   to define name of the file for DE200/LE200 ephemeris }

 Procedure ToGetCoeFileName ; { all variables are external }
 Const
   numfile   = 3 ;
   namefile  : Array [1..numfile] of String [12] =
             ( 'EphTheoL.dat'  ,
               'EphemAll.jpl' , 'Ephem11.jpl'  ) ;
 Var
   num  : Byte ;
   Pos  : LongInt;
   boo  : Boolean ;
   ior  : Integer ;
   stn  : String ;
 Begin
   BooOpenCoeFile:=FALSE;
   BooExistEph:=FALSE;
   BooExistPos:=FALSE;
   num:=0;
   Repeat  
     num:=num+1;
     stn:=namefile[num];
     CoeFileName:=stn;
     Assign(CoeFile,CoeFileName);
 {$I-}
     ReSet(CoeFile);
 {$I+}
     ior:=IOResult;
     If  ior IN [ 0 , 5 ]
       Then
         BooExistEph:=TRUE
       Else
         Begin
           ToUseSomePath(stn,CoeFileName,boo);
           If  boo
             Then
               Begin
                 Assign(CoeFile,CoeFileName);
 {$I-}
                 ReSet(CoeFile);
 {$I+}
                 ior:=IOResult;
                 If  ior IN [ 0 , 5 ]
                   Then
                     BooExistEph:=TRUE;
               End;
         End;
     If  ior = 5
       Then { file read only }
         Begin
           FileMode:=0;
           ReSet(CoeFile); { file is opened for reading }
           FileMode:=2;
         End;
   Until  ( ( BooExistEph ) OR ( num = numfile ) ) ;
   If  NOT BooExistEph
     Then { no ephemeris file DE200/LE200 }
       Exit;
   { file is opened DE200/LE200 }
   Pos:=0; Seek(CoeFile,Pos);
   Read(CoeFile,CoefEph); TinitEph:=CoefEph[1];
   Pos:=FileSize(CoeFile)-1; Seek(CoeFile,Pos);
   Read(CoeFile,CoefEph); TlastEph:=CoefEph[1]+32;
   BooOpenCoeFile:=TRUE;
   NofRec:=-1;
   irecmax:=FileSize(CoeFile)-1;
 End;

 { to read file with the DE200 data }

 Procedure ToReaDEphemeris ( TinTDB  : Extended ) ;
 Var
   irec    : Integer ;
   r       : Extended;
 Begin
   BooExistPos:=FALSE; { previous }
   If  NOT BooOpenCoeFile
     Then
       ToGetCoeFileName;
   If  NOT BooExistEph
     Then { no ephemeris file DE200/LE200 }
       Exit;
   If  ( ( TinitEph <= TinTDB ) AND ( TinTDB <= TlastEph ) )
     Then
       Begin
         r:=(TinTDB-TinitEph)/32.e0;
         irec:=Trunc(r);
         If  irec > irecmax
           Then
             irec:=irecmax;
         If  irec <> NofRec
           Then
             Begin
               Seek(CoeFile,irec);
               Read(CoeFile,CoefEph);
               NofRec:=irec;
             End;
         BooExistPos:=TRUE; { there are ephemeris data }
       End;
  { DE200/LE200 Ephemeris has been read in massive  CoefEph(745) }
 End;

 { ClcEphEarth is a procedure
   to obtain the position of the Earth reffered to the barycenter
      of the Solar system from DE200/LE200 Ephemeris }

 Procedure ClcEphEarth ( TinTDB : Extended ;
                         Var  PosEarth , VelEarth : TVect3 ) ;
 Var
   PosMoon,VelMoon,
   PosBarEM,VelBarEM : TVect3 ;
   i                 : Byte ;
 Begin
   ClcPosVel(NMoon,TinTDB,PosMoon,VelMoon);
   If  NOT BooExistPos
     Then { no ephemeris for this date TinTDB }
       Exit ;
   ClcPosVel(NEarth,TinTDB,PosBarEM,VelBarEM);
   For i:=1 To 3 Do
     Begin
       PosEarth[i]:=PosBarEM[i]-CoefMassRatio*PosMoon[i];
       VelEarth[i]:=VelBarEM[i]-CoefMassRatio*VelMoon[i];
     End;
 End;

 END.
