unit UnButIni;

interface

uses
  Graphics , // for TColor
  SysUtils ; // for IntToStr function

const
  NumButt  = 22 ; // number of simple buttons

const
  ChButS  : array[1..NumButt] of Char =
          ( 'F' ,      // folder and name for catalogue
            '&' ,      // anchor & or L to view from satellite
            '1' , '2' , 'U' ,  // 1 for date 2 for time U for UTC
            'P' , 'C' , 'E' , 'T' , // P 6 , C 7 , E 8 , T 9
            '+' , '-' ,             // forward 10 , backward 11
            'S' , // for step
            'H' , // help
            'Q' , // quit
            'X' , // X exit top panel 15 buttons 12 13 14 15
            'Y' ,  // Y centre objects position small rectangle
            '''' , // ' for sat ephemeris
            '#' , // # triangular
            '%' , // 19 size of area   bottom panel 7 buttons 15+7 = 22
            'I' , ' ' , // I ICRS information about objects
            'M' ) ;    // M for world map
  ChButW  : array[1..NumButt] of Char =
          ( 'V' , 'V' ,
            '1' , ' ' , ' ' ,       // 1 for date
            'L' , ' ' , ' ' , ' ' , // L as & for look from satellite
            '+' , '-' ,
            'S' , 'H' , 'Q' , 'X' , // top panel 15 buttons 12 13 14 15
            'A' , '''' ,
            ' ' ,
            'A' ,
            'A' , ' ' , // bottom panel 7 buttons 15+7 = 22
            'M' ) ;    // M for world map
  ChButL  : array[1..NumButt] of Char = // to look from the satellite
          ( 'F' ,                   // folder and name for catalogue
            '&' ,                   // blank
            '1' , '2' , 'U' ,       // 1 for date 2 for time U for UTC
            'L' , 'C' , 'E' , 'T' , // L or & 6 , C 7 , E 8 , T 9
            '+' , '-' ,             // forward 10 , backward 11
            'S' , // for step
            'H' , // help
            'Q' , // quit
            'X' , // X exit top panel 15 buttons 12 13 14 15
            'Y' , // Y centre objects position small rectangle
            ' ' ,
            '#' , // # triangular
            '%' , // 19 size of area   bottom panel 7 buttons 15+7 = 22
            ' ' , ' ' ,
            'M' ) ;    // M to the sky backward
type
  TRectBut  = Record
                x1,x2,y1,y2  : Integer ;
              End;
  TButtRec  = Record // for position of a button
                cb  : Char ; // one symbol for button
                sb  : ShortString ; // special string for button
                sh  : ShortString ; // hint for button
                rb  : TRectBut ; // rectangle
                bo  : Boolean ; // yes or no hint
              End;
  TParmSky  = Record // type with some integers for figure
                xh , yh  : Integer ; // position of the half of SkyRect
                xa , xb  : Integer ; // position left right for x
                ya , yb  : Integer ; // position top bottom for y
                dx , dy  : Integer ; // maximum distance from the half
                rx , ry  : ShortInt ; // for reversal x or y position
              End;

var
  UpperR    : TRectBut ; // for small strip top
  BottomR   : TRectBut ; // for small strip bottom
  SkyRect   : TRectBut ; // for rectangular for sky view
  SkyParm   : TParmSky ; // some integeres for figure in SkyRect
  AllButt   : array[1..NumButt] of TButtRec ;
  HintButt  : array[1..NumButt] of ShortString ;
  SpecButt  : array[1..NumButt] of ShortString ;
  LongButt  : array[1..NumButt] of Byte ;
  SkyBooX   : Boolean ;

function PointInRect ( x , y  : Integer ;  r  : TRectBut ) : Boolean ;

procedure HintForButtons ;

procedure SpecForButtons ;

procedure TopPanelButtons ; // unit UnButIni

procedure BotPanelButtons ; // bottom panel

procedure ForHintOfSize ; // when X Y inDegC are known

procedure HintForStarOnScreen ; // when value NumStarsInRect is known

procedure ForSunHightButton ( ch  : Char ;  alt  : Extended ) ;

procedure ForPositionButton ( s  : ShortString ;  a , d  : Extended ) ;

procedure ToScreenPosButt ( s  : ShortString ;  a , d  : Extended ) ;

procedure ToScreenCnsButt ( a , d  : Extended ) ; // for constellation

implementation

uses
  UnitMain , // for fmPuz
  UnConTyp , // for const GraRad
  UnCoTVar , // for var StrNameTychoCat
  UnFigDat , // for InitDateTime for JulianDateToStr
  UnGloVar , // for global variable JulianDate
  PUnCRead , // for boolean variable BrightStarsOnly
  UnForStr , // for simple function BlanksToStr
  UnForFun , // for procedure PrDegrAngle
  UnitFixO , // for stFixObject name of the object that fixed
  UW_Cns   , // for procedure Identify (constellation)
  UW_CnsT  ; // for DimConStel with russian and latinian names

function PointInRect ( x , y  : Integer ;  r  : TRectBut ) : Boolean ;
begin // type TRectBut from UnFigIni  x1 , x2 , y1 , y2  integer
  if  ( ( ( r.x1 < x ) and ( x < r.x2 ) )
       and ( ( r.y1 < y ) and ( y < r.y2 ) ) )
    then // current point x ,y is in this rectangle
      PointInRect:=True
    else
      PointInRect:=False;
end;

procedure HintForButtonsSky ;
begin
  HintButt[1]:='';
  HintButt[2]:=' ';
  HintButt[6]:='P   ';
  HintButt[7]:='C   ';
  HintButt[8]:='E     ';
  HintButt[9]:='T  ';
  HintButt[16]:=' ';
  HintButt[17]:=' ';
  HintButt[18]:='  ';
  HintButt[19]:=' ';
  HintButt[20]:='';
  HintButt[21]:=' ';
end;

procedure HintForButtonsMap ;
begin
  HintButt[1]:=''; // 'F'
  HintButt[2]:='';
  HintButt[6]:='  '; // '&' or 'L'
  HintButt[7]:='';
  HintButt[8]:='';
  HintButt[9]:=''; // 'T'
  HintButt[16]:=' ';
  HintButt[17]:='';
  HintButt[18]:='';
  HintButt[19]:='';
  HintButt[20]:='';
  HintButt[21]:='';
end;

procedure HintForButtonsLook ;
begin
  HintButt[1]:='';
  HintButt[2]:='   ';
  HintButt[6]:='  '; // '&'
  HintButt[7]:='C   ';
  HintButt[8]:='E     ';
  HintButt[9]:='T  ';
  HintButt[16]:=' ';
  HintButt[17]:='';
  HintButt[18]:=' ';
  HintButt[19]:=' ';
  HintButt[20]:='';
  HintButt[21]:=' ';
end;

procedure HintForButtons ;
begin
  HintButt[3]:='';
  HintButt[4]:='';
  HintButt[5]:='  ';
  HintButt[10]:='+   ';
  HintButt[11]:='-   ';
  HintButt[12]:='S     ';
  HintButt[13]:='H  ';
  HintButt[14]:='Q    ';
  HintButt[15]:='X   ';
  HintButt[22]:=' ';
  case  CharForView  of  // character 'S' 'M' 'L from UnGloVar
    'S' : HintForButtonsSky;
    'W' : HintForButtonsMap;
    'L' : HintForButtonsLook;
  end;
end;

procedure SpecForButtonsSky ;
var
  s  : ShortString ;
begin
  if  BrightStarsOnly // boolean variable from PUnCRead
    then  SpecButt[1]:='FK5+ '
    else  SpecButt[1]:=StrNameTychoCat+' '; // is declared UnCoTVar
  s:=PlaceCoor.Name;
  SpecButt[2]:=BlanksToStr(s,12)+' '; // simple function from UnForStr
  ForPositionButton('',AzimutC,AltitudeC); // var from UnGloVar
  SpecButt[19]:='  ';
  SpecButt[20]:='         ';
  SpecButt[21]:=' ';
  SpecButt[22]:=' ';
end;

procedure SpecForButtonsMap ;
begin
  SpecButt[1]:='    ';
  SpecButt[2]:=BlanksToStr(stFixObject,12)+' '; //  function from UnForStr
  ForPositionButton('',45.0,0.0); // var from UnGloVar
  SpecButt[19]:='      ';
  SpecButt[20]:='                  ';
  SpecButt[21]:='        ';
  SpecButt[22]:=' ';
end;

procedure SpecForButtonsLook ;
var
  s  : ShortString ;
begin
  if  BrightStarsOnly // boolean variable from PUnCRead
    then  SpecButt[1]:='FK5+ '
    else  SpecButt[1]:=StrNameTychoCat+' '; // is declared UnCoTVar
  s:=SatElemLook.satname; // global var from UnCotVar
  SpecButt[2]:=BlanksToStr(s,12)+' '; // simple function from UnForStr
  ForPositionButton('',AzimutC,AltitudeC); // var from UnGloVar
  SpecButt[19]:='  ';
  SpecButt[20]:='         ';
  SpecButt[21]:='    ';
  SpecButt[22]:='   ';
end;

procedure SpecForButtons ;
var
  n  : Byte ;
begin
  JulianDateToStr(JulianDate, // unit UnFigDat , JulianDate from UnGloVar
                  SpecButt[3],HintButt[3],SpecButt[4],HintButt[4]);
  SpecButt[5]:='UTC ';
  SpecButt[6]:='';
  SpecButt[7]:='';
  SpecButt[8]:='';
  SpecButt[9]:='';
  SpecButt[10]:='';
  SpecButt[11]:='';
  SpecButt[12]:='';
  SpecButt[13]:='';
  SpecButt[14]:='';
  SpecButt[15]:='';
  SpecButt[16]:=' ';
  SpecButt[17]:='283 24 11.9  ';
  SpecButt[18]:='+ 2 18 42.3 ';
  case  CharForView  of  // character 'S' 'M' 'L' from UnGloVar
    'S' : SpecForButtonsSky;
    'W' : SpecForButtonsMap;
    'L' : SpecForButtonsLook;
  end;
  for n:=1 to NumButt do  LongButt[n]:=Length(SpecButt[n]);
end;

procedure SimpTopButt ;
var
  n  : Byte ;
  x  : Integer ;
  w  : Integer ;
  r  : TRectBut ;
begin
  w:=fmPuz.Canvas.TextWidth('X');
  fmPuz.Canvas.Brush.Color:=clWhite;
  x:=UpperR.x2;
  for n:=15 downto 6 do
    begin
      case  CharForView  of
        'S' : AllButt[n].cb:=ChButS[n];
        'W' : AllButt[n].cb:=ChButW[n];
        'L' : AllButt[n].cb:=ChButL[n];
      end;
      AllButt[n].sb:=SpecButt[n];
      AllButt[n].sh:=HintButt[n];
      AllButt[n].rb.y1:=UpperR.y1;
      AllButt[n].rb.y2:=UpperR.y2;
      AllButt[n].rb.x2:=x;
      AllButt[n].rb.x1:=x-3*w;
      AllButt[n].bo:=FALSE;
      x:=x-3*w;
      r:=AllButt[n].rb;
      fmPuz.Canvas.Pen.Color:=clLime;
      fmPuz.Canvas.Rectangle(r.x1,r.y1,r.x2,r.y2);
      fmPuz.Canvas.Font.Color:=clBlack;
      fmPuz.Canvas.TextOut(r.x1+w,r.y1+2,AllButt[n].cb);
    end;
end;

procedure TopSymbolColor ( w  : Integer ) ; // clSilver for h m s AllButt[4]
var
  x,y  : Integer ;
begin
  x:=AllButt[4].rb.x1+w;
  y:=AllButt[4].rb.y1+2;
  fmPuz.Canvas.Font.Color:=clSilver;
  x:=x+2*w; // two symbol forward for hour
  fmPuz.Canvas.TextOut(x,y,'h');
  x:=x+3*w; // two symbol forward for minute
  fmPuz.Canvas.TextOut(x,y,'m');
  x:=x+6*w; // two symbol forward for second
  fmPuz.Canvas.TextOut(x,y,'s');
end;

procedure SpecTopButt ;
var
  n  : Byte ;
  x  : Integer ;
  w  : Integer ;
  r  : TRectBut ;
begin
  w:=fmPuz.Canvas.TextWidth('O');
  fmPuz.Canvas.Brush.Color:=clNavy;
  x:=UpperR.x1;
  for n:=1 to 5 do
    begin
      case  CharForView  of
        'S' : AllButt[n].cb:=ChButS[n];
        'W' : AllButt[n].cb:=ChButW[n];
        'L' : AllButt[n].cb:=ChButL[n];
      end;
      AllButt[n].sb:=SpecButt[n];
      AllButt[n].sh:=HintButt[n];
      AllButt[n].rb.y1:=UpperR.y1;
      AllButt[n].rb.y2:=UpperR.y2;
      AllButt[n].rb.x1:=x;
      AllButt[n].rb.x2:=x+LongButt[n]*w;
      AllButt[n].bo:=FALSE;
      x:=x+LongButt[n]*w;
      r:=AllButt[n].rb;
      if  n = 2
        then
          fmPuz.Canvas.Font.Color:=clLime
        else
          if  n = 5
            then
              fmPuz.Canvas.Font.Color:=clSilver
            else
              fmPuz.Canvas.Font.Color:=clWhite;
      fmPuz.Canvas.TextOut(r.x1+w,r.y1+2,AllButt[n].sb);
    end;
  TopSymbolColor(w); // clSilver for h m s in AllButt[4]
end;

procedure TopPanelButtons ; // to fix buttons for top panel
begin
  fmPuz.Canvas.Font.Style:=[fsBold];
  SimpTopButt ;
  SpecTopButt ;
end;

procedure BotSymbolColor ( w  : Integer ) ; // clSilver for o ' ''
var  // for AllButt[17] azimut AllButt[18] altitude
  x,y  : Integer ;
begin
  x:=AllButt[17].rb.x1+w;  // .283 24 11.9  .
  y:=AllButt[17].rb.y1;
  fmPuz.Canvas.Font.Color:=clSilver;
  x:=x+3*w; // three symbols forward for degree
  fmPuz.Canvas.TextOut(x,y+2,'');
  x:=x+3*w; // three symbols forward for minute
  fmPuz.Canvas.TextOut(x,y+2,'''');
  x:=x+5*w; // five symbols forward for arc second
  fmPuz.Canvas.TextOut(x,y+2,'"');
  x:=AllButt[18].rb.x1+w; // .+ 2 18 42.3 .
  y:=AllButt[18].rb.y1;
  fmPuz.Canvas.Font.Color:=clSilver;
  x:=x+3*w; // three symbols forward for degree
  fmPuz.Canvas.TextOut(x,y+2,'');
  x:=x+3*w; // three symbols forward for minute
  fmPuz.Canvas.TextOut(x,y+2,'''');
  x:=x+5*w; // five symbols forward for arc second
  fmPuz.Canvas.TextOut(x,y+2,'"');
end;

procedure SpecBotButt ;
var
  k  : Byte ;
  n  : Byte ;
  a  : Integer ;
  x  : Integer ;
  z  : Integer ;
  w  : Integer ;
  r  : TRectBut ;
begin
  w:=fmPuz.Canvas.TextWidth('O');
  fmPuz.Canvas.Brush.Color:=clNavy;
  x:=BottomR.x1;
  z:=BottomR.x2-w-w div 2;
  for k:=1 to 7 do
    begin
      if  k < 4  then  n:=15+k  else  n:=26-k;
      case  CharForView  of
        'S' : AllButt[n].cb:=ChButS[n];
        'W' : AllButt[n].cb:=ChButW[n];
        'L' : AllButt[n].cb:=ChButL[n];
      end;
      AllButt[n].sb:=SpecButt[n];
      AllButt[n].sh:=HintButt[n];
      AllButt[n].rb.y1:=BottomR.y1;
      AllButt[n].rb.y2:=BottomR.y2;
      AllButt[n].bo:=FALSE;
      a:=LongButt[n]*w;
      if  k < 4
        then
          begin
            AllButt[n].rb.x1:=x;
            AllButt[n].rb.x2:=x+a;
            x:=x+a;
          end
        else
          begin
            AllButt[n].rb.x2:=z;
            AllButt[n].rb.x1:=z-a;
            z:=z-a;
          end;
      r:=AllButt[n].rb;
      if  n in [ 17 , 18 ]
        then
          fmPuz.Canvas.Font.Color:=clYellow
        else
          if  n in [ 16 , 19 , 22 ]
            then
              fmPuz.Canvas.Font.Color:=clSilver
            else
              if  n = 20
                then
                  fmPuz.Canvas.Font.Color:=clLime
                else
                  fmPuz.Canvas.Font.Color:=clWhite;
      fmPuz.Canvas.TextOut(r.x1+w,r.y1+2,AllButt[n].sb); // constellation
    end;
  BotSymbolColor(w);
end;

procedure BotPanelButtons ; // to fix buttons for bottom panel
begin
  SpecBotButt;
  fmPuz.Canvas.Font.Style:=[];
end;

{ 2*XinDegC is size of area in degree for X axis
  2*YinDegC is size of area in degree for Y axis }

procedure ForHintOfSize ; // when X Y inDegC are known
var
  sa,st  : ShortString ;
  id,im  : Integer ;
  x,y,s   : Extended ;
begin                                 // but not for map world
  if  CharForView = 'W'  Then  Exit ; // only for sky and look
  x:=2*GraRad*XinDegC; // to radian const GraRad from UnConTyp
  y:=2*GraRad*YinDegC; // to radian
  PrDegrAngle(x,st,id,im,s); // procedure PrDegrAngle
  sa:=' '+IntToStr(id)+''+IntegerToFixStr(im,2)+'''';
  PrDegrAngle(y,st,id,im,s); // from unit UnForFun
  sa:=sa+' x '+IntToStr(id)+''+IntegerToFixStr(im,2)+'''';
  HintButt[19]:=sa; // function IntegerToFixStr from unit UnForStr
end;

procedure HintForStarOnScreen ; // when value NumStarsInRect is known
begin // NumStarsInRect from UnGloVar and value from ViewPlac UnPlacea
  HintButt[1]:='    '+IntToStr(NumStarsInRect)+' ';
end;

procedure ForSunHightButton ( ch  : Char ;  alt  : Extended ) ;
var
  st  : ShortString ;
begin // SpecButton
  Str(alt:5:1,st); // altitude to string
  case  ch  of
   'D' : begin
           SpecButt[21]:='    ';
           HintButt[21]:='  '+st+'';
         end;
   'T' : begin
           SpecButt[21]:=' ';
           HintButt[21]:='  '+st+'';
         end;
   'N' : begin
           SpecButt[21]:='    ';
           HintButt[21]:='  '+st+'';
         end;
  end; // case  ch
end;

procedure ForPositionButton ( s  : ShortString ;  a , d  : Extended ) ;
var
  st  : ShortString ;
  id,im  : Integer ;
  sec  : Extended ;
begin // to place information linked with SpecButt[16] [17] [18] to screen
  SpecButt[16]:=BlanksToStr(s,7);
  PrDegrAngle(GraRad*a,st,id,im,sec); // procedure PrDegrAngle from UnForFun
  SpecButt[17]:=IntegerToFixStr(id,3)+' '  // simple string function
               +IntegerToFixStr(im,2)+' ' // from unit UnForStr
               +FloatToFixStr(sec,4,1)+'  ';
  PrDegrAngle(GraRad*d,st,id,im,sec); // from unit UnForFun
  SpecButt[18]:=st+IntegerToFixStr(id,2)+' '  // simple string function
               +IntegerToFixStr(im,2)+' ' // from unit UnForStr
               +FloatToFixStr(sec,4,1)+' ';
end;

procedure ToScreenPosButt ( s  : ShortString ;  a , d  : Extended ) ;
var
  n  : Byte ;
  w  : Integer ;
  r  : TRectBut ;
begin // topocentric a d azimuth altitude in degree
  ForPositionButton(s,a,d); // a d in degree
  w:=fmPuz.Canvas.TextWidth('O');
  fmPuz.Canvas.Brush.Color:=clNavy;
  for n:=16 to 18 do
    begin
      r:=AllButt[n].rb;
      if  n in [ 17 , 18 ]
        then
          fmPuz.Canvas.Font.Color:=clYellow  // n = 17 18
        else
          fmPuz.Canvas.Font.Color:=clSilver; // n = 16
      fmPuz.Canvas.TextOut(r.x1+w,r.y1+2,SpecButt[n]);
    end;
  BotSymbolColor(w);
end;

procedure ToScreenCnsButt ( a , d  : Extended ) ; // for constellation
var // Identify from UW_Cns SpecButt[20]:='       ';  a , d
  n  : Byte ;
  w  : Integer ;
  r  : TRectBut ;
  s  : ShortString ; // three symbol for identification s='And' for example
  z  : ShortString ;
begin // a d in degree topocentric alpha delta refer to fixed equator
  if  CharForView = 'W'  then  Exit ; // only for sky and look
  SpecButt[20]:='       '; // 17 symbols
  HintButt[20]:=''; // a d from degree to radian
  Identify(2000.0e0,GraRad*a,GraRad*d,s); // procedure from unit UW_Cns
  for n:=1 To 88 do
    if  DimConStel[n].LatN = s // const DimConStel from unit UW_CnsT
      then
        begin
          z:=DimConStel[n].RusN; // russian name from UW_CnsT
          SpecButt[20]:=BlanksToStr(z,18); // function from UnForStr
          HintButt[20]:=' '+DimConsteLatin[n].Latin; // latinian
        end;
  w:=fmPuz.Canvas.TextWidth('O');
  fmPuz.Canvas.Brush.Color:=clNavy;
  r:=AllButt[20].rb; // place for constellation
  fmPuz.Canvas.Font.Color:=clLime;  // color for text n = 20
  fmPuz.Canvas.TextOut(r.x1+w,r.y1+2,SpecButt[20]);
  AllButt[20].sh:=HintButt[20];
end;

end.
