
unit UnClaMap ; // class for simple map of world

interface

uses
  Graphics , // for TFontStyles
  Windows  , // for TPoint type record with  X , Y  LongInt
  UnitStat ; // for MaxStationL NumStationL StationList

type
  TPointMap  = record  // type for map position of the station
                 x  : Integer ;    // x screen position
                 y  : Integer ;    // y screen position
                 s  : String[14] ; // string for name of station
               end;
  TPointFix  = record  // type for map position for fixed point
                 xf , yf  : Integer ;  // screen position pixels
                 ff , vf  : Extended ; // latitude longitude degree
                 sf  : ShortString ;   // name of the point if it exists
                 sd  : ShortString ;   // day , twilight or night 
               end;

type

  TMapW = class

    curFontName  : ShortString ;
    curFontSize  : Integer ;
    curFontBold  : TFontStyles ;

    procedure ForSizeMap ( x1 , y1 , x2 , y2  : Integer ) ;
    procedure SimpleEllipse ;
    procedure SimpleMeridians ;
    procedure CotangParallels ;
    procedure ForElliMap ;
    function ToGetPhi ( rx , ry  : Extended ) : Extended ;
    procedure CoorToMap ;
    function MapColor ( mc  : Byte ) : TColor ;
    procedure LineToMap ;
    procedure DistinguishPoint ; // if BooMapPoint true from UnGloVar

    private
      mapn  : SmallInt ;
      mapb  : array[1..3113] of Byte ;
      mapf  : array[1..3113] of Single ;
      mapl  : array[1..3113] of Single ;
      procedure PlanetZenith ( cs  : TColor ;
                               fs , vs  : Extended ;
                               var  xs , ys  : Integer ) ;

    public
      NulH,NulW  : Integer ;  // x y position of the centre of the screen
      EllA,EllB  : Extended ; // a and b semi-axis of the ellipse with map
      PointMapS  : array[1..MaxStationL] of TPointMap ; // point map position
      PointMapF  : TPointFix ; // point that fixed at the map
      procedure LineToMem ;
      procedure FilaPoint ( f , v  : Extended ;  var  x , y  : Extended ) ;
      procedure PointFila ( xc , yc  : Integer ; // current position in screen
                            var  bo  : Boolean ; // in ellipse or not
                            var  rf , rl  : Extended ) ; // lati longi tude
      procedure SimpleMapWorld ( x1 , y1 , x2 ,y2  : Integer ) ;
      procedure NightPoints ;
      procedure DayNightLine ( fs , vs , fm , vm  : Extended ;
                               var  xs , ys , xm , ym  : Integer ) ;
      procedure PointName ( xs , ys  : Integer ) ; // from UnMouSky
      procedure AboutDayN ( st  : ShortString ) ;  // from UnMouSky
      procedure AboutSatV ( st  : ShortString ) ;  // from UnMouSky
      procedure ToSelectP ( x , y  : Integer ;  f , v  : Extended ) ;

          end; // class TMapW

var
  MapW  : TMapW ;

implementation

uses
  UnitMain , // for fmPuz
  UnConTyp , // for type TMatr33 TVect3
  UnGloVar , // for boolean var BooMapPoint JulianDate
  UnButIni , // for variable AllBut
  UnForFun , // for procedure SimpleRotMatr
  UnForCoo , // for procedure ClcSpherCoorInDegree
  UnForPos , // function stDayOrNight
  UnForStr ; // for function BlanksToStr

procedure TMapW.ForSizeMap ( x1 , y1 , x2 , y2  : Integer ) ;
var
  rh,rw  : Integer ;
  mw,mh  : Integer ;
begin
  rh:=y2-y1;
  rw:=x2-x1;
  mh:=rh-6;
  mw:=4*(mh div 2);
  if  mw > rw
    then
      begin
        mw:=rw-6;
        mh:=2*(mw div 4);
      end;
  EllB:=mh div 2; // b semi-axis
  EllA:=mw div 2; // a semi-axis
  NulH:=y1+rh div 2; // half of screen y axis
  NulW:=x1+rw div 2; // half of screen x axis
  curFontName:=fmPuz.Canvas.Font.Name; // to memory Font before map
  curFontSize:=fmPuz.Canvas.Font.Size; // to memory Size before map
  curFontBold:=fmPuz.Canvas.Font.Style;
end;

procedure TMapW.SimpleEllipse ;
var
  x1,x2  : Integer ;
  y1,y2  : Integer ;
begin
  x1:=NulW-Round(EllA);
  y1:=NulH-Round(EllB);
  x2:=NulW+Round(EllA);
  y2:=NulH+Round(EllB);
  fmPuz.Canvas.Ellipse(x1,y1,x2,y2);
end;

procedure TMapW.SimpleMeridians ;
var
  x1,x2  : Integer ;
  y1,y2  : Integer ;
  xs,ys  : Integer ;
begin
  y1:=NulH-Round(EllB);
  y2:=NulH+Round(EllB);
  xs:=Round(EllA) div 6;
  x1:=NulW-xs div 2; // meridian 30 degree
  x2:=NulW+xs div 2; // meridian 60 degree
  for ys:=1 to 6 do
    begin
      fmPuz.Canvas.Arc(x1,y1,x2,y2,x2,y2,x2,y2);
      x1:=x1-xs; // meridian from  0 to -120 degree
      x2:=x2+xs; // meridian from 90 to +210 degree
    end;
end;

procedure TMapW.FilaPoint ( f , v  : Extended ;  var  x , y  : Extended ) ;
var // f v in degree  x y position from the centre of the screen
  am,bm  : Extended ; // ellipse for meridians
  ap,bp  : Extended ; // ellipse for parallels
  a,b,c  : Extended ; // for equation a*x^2+b*x+c=0
  rx,ry  : Extended ; // for roots x y of equation
  rf,rl  : Extended ; // the same as latitude longitude in degree
begin // from latitude longitude in degree to x y position
  rf:=f; // latitude  in degree
  rl:=v; // longitude in degree
  if  rl > 225.0  then  rl:=rl-360.0; // this is our condition for map
  bm:=EllB; // axis b of ellipse for meridians
  am:=EllA*Abs(rl-45.0)/180.0; // on equator
  if  Abs(rf) < 0.01
    then // equator
      begin
        if  rl < 45.0  then  x:=-am  else  x:=+am;
        y:=0.0;
        Exit;
      end;
  if  (90-Abs(rf)) < 0.01
    then // pole north or south
      begin
        x:=0.0;
        if  rf < 0.0  then  y:=-bm  else  y:=+bm;
        Exit;
      end;
  bp:=EllB*(90.0-Abs(rf))/90.0; // axis b of ellipse for parallels
  ap:=EllA*Cos(Pi*rf/180.0)/Sin(Pi*rf/180.0); // axis a
  a:=Sqr(am/bm)-Sqr(ap/bp);
  b:=2*bm*Sqr(ap/bp);
  c:=Sqr(ap)-Sqr(am)-Sqr(bm)*Sqr(ap/bp);
  ry:=(-b+Sqrt(Sqr(b)-4*a*c))/(2*a); // positive root
  rx:=Sqrt(Sqr(am)-Sqr(ry)*Sqr(am/bm));
  if  rl < 45.0  then  x:=-rx  else  x:=+rx;
  if  rf <  0.0  then  y:=-ry  else  y:=+ry;
end;

procedure TMapW.CotangParallels ;
var
  ua,za  : Extended ;
  x1,x2  : Integer ;
  y1,y2  : Integer ;
  x3,x4  : Integer ;
  y3,y4  : Integer ;
  rx,ry  : Extended ;
begin
  ua:=-90.0;
Repeat
  ua:=ua+30.0;
  if  Abs(ua) < 1.0e-1 // equator
    then
      begin
        x1:=NulW-Round(EllA);
        y1:=NulH;
        fmPuz.Canvas.MoveTo(x1,y1);
        x2:=NulW+Round(EllA);
        y2:=NulH;
        fmPuz.Canvas.LineTo(x2,y2); // equator 0 degree
      end
    else
      begin
        za:=Pi*Abs(ua)/180.0; // to radian
        za:=Cos(za)/Sin(za);
        x1:=NulW-Round(EllA*za);
        x2:=NulW+Round(EllA*za);
        if  ua < 0.0
          then
            y2:=NulH+Round(EllB)
          else
            y2:=NulH-Round(EllB);
        y1:=y2-Round(EllB*(1-Abs(ua)/90.0));
        y2:=y2+Round(EllB*(1-Abs(ua)/90.0));
        FilaPoint(ua,225.0,rx,ry);
        x3:=NulW-Round(rx);
        y3:=NulH-Round(ry);
        x4:=NulW+Round(rx);
        y4:=NulH-Round(ry);
        if  ua > 0.0
          then
            fmPuz.Canvas.Arc(x1,y1,x2,y2,x3,y3,x4,y4)
          else
            fmPuz.Canvas.Arc(x1,y1,x2,y2,x4,y4,x3,y3);
      end;
until ua > 55.0;
end;

procedure TMapW.ForElliMap ;
begin
  fmPuz.Canvas.Pen.Color:=clBlack;  // colors
  fmPuz.Canvas.Brush.Color:=clAqua; // from Graphics
  fmPuz.Canvas.Font.Color:=clBlack;
  SimpleEllipse;
end;

function TMapW.ToGetPhi ( rx , ry  : Extended ) : Extended ;
var
  it  : Byte ;
  rr,rf,xa,yb  : Extended ;
  ff,df,sf,cf  : Extended ;
  hf,qf,qr,qq  : Extended ;
begin
  rf:=-90.0*ry/EllB; // initial approximately latitude in degree
  rr:=Pi*rf/180.0; // latitude in radian
  xa:=Sqr(rx/EllA);
  yb:=Sqr((Abs(ry)-EllB)/EllB);
  hf:=0.1;
  it:=0;
repeat
  it:=it+1;
  sf:=Sin(rr);
  cf:=Cos(rr);
  ff:=xa*Sqr(sf/cf)+yb*Sqr(90.0/(90.0-Abs(rf)))-1;
  qf:=rf+hf;
  qr:=Pi*qf/180.0;
  sf:=Sin(qr);
  cf:=Cos(qr);
  qq:=xa*Sqr(sf/cf)+yb*Sqr(90.0/(90.0-Abs(qf)))-1;
  df:=(qq-ff)/hf*180.0/Pi;
  rr:=rr-ff/df;
  rf:=180.0*rr/Pi;
until  it = 4 ;
  ToGetPhi:=rf;
end;

procedure TMapW.PointFila ( xc , yc  : Integer ; // current position in screen
                            var  bo  : Boolean ; // in ellipse or not
                            var  rf , rl  : Extended ) ; // lati longi tude
var
  rx,ry  : Extended ;
  ra,rr  : Extended ;
begin
  bo:=False; // initial point is not in ellipse
  rf:=0.0;   // initial latitude centre
  rl:=+45.0; // initial longitude centre in degree
  rx:=xc-NulW; // x y position refer
  ry:=yc-NulH; // to the centre of the ellipse map
  rr:=1-Sqr(ry/EllB);
  if  ( Abs(rr) < 1.0e-5 )  then  Exit ; // not in ellipse
  ra:=Sqr(rx)/rr; // ellipse for meridian square semiaxis a
  if  ra < 0.0  then  Exit ; // not in ellipse
  ra:=Sqrt(ra); // meridian ellipse semiaxis a
  if  rx < 0.0  then  ra:=-ra;
  rr:=45.0+180.0*ra/EllA; // value longitude in degree
  if  ( rr < -135.0 ) or ( rr > +225.0 )  then Exit ; // not in ellipse
  bo:=True; // our point xc yc in ellipse
  rl:=rr; // current value of longitude in degree
  rf:=ToGetPhi(rx,ry);
end;

procedure TMapW.CoorToMap ;
var
  st  : String[13];
  nm,mm  : Integer ;
  rf,rl  : Extended ;
  rx,ry  : Extended ;
  xc,yc  : Integer ;
begin
  if  curFontSize < 12  then  mm:=11  else  mm:=13;
  fmPuz.Canvas.Font.Name:='Small Fonts'; // to change Font
  fmPuz.Canvas.Font.Size:=6;             // to change Size
  fmPuz.Canvas.Font.Style:=[];
  for nm:=1 to NumStationL do
    begin
      rf:=StationList[nm].f; // latitude in degree    StationList
      rl:=StationList[nm].l; // latitude in degree    from UnitStat
      st:=StationList[nm].name; // name of the point  TPlaceCooRec
      While  Pos(' ',st) > 0  Do  Delete(st,Pos(' ',st),1);
      if  rl > 225.0  then  rl:=rl-360.0;
      FilaPoint(rf,rl,rx,ry); // from the centre
      xc:=NulW+Round(rx);     // points in screen
      yc:=NulH-Round(ry);
      with  MapW.PointMapS[nm]  do
        begin // var array for map position of the station
          x:=xc; // x position on the screen
          y:=yc; // y position on the screen
          s:=st; // name of the station
        end;
      fmPuz.Canvas.Brush.Color:=clAqua;
      fmPuz.Canvas.Ellipse(xc-2,yc-2,xc+2,yc+2);
      if  ( rf > 45.0 ) or ( ( 65.0 < rl ) and ( rl < 85.0 ) )
        then
          fmPuz.Canvas.Brush.Color:=clWhite
        else
          fmPuz.Canvas.Brush.Color:=clAqua;
      if  nm <= mm  then  fmPuz.Canvas.TextOut(xc+5,yc-4,st);
    end;
  fmPuz.Canvas.Font.Name:=curFontName; // to restore Name
  fmPuz.Canvas.Font.Size:=curFontSize; // to restore Size
  fmPuz.Canvas.Font.Style:=curFontBold;
end;

function TMapW.MapColor ( mc  : Byte ) : TColor ;
var
  c  : TColor;
begin
  c:=clWhite;
  case  mc  of
    1,2   : c:=clSilver; // Afric Madagaskar
    3,4,5 : c:=clLime; // Australia New ZeeLand
    6,7 : c:=clRed;    // South Amrica
    8,9 : c:=clFuchsia;  // Great Britain
    10,11 : c:=clYellow; // North America
    12,13 : c:=clOlive;  // Kuba
    14,15 : c:=clTeal;   // GreenLand
    16,17,18,19,20,21 : c:=clGreen;
    22,23 : c:=clPurple;
    24,25 : c:=clMaroon;
    27,28,29,30,31,32,34 : c:=clLime;
    33 : c:=clYellow;
    37 : c:=clAqua;
  end; // case mp
  MapColor:=c;
end;

procedure TMapW.LineToMem ;
var
  ft  : TextFile ;
begin
  AssignFile(ft,'linemaps.txt');
  ReSet(ft);
  mapn:=0;
  while  not EOF(ft)  do
    begin
      mapn:=mapn+1;
      if  mapn > 3113  then  mapn:=3113;
      ReadLn(ft,mapb[mapn],mapf[mapn],mapl[mapn]);
      if  mapl[mapn] > 225.0  then  mapl[mapn]:=mapl[mapn]-360.0;
    end;
  CloseFile(ft);
end;

procedure TMapW.LineToMap ;
var
  mb  : Byte ;
  mc  : Byte ;
  rf,rl  : Extended ;
  rx,ry  : Extended ;
  xc,yc  : Integer ;
  na,nm  : SmallInt ;
  xy  : Array[1..1111] of TPoint ;
begin
  na:=0;
  mc:=0;
  for nm:=1 to mapn do
    begin
      mb:=mapb[nm];
      rf:=mapf[nm];
      rl:=mapl[nm];
      if  rl > 225.0  then  rl:=rl-360.0;
      FilaPoint(rf,rl,rx,ry); // from the centre
      xc:=NulW+Round(rx);     // points in screen
      yc:=NulH-Round(ry);
      if  mb = 0
        then
          begin
            if  na > 0
              then
                begin
                  mc:=mc+1;
                  fmPuz.Canvas.Pen.Color:=clBlack;
                  fmPuz.Canvas.Brush.Color:=MapColor(mc);
                  fmPuz.Canvas.Polygon(Slice(xy,na));
                  na:=1;
                  xy[na].x:=xc;
                  xy[na].y:=yc;
                end
            end
          else
            begin
              na:=na+1;
              xy[na].x:=xc;
              xy[na].y:=yc;
            end;
    end;
end;

procedure TMapW.DistinguishPoint ;  // if BooMapPoint
begin                               // screen x y position in record PointMapF
  if  not BooMapPoint  then  Exit ; // no fixed point at the map
  fmPuz.Canvas.Brush.Color:=clBlue;
  fmPuz.Canvas.Pen.Color:=clBlue;
  with  PointMapF  do
    fmPuz.Canvas.Ellipse(xf-3,yf-3,xf+3,yf+3);
  fmPuz.Canvas.Brush.Color:=clYellow;
  with  PointMapF  do
    fmPuz.Canvas.Ellipse(xf-2,yf-2,xf+2,yf+2);
end;

procedure TMapW.SimpleMapWorld ( x1 , y1 , x2 ,y2  : Integer ) ;
begin
  ForSizeMap(x1,y1,x2,y2);
  ForElliMap ;
  LineToMap ;
  CoorToMap ;
  SimpleMeridians ;
  CotangParallels ;
  DistinguishPoint ; // if BooMapPoint is true pos PointMapF. xf yf ff vf
end;

procedure TMapW.PlanetZenith ( cs  : TColor ;
                               fs , vs  : Extended ;
                               var  xs , ys  : Integer ) ;
var
  rf,rl,rx,ry  : Extended ;
  xa,ya,xc,yc  : Integer ;
begin
  rf:=fs; // latitude  of the planet in degree
  rl:=vs; // longitude of the planet in degree
  FilaPoint(rf,rl,rx,ry); // planet projection refer to the centre of screen
  xs:=NulW+Round(rx);     // x y position in screen corresponding
  ys:=NulH-Round(ry);     // values of long latitude on the Earth
  xa:=xs-3; // left   for the planet circle on the screen
  ya:=ys-3; // top    for the planet circle on the screen
  xc:=xs+3; // right  for the planet circle on the screen
  yc:=ys+3; // bottom for the planet circle on the screen
  fmPuz.Canvas.Pen.Color:=clFuchsia;   // to prepare Pen
  fmPuz.Canvas.Brush.Color:=cs;  // and Brush for the simple circle
  fmPuz.Canvas.Ellipse(xa,ya,xc,yc);  // in the place of projection
end;

procedure TMapW.DayNightLine ( fs , vs , fm , vm  : Extended ;
                               var  xs , ys , xm , ym  : Integer ) ;
var // this procedure is called by FiguViewMap from UnPlaceA
  lp  : Integer ;     // fs is  latitude of the Sun refer to rotating Earth
  xc,yc  : Integer ;  // vs is longitude of the Sun refer to rotating Earth
  xa,ya  : Integer ;  // xa,ya,xc,yc current position on the sceen
  vp,vz  : TVect3 ;   // UnConTyp vector from the Earth centre to the Sun
  p,q,r  : TMatr33 ;  // UnConTyp matrixes to rotate space system
  x,y,z  : Extended ; // simple usefull variables for any calculations
  rf,rl  : Extended ; // latitude longitude in degree point on the Earth
  rx,ry  : Extended ; // simple usefull variables for any calculations
begin
  PlanetZenith(clYellow,fs,vs,xs,ys); // for the Sun
  PlanetZenith(clGray,fm,vm,xm,ym); // for the Moon
  fmPuz.Canvas.Pen.Width:=2; // to prepare width of Pen to figure line
  x:=90-fs; // the first  rotation in degree angle (90 - \varphi_Sun)
  SimpleRotMatr(1,x,p); // UnForFun to rotate around x axis
  z:=90-vs; // the second rotation in degree angle (90 - \lambda_Sun)
  SimpleRotMatr(3,z,q); // UnForFun to rotate around z axis
  MatrMultMatr(q,p,r);  // UnForFun to multiply matrixes
  xa:=0; // to prevent messages
  ya:=0; // simple init. values
  lp:=-2; // simple integer as current longitude in degree
  while  lp < 360  do // along the day night line equator
    begin             // direction from the Earth to the Sun
      lp:=lp+2;       // is simple perpendicular to this equator
      y:=GraRad*lp;   // the same in radians
      vp[1]:=cos(y);  // x position along to Sun direction equator
      vp[2]:=sin(y);  // y position the same but  z = 0 on equator
      vp[3]:=0.0; // vector is perpendicular to direction to the Sun
      MatrMultVector(r,vp,vz); // vector in Earth rotating system
      ClcSpherCoorInDegree(vz,rl,rf,z); // long latitude on the Earth
      FilaPoint(rf,rl,rx,ry); // position refer to the centre of screen
      xc:=NulW+Round(rx);     // x y position in screen corresponding
      yc:=NulH-Round(ry);     // values of long latitude on the Earth
      if  lp = 0 // simple check to figure lines
        then // longitude on day night equator
          fmPuz.Canvas.MoveTo(xc,yc) // initial point for line
        else
          if ( Abs(xc-xa) > 50 ) or ( Abs(yc-ya) > 20 )
            then // the distance between points is too large
              fmPuz.Canvas.MoveTo(xc,yc) // without line
            else // distance is small
              fmPuz.Canvas.LineTo(xc,yc); // it is our line
       xa:=xc; // current value for x position to memory to compare
       ya:=yc; // current value for y position to memory to compare
    end;
  fmPuz.Canvas.Pen.Width:=1; // to return to the normal Pen width
  MapW.NightPoints ;
end;

{ a try to fill night region of the map by points }
procedure TMapW.NightPoints ;
var
  x1,x2  : Integer ;
  y1,y2  : Integer ;
  xc,yc  : Integer ;
  xo,yo  : Integer ;
  xm,ym  : Integer ;
  xi,yi  : Integer ;
  rf,rl  : Extended ; // latitude longitude in degree point on the Earth
  bo     : Boolean ;
begin
  x1:=NulW-Round(EllA); { start  for x }
  y1:=NulH-Round(EllB); { finish for x }
  x2:=NulW+Round(EllA); { start  for y }
  y2:=NulH+Round(EllB); { finish for y }
  xm:=60;               { number of points for x }
  xo:=(x2-x1) div xm;   { step for x }
  yo:=xo;               { step for y }
  ym:=(y2-y1) div yo;   { number of points for y }
  xc:=x1; { initial for x }
  for xi:=1 to xm+10 do
    begin            { count for x }
      xc:=xc+xo;     { one step for x }
      yc:=y1; { initial for y }
      for yi:=1 to ym+1 do
        begin        { count for y }
          yc:=yc+yo; { one step for y }
          MapW.PointFila(xc,yc,bo,rf,rl); { latitude longitude }
          if  bo  { point among the map  UnGloVar }
              and ( Pos('',stDayOrNight(JulianDate,rl,rf)) = 0 )
            then  { and night at the point  UnPrtPos }
              fmPuz.Canvas.Pixels[xc,yc]:=clFuchsia;
        end;
    end;
end;

procedure TMapW.PointName ( xs , ys  : Integer ) ; // from UnMouSky
var // to screen name of the point where cursor is fixed if it is
  n  : Integer ;     // count  xs ys is the cursor position on the screen
  w  : Integer ;     // to compare with the PointMapS. x y
  r  : TRectBut ;
  z  : ShortString ;
begin
  if  not BooMapPoint // no fixed point at the map
    then              // BooMapPoint UnGloVar
      begin
        z:=' ';
        for n:=1 To NumStationL do   // var NumStationL is declared in UnitStat
        if  ( Abs(xs-PointMapS[n].x) < 2 )
            and ( Abs(ys-PointMapS[n].y) < 2 )
          then // cursor is near station on the map
            z:=PointMapS[n].s; // station name
        if  Length(z) > 3
          then
            MapW.PointMapF.sf:=z
          else
            MapW.PointMapF.sf:='';
      end
    else
      z:=MapW.PointMapF.sf; // name of the fixed point
  z:=BlanksToStr(z,18); // function from UnForStr
  w:=fmPuz.Canvas.TextWidth('O');
  r:=AllButt[20].rb; // place for station name
  fmPuz.Canvas.Brush.Color:=clNavy;
  fmPuz.Canvas.Font.Color:=clLime;  // color for text n = 20
  fmPuz.Canvas.Font.Style:=[fsBold];
  fmPuz.Canvas.TextOut(r.x1+w,r.y1+2,z);
end;

procedure TMapW.AboutDayN ( st  : ShortString ) ; // from UnMouSky
var
  w  : Integer ;
  r  : TRectBut ;
  z  : ShortString ;
begin
  z:=BlanksToStr(st,8); // function from UnForStr
  w:=fmPuz.Canvas.TextWidth('O');
  r:=AllButt[21].rb; // place in bottom strip for day night or twilight
  fmPuz.Canvas.Brush.Color:=clNavy;
  fmPuz.Canvas.Font.Color:=clWhite;  // color for text n = 21
  fmPuz.Canvas.Font.Style:=[fsBold];
  fmPuz.Canvas.TextOut(r.x1+w,r.y1+2,z);
end;

procedure TMapW.AboutSatV ( st  : ShortString ) ; // from UnMouSky
var   // information about all number of satellites
  w  : Integer ;      // that are visible from this point of view
  r  : TRectBut ;
  z  : ShortString ;
begin
  z:=BlanksToStr(st,17);
  w:=fmPuz.Canvas.TextWidth('O') div 2;
  r:=AllButt[1].rb; // place in top strip about number of satellite
  fmPuz.Canvas.Brush.Color:=clNavy;
  fmPuz.Canvas.Font.Color:=clLime;  // color for text n = 21
  fmPuz.Canvas.Font.Style:=[fsBold];
  fmPuz.Canvas.TextOut(r.x1+w,r.y1+2,z); // about number of satellite
end;

procedure TMapW.ToSelectP ( x , y  : Integer ;  f , v  : Extended ) ;
var
  n  : Integer ;
  b  : Boolean ;
begin  // one point is fixed at the map f v lati longi tude in degree
  with  PointMapF  do  // it is called from UnMouSky by MouseLeftClickMap
    begin  // type TPointFix
      xf:=x; // x screen position
      yf:=y; // y screen position
      ff:=f; // latitude  degree
      vf:=v; // longitude degree
    end;
  PointMapF.sf:=''; // default
  PointMapF.sd:=' ';     // default
  n:=0;
  b:=False; // to compare point on map with the known station position
  while  ( not b ) and ( n < NumStationL )  do
    begin  // var NumStationL is declared in UnitStat
      n:=n+1;
      if  ( Abs(x-PointMapS[n].x) < 2 ) and ( Abs(y-PointMapS[n].y) < 2 )
        then // cursor is near station on the map
          begin
            PointMapF.sf:=PointMapS[n].s; // station name
            PlaceCoor:=StationList[n];    // to change station position
            b:=True; // this point at the map is our known station
          end;  // if not b then it is possible
    end;        // to name this point across UnitStat UnSatMap
  if  not b  then  StationFromMap('N',f,v,PlaceCoor); // proc from UnitStat
  PointMapF.sf:=PlaceCoor.name;  // 'N' may be name of this point
  DistinguishPoint ; // at the map if BooPointMap UnGloVar
end;

end.
