
 UNIT UnForCoo ; // simple procedure to change coordinate systems

 INTERFACE

 Uses
   UnConTyp , // for type TMatr33 TVect3  const JD2000 JulianC
   UnGloVar ; // for type TPlaceCooRec

 Procedure ClcPrecMatr ( Epoch , TCur  : Extended ;
                         Var  PrecMatr : TMatr33 );

 Procedure ClcTrueRotMatr ( Epoch , TCur  : Extended ;
                            Var  RotMatr , RosMatr  : TMatr33 );

 Procedure ClcThreeRotMatr ( Epoch , TCur   : Extended ;
                             PlaceCoor      : TPlaceCooRec ;
                             Var
                               RotMatr ,
                               RosMatr ,
                               TopMatr      : TMatr33 );

 Procedure ClcSpherCoor( Pos : TVect3 ;
                         Var Alfa , Delta, Ro : Extended );

 Procedure ClcSpherCoorInDegree( Pos : TVect3 ;
                                 Var Alfa , Delta, Ro : Extended );

 Procedure ClcOrbCoorInDegree ( Pos  : TypeDim3 ;
                                Var  Azt , Zan, Ro : Extended );

 Procedure ClcTopCoorInDegree ( Pos : TVect3 ;
                                Var Azt , Alt, Ro : Extended );

 Procedure GeoSpherFromCart ( x , y , z          : Extended ;
                              Var   f , v ,  h   : Extended ) ;

 Procedure GeoCartFromSpher ( f , v , h         : Extended ;
                              Var   x , y , z   : Extended ) ;

 Procedure DescFromSpherCoor ( Aln , Alt , Ro    : Extended ;
                               Var  PosDescart  : TVect3 ) ;

 procedure DescFromHorizCoor ( Az , Alt , Ro    : Extended ;
                               var  PosDescart  : TVect3 ) ;

 IMPLEMENTATION

 Uses
   UnCoTVar , // for global variable SatElemLook of TElemRec
   UnForFun , // procedure ArcTanTwoArg for example
   UnForTim , // function ToGetGrMSTime
   PUNutC   , // for nutation procedure
   UnitEilP , // for EilXV  procedure
   UnitElmK , // ToObtainOrbitalMatr procedure
   UnRefrOl ; // for ToGetRefraction function

  { ClcPrecMatr is a procedure
    to calculate the Matrix of precession
       for change from   Epoch  to  TCur  or another words
       from the fixed equator and ecliptic to the instantaneous ones.
       Fundamental  EPOCH  is  J2000.0
         but  Epoch  as parameter
              may be any fixed epoch in  JD
                 and  TCur  is epoch of the Date in  JD .
       PrecMatr[1..3,1..3] is the resulting massive .
       global const - SecRad - for change from arcsec to radians . }

 Procedure ClcPrecMatr ( Epoch , TCur : Extended ;
                         Var  PrecMatr : TMatr33 );
 Var
    Dzita0,Teta,Zet   : Extended;
    de,de2,dt,dt2,dt3 : Extended;
    r,u1,u2,u3,s1,s2,s3,c1,c2,c3  : Extended;
 Begin
    de:=(Epoch-JD2000)/JulianC; de2:= de*de; // constants from UnConTyp
    dt:=(TCur-Epoch)/JulianC; dt2:= dt*dt; dt3:=dt2*dt;
    r:=(2306.2181+1.39656*de-0.000139*de2)*dt;
    Dzita0:=r+(0.30188-0.000344*de)*dt2+0.017998*dt3;
    Teta:=(2004.3109-0.85330*de-0.000217*de2)*dt
           -(0.42665+0.000217*de)*dt2-0.041833*dt3;
    Zet:=r+(1.09468+0.000066*de)*dt2+0.018203*dt3;
    u1:=SecRad*Dzita0; u2:=SecRad*Teta; u3:=SecRad*Zet;
    s1:=Sin(u1); s2:=Sin(u2); s3:=Sin(u3);
    c1:=Cos(u1); c2:=Cos(u2); c3:=Cos(u3);
    PrecMatr[1,1]:=+c3*c2*c1-s3*s1;
    PrecMatr[1,2]:=-c3*c2*s1-s3*c1; PrecMatr[1,3]:=-c3*s2;
    PrecMatr[2,1]:=+s3*c2*c1+c3*s1;
    PrecMatr[2,2]:=-s3*c2*s1+c3*c1; PrecMatr[2,3]:=-s3*s2;
    PrecMatr[3,1]:=+s2*c1; PrecMatr[3,2]:=-s2*s1; PrecMatr[3,3]:=+c2;
 End;

  { ClcTrueRotMatr is a procedure
       to calculate the Matrix to change
       from the fixed equator and ecliptic
       to the system of the true equator and instantaneous ecliptic.
       Fundamental  EPOCH  is  J2000.0
       TCur  is epoch of the Date in  JD .
       PrecMatr and MatrNut is declared in UnGloVar
       RotMatr[1..3,1..3] is the resulting massive .
       RosMatr[1..3,1..3] is the matrix for the sidereal time }

 Procedure ClcTrueRotMatr ( Epoch , TCur  : Extended ;
                            Var  RotMatr , RosMatr  : TMatr33 );
 Var
   i,j,k                         : Byte ;
   uf,ul,sf,cf,
   sl,cl,st,ct,
   e,s,c,se,ce,ts,SidTime,
   DeltaPsi,DeltaEps,EpsMean     : Extended ;
   MatrCur,MatrEps,MatrPsi       : TMatr33 ;
 Begin
   ClcNut(TCur,DeltaPsi,DeltaEps,EpsMean); { in radian }
   se:=Sin(EpsMean);
   ce:=Cos(EpsMean);
   MatrEps[1,1]:=+1;
   MatrEps[1,2]:=0;
   MatrEps[1,3]:=0;
   MatrEps[2,1]:=0;
   MatrEps[2,2]:=+ce;
   MatrEps[2,3]:=+se;
   MatrEps[3,1]:=0;
   MatrEps[3,2]:=-se;
   MatrEps[3,3]:=+ce;
   For i:=1 To 3 Do
     For j:=1 To 3 Do
       MatrEcl[i,j]:=MatrEps[i,j];
   s:=Sin(DeltaPsi);
   c:=Cos(DeltaPsi);
   MatrPsi[1,1]:=+c;
   MatrPsi[1,2]:=-s;
   MatrPsi[1,3]:=0;
   MatrPsi[2,1]:=+s;
   MatrPsi[2,2]:=+c;
   MatrPsi[2,3]:=0;
   MatrPsi[3,1]:=0;
   MatrPsi[3,2]:=0;
   MatrPsi[3,3]:=+1;
   For i:=1 To 3 Do
     For j:=1 To 3 Do
       Begin
         MatrCur[i,j]:=0;
         For k:=1 To 3 Do
           MatrCur[i,j]:=MatrCur[i,j]+MatrPsi[i,k]*MatrEps[k,j];
       End;
   e:=EpsMean+DeltaEps;
   s:=Sin(e);
   c:=Cos(e);
   MatrEps[1,1]:=+1;
   MatrEps[1,2]:=0;
   MatrEps[1,3]:=0;
   MatrEps[2,1]:=0;
   MatrEps[2,2]:=+c;
   MatrEps[2,3]:=-s;
   MatrEps[3,1]:=0;
   MatrEps[3,2]:=+s;
   MatrEps[3,3]:=+c;
   For i:=1 To 3 Do
     For j:=1 To 3 Do
       Begin
         MatrNut[i,j]:=0;
         For k:=1 To 3 Do
           MatrNut[i,j]:=MatrNut[i,j]+MatrEps[i,k]*MatrCur[k,j];
       End;
   ClcPrecMatr(Epoch,TCur,PrecMatr);
   For i:=1 To 3 Do
     For j:=1 To 3 Do
       Begin
         RotMatr[i,j]:=0;
         For k:=1 To 3 Do
           RotMatr[i,j]:=RotMatr[i,j]+MatrNut[i,k]*PrecMatr[k,j];
       End;
   SidTime:=ToGetGrMSTime(TCur); { mean sidereal time unit UnForTim }
   ts:=SidTime+DeltaPsi*ce;    { true sidereal time }
   s:=Sin(ts); // RosMatr matrix of rotation R_z(teta)
   c:=Cos(ts); // from true equinox to greenwich point
   RosMatr[1,1]:=+c;
   RosMatr[1,2]:=+s;
   RosMatr[1,3]:=0;
   RosMatr[2,1]:=-s;
   RosMatr[2,2]:=+c;
   RosMatr[2,3]:=0;
   RosMatr[3,1]:=0;
   RosMatr[3,2]:=0;
   RosMatr[3,3]:=+1;
 End;
// for option 'S' 'W' topocentric matrix
procedure ClcTopMatr ( pc  : TPlaceCooRec ; // for station
                       rm  : TMatr33 ;      // rotation matrix
                       var  ps  : TVect3 ;  // station position
                       var  rt  : TMatr33 ) ; // topo matrix
Var
  i,j        : Byte ;
  uf,sf,cf   : Extended ;
  ul,sl,cl   : Extended ;
  st,ct,s,c  : Extended ;
  PosGeodet  : TVect3 ;
begin
  With  pc  Do
    Begin // pc with information about station
      uf:=GraRad*f;           { latitude  to radian }
      ul:=GraRad*l;           { longitude to radian }
      PosGeodet[1]:=1.0e-3*x; { in km }
      PosGeodet[2]:=1.0e-3*y; { geo rotating system }
      PosGeodet[3]:=1.0e-3*z; { in km }
    End;
  { station position from greenwich to true equator }
  For i:=1 To 3 Do
    Begin { station position from greenwich to true equator }
      ps[i]:=0; // refer to the geo centre
      For j:=1 To 3 Do
        ps[i]:=ps[i]+rm[j,i]*PosGeodet[j];
    End;
  { teta greenwich true sidereal time }
  c:=rm[1,1]; { cos(teta) }
  s:=rm[1,2]; { sin(teta) }
  sf:=Sin(uf);
  cf:=Cos(uf);
  sl:=Sin(ul);
  cl:=Cos(ul);
  st:=s*cl+c*sl; { sin ( teta + lambda ) }
  ct:=c*cl-s*sl; { cos ( teta + lambda ) }
  ArcTanTwoArg(st,ct,LocalSTime); { local sidereal time in radians }
  { rt matrix to change
       from true equator system
         to topocentric horizontal system
         R_z(180)*R_y(90-phi)*R_z(teta+lambda)
         where phi latitude lambda longitude of the point
         after this transformation
         axis OX direct to the north
         axis OZ direct to the zenith }
  rt[1,1]:=-sf*ct;
  rt[1,2]:=-sf*st;
  rt[1,3]:=+cf;
  rt[2,1]:=+st;
  rt[2,2]:=-ct;
  rt[2,3]:=0;
  rt[3,1]:=+cf*ct;
  rt[3,2]:=+cf*st;
  rt[3,3]:=+sf;
end;

{ om orbital matrix to change
     from true equator system
     to orbital system refer to the satellite
     elements of the satellite are in record SatElemLook UnCoTVar
     axis OX direct as transversal from the satellite
     axis OZ direct along radius-vector
     R_x(90)*R_z(90)*R_z(u)*R_x(i)*R_z(Omega)
     where simple satellite orbit parameters
     u is argument of latitude
     i is angle of inclination
     Omega is longitude of ascending node }
procedure ClcOrbMatr ( tc  : Extended ; // moment julian date
                       var  po  : TVect3 ; // object position
                       var  om  : TMatr33 ) ; // orbital matr
var
  x,v  : TVect3 ; // position velosity in km satellite
begin  // record SatElemLook UnCoTVar satellite in true equator
  EilXV(tc,SatElemLook,x,v); // UnitEilP  refer to the geo centre
  po:=x; // satellite position the same as StationPos[..]
  ToObtainOrbitalMatr(x,v,om); // our matrix to change to orbital
  x:=om[1]; // to replace elements of the orbital matrix UnitElmK
  om[1]:=om[2]; // x' = y
  om[2]:=om[3]; // y' = z
  om[3]:=x;     // z' = x
end;                           // from UnitEilP instead TopMatr

  { ClcThreeRotMatr is a procedure
       to calculate the Matrix to change
       from the fixed equator and ecliptic
       to the system of the true equator and instantaneous ecliptic.
       Fundamental  EPOCH  is  J2000.0
       TCur  is epoch of the Date in  JD .
       PrecMatr and MatrNut is declared in UnGloVar
       RotMatr[1..3,1..3] is the resulting massive .
       RosMatr[1..3,1..3] is the matrix for the sidereal time
       TopMatr[1..3,1..3] is the matrix
              to change to topocentric system for 'S' sky
              to change to orbital system for 'L' look option }

 Procedure ClcThreeRotMatr ( Epoch , TCur   : Extended ;
                             PlaceCoor      : TPlaceCooRec ;
                             Var
                               RotMatr ,
                               RosMatr ,
                               TopMatr      : TMatr33 );
 Begin
   ClcTrueRotMatr(Epoch,TCur,RotMatr,RosMatr);
   { 'S' option
     TopMatr[...] to change from true equator to horizont
     'L' option
     TopMatr[...] is simple orbital matrix
                  to change from true equator to object system }
   case  CharForView  of // for 'S' 'W' topocentric 'L' orbital
     'S' , 'W' : ClcTopMatr(PlaceCoor,RosMatr,StationPos,TopMatr);
     'L' : ClcOrbMatr(TCur,StationPos,TopMatr); // for satellite
   end;
 End;

 { ClcSpherCoor is a procedure
   to obtain the spherical coordinates of the planet
      from cartesian ones }

 Procedure ClcSpherCoor( Pos : TVect3 ;
                         Var Alfa , Delta, Ro : Extended );
 Var
   xr : TVect3;
   r,rr,q,qq,p,f,s,c,a,b : Extended;
   i : Integer;
 Begin
   for i:=1 to 3 do  xr[i]:=Pos[i];
   qq:=Sqr(xr[1])+Sqr(xr[2]); rr:=qq+Sqr(xr[3]); r:=Sqrt(rr);
   q:=Sqrt(qq); p:=1/q; f:=1/r;
   c:=p*xr[1]; s:=p*xr[2];    ArcTanTwoArg(s,c,a); // unit UnForFun
   c:=q*f;     s:=xr[3]*f;    ArcTanTwoArg(s,c,b);
   if  a < 0  then  a:=a+Pi22; Alfa:=a; Delta:=b; Ro:=r;
 End;

 { ClcSpherCoorInDegree is a procedure
   to obtain the spherical coordinates of the planet from cartesian ones }

 Procedure ClcSpherCoorInDegree ( Pos : TVect3 ;
                                  Var Alfa , Delta, Ro : Extended );
 Var
   xr : TVect3;
   r,rr,q,qq,p,f,s,c,a,b : Extended;
   i : Integer;
 Begin
   for i:=1 to 3 do  xr[i]:=Pos[i];
   qq:=Sqr(xr[1])+Sqr(xr[2]); rr:=qq+Sqr(xr[3]); r:=Sqrt(rr);
   q:=Sqrt(qq); p:=1/q; f:=1/r;
   c:=p*xr[1]; s:=p*xr[2];    ArcTanTwoArg(s,c,a);
   c:=q*f;     s:=xr[3]*f;    ArcTanTwoArg(s,c,b);
   if  a < 0  then  a:=a+Pi22;
   Alfa:=RadGra*a;
   Delta:=RadGra*b;
   Ro:=r;
 End;

 { ClcOrbCoorInDegree is a procedure
   to obtain the orbital coordinates in spherical form }

 Procedure ClcOrbCoorInDegree ( Pos  : TypeDim3 ;
                                Var  Azt , Zan, Ro : Extended );
 Var
   xr : TypeDim3;
   r,rr,q,qq,p,f,s,c,a,b : Extended;
 Begin
   xr[1]:=Pos[2];
   xr[2]:=Pos[3];
   xr[3]:=Pos[1];
   qq:=Sqr(xr[1])+Sqr(xr[2]);
   rr:=qq+Sqr(xr[3]);
   r:=Sqrt(rr);
   q:=Sqrt(qq);
   if  ( r < 0.01 ) or ( q < 0.01 ) // to compare distance in km
     then  // distance in kilometer is too small
       begin
         Azt:=0.0;
         Zan:=-100.0;
         Ro:=0.0;
         Exit; // it is necessary to exclude this point
       end;
   p:=1/q;
   f:=1/r;
   { special rotation }
   c:=+p*xr[1]; s:=-p*xr[2];    ArcTanTwoArg(s,c,a); // UnForFun
   c:=+q*f;     s:=+xr[3]*f;    ArcTanTwoArg(s,c,b); // UnForFun
   if  a < 0  then  a:=a+Pi22; // const UnConTyp
   Azt:=RadGra*a; { azimut in degree UnConTyp }
   Zan:=90-RadGra*b; { zenith distance in degree }
   Ro:=r;
 End;

 { ClcTopCoorInDegree is a procedure
   to obtain the topocentric coordinates in spherical form }

 Procedure ClcTopCoorInDegree ( Pos  : TVect3 ;
                                Var  Azt , Alt, Ro  : Extended );
 Var
   xr : TVect3;
   r,rr,q,qq,p,f,s,c,a,b : Extended;
   i   : Integer;
   zr  : Extended ;
 Begin
   for i:=1 to 3 do  xr[i]:=Pos[i];
   qq:=Sqr(xr[1])+Sqr(xr[2]);
   rr:=qq+Sqr(xr[3]);
   r:=Sqrt(rr); // distance in km
   q:=Sqrt(qq);
   if  ( r < 0.01 ) or ( q < 0.01 ) // to compare distance in km
     then  // distance in kilometer is too small
       begin
         Azt:=0.0;
         Alt:=-100.0;
         Ro:=0.0;
         Exit; // it is necessary to exclude this point
       end;
   p:=1/q;
   f:=1/r;
   c:=+p*xr[1]; s:=-p*xr[2];   ArcTanTwoArg(s,c,a);
   c:=q*f;      s:=xr[3]*f;    ArcTanTwoArg(s,c,b);
   if  a < 0  then  a:=a+Pi22;
   Azt:=RadGra*a; // azimuth  in degree
   Alt:=RadGra*b; // altitude in degree
   Ro:=r;
   If  ( CharForView = 'L' ) or ( not BooRefraction )
      Then  Exit ; // without refraction correction
   { to add refraction }
   zr:=ToGetRefraction(Alt);
   Alt:=Alt+zr;
 End;

 { to obtain spherical coordinates ( h in metr )
      of the station from the cartesian ones in metr }

 Procedure GeoSpherFromCart ( x , y , z          : Extended ;
                              Var   f , v ,  h   : Extended ) ;
 Var
   irf                : Byte ;
   a,e,r,ff,vv,
   pp,hg,gg,sv,cv,
   sf,cf,sff,cff,cc   : Extended ;
 Begin
   r:=6378144.11; { in metr }
   a:=1.0/298.257;
   e:=2*a-a*a;
   pp:=Sqrt(x*x+y*y);
   If pp < 1.0e4
     Then
       r:=1.0e-3*r; { in km }
   cv:=x/pp;
   sv:=y/pp;
   hg:=0.0;
   For irf:=1 To 4 Do
     Begin
       sff:=z*(1+hg);
       cff:=pp*(1-e+hg);
       cc:=Sqrt(sff*sff+cff*cff);
       sf:=sff/cc;
       cf:=cff/cc;
       gg:=r/Sqrt(1-e*sf*sf);
       hg:=pp/(gg*cf)-1.0;
     End;
   ArcTanTwoArg(sf,cf,ff);
   f:=RadGra*ff;
   ArcTanTwoArg(sv,cv,vv);
   v:=RadGra*vv;
   If  v < 0.0
     Then
       v:=360+v;
   h:=hg*gg;
 End;

 { to obtain cartesian coordinates in metr
      of the station from the spherical ones }

 Procedure GeoCartFromSpher ( f , v , h         : Extended ;
                              Var   x , y , z   : Extended ) ;
 Var
   a,e,r,q,
   ff,vv,sf,cf,gg  : Extended ;
 Begin
   r:=6378144.11;
   a:=1.0/298.257;
   e:=2*a-a*a;
   q:=1-e;
   ff:=GraRad*f;
   vv:=GraRad*v;
   sf:=Sin(ff);
   cf:=Cos(ff);
   gg:=r/Sqrt(1-e*sf*sf);
   x:=(gg+h)*cf*Cos(vv);
   y:=(gg+h)*cf*Sin(vv);
   z:=(gg*q+h)*sf;
 End;
 { to obtain DesCart
   or rectangular position
   when sheric coordinates in right system are known
   for example longitude latitude right ascension declination }
 Procedure DescFromSpherCoor ( Aln , Alt , Ro    : Extended ;
                               Var  PosDescart  : TVect3 ) ;
 Var
   a,b,cb  : Extended ;
 Begin // by usual formula is called in UnForPos UnPhysOp
   a:=GraRad*Aln;  { longitude in radian  0 <= azimuth <= 360 }
   b:=GraRad*Alt;  { latitude  in radian -90 <= altit <= +90 }
   cb:=Cos(b);     { cosinus altitude }
   PosDescart[1]:=Ro*Cos(a)*cb; { x coordinate }
   PosDescart[2]:=Ro*Sin(a)*cb; { y coordinate }
   PosDescart[3]:=Ro*Sin(b);    { z coordinate }
 End;

 { to obtain DesCart
   or rectangular horizontal position
   when sheric coordinates in left system are known
   for example horizontal azimuth from the north altitude }
 procedure DescFromHorizCoor ( Az , Alt , Ro    : Extended ;
                               var  PosDescart  : TVect3 ) ;
 begin // proc is called in UnForSys by FromTopToTrueEqu
   DescFromSpherCoor(Az,Alt,Ro,PosDescart); { but left azimuth }
   PosDescart[2]:=-PosDescart[2];   { from left to right angle }
 end;

 END.