
 UNIT UnForPos ; // only for Solar system planet

 INTERFACE

 Uses
   UnConTyp ; // for type TVect3

 procedure ClcPlanetMagnitude ( np  : Byte ;
                                PosP , PosS  : TVect3 ;
                                Var  vmag  : Single ;
                                Var  phas  : Extended ) ;

 Function DayOrNightNow : Char ; // current date in JulianDate PlaceCoor

 function chDayOrNight ( tt  : Extended ) : Char ;

 function stDayOrNight ( tt , aa , dd  : Extended ) : ShortString ;

 Procedure PlanetsToScreen ; // current date in JulianDate

 Procedure PosGeoPlanet ( np  : Byte ;  tc  : Extended ;  var  xp  : TVect3 ) ;

 Function CentreWithFixPlanet : Boolean ; // called from UnitFixO

 Procedure RotatingGeoPos ( np  : Byte ;  // number of planet
                            Var  pp  : TVect3 ;
                            Var  fs , vs  : Extended ) ; // lat longitude degree

 IMPLEMENTATION

 Uses
   UnCotVar , // for types
   UnGloVar , // for current date variable JulianDate PrecMatr MatrNut
   UnForTim , // for FromUTCtoTT function ToGetTEph
   PUnDE200 , // for ephemeris
   UnForRed , // for aberration correction
   UnitShad , // for byte function ToGetShadowCase
   UnForFun , // for UMatrMultVect MatrMultVector procedure
   UnForCoo , // for procedure ClcTopCoorInDegree ClcThreeRotMatr
   UnitSort , // for procedure ShellMethodForOrdering
   UnPlaceA , // for RectCoordinates
   UnPlateC , // for IdealCoordinates
   UnGraAll , // for procedure FigurePlanet
   UnButIni ; // for SpecButt[21] and HintButt[21] day twilight night

 var // PUnD200 UnCoTVar
   TopArray  : Array[1..NumberOfPlanets] of TObjectTopRec;
   IntOrder  : TIntegerNumber ; // UnitSort
   RealSort  : TRealNumber ;    // UnitSort

 // after ClcThreeRotMatr from unit UnForCoo
 // there are matrix to transform to true equator for current date

 Function MagnitudeValue ( n  : Byte ;
                           ro , rop , phase  : Extended ) : Single ;
 Var
   g,f  : Single ;
 Begin
   g:=99.0;
   f:=RadGra*phase; { in degree UnConTyp}
   case  n  of
     NMercury :  g:=-0.38+5*Log10(ro*rop)+0.0380*f
                    -0.000273*f*f+0.000002*f*f*f;
     NVenus   :  g:=-4.41+5*Log10(ro*rop)+0.01314*f
                    +0.0000004351*f*f*f;
     NMars    :  g:=-1.51+5*Log10(ro*rop)+0.01486*f;
     NJupiter :  g:=-9.40+5*Log10(ro*rop);
     NSaturn  :  g:=-8.88+5*Log10(ro*rop);
     NUranus  :  g:=-7.19+5*Log10(ro*rop);
     NNeptune :  g:=-6.87+5*Log10(ro*rop);
     NPluto   :  g:=-1.00+5*Log10(ro*rop);
   end; { case n }
   MagnitudeValue:=g;
 End;

 Function  MagSaturnRing  : Single ;
 Var
   i         : Byte ;
   rotm      : TMatr33 ; // UnConTyp
   pq,pp     : TVect3 ;
   pe,ps,pt  : TVect3 ;
   ap,dp,rp  : Extended ;
   ae,de,re  : Extended ;
   az,ds,rs  : Extended ;
   dt,sde    : Extended ;
 Begin // PrecMatr and MatrNut from UnGloVar are assigned by ToClcThreeRot...
   dt:=(JulianDate-JD2000)/36525.00; // difference in century
   ap:=40.58-0.036*dt;  // north pole of the Saturn in J2000.0 equator
   dp:=83.54-0.004*dt;  // in degree right ascension declination
   rp:=1.0; // unit distance  value for PrecMatr exists
   DescFromSpherCoor(ap,dp,rp,pq); // UnForCoo pole descart position
   MatrMultVector(PrecMatr,pq,pp); //  Saturn north pole to equator of date
   ClcSpherCoorInDegree(pp,ap,dp,rp); // descart position in equator of date
   RotMatrFromPole(ap,dp,rotm); // UnForFun rotation matrix to Saturn equator
   pq:=PosDim[NSaturn]; // topocentric descart position of Saturn
   UMatrMultVect(TopMatr,pq,pt); // topocentric but true equator
   UMatrMultVect(MatrNut,pt,pe); // the same but equator of date for Saturn
   pq:=PosDim[NSun]; // topocentric descart position of the Sun
   UMatrMultVect(TopMatr,pq,pt); // topocentric but true equator
   UMatrMultVect(MatrNut,pt,ps); // the same but equator of date for the Sun
   For i:=1 To 3 Do
     Begin // change from station to the Saturn in equator of date
       ps[i]:=-pe[i]+ps[i];
       pe[i]:=-pe[i];
     End;
   MatrMultVector(rotm,pe,pq); // to the Saturn equator rotation matrix
   ClcSpherCoorInDegree(pq,ae,de,re); // the Earth on the Saturn equator
   MatrMultVector(rotm,ps,pq); // to the Saturn equator rotation matrix
   ClcSpherCoorInDegree(pq,az,ds,rs); // the Sun on the Saturn equator
   sde:=Sin(GraRad*de);
   MagSaturnRing:=+0.044*Abs(az-ae)-2.60*Abs(sde)+1.25*Sqr(sde);
 End;

 { np    number of planet
   PosP  position planet  refer to our point
   PosS  position the Sun refer to our point }

 procedure ClcPlanetMagnitude ( np  : Byte ;
                                PosP , PosS  : TVect3 ;
                                Var  vmag  : Single ;
                                Var  phas  : Extended ) ;
 var
   i  : Byte ;
   a,b,c  : Extended ;
   s,f    : Extended ;
   ra,rb  : TVect3 ;
 begin
   if  ( np = NEarth) or ( np = NSun )  then
     begin
       vmag:=99;
       phas:=1.0;
       Exit;
     end;
   for i:=1 to 3 do
     begin
       ra[i]:=-PosP[i]; { the point from the planet }
       rb[i]:=+PosS[i]-PosP[i]; { the Sun in topocentric }
     end;
   a:=Sqrt(Sqr(ra[1])+Sqr(ra[2])+Sqr(ra[3]));
   b:=Sqrt(Sqr(rb[1])+Sqr(rb[2])+Sqr(rb[3]));
   c:=(ra[1]*rb[1]+ra[2]*rb[2]+ra[3]*rb[3])/(a*b); { cos(Phase) }
   phas:=0.5*(1+c);
   if  np <> NMoon  then
     begin
       s:=Sqrt(1-Sqr(c));
       f:=DATan2(s,c); // function from UnForFun
       a:=a/AstrUnit;
       b:=b/AstrUnit;
       vmag:=MagnitudeValue(np,a,b,f);
       if  np = NSaturn  then
         vmag:=vmag+MagSaturnRing;
     end
   else
     vmag:=99; // for the Moon
 end;

 procedure ForSimplePos ( np  : Byte ;     // number of planet
                          tb  : Extended ;  // moment in TDB scale
                          pf , pb  : TVect3 ; // point pf to the Earth
                          Var  pp  : TVect3 ) ; // pb to barycenter
 var // in pp[..] position refer to station or other object with pf[.]
   i  : Byte ;
   v  : TVect3 ;
 begin
   ClcPosVel(np,tb,pp,v); // the planet to Solar system barycenter
   for i:=1 to 3 do // position of the planet in fixed equator
     if  np <> NMoon // NMoon ClcPosVel from PUnDE200
       then // for all planets and the Sun
         pp[i]:=pp[i]-pb[i] // refer to the current point
       else // only for the Moon refer to the centre of the Earth
         pp[i]:=pp[i]-pf[i]; // refer to the current point
 end;

 Procedure PosReferToPoint ( np  : Byte ;     // number of planet
                             tb  : Extended ;  // moment in TDB scale
                             pf , pb  : TVect3 ; // point pf to the Earth
                             Var  pp  : TVect3 ) ; // pb to barycenter
 Var // pp[1..3] position of the planet refer to the point in fixed equator
   iter  : Byte ;
   rs,dt,tcur  : Extended ;
 Begin
   ForSimplePos(np,tb,pf,pb,pp); // here above
   For iter:=1 To 3 Do // to get position of the planet
     Begin // with light delay by simple iteration
       rs:=VectorModul(pp); // range from point to the planet in km
       dt:=(rs/VelOfLight)/86400; // light delay in part of day UnConTyp
       tcur:=tb-dt; // moment with light delay
       ForSimplePos(np,tcur,pf,pb,pp); // here above
     End; // in fixed equator
   ForSimplePos(np,tcur,pf,pb,pp); // here above
 End;

 Function DayOrNightNow  : Char ; // current date in JulianDate
 Var
   i  : Byte ;
   azt,alt,ro,rs  : Extended ;
   TinTDT,TinTDB,DeltaTA  : Extended ;
   PosE,PosS,Vel,pof,pob  : TVect3 ;
   ch  : Char ;
 Begin // current station position in km StationPos in true equator system
   DayOrNightNow:='N'; // may be no ephemeris file
   FromUTCtoTT(JulianDate,TinTDT,DeltaTA);  // unit UnForTim
   TinTDB:=ToGetTEph(TinTDT); // function from unit UnForTim
   ClcEphEarth(TinTDB,PosE,Vel); // the Earth to Solar system barycentre
   If  NOT BooExistPos  Then  Exit ; // boolean var from unit PUnDE200
   ClcThreeRotMatr(JD2000,JulianDate,PlaceCoor,RotMatr,RosMatr,TopMatr);
   UMatrMultVect(RotMatr,StationPos,pof); // station to fix equator
   For i:=1 To 3 Do // vector for station position in fixed equator in km
     pob[i]:=PosE[i]+pof[i]; // refer to Solar system barycentre
   PosReferToPoint(NSun,TinTDB,pof,pob,PosS);
   MatrMultVector(RotMatr,PosS,PosE); // from fixed to true equator UnForFun
   MatrMultVector(TopMatr,PosE,PosS); // to topocentric horizontal system
   ClcTopCoorInDegree(PosS,azt,alt,ro); // UnForCoo azimut altitude range
   rs:=RadGra*RadiusOfPlanet[NSun]/ro; // half Sun from station in degree
   If  alt > -rs // the Sun height above horizon with the refraction
     Then  ch:='D' // correction and with the Solar radius
     Else         // now is the day the Sun above horizon
       If  alt > -6.0   // in degree
         Then  ch:='T'  // astronomical twilight
         Else  ch:='N'; // night for faint stars
   ForSunHightButton(ch,alt); // procedure from unit UnButIni
   DayOrNightNow:=ch;
 End;

 function chDayOrNight ( tt  : Extended ) : Char ;
 Var // it is possible to call this function after proc ClcThreeRotMatr
   i  : Byte ;
   azt,alt,ro,rs  : Extended ;
   TinTDT,TinTDB,DeltaTA  : Extended ;
   PosE,PosS,Vel,pof,pob  : TVect3 ;
   ch  : Char ;
 Begin // current station position in km StationPos in true equator system
   chDayOrNight:='N'; // may be no ephemeris file
   FromUTCtoTT(tt,TinTDT,DeltaTA);  // unit UnForTim
   TinTDB:=ToGetTEph(TinTDT); // function from unit UnForTim
   ClcEphEarth(TinTDB,PosE,Vel); // the Earth to Solar system barycentre
   If  NOT BooExistPos  Then  Exit ; // boolean var from unit PUnDE200
   UMatrMultVect(RotMatr,StationPos,pof); // station to fix equator
   For i:=1 To 3 Do // vector for station position in fixed equator in km
     pob[i]:=PosE[i]+pof[i]; // refer to Solar system barycentre
   PosReferToPoint(NSun,TinTDB,pof,pob,PosS);
   MatrMultVector(RotMatr,PosS,PosE); // from fixed to true equator UnForFun
   MatrMultVector(TopMatr,PosE,PosS); // to topocentric horizontal system
   ClcTopCoorInDegree(PosS,azt,alt,ro); // UnForCoo azimut altitude range
   rs:=RadGra*RadiusOfPlanet[NSun]/ro; // half Sun from station in degree
   If  alt > -rs // the Sun height above horizon with the refraction
     Then  ch:='D' // correction and with the Solar radius
     Else         // now is the day the Sun above horizon
       If  alt > -6.0   // in degree
         Then  ch:='T'  // astronomical twilight
         Else  ch:='N'; // night for faint stars
   chDayOrNight:=ch;
 End;

 function stDayOrNight ( tt , aa , dd  : Extended ) : ShortString ;
 Var // tt current moment aa longitude dd latitude of the point on the Earth
   i  : Byte ;
   azt,alt,ro,rs  : Extended ;
   TinTDT,TinTDB,DeltaTA  : Extended ;
   PosE,PosS,Vel,pof,pob  : TVect3 ;
   pl  : TPlaceCooRec ;
   st  : ShortString ;
 Begin // current station position in km StationPos in true equator system
   stDayOrNight:='    '; // may be no ephemeris file
   pl.num:=0;
   pl.name:='';
   pl.f:=dd;  // latitude in degree
   pl.l:=aa;  // logitude in degree
   pl.h:=0.0; // height in meter
   with  pl  do  GeoCartFromSpher(f,l,h,x,y,z); // proc from UnForCoo
   ClcThreeRotMatr(JD2000,tt,pl,RotMatr,RosMatr,TopMatr);
   FromUTCtoTT(tt,TinTDT,DeltaTA);  // unit UnForTim
   TinTDB:=ToGetTEph(TinTDT); // function from unit UnForTim
   ClcEphEarth(TinTDB,PosE,Vel); // the Earth to Solar system barycentre
   If  NOT BooExistPos  Then  Exit ; // boolean var from unit PUnDE200
   UMatrMultVect(RotMatr,StationPos,pof); // station to fix equator
   For i:=1 To 3 Do // vector for station position in fixed equator in km
     pob[i]:=PosE[i]+pof[i]; // refer to Solar system barycentre
   PosReferToPoint(NSun,TinTDB,pof,pob,PosS);
   MatrMultVector(RotMatr,PosS,PosE); // from fixed to true equator UnForFun
   MatrMultVector(TopMatr,PosE,PosS); // to topocentric horizontal system
   ClcTopCoorInDegree(PosS,azt,alt,ro); // UnForCoo azimut altitude range
   rs:=RadGra*RadiusOfPlanet[NSun]/ro; // half Sun from station in degree
   If  alt > -rs // the Sun height above horizon with the refraction
     Then  st:='    ' // correction and with the Solar radius
     Else         // now is the day the Sun above horizon
       If  alt > -6.0   // in degree
         Then  st:=' '  // astronomical twilight
         Else  st:='    '; // night for faint stars
   stDayOrNight:=st;
 End;

 Procedure PlanetsToHeapWithFigure ( numc  : Integer ) ;
 Var
   nc  : Integer ;
   np  : Integer ;
 Begin
 nc:=0;
 Repeat
   nc:=nc+1;
   np:=IntOrder[nc];
   ObjectTopCur:=TopArray[np];
   Inc(NumObjectInHeap);
   ObjectTopHeap^[NumObjectInHeap]:=ObjectTopCur;
   With  ObjectTopCur  do
     If  booscreen and BooPosInRect(xpos,ypos)
       Then  FigurePlanet; // unit UnGraAll
 Until  nc = numc ;
 End;

 Procedure PlanetForOrder ( np  : Byte ;
                            az , al , ro  : Extended ;
                            vmag  : Single ;
                            phasep  : Extended ;
                            ishad  : Byte ;
                            Var  numc  : Integer ) ;
 Var
   x,y  : Integer ;
   ksi,eta  : Extended ;
   top  : TVect3 ; // UnConTyp topocentric horizon descart
   bos  : Boolean ; // our planet may be in the other hemisphere
 Begin
   top:=PosDim[np]; // topocentric position of current planet
   { PosHorizoC UnGloVar topo horizon vector of the centre
     our planet may be out of our hemisphere
     ScalarMunt simple cosinus between two vectors
     if cosinus > 0.5 then planet is in our hemishere
     if cosinus > 0.5 then planet may be in our field of view }
   bos:= ( ScalarMunt(top,PosHorizoC) > 0.5 ) ; // UnForFun
   if  bos        // planet is in our hemishere
     then         // this planet is in our hemisphere
       begin      // this planet may be in our field of view
         IdealCoordinates(az,al,ksi,eta); // from UnPlateC
         RectCoordinates(ksi,eta,x,y);    // from UnPlaceA
       end // position projection to the plate
     else     // this planet is out our hemisphere
       begin  // this palnet is out our field if view
         x:=-30000; // default large minus position
         y:=-30000; // default large minus position
       end;   // to exlude this planet
   ObjectTopCur.nall:=np; // number
   ObjectTopCur.chps:='P'; // there is planet
   ObjectTopCur.name:=PlanetNameR[np]; // PUnDE200
   ObjectTopCur.azt:=az; // current azimuth
   ObjectTopCur.alt:=al; // current altitude
   ObjectTopCur.rot:=ro; // range default for star
   ObjectTopCur.Mag:=vmag; // magnitude
   ObjectTopCur.phase:=phasep; // for planet
   ObjectTopCur.IndexShadow:=ishad; // default
   If  al > 0.0                    // altitude above horizon
     Then
       ObjectTopCur.boovis:=True   // visibility
     Else
       ObjectTopCur.boovis:=False; // no visibility
   ObjectTopCur.booscreen:=bos;    // screen condition
   ObjectTopCur.xpos:=x; // x on screen
   ObjectTopCur.ypos:=y; // y on screen
   Inc(numc);
   TopArray[numc]:=ObjectTopCur;
   IntOrder[numc]:=numc; // initial order in array
   RealSort[numc]:=ObjectTopCur.rot; // to order with range in km
 End;

 Procedure PlanetsToScreen ; // current date in JulianDate
 Var                         // proc is called by ViewPlac
   i  : Byte ;
   np  : Byte ;
   numc  : Integer ;
   vmag  : Single ;
   ishad  : Byte ;
   azt,alt,ro,phase  : Extended ;
   TinTDT,TinTDB,DeltaTA  : Extended ;
   PosE,PosP,PosS,Vel,pof,pob  : TVect3 ;
 Begin // current station position in km StationPos in true equator system
   FromUTCtoTT(JulianDate,TinTDT,DeltaTA);  // unit UnForTim
   TinTDB:=ToGetTEph(TinTDT); // function from unit UnForTim
   ClcEphEarth(TinTDB,PosE,Vel); // the Earth to Solar system barycentre
   If  NOT BooExistPos  Then  Exit ; // boolean var from unit PUnDE200
   UMatrMultVect(RotMatr,StationPos,pof); // station to fix equator
   For i:=1 To 3 Do // vector for station position in fixed equator in km
     pob[i]:=PosE[i]+pof[i]; // refer to Solar system barycentre
   PosReferToPoint(NSun,TinTDB,pof,pob,PosS); // the Sun refer to point
   MatrMultVector(RotMatr,PosS,Vel); // from fixed to true equator
   MatrMultVector(TopMatr,Vel,PosS); // to topocentric horizont the Sun
   numc:=0; // inital nullo before order all planets with range
   For np:=NMercury To NSun Do
     If  np <> NEarth  Then  // pof point refer to the Earth
       Begin  // pob point refer to the Solar system barycenter
         PosReferToPoint(np,TinTDB,pof,pob,PosP); // PosP refer to point
         ForAberrationCorrection(PosP); // UnForRed correction in fix equator
         MatrMultVector(RotMatr,PosP,Vel); // from fixed to true equator
         MatrMultVector(TopMatr,Vel,PosP); // to topocentric horizont
         PosDim[np]:=PosP; // topocentric position of the planet to array
         ClcPlanetMagnitude(np,PosP,PosS,vmag,phase);
         ClcTopCoorInDegree(PosP,azt,alt,ro); // UnForCoo
         ishad:=ToGetShadow(np); // UnitShad
         PlanetForOrder(np,azt,alt,ro,vmag,phase,ishad,numc);
       End;
   For i:=1 To 3 Do  PosDim[NEarth][i]:=0.0; // array PosDim from PUnDE200
   ShellMethodForOrdering(numc,RealSort,IntOrder); // UnitSort with range
   PlanetsToHeapWithFigure(numc);
 End;

 Procedure PosTopPlanet ( np  : Byte ;  tcur  : Extended ;
                          Var  azt , alt  : Extended ) ;
 Var
   i  : Byte ;
   ro,TinTDT,TinTDB,dtmp  : Extended ;
   PosE,PosP,tmp,pof,pob  : TVect3 ;
 Begin
   ClcThreeRotMatr(JD2000,tcur, // UnConTyp UnGloVar for StationPos true
                   PlaceCoor,RotMatr,RosMatr,TopMatr); // UnForCoo  equator
   ToObtainAberrationParm(tcur); // UnForRed
   FromUTCtoTT(tcur,TinTDT,dtmp);  // unit UnForTim
   TinTDB:=ToGetTEph(TinTDT); // function from unit UnForTim
   ClcEphEarth(TinTDB,PosE,tmp); // the Earth to Solar system barycentre
   If  NOT BooExistPos  Then  Exit ; // boolean var from unit PUnDE200
   UMatrMultVect(RotMatr,StationPos,pof); // station to the Earth fix equator
   For i:=1 To 3 Do // vector for station position in fixed equator in km
     pob[i]:=PosE[i]+pof[i]; // pob refer to Solar system barycentre
   PosReferToPoint(np,TinTDB,pof,pob,PosP); // PosP refer to station
   ForAberrationCorrection(PosP); // UnForRed correction in fix equator
   MatrMultVector(RotMatr,PosP,tmp); // from fixed to true equator UnForFun
   MatrMultVector(TopMatr,tmp,PosP); // to topocentric horizont UnForFun
   ClcTopCoorInDegree(PosP,azt,alt,ro); // UnForCoo
 End;

 Procedure PosTopSop ( np  : Byte ; // number of Solar system object
                       tcur  : Extended ; // current moment
                       posc  : TPlaceCooRec ; // station geodetic position
                       Var  ptop  : TVect3 ) ; // in topocentric
 Var
   i  : Byte ;
   TinTDT,TinTDB,dtmp  : Extended ;
   pose,posp,tmp,pof,pob  : TVect3 ;
 Begin // StationPos[..] from ClcThreeTorMatr with posc
   ClcThreeRotMatr(JD2000,tcur, // UnConTyp UnGloVar for StationPos true
                   posc,RotMatr,RosMatr,TopMatr); // UnForCoo  equator
   ToObtainAberrationParm(tcur); // UnForRed
   FromUTCtoTT(tcur,TinTDT,dtmp);  // unit UnForTim
   TinTDB:=ToGetTEph(TinTDT); // function from unit UnForTim
   ClcEphEarth(TinTDB,pose,tmp); // the Earth to Solar system barycentre
   If  NOT BooExistPos  Then  Exit ; // boolean var from unit PUnDE200
   UMatrMultVect(RotMatr,StationPos,pof); // station to the Earth fix equator
   For i:=1 To 3 Do // vector for station position in fixed equator in km
     pob[i]:=pose[i]+pof[i]; // pob refer to Solar system barycentre
   PosReferToPoint(np,TinTDB,pof,pob,posp); // PosP refer to station
   ForAberrationCorrection(posp); // UnForRed correction in fix equator
   MatrMultVector(RotMatr,posp,tmp); // from fixed to true equator UnForFun
   MatrMultVector(TopMatr,tmp,ptop); // to topocentric horizont UnForFun
 End;

procedure PosGeoPlanet ( np  : Byte ;  tc  : Extended ;  var  xp  : TVect3 ) ;
var // position of the planet refer to the centre of the Earth with delay
  ic,it  : Byte ;
  tt,tb  : Extended ;
  rr,dc  : Extended ;
  xe,xg  : TVect3 ;
  xs,vs  : TVect3 ;
begin
  FromUTCtoTT(tc,tt,tb);  // unit UnForTim tt scale TDT
  tb:=ToGetTEph(tt); // function from unit UnForTim  tb scale TDB
  if  np <> NMoon
    then
      ClcEphEarth(tb,xe,vs) // PUnDE200 the Earth refer to Solar barycentre
    else
      for ic:=1 to 3 do  xe[ic]:=0.0; // for the Moon refer to the Earth
  if  not BooExistPos  then  Exit; // global boolean var from PUnDE200
  dc:=tb; // for iteration to obtain light delay
  ClcPosVel(np,dc,xs,vs); // for the planet or the Sun from PUnDE200
  for it:=1 to 3 do // xs[..] the planet refer to Solar system barycentre
    begin // the Sun refer to the centre of the Earth
      for ic:=1 to 3 do  xg[ic]:=xs[ic]-xe[ic]; // geocentric for the planet
      rr:=VectorModul(xg); // function from UnForFun
      dc:=tb-(rr/VelOfLight)/86400; // time delay const UnConTyp
      ClcPosVel(np,dc,xs,vs); // the planet or the Sun position with delay
    end;
  for ic:=1 to 3 do  xp[ic]:=xs[ic]-xe[ic]; // fix equator geocentric the planet
end;

 Function CentreWithFixPlanet : Boolean ; // called from UnitFixO
 Var // to obtain AzimutC AltitudeC centre of the screen in horizon
   tmin,tmax,tcur,step,azt,alt  : Extended ;
 Begin
   CentreWithFixPlanet:=False; // default
   If  ( PlanetNumFix < NMercury ) or ( PlanetNumFix > NSun )
     Then  // it is strange number
       Exit ; // number of planet may be within NMercuty..NSun
   if  CharForView = 'L' // 'L' look option without visibility
     then                // to  look around from the satellite
       begin             // full sky above anfd lower horizon
         CentreWithFixPlanet:=True; // well on current date
         PosTopPlanet(PlanetNumFix, // selected planet
                      JulianDate,   // current moment
                      AzimutC,AltitudeC); // new centre
         Exit; // only for look option full sky lower horizon
       end;
   step:=0.5/24.0; // for step in day to find moment of visibility
   tmin:=JulianDate;       // start  moment to find
   tmax:=tmin+60.0*step;   // finish moment to find
   tcur:=tmin-step;
 Repeat // step by step to find
   tcur:=tcur+step; // visibility from a ststion
   PosTopPlanet(PlanetNumFix,tcur,azt,alt); // selected planet
   If  NOT BooExistPos  Then  Exit ; // boolean var from unit PUnDE200
 Until  ( alt > 0.0 ) or ( tcur > tmax ) ;  // may be visibility
   if  alt > 0.0 // altitude more than 0.0 degree for visibility
     then
       begin     // new centre for field of view
         CentreWithFixPlanet:=True; // well
         JulianDate:=tcur; // for new date
         AzimutC:=azt;     // new centre azimuth
         AltitudeC:=alt;   // new centre altitude
         If  AltitudeC < RinDegC-1 // but above horizon
           Then // all these variables from UnGloVar
             AltitudeC:=RinDegC-1; // above horizon
       end;
 End;

 Procedure RotatingGeoPos ( np  : Byte ;  // number of planet
                            Var  pp  : TVect3 ;
                            Var  fs , vs  : Extended ) ; // lat longitude degree
 Var
   i,iter   : Byte ;
   tdt,tdb  : Extended ;
   tb,rs,dt : Extended ;
   pe,vp    : TVect3 ;
 Begin
   ClcTrueRotMatr(JD2000,JulianDate,RotMatr,RosMatr);  // UnForCoo  equator
   FromUTCtoTT(JulianDate,tdt,dt);  // unit UnForTim
   tdb:=ToGetTEph(tdt); // function from unit UnForTim
   case  np  of
     NMoon :  ClcPosVel(np,tdb,pp,vp); // the Moon refer to the Earth now
     else
 begin
   ClcEphEarth(tdb,pe,vp); // the Earth to Solar system barycentre
   If  NOT BooExistPos  Then  Exit ; // boolean var from unit PUnDE200
   ClcPosVel(np,tdb,pp,vp); // the Sun to Solar system barycenter
   For i:=1 To 3 Do // position of the Sun in fixed equator
     pp[i]:=pp[i]-pe[i]; // refer to the centre of the Earth
   For iter:=1 To 3 Do // to get position of the Sun
     Begin // with light delay by simple iteration VectorModul UnForFun
       rs:=VectorModul(pp); // range from point to the planet in km
       dt:=(rs/VelOfLight)/86400; // light delay in part of day UnConTyp
       tb:=tdb-dt; // moment with light delay  PUnDE200
       ClcPosVel(np,tb,pp,vp); // the Sun position with light delay
       For i:=1 To 3 Do  pp[i]:=pp[i]-pe[i]; // refer to Earth centre
     End; // in fixed equator
 end;
   end; // case  np
   MatrMultVector(RotMatr,pp,vp); // from fixed to true equator
   MatrMultVector(RosMatr,vp,pp); // to rotating Earth equator UnForFun
   ClcSpherCoorInDegree(pp,vs,fs,rs); // long latitude of the Sun UnForCoo
 End;

 END.
