unit UnForRed ; // for the reductions of the positions

interface

uses
  UnConTyp ; // for type TVect3 const VelOfLight

var
  VelBarEarth  : TVect3 ;

 Procedure ToObtainAberrationParm ( JulUTC  : Extended ) ;

 Procedure ForAberrationCorrection ( Var  PosFix  : TVect3 ) ;

implementation

uses
  UnForTim , // to change some time scale
  PUnDE200 ; // for procedure ClcEphEarth

 { to calculate aberration it is necessary
   to calculate the position and velosity of the Earth }

 Procedure ToObtainAberrationParm ( JulUTC  : Extended ) ;
 Var
   i  : Byte ;
   dt,tdt,TinTDB  : Extended ;
   PosBar,VelBar  : TVect3 ;
 Begin
   FromUTCtoTT(JulUTC,tdt,dt); // UnForTim time terrestrial
   TinTDB:=ToGetTEph(tdt);     // UnForTim time ephemeris
   ClcEphEarth(TinTDB,PosBar,VelBar); { to the barycentre }
   If  NOT BooExistPos  Then  Exit ; { no ephemeris }
   For i:=1 To 3 Do
     VelBarEarth[i]:=VelBar[i]/86400.0; { to km/s }
 End;

 { to correct the geocentric vector for aberration }

 Procedure ForAberrationCorrection ( Var  PosFix  : TVect3 ) ;
 Var
   i    : Byte ;
   u,v  : Extended ;
   b,p  : Extended ;
   q,r  : Extended ;
   pos  : TVect3 ;
   vos  : TVect3 ;
 Begin
   If  NOT BooExistPos  Then  Exit ; // boolean from PUnDE200
   u:=Sqrt(Sqr(PosFix[1])+Sqr(PosFix[2])+Sqr(PosFix[3]));
   If  u < 1.0e-10  Then  Exit ; { too small }
   For i:=1 To 3 Do
     Begin
       pos[i]:=PosFix[i]/u; { to unit vector }
       vos[i]:=VelBarEarth[i]/VelofLight; // VelOfLight from UnConTyp
     End;
   v:=Sqr(vos[1])+Sqr(vos[2])+Sqr(vos[3]); { V*V }
   b:=Sqrt(1-v); { 1/beta = Sqrt(1-V*V) }
   p:=pos[1]*vos[1]+pos[2]*vos[2]+pos[3]*vos[3]; { scalar multiplication }
   q:=p/(1+b);
   r:=1/(1+p);
   For i:=1 To 3 Do
     Begin
       pos[i]:=r*(b*pos[i]+vos[i]+q*vos[i]); { the unit vector }
       PosFix[i]:=u*pos[i]; { to initial distance }
     End;
 End;

end.