unit UnForSat; // to deal with artificial satellite of the Earth

interface

uses
  UnConTyp , // for TypeDim3 TVect3 const VelOfLight
  UnCoTVar , // type TElemRec  SatElemHeap  NumSatElemInHeap
  UnGloVar ; // type TPlaceCooRec StationPos ClcThreeRotMatr

function ToGetSatPhase ( xp , xs  : TVect3 ) : Single ; // type UnConTyp

function ToGetSatMag ( vm : Single ; ro : Extended ; ph : Single ) : Single ;

procedure SatPosWithLightDelay ( ts  : Extended ;   // moment julian date
                                 el  : TElemRec ;   // satellite elements
                                 pc  : TVect3 ;     // station position
                                 var                // satellite position
                                   xe , xs  : TVect3 ) ; // true equator

function ShadLookSat : Byte ; // our satellite may be in shadow

procedure SatellitesAmongStars ; // current date in JulianDate

procedure PosTopSat ( tcur  : Extended ; // satellite topocentric position
                      elem  : TElemRec ; // type UnCoTVar record elements
                      posc  : TPlaceCooRec ; // UnGloVar geodet position
                      var  xp  : TVect3 ) ; // sat topocentric horizont pos

procedure GeoTopSat ( tcur  : Extended ; // satellite topocentric position
                      elem  : TElemRec ; // type UnCoTVar record elements
                      posc  : TPlaceCooRec ; // UnGloVar geodet position
                      var  xe  : TVect3 ;    // sat geocentric true eqautor
                      var  xp  : TVect3 ) ;  // sat topocentric horizont pos

function CentreWithFixSatellite : Boolean ; // called from UnitFixO

procedure SatelliteToMapWorld ( xs , ys ,xm , ym  : Integer ) ;

function stNumSatlVis ( a , d  : Extended ) : ShortString ;

implementation

uses
  UnitEilP , // for ElemX procedure
  PUnDE200 , // for const NSun
  UnitShad , // for byte function ToGetShadowCase
  UnForFun , // for MatrMultVect procedure function VectorModul
  UnForCoo , // for ClcTopCoorInDegree GeoSpherFromCart
  UnPlaceA , // for RectCoordinates procedure
  UnPlateC , // for IdealCoordinates procedure
  UnForZem , // variables PosZemelaC PosHorizoC
  UnGraAll ; // for procedure FigureSatellite ;

{ ns simple number
  az al ro topocentric horizon spheric coordinates
  xp topocentric horizon descart vector
  vmag phasep magnitude phase
  ishad index for shadow
  our satellite may be on screen
      or may be out our hemisphere
  PosHorizoC UnGloVar topo horizon vector of the centre
     our planet may be out of our hemisphere
     ScalarMunt simple cosinus between two vectors
     if cosinus > 0.5 then planet is in our hemishere
     if cosinus > 0.5 then planet may be in our field of view
  for 'L' option
     when objects are observed from the satellite
     may occur situation of cover these objects by the Earth
  PosZemelaC descart position of the Earth's centre UnForZem
  AngleZemeL angle in degree between two directions
     from the satellite to the Earth's centre
     from the satellite to the special point on the surface
     if cosinus between
            direction from the satellite to our object
        and direction from the satellite to the Earth's centre
            greater than cos(AngleZemeL)
            then all objects are covered by the Earth }

procedure SatelliteToHeap (  ns  : LongInt ;
                             az , al , ro  : Extended ;
                             xp  : TVect3 ;  // type UnConTyp
                             vmag , phasep   : Single ;
                             ishad  : Byte ) ;
var
  x,y  : Integer ;
  ksi,eta  : Extended ;
  bos  : Boolean ; // satellite in or out our hemisphere
begin
   bos:= ( ScalarMunt(xp,PosHorizoC) > 0.5 ) ; // UnForFun
   if  CharForView = 'L' // look from the satellite
     then                // only for look option
       bos:= bos         // boolean variable  to compare
             and ( (  ScalarMunt(xp,PosZemelaC)    // cosinus one
                      < Cos(GraRad*AngleZemeL) )  // cosinus two
                  or ( ro < SurfaRange ) ) ; // UnForZem
   if  bos        // satellite is in our hemishere
     then         // this satellite is in our hemisphere
       begin      // this satellite may be in our field of view
         IdealCoordinates(az,al,ksi,eta); // from UnPlateC
         RectCoordinates(ksi,eta,x,y);    // from UnPlaceA
       end // position projection to the plate
     else     // this planet is out our hemisphere
       begin  // this palnet is out our field if view
         x:=-30000; // default large minus position
         y:=-30000; // default large minus position
       end;   // to exlude this planet
  ObjectTopCur.nall:=ns; // number of satellite in SatElemHeap^
  ObjectTopCur.chps:='S'; // there is Earth artificial satellite
  ObjectTopCur.name:=SatElemCur.satname; // SatElemCur from UnCoTVar
  ObjectTopCur.azt:=az; // current azimuth
  ObjectTopCur.alt:=al; // current altitude
  ObjectTopCur.rot:=ro; // current range in km
  ObjectTopCur.Mag:=vmag; // magnitude
  ObjectTopCur.phase:=phasep; // for satellite
  ObjectTopCur.IndexShadow:=ishad; // default
  if  al > 0.0
    then
      ObjectTopCur.boovis:=True // visibility
    else
      ObjectTopCur.boovis:=False; // no visibility
  if  BooPosInRect(x,y)
    then
      ObjectTopCur.booscreen:=True
    else
      ObjectTopCur.booscreen:=False;
  ObjectTopCur.xpos:=x; // x on screen
  ObjectTopCur.ypos:=y; // y on screen
  Inc(NumObjectInHeap);
  if  ns = 1  then  NumFirstSatHeap:=NumObjectInHeap; // var UnCoTVar for sort
  ObjectTopHeap^[NumObjectInHeap]:=ObjectTopCur;
  if  ObjectTopCur.booscreen
    then
      FigureSatellite ; // unit UnGraAll
end;

function ToGetSatPhase ( xp , xs  : TVect3 ) : Single ; // type UnConTyp
var // xp[..] satellite xs[..] the Sun topocentric true equator position
  i       : Byte ;
  ra,rb   : TVect3 ;
  c,a,b   : Extended ;
begin
  for i:=1 to 3 do
    begin // to satellite xp is satellite topocentric position
       ra[i]:=-xp[i]; // simple view from the satellite to station
       rb[i]:=+xs[i]-xp[i]; // simple view from the satellite to the Sun
    end; // by PlanetsToScreen in UnForPos as the Sun topocentric position
  a:=VectorModul(ra); // range from satellite to station
  b:=VectorModul(rb); // range from satellite to the Sun
  c:=(ra[1]*rb[1]+ra[2]*rb[2]+ra[3]*rb[3])/(a*b); // cos(Phase) scalar mult
  ToGetSatPhase:=0.5*(1+c); // default
end;

function ToGetSatMag ( vm : Single ; ro : Extended ; ph : Single ) : Single ;
var // vm may be known value for some satellites
  vc  : Single ; // ro is range from satellite to site in km
begin            // ph is phase without unit
  vc:=99;
  if ( vm < 50.0 ) and ( ph > 0.01 )
    then
      vc:=vm-15.75+2.5*Log10(Sqr(ro)/ph); // Log10 in UnForFun
  ToGetSatMag:=vc;
 End;

procedure SatPosWithLightDelay ( ts  : Extended ;   // moment julian date
                                 el  : TElemRec ;   // satellite elements
                                 pc  : TVect3 ;     // station position
                                 var                // satellite position
                                   xe , xs  : TVect3 ) ; // true equator
var  // xe - geocentric satellite position , xs - topocentric
  it  : Byte ;
  tc  : Extended ;
  rc  : Extended ;
begin
  it:=0;
  tc:=ts;
repeat
  it:=it+1;
  ElemX(tc,el,xe); // unit UnitEilP refer to the Earth
  xs[1]:=xe[1]-pc[1]; // in true equator system
  xs[2]:=xe[2]-pc[2]; // topocentric satellite position
  xs[3]:=xe[3]-pc[3]; // in true equator system
  rc:=VectorModul(xs); // range in km function from UnForFun
  tc:=ts-(rc/VelOfLight)/86400.0; // light delay in part of day
until  it = 1 ; // simple iteration for delay
  ElemX(tc,el,xe); // unit UnitEilP refer to the Earth
  xs[1]:=xe[1]-pc[1]; // in true equator system
  xs[2]:=xe[2]-pc[2]; // topocentric satellite position
  xs[3]:=xe[3]-pc[3]; // in true equator system
end;

function ShadLookSat : Byte ; // our satellite may be in shadow
var
  t  : Extended ;
  x  : TVect3 ;
begin
  t:=JulianDate; // current moment for our satellite
  ElemX(t,SatElemLook,x); // UnitEilP refer to the Earth
  ShadLookSat:=ToGetShadowCase(t,x); // UnGloVar UnitShad
end;

procedure SatellitesAmongStars ; // current date in JulianDate UnGloVar
var
  ns  : LongInt ;  // count for satellites from heap
  tc  : Extended ; // current moment julian date
  xe  : TypeDim3 ; // position of the satellite in km in true equator
  xs  : TVect3 ;   // satellite refer to the station  in true equator
  xp  : TVect3 ;   // satellite topocentric horizont position
  pc  : TVect3 ;   // true equator position of the station or satellite
  azt,alt,ro  : Extended ;
  vmag,phase  : Single ;
  ishad  : Byte ; // 0 no shadow 1 semishadow 2 shadow
begin // current station position in km StationPos in true equator system
  if  JulianDate < 2436117.0  then  Exit ; // 5 october 1957 year
  tc:=JulianDate; // global var UnGloVar
  case  CharForView  of   // 'S' sky or 'L' look options
          // a look from the station with StationPos in true equator
    'S' : pc:=StationPos; // for sky a look from the station
          // satellite for looking SatElemLook of TElemRec true equa
    'L' : ElemX(tc,SatElemLook,pc); // UnitEilP refer to the Earth
  end; // case 'L' or 'S' pc
  ns:=0;
repeat  // xe - geocentric  xs - topocentric or satellitecentric
  ns:=ns+1;
  SatElemCur:=SatElemHeap^[ns]; // heap is filled in UnitSatF ToFind...
  if  ( CharForView = 'L' )
    then // 'L' to look from the satellite
      if  SatElemCur.satname = SatElemLook.satname
        then // it is the same satellite no looking
          begin // the record may be the last in heap
            if  ns = NumSatElemInHeap  then  Exit ; // it is last
            ns:=ns+1; // the next number for record from heap
            SatElemCur:=SatElemHeap^[ns]; // the next record
          end;
  SatPosWithLightDelay(tc,SatElemCur,pc,xe,xs); // refer
  MatrMultVector(TopMatr,xs,xp); // to topocentric horizon
  ClcTopCoorInDegree(xp,azt,alt,ro); // UnForCoo with refraction corr.
  phase:=ToGetSatPhase(xp,PosDim[NSun]); // with PosDim[NSun] PUnDE200
  vmag:=ToGetSatMag(SatElemCur.stdmag,ro,phase);
  ishad:=ToGetShadowCase(tc,xe); // function from UnitShad  xp vect
  SatelliteToHeap(ns,azt,alt,ro,xp,vmag,phase,ishad); // topo horiz
until  ns = NumSatElemInHeap ;
end;

procedure PosTopSat ( tcur  : Extended ; // satellite topocentric position
                      elem  : TElemRec ; // type UnCoTVar record elements
                      posc  : TPlaceCooRec ; // UnGloVar geodet position
                      var  xp  : TVect3 ) ; // sat topocentric horizont pos
var
  xe  : TVect3 ; // position of the satellite in km in true equator
  xs  : TVect3 ; // satellite refer to the station  in true equator
begin // value for StationPos[1..3] is obtained by proc ClcThreeRotMatr
  ClcThreeRotMatr(JD2000,tcur, // UnConTyp UnGloVar for StationPos true
                  posc,RotMatr,RosMatr,TopMatr); // UnForCoo  equator
  SatPosWithLightDelay(tcur,elem,StationPos,xe,xs); // above xe geocentric
  MatrMultVector(TopMatr,xs,xp); // to topocentric horizon
end;

procedure GeoTopSat ( tcur  : Extended ; // satellite topocentric position
                      elem  : TElemRec ; // type UnCoTVar record elements
                      posc  : TPlaceCooRec ; // UnGloVar geodet position
                      var  xe  : TVect3 ;    // sat geocentric true eqautor
                      var  xp  : TVect3 ) ;  // sat topocentric horizont pos
var
  xs  : TVect3 ; // satellite refer to the station  in true equator
begin // value for StationPos[1..3] is obtained by proc ClcThreeRotMatr
  ClcThreeRotMatr(JD2000,tcur, // UnConTyp UnGloVar for StationPos true
                  posc,RotMatr,RosMatr,TopMatr); // UnForCoo  equator
  SatPosWithLightDelay(tcur,elem,StationPos,xe,xs); // above xe geocentric
  MatrMultVector(TopMatr,xs,xp); // to topocentric horizon
end;

function CentreWithFixSatellite : Boolean ; // called from UnitFixO
var
  tmin,tmax,tcur,step,pers,azt,alt,rot  : Extended ;
  top  : TVect3 ; // sat position to topocentric horizon
begin             // with refraction
  CentreWithFixSatellite:=False; // default
  if  CharForView = 'L' // for look option
    then                // to look around full sky
      begin             // without visibility
         PosTopSat(JulianDate,     // current date
                   SatElemFix,     // selected satellite as centre
                   PlaceCoor,top); // topocentric horizon
         ClcTopCoorInDegree(top,   // above or lower horizon
                            AzimutC,AltitudeC,rot); // UnForCoo
         CentreWithFixSatellite:=True; // well angle in degree
         Exit; // to figure with new centre
      end;
  { 'S' sky from station all objects only above horizon }
  pers:=1.0/SatElemFix.an; // period of revolution in day
  step:=pers/20.0; // step to find visibility for satellite
  tmin:=JulianDate; // start  moment in day
  tmax:=tmin+1.2;   // finish moment in day
  tcur:=tmin-step;  // initial moment to find visibility
repeat
  tcur:=tcur+step; // global var PlaceCoor from UnGloVar
  PosTopSat(tcur,SatElemFix,PlaceCoor,top); // here above
  ClcTopCoorInDegree(top,azt,alt,rot); // UnForCoo in degree and km
until  ( alt > 0.0 ) or ( tcur > tmax ) ; // may be visibility
  if  alt > 0.0 // our satellite above horizon
    then        // it is visibility
      begin     // at current moment     now to new centre
        CentreWithFixSatellite:=True; // of the field of view
        JulianDate:=tcur; // UnGloVar this moment
        AzimutC:=azt;     // new centre azimuth  in degree
        AltitudeC:=alt;   // new centre altitude in degree
        if  AltitudeC < RinDegC-1 // to compare with half of field
          then                    // only above horizon
            AltitudeC:=RinDegC-1; // centre
      end;
end;

procedure PosGeoSat ( tcur  : Extended ; // current moment in julian day
                      elem  : TElemRec ; // type UnCoTVar current elements
                      var  xr  : TVect3 ; // satl pos refer rotating Earth
                      var  rlon , rlat , rokm  : Extended ) ; // in degree
var // longitude latitude refer to the Earth
  xe  : TypeDim3 ; // position of the satellite in km in true equator
begin
  ClcTrueRotMatr(JD2000,tcur, // UnConTyp UnGloVar
                 RotMatr,RosMatr); // UnForCoo  equator
  ElemX(tcur,elem,xe); // unit UnitEilP refer to the Earth true equator
  MatrMultVector(RosMatr,xe,xr); // UnForFun to the rotating Earth position
  ClcSpherCoorInDegree(xr,rlon,rlat,rokm); // UnForCoo in degree and km
end;

procedure SatelliteToMapWorld ( xs , ys , xm , ym  : Integer ) ;
var  // is called from UnPlaceA  xs ys xm ym  on screen the Sun the Moon
  ns,np  : Byte ; // global var MapSatNumber SatElemMap[] from UnGloVar
  tcur,rlon,rlat,rokm  : Extended ;
begin
  if  JulianDate < 2436117.0 // 5 october 1957 year
    then  MapSatNumber:=0;   // no satellites
  tcur:=JulianDate; // current moment UnGloVar
  for ns:=1 to MapSatNumber do  // MapSatNumber assigned in UnForBut
    begin                       // MapSatGeoPos[1..8][1..3] UnGloVar
      SatElemFix:=SatElemMap[ns]; // UnCoTVar UnGloVar assigned UnForBut
      PosGeoSat(tcur,SatElemFix,MapSatGeoPos[ns],rlon,rlat,rokm); // above
      MapSatShadow[ns]:=ToMapShadowCase(MapSatGeoPos[ns], // UnitShad
                                        MapSatGeoPos[MapSatNumber+1]);
      FigureImageSat(ns,rlon,rlat,rokm); // from UnGraAll
    end;
  np:=MapSatNumber+1; // for the Sun
  MapSatX[np]:=xs;
  MapSatY[np]:=ys;
  np:=np+1; // for the Moon
  MapSatX[np]:=xm;
  MapSatY[np]:=ym;
end;

function stNumSatlVis ( a , d  : Extended ) : ShortString ;
var              // a longitude b latitude in degree of the point
  ns  : Byte ;
  nm  : Byte ;
  nv  : Byte ;
  uc  : Extended ;
  us  : Extended ;
  st  : ShortString ; // auxiliary
  xp  : TVect3 ; // point position in rotating Earth
  xs  : TVect3 ; // satellite position refer to point rotating Earth
begin            // about number of visible satellite from this point
  GeoCartFromSpher(d,a,0.0,xp[1],xp[2],xp[3]); // proc from UnForCoo
  for ns:=1 To 3 do         // the center of the Earth refer to the point
    xp[ns]:=-1.0e-3*xp[ns]; // from meter to kilometer for the point
  nv:=0; // default number for visible satellites
  for nm:=1 to MapSatNumber do // UnGloVar
    begin                      // current satellite from the map
      for ns:=1 to 3 do        // satellite position refer to the point
        xs[ns]:=MapSatGeoPos[nm][ns]+xp[ns]; // in rotating Earth system
      uc:=ScalarMunt(xs,xp);   // UnForFun cosinus between two vectors
      us:=Sqrt(1-Sqr(uc));     // sinus   MapSatAltitude from UnGloVar
      MapSatAltitude[nm]:=ATanDegree(us,uc)-90.0;  // altitude in degree
      if  MapSatAltitude[nm] > 0.0  then  nv:=nv+1; // it is visibility
    end;
  Str(nv:1,st);
  if  MapSatNumber = 0
    then  st:=' '
    else
      case  nv  of
        0  : st:=' ';
        1  : st:=' '+st+' ';
        2,3,4  : st:=' '+st+' ';
        5,6,7,8  : st:=' '+st+' -';
      end; // case nv
  stNumSatlVis:=st;
end;

end.
