
 UNIT UnForSta ; // a few procedures for stars coordinates

 INTERFACE

 Uses
   UnConTyp ; // for type TVect3

 Procedure StarToMemory ; // NumberOfStars around AlphaC DeltaC to memory

 Procedure StarToFixEquator ( Var  PosFix  : TVect3 ) ;

 Procedure StarToTrueEquator ( Var  PosTrue  : TVect3 ) ;

 Procedure StarHorizontCoor ( n  : Word ; // simple current number
                              Var  azt , alt  : Extended ; // degree
                              Var  bos  : Boolean ) ; // yes or no

 Function CentreWithFixStar : Boolean ; // called from UnitFixO

 IMPLEMENTATION

 Uses
   UnGloVar , // for variable JulianDate
   UnCotVar , // for record StarOneCur StarAllHeap^[]
   UnForSys , // proc FromTopToFixEqu
   PUnCRead , // proc ToReadStarsFrom B or T catalogue
   UnForRed , // proc ForAberrationCorrection
   UnForFun , // for MatrMultVector
   UnForCoo ; // proc ClcTopCoorInDegree

 { variables in degree
   AzimutC AltitudeC AlphaC DeltaC are declared in unit UnGloVar
   PosHorizoC is topocentric horizontal
     rectangular position of the centre of the field of view }
 Procedure StarToMemory ; // NumberOfStars around AlphaC DeltaC to memory
 Begin // AlphaC DeltaC centre of the field of view in J2000 fixed equator
   FromTopToFixEqu(AzimutC,AltitudeC, // from horizon to equator
                   AlphaC,DeltaC,PosHorizoC); // UnForSys in degree
   If  not BrightStarsOnly // procedure from PUnCRead
     Then // NumBrStarsInHeap from UnCoTVar NumberOfStars from UnGloVar
       ToReadStarsFromT(NumBrStarsInHeap,NumberOfStars) // stars in memory
     Else // proc ToReadStars... from PUnCRead
       NumberOfStars:=NumBrStarsInHeap;
 End;

 { to change stellar position
   refer to fixed equator to take into account proper motion
   information about star in record StarOneCur }
 Procedure StarToFixEquator ( Var  PosFix  : TVect3 ) ;
 Var
   a,d,dt,rr,
   sa,ca,sd,cd,
   va,vd,vx,vy,vz,
   x,y,z,xx,yy,zz   : Extended ;
 Begin
   dt:=(JulianDate-JD2000)/365.25; { differences in year from J2000.0}
   With  StarOneCur  Do // variable from unit UnCotVar
     Begin
       a:=GraRad*Alpha; // to radian right ascension
       d:=GraRad*Delta; // to radian declination
     End;
   With  StarOneCur  Do // variable from unit UnCotVar const from UnConTyp
     Begin
       va:=GraRad*AlphaMotion/3600.0; { in radian per year }
       vd:=GraRad*DeltaMotion/3600.0; { vd but va*cos(delta) }
     End;
   sa:=Sin(a);
   ca:=Cos(a);
   sd:=Sin(d);
   cd:=Cos(d);
   x:=ca*cd; { position at J2000.0 }
   y:=sa*cd;
   z:=sd;
   vx:=-va*sa-vd*sd*ca;
   vy:=+va*ca-vd*sd*sa;
   vz:=+vd*cd;
   xx:=x+vx*dt; { new position at JulianDate but in J2000.0}
   yy:=y+vy*dt;
   zz:=z+vz*dt;
   rr:=Sqrt(xx*xx+yy*yy+zz*zz);
   PosFix[1]:=xx/rr;
   PosFix[2]:=yy/rr;
   PosFix[3]:=zz/rr;
 End;

 { two operations for change stellar position
   to take into account
   proper motion , precession , nutation , aberration
   information about star in record StarOneCur }
 procedure StarToTrueEquator ( var  PosTrue : Tvect3 ) ;
 var
   PosFix  : TVect3 ;
 begin
   StarToFixEquator(PosFix); // by using record StarOneCur
   ForAberrationCorrection(PosFix); // UnForRed correction in fix equator
   MatrMultVector(RotMatr,PosFix,PosTrue); // from fixed to true UnForFun
 end;

 { for current star from StarHeap record StarOneCur
   to obtain topocentric horizontal spheric azimuth altitude
   to obtain the star being in the field of view or no
      another words a star may be out our hemisphere
   this procedure is called by proc StarToScreen from UnPlaceA }

 Procedure StarHorizontCoor ( n  : Word ; // simple current number
                              Var  azt , alt  : Extended ; // degree
                              Var  bos  : Boolean ) ; // yes or no
 Var  // for horizontal azimuth altitude
   PosTrue,PosTop  : TVect3 ;
   ro  : Extended ;
 Begin  // procedure ToGetStarFromHeap from unit PUnCRead
   ToGetStarFromHeap(n); // information to record StarOneCur
   StarToTrueEquator(PosTrue); // from unit UnForSta with aberration
   MatrMultVector(TopMatr,PosTrue,PosTop); // from unit UnForFun
   ClcTopCoorInDegree(PosTop,azt,alt,ro); // unit UnForCoo in Degree
   { PosHorizoC UnGloVar topo horizon vector of the centre
     our star may be out of field of view (hemisphere)
     ScalarMunt simple cosinus between two vectors
     if  cosinus > 0.5 then our star is in our hemisphere
         and more our star may be in our field of view }
   bos:= ( ScalarMunt(PosTop,PosHorizoC) > 0.5 ) ; // UnForFun
   { for 'L' option
     when objects are observed from the satellite
     may occur situation of cover these objects by the Earth
     PosZemelaC descart position of the Earth's centre UnForZem
     AngleZemeL angle in degree between two directions
     from the satellite to the Earth's centre
     from the satellite to the special point on the surface
     if cosinus between
            direction from the satellite to our object
        and direction from the satellite to the Earth's centre
        greater than cos(AngleZemeL)
        then all objects are covered by the Earth }
   if  CharForView = 'L' // look from the satellite
     then                // only for look option
       bos:= bos         // boolean variable  to compare
             and ( ScalarMunt(PosTop,PosZemelaC) // cosinus one
                   < Cos(GraRad*AngleZemeL) ) ;  // cosinus two
 End;

 Function CentreWithFixStar : Boolean ; // called from UnitFixO
 Var  // for horizontal azimuth altitude
   ro  : Extended ;
   pe  : TVect3 ;
   pt  : TVect3 ;
 Begin  // inform about fixed star is in record StarOneFix
   StarOneCur:=StarOneFix; // information to record StarOneCur
   StarToTrueEquator(pe); // from unit UnForSta  to equator
   MatrMultVector(TopMatr,pe,pt); // from unit UnForFun  to horizon
   ClcTopCoorInDegree(pt,AzimutC,AltitudeC,ro); // unit UnForCoo
   if  CharForView = 'S' // only 'S but 'L' look option full sky
     then // 'S' for sky option stars above horizon
       if  AltitudeC < RinDegC-1
         then // all these variables from UnGloVar
           AltitudeC:=RinDegC-1;
   CentreWithFixStar:=True;
 End;

 END.
