
 UNIT UnForSys ;

 INTERFACE

 Uses
   UnConTyp ; // for type TVect3

 Procedure FromTopToTrueEqu ( Azt , Alt  : Extended ; // degree
                              Var  Alpha , Delta  : Extended ) ;

 Procedure FromTopToFixEqu ( Azt , Alt  : Extended ; // horiz degree
                             Var  Alpha , Delta  : Extended ; // degr
                             Var  PosTop  : TVect3 ) ; // horizon

 procedure FromFixEquToEcl ( pf  : TVect3 ;  var  pe  : TVect3 ) ;

 procedure FromFixEquToGal ( pf  : TVect3 ;  var  pe  : TVect3 ) ;

 IMPLEMENTATION

 Uses
   UnGloVar , // for matrix variables RotMatr TopMatr MatrEcl
   UnForCoo , // for DescFromSpherCoor
   UnForFun ; // for matrix procedures MatrMultVector UMatrMultVect

 { from topocentric azimuth altitude for a star
   to true equator right ascension declination range = 1 }
 Procedure FromTopToTrueEqu ( Azt , Alt  : Extended ; // in degree
                              Var  Alpha , Delta  : Extended ) ; // degree
 Var
   ro  : Extended ;
   rtop,requ  : TVect3 ;
 Begin
   ro:=1.0e0; // default only angle variables    but azimuth left
   DescFromHorizCoor(Azt,Alt,ro,rtop); // from UnForCoo direction
   UMatrMultVect(TopMatr,rtop,requ); // from UnForFun
   ClcSpherCoorInDegree(requ,Alpha,Delta,ro); // in degree true equator
 End;

 { from topocentric azimuth altitude for a star
   to fixed equator right ascension declination range = 1
   this procedure is called by StarToMemory in UnForSta
   to obtain fixed equator position of the images centre
   but vector PosTop is topocentric horizon descart position }
 Procedure FromTopToFixEqu ( Azt , Alt  : Extended ; // horiz degree
                             Var  Alpha , Delta  : Extended ; // degr
                             Var  PosTop  : TVect3 ) ; // horizon
 Var
   ro  : Extended ; // simple range 1
   re  : TVect3 ;   // true  equator position
   rf  : TVect3 ;   // fixed equator position
 Begin              // Alpha Delta in degree fixed equator
   ro:=1.0e0; // default only angle variables    but azimuth left
   DescFromHorizCoor(Azt,Alt,ro,PosTop); // from UnForCoo direction
   UMatrMultVect(TopMatr,PosTop,re); // UnForFun to true equator
   UMatrMultVect(RotMatr,re,rf); // UnForFun to fixed equator
   ClcSpherCoorInDegree(rf,Alpha,Delta,ro); // degree fix equator
 End;

 procedure FromFixEquToEcl ( pf  : TVect3 ;  var  pe  : TVect3 ) ;
 begin // position from standard equator to standard ecliptic
   MatrMultVector(MatrEcl,pf,pe); // UnForFun UnGloVar
 end; // MatrEcl is filled by procedure ClcThreeRotMatr from UnForCoo

 procedure FromFixEquToGal ( pf  : TVect3 ;  var  pe  : TVect3 ) ;
 begin // position from standard equator to standard galactic
   UMatrMultVect(MatrGal,pf,pe); // UnForFun UnGloVar to galactic
 end; // Const MatrGal from UnGloVar

 END.