
 UNIT UnGetIni ; // initial for figure

 INTERFACE

 Procedure TimeVarInit ( istep  : Byte ) ; // some time variables

 Procedure CentreCoorH ; // for initial azimut altitude

 Procedure ForIniC ( ic  : Byte ) ; // ic =0 before Form1 is opened

 IMPLEMENTATION

 Uses
   SysUtils , // or Date function, DeCodeDate , DeCodeTime procedures
   UnConTyp ,
   UnGloVar , // for TPlaceCooRec variable JulianDate  RotMatr RosMatr
   PUnCRead , // VAllMag for example
   UnForCoo , // for GeoSpherFromCart  ClcThreeRotMatr
   UnitStat , // for intial NumStationL and StationList[] of TPlaceCooRec
   UnForDat , // unit for TransDateToJD
   UnForTim , // unit for TransUTCtoTDT
   UnMuDraw , // for boolean boomoud
   UnRefrOl , // refraction procedure
   UnitSatF , // for SatNoradFileFind
   UnitFixO , // for InitForFixCase procedure
   PUnDE200 ; // for ToGetCoeFileName ToReadEphemeris

 Function CatalogueExist : Boolean ;
 Begin
   VAllMin:=+25; VAllMax:=-25;
   VMagMin:=-5; VMagMax:=25;
   If  ExistBrFile // boolean variable from unit PUnCRead
     Then
       Begin
         RinDegC:=15; // in degree for full field of view UnGloVar
         BrightStarsOnly:=TRUE; //  bright stars catalogue only for demo
         CatalogueExist:=True;
       End
     Else
       If  ExistTRCFile
         Then
           Begin
             RinDegC:=1.5; // in degree for small field of view  UnGloVar
             BrightStarsOnly:=FALSE; // boolean variable from unit PUnCRead
             CatalogueExist:=True;
           End
         Else { there is no catalogues }
           CatalogueExist:=False;
 End;

 Procedure SomeInitVar ;
 Begin
   WhiteColorOnly:=FALSE;
   BooNegative:=FALSE;
   BooReticulum:=FALSE;
   BooHourImage:=FALSE;
   BooRefraction:=TRUE; { to refraction take into account }
   SimpleDataForRefraction ; { from UnRefrOl }
   BooAddInform:=TRUE; { to add information on the screen }
   BooDayNight:=FALSE;
   BooAzNorthS:=TRUE; { geodetic azimuth from the north point }
   BooAlphaHourA:=TRUE; { right ascension for the equator system }
   CharDayNight:=' ';
   CharDayNightMem:=' ';
   StrDayNight:='';
   BooPlanetLook:=FALSE; { no real unit if objects is figured }
   BooAutoControl:=FALSE;
   CharReferSystem:='H'; { H - horizontal , A - equator }
   CharRefSysteMem:=' ';
   StrRefSystem:=' horizon';
   BooRusLat:=True; { FALSE for latinian  TRUE for russian }
   CharForView:='S'; { to view sky but 'W' for world map }
   VMagMax:=25;    { to limit stars with magnitude }
   boomoud:=False; { UnMuDraw to draw by mouse if true and Ctrl }
   boomaif:=True;  { UnGloVar main form fmPuz is active }
 End;

 Procedure ForStatInit ; // initial data for station default
 Var
   rx,ry,rz,rf,rl,rh  : Extended ;
 Begin
   rx:=2886365.206; // cartesian
   ry:=2155941.870; // position
   rz:=5245817.642; // in metre for station
   GeoSpherFromCart(rx,ry,rz,rf,rl,rh); // from unit UnForCoo
   With  PlaceCoor  Do
     Begin
       num:=1872;
       name:='';  // default
       x:=rx; // metre
       y:=ry; // metre
       z:=rz; // metre
       f:=rf; // geodetic latitude in degree
       l:=rl; // geodetic longitude in degree
       h:=rh; // geodetic height in metre
       c:=2;  // hour zone civil time
       d:=1;  // decret hour
       s:=1;  // summer hour default for Zvenigorod
     End;
   NumStatValue:=1; // number of station in list Zvenigorod default
   NumSizeValue:=1; // initial number of area size in list
   NumStationL:=1; // var from UnitStat
   StationList[NumStationL]:=PlaceCoor;
 End;

 Procedure TimeVarInit ( istep  : Byte ) ; // simple initial values
 Begin                                     // of some time variables
   CharTimeSystem:='U'; { U - UTC,M - MDT,C - civil time C.T,D - CDT }
   DCivilTime:=PlaceCoor.c/24.0; { for Moscow civil zone time }
   DecreTMoscow:=DCivilTime+PlaceCoor.d/24.0; { for Moscow decret time }
   DHourImage:=1.0/24.0; { one hour earlier }
   DecretTime:=DecreTMoscow;
   If  BooFixObject or ( istep <> 1 )  // BooFixObject from UnGloVar
     Then         // current number of this value in step list
       Begin      // NumStepValue from unit UnitSelH
         StepWithTime:=1.0/24.0; // one hour step default UnGloVar
         NumStepValue:=18; // current number of this value in step list
       End;
   SummerHour:=PlaceCoor.s; // all these global variables from unit UnGloVar
   if  PlaceCoor.s = 1
     then
       BooSummerJ:=True { hour jump for summer time for Russia our Country }
     else
       BooSummerJ:=False;
   BooRiSetP:=False;
   InitForFixCase; // chFixObject:=' ' BooFixObject:=False UnitFixO
 End;

 Procedure CurDateTime ; // to get current date and time in julian day
 Var
   Day,Month,Year      : Word ;
   Hour,Min,Sec,SecT   : Word ;
   iday,imonth,iyear   : Integer ;
 Begin
   DeCodeDate(Date,Year,Month,Day); // from SysUtils for current date
   iday:=Day;
   imonth:=Month;
   iyear:=Year;
   TransDateToJD(iday,imonth,iyear,JulianDate); // unit UnForDat
   DeCodeTime(Time,Hour,Min,Sec,SecT); // from SysUtils for current time
   JulianDate:=JulianDate+(Hour+Min/60.0)/24.0-DecretTime; // to London
   If  BooSummerJ // one hour jump is possible for this station
     Then
       If  BooHourJump(JulianDate) // from unit UnForTim
         Then
           JulianDate:=JulianDate-SummerHour/24.0; // one hour jump
   ClcThreeRotMatr(JD2000,JulianDate, // UnConTyp UnGloVar
                   PlaceCoor,RotMatr,RosMatr,TopMatr); // UnForCoo
 End;

 Procedure CentreCoorH ; // for initial azimut altitude
 Begin
   AzimutC:=180; // topocentric horizontal azimut in degree
   If  PlaceCoor.f > 0
     Then // north hemisphere
       AltitudeC:=90-PlaceCoor.f
     Else // south hemisphere
       Begin
         AltitudeC:=90+PlaceCoor.f;
         AzimutC:=0.0;
       End;
   If  AltitudeC < RinDegC-1
     Then // initial altitude topocentric horizontal in degree
       AltitudeC:=RinDegC-1;
 End;

 Procedure EphemInit ; // for DE200/LE200 ephemeris
 Var
   DeltaTA,TinTDT,TinTDB  : Extended ;
 Begin // if  NOT BooExistPos then there is no ephemeris
   BooOpenCoeFile:=FALSE;
   BooExistEph:=FALSE;
   BooExistPos:=FALSE;
   BooCircle:=TRUE; // two circles for the Moon eclipse case
   FromUTCtoTT(JulianDate,TinTDT,DeltaTA); // unit UnForTim in julian day
   TinTDB:=ToGetTEph(TinTDT); // unit UnForTim in julian day
   ToGetCoeFileName; // from unit PUnDE200
   ToReadEphemeris(TinTDB);  // from unit PUnDE200
 End;

 Procedure EphemCurrent ; // for DE200/LE200 ephemeris
 Var
   DeltaTA,TinTDT,TinTDB  : Extended ;
 Begin // if  NOT BooExistPos then there is no ephemeris
   BooCircle:=TRUE; // two circles for the Moon eclipse case
   FromUTCtoTT(JulianDate,TinTDT,DeltaTA); // unit UnForTim in julian day
   TinTDB:=ToGetTEph(TinTDT); // unit UnForTim in julian day
   ToReadEphemeris(TinTDB);  // from unit PUnDE200
 End;

 Procedure ForIniC ( ic  : Byte ) ; // ic =0 before Form1 is opened
 Begin
   If  NOT CatalogueExist  Then  Exit ; // no stars catalogue
   SomeInitVar ; // initial values for boolean char and string variables
   if  ic = 0
     then
       ForStatInit ; // initial for station position and identification
   TimeVarInit(ic) ; // initial values for some time boolean char variables
   CurDateTime ; // to get current date and time in julian day
   CentreCoorH ; // for initial azimut altitude
   If  ic > 0
     Then // file with ephemeris is opened
       EphemCurrent
     Else
       Begin
         EphemInit ; // to define and open file with the planet ephemeris
         SatNoradFileFind ; // to find file with *.tle in UnitSatF
       End;
 End;

 END.
