
 UNIT UnGloVar ; // for global variables

 INTERFACE

 Uses
   UnConTyp , // for TypeDim3 TpeDim33
   UnCoTVar ; // for TElemRec

 Type
   { for the different coordinates system }
   TAllSystemRec   = Record
                       ro            : Extended ;
                       ah , hh       : Extended ; { horizontal }
                       aa , da       : Extended ; { equator }
                       le , be       : Extended ; { ecliptic }
                       lg , bg       : Extended ;
                     End;

 Const
   MatrUnit       : TypeDim33  =
                  ( ( 1.0 , 0.0 , 0.0 ) ,
                    ( 0.0 , 1.0 , 0.0 ) ,
                    ( 0.0 , 0.0 , 1.0 ) ) ;
  { to change from galactic to equatorial system from HIP v.1,p.92 }
   MatrGal        : TypeDim33 =
                  ( ( -0.0548755604 , +0.4941094279 , -0.8676661490 ) ,
                    ( -0.8734370902 , -0.4448296300 , -0.1980763734 ) ,
                    ( -0.4838350155 , +0.7469822445 , +0.4559837762 ) ) ;

 Type
   TypeFundArg       = array [1..5] of Extended ;

 Type
   TPlaceCooRec   = Record
                      num        : Integer ;
                      name       : ShortString ;
                      x , y , z  : Extended ; // position in metr
                      f , l , h  : Extended ; // lat long degree h metr
                      c , d , s  : ShortInt ; // hour : zone decret summer
                    End;

 Type
   TEclipseRec    = Record
                      centreshad  : TypeDim3 ;
                      fullshadow  : Extended ;
                      semishadow  : Extended ;
                    End ;

 Var
   NumberOfStars    : Word ;
   NumStarsInRect   : Word ;
   SmallStarSize    : ShortInt ;

 Var
   BooAddInform     : Boolean ;
   BooNegative      : Boolean ;
   BooReticulum     : Boolean ;
   BooFixObject     : Boolean ;
   BooPlanetLook    : Boolean ;
   BooApp,
   BooApprox,
   BooRiSetP,
   BooSatElem,
   BooExistSat,
   BooDayNight,
   BooHourImage,
   BooRefraction,
   BooAutoControl   : Boolean ;

 Var
   BooRusLat,
   BooAzNorthS,
   BooAlphaHourA    : Boolean ;
   CharDayNight,
   CharDayNightMem  : Char ; { D - day , T - twilight , N - night }
   StrDayNight,
   StrDayNightMem   : String [8] ;
   StrRefSystem     : String [8] ;
   TheSunHight      : Extended ; { the hight of the sun under horizon }
   StrTheSunHight   : String [30] ;
   CharReferSystem  : Char ;
   CharRefSysteMem  : Char ;
   CharTimeSystem   : Char ;
   DecreTMoscow     : Extended ;
   SummerHour       : ShortInt ; { for one hour jump of time if exists }
   StepWithTime     : Extended ;
   SizeForArea      : Extended ;

 Var
   JulianDate       : Extended ; // the main variable for time
   LocalSTime       : Extended ; // local sidereal time
   DHourImage       : Extended ;
   StationPos       : TVect3 ; { in the true equatorial system }
   MatrEcl          : TMatr33 ; // from fixed equator to ecliptic
   PrecMatr         : TMatr33 ; // precession matrix
   MatrNut          : TMatr33 ; // nutation matrix
   RotMatr          : TMatr33 ; // from fixed to true equator
   TopMatr          : TMatr33 ; // from true to horizon or orbital
   RosMatr          : TMatr33 ; // true sidereal time matrix
   PlaceCoor        : TPlaceCooRec ; // information about station 
   RinDegC          : Extended ; // general radius of field of view
   XinDegC,YinDegC  : Extended ; // current scale for X Y axis
   AzimutC          : Extended ; // horizon degree centre image
   AltitudeC        : Extended ; // horizon degree centre image
   PosHorizoC       : TVect3 ;   // centre horizon  but descart
   AlphaC,DeltaC    : Extended ; // equator degree centre image
   PosZemelaC       : TVect3 ;   // orbital centre of the Earth
   AngleZemeL       : Extended ; // 'L' the Earth from satellite
   SurfaRange       : Extended ; // special range from satellite
   DecretTime       : Extended ;
   DCivilTime       : Extended ;
   BooSummerJ       : Boolean ;  // summer hour yes or no

 Var                             // is called PUnCRead UnitMain 
   VMagMin,VMagMax  : Single ;   // to limit stars magnitude
   VAllMin,VAllMax  : Single ;   // stars magnitude

 Var
   VEclipse         : TEclipseRec ;
   BooCircle        : Boolean ;

 Var
   CharForView      : Char ; // 'S' for sky 'W' for world map and 'L'
   CharForEphem     : Char ; // 'P' for point 'O' for object
   CharBodEph       : Char ; // 'S' satellite 'P' planet as object
   NumberBody       : LongInt ; // number of choosed planet in case 'O'
   chElemFormat     : Char ; // 'N' NORAD 'I' IRVS 'K' KDr 'M' no
   stElementFile    : ShortString ; // name of file with elements
   stCatalogFile    : ShortString ; // name of catalogue
   stPathCat        : ShortString ; // for path to folder with catalogue
   stPathElem       : ShortString ; // for path to folder with elements
   stPathResult     : ShortString ; // for path to folder for result
   stPathDefault    : ShortString ; // for path to folder with program
   stPathSite       : ShortString ; // for path to folder with site position

 Const
   MaxNameValue     = 97 ; // to select step , area size and others
 Type
   TNameValue       = Record
                        strname  : ShortString ;
                        valname  : Extended ;
                      End;
 Var
   NumNameValue     : SmallInt ; // current maximum number
   NumStatValue     : SmallInt ; // current number in list station
   NumStepValue     : SmallInt ; // current number in list of time step
   NumSizeValue     : SmallInt ; // current number in list of area size
   NameValue        : Array[1..MaxNameValue] of TNameValue ;

 var // variable for map world
   MapSatNumber     : Byte ;  // less equal 8 and 2 for the Sun the Moon
   MapSatGeoPos     : Array[1..10] of TVect3 ;   // for position
   MapSatAltitude   : Array[1..10] of Extended ; // for altitude in degree
   MapSatShadow     : Array[1..10] of Byte ;     // o 1 2
   MapSatX,MapSatY  : Array[1..10] of Integer ;  // for position of imagesat
   RangeSatMap      : Array[1..10] of Extended ; // UnGraAll for range 'W'
   SatElemMap       : Array[1..10] of TElemRec ; // elements
   BooMapPoint      : Boolean ;

 var
   xpMouse  : Integer ;
   ypMouse  : Integer ;

 var
   boomaif  : Boolean ; { false if fmPuz not active }

 IMPLEMENTATION

 END.