
 Unit UnGraAll ; // to figure stars and other objects in SkyRect with Canvas

 INTERFACE

 Procedure FigureStar ( x , y  : Integer ) ;

 procedure FigurePlanet ;

 procedure FigureSatellite ;

 procedure FigureImageSat ( nsat  : Byte ;  rlon , rlat , rokm  : Extended ) ;

 IMPLEMENTATION

 Uses
   Windows  , // for TRect
   Graphics , // for main graphics procedures and functions
   ClipBrd  , // for ClipBoard function
   UnitMain , // for fmPuz.Canvas...
   UnClaMap , // for Methods of the Class TMapW
   UnConTyp , // for const GeoR0 ParmA
   UnCoTVar , // for TObjectTopRec
   UnGloVar , // for simple variable SmallStarSize
   UnButIni , // for SkyParm record
   PUnCRead , // for boolean global variable BrightStarsOnly
   PUnDE200 , // for planets parameters
   UnPhysOp ; // for self body system parallel meridian  bodbmp TBitMap

 Function StarColorFromBV : TColor ; // with using record StarMagCur .bt .vt
 Var
   dif  : Single ; // differences B-V for color with scale by sir Allan
 Begin
   With  StarOneCur  Do  dif:=bt-vt;
   StarColorFromBV:=clMaroon;
   If  dif < +1.44  Then  StarColorFromBV:=clRed;
   If  dif < +0.98  Then  StarColorFromBV:=clYellow;
   If  dif < +0.59  Then  StarColorFromBV:=clLime;
   If  dif < +0.30  Then  StarColorFromBV:=clWhite;
   If  dif < +0.00  Then  StarColorFromBV:=clFuchsia;
   If  dif < -0.28  Then  StarColorFromBV:=clFuchsia;
 End;

 function PlanetColor ( np  : Byte ) : TColor ;
 begin // name of planets from unit PUnDE200
   PlanetColor:=clLime;
   case  np  of
     NMercury  : PlanetColor:=clSilver; // clOlive;
     NVenus    : PlanetColor:=clSilver; // clAqua;
     NEarth    : PlanetColor:=clGreen;
     NMars     : PlanetColor:=clRed;
     NJupiter  : PlanetColor:=clAqua;
     NSaturn   : PlanetColor:=clYellow;
     NUranus   : PlanetColor:=clTeal;
     NNeptune  : PlanetColor:=clMaroon;
     NPluto    : PlanetColor:=clPurple;
     NMoon     : PlanetColor:=clSilver;
     NSun      : PlanetColor:=clYellow;
   end; { case for color }
 end;

 Function ScreenRadiusFromV ( v  : Single ) : Integer ;
 Var
   z  : Single ;
   r  : Integer ;
 Begin
   If  BrightStarsOnly // from unit PUnCRead
     Then  z:=0.0
     Else  z:=3.5;
   r:=-1;
   if  v < +6.5+z  then  r:=0;
   if  v < +5.5+z  then  r:=1;
   if  v < +4.5+z  then  r:=2;
   if  v < +3.5+z  then  r:=3;
   if  v < +2.5+z  then  r:=4;
   if  v < +1.5+z  then  r:=5;
   if  v < +0.5+z  then  r:=6;
   if  v < -0.5+z  then  r:=7;
   r:=r+SmallStarSize; // variable form UnGloVar
   if  r < 0  then  r:=0;
   ScreenRadiusFromV:=r;
 End;

 { for some special body with the phase }
 function SpecScreenRadius  : Integer ;
 var // np number of planet dp distance to planet
   np  : Byte ;
   dp  : Extended ;
   rr  : Extended ;
   rm  : Integer ;
   ra  : Integer ;
 begin // global record SkyParm is declared in UnButIni
   np:=ObjectTopCur.nall;
   dp:=ObjectTopCur.rot; // UnCoTVar is assigned
   ra:=10; // but is assigned by procedure ForSkyFiguParm in UnFigIni
   if  ( not ( dp < 1.0e1 ) ) // integer SkyParm.dy is half of y axes
     then  // RadGra from UnConTyp radian to degree
       begin // YinDegC from UnGloVar y axes scale in degree
         rr:=RadGra*RadiusOfPlanet[np]/dp; // PUnDE200 radius in degree
         ra:=Round(rr/YinDegC*SkyParm.dy); // radius on screen
       end;
   case  np  of   // for some special body
     NMoon , NSun      :  if  ra < 10  then  ra:=10; // non real
    NVenus , NMercury  :  // real or non real radius of image
     begin // on screen
       rm:=ScreenRadiusFromV(ObjectTopCur.Mag);
       if  ra < rm  then  ra:=rm; // non real radius of image
     end;
   end; // case for some body
   SpecScreenRadius:=ra;
 end;

 procedure SpecImageView ( n  : Byte ;   // number of object the Sun Moon
                           c  : TColor ; // color
                           x , y , r  : Integer ) ; // screen coordinates
 var // by using TBitMap as TCanvas  global var bmp from UnPhysOp
   a,b  : Integer ;
   dst  : TRect ;   // screen rectangular
   src  : TRect ;   // screen rectangular
 begin // for the Sun the Moon and the Earth if possible
   bodbmp:=TBitMap.Create;
   bodbmp.Width:=SkyRect.x2-SkyRect.x1-2;
   bodbmp.Height:=SkyRect.y2-SkyRect.y1-2;
   src.Left:=SkyRect.x1+1;
   src.Top:=SkyRect.y1+1;
   src.Right:=SkyRect.x2-1;
   src.Bottom:=SkyRect.y2-1;
   dst.Left:=0;
   dst.Top:=0;
   dst.Right:=bodbmp.Width;
   dst.Bottom:=bodbmp.Height;
   bodbmp.Canvas.CopyRect(dst,fmPuz.Canvas,src);
 try
   bodbmp.Canvas.Brush.Color:=c;
   bodbmp.Canvas.Pen.Color:=c;
   a:=x-SkyRect.x1;
   b:=y-SkyRect.y1;
   bodbmp.Canvas.Ellipse(a-r,b-r,a+r,b+r);
   RotoBody(n,a,b,r); // UnPhysOp body parallels meridians or phase
   fmPuz.Canvas.Draw(SkyRect.x1+1,SkyRect.y1+1,bodbmp);
 finally
   bodbmp.free; // bodbmp TBitMap from UPhysOp
 end;
 end;

 { for the Sun the Moon
       the Venus the Mercury
       and phase n number of body c color }
 procedure SpecialImageFigure ( n  : Byte ;  c  : TColor ) ;
 var
   r,x,y  : Integer ;
 begin // information about Sun or Moon in record ObjectTopCur
   r:=SpecScreenRadius ; // radius in pixels real or not
   x:=ObjectTopCur.xpos; // x axis centre position on screen
   y:=ObjectTopCur.ypos; // y axis centre position on screen
   SpecImageView(n,c,x,y,r);
 end;

 Procedure FigureStar ( x , y  : Integer ) ;
 Var
   rad  : Integer ;
   col  : TColor ; // type from Graphics
   clb  : TColor ; // for BruchColor
   clp  : TColor ; // for PenColor
 Begin
   rad:=ScreenRadiusFromV(StarOneCur.vt); // record StarOneCur.vt
   col:=StarColorFromBV ; // with using record StarMagCur .bt .vt
   clb:=fmPuz.Canvas.Brush.Color;
   clp:=fmPuz.Canvas.Pen.Color; // the main colors to keep
   fmPuz.Canvas.Brush.Color:=col;
   fmPuz.Canvas.Pen.Color:=col; // new colors for star
   If  rad = 0
     Then // only simple point
       fmPuz.Canvas.Pixels[x,y]:=col
     Else // a try to figure ellipse
       fmPuz.Canvas.Ellipse(x-rad,y-rad,x+rad,y+rad);
   fmPuz.Canvas.Brush.Color:=clb;
   fmPuz.Canvas.Pen.Color:=clp; // the main colors to replace
 End;

 procedure ToArrowObject ( x , y  : Integer ;  c  : TColor );
 var
   i  : Byte ;
 begin // simple arrow for to select object
   for i:=1 to 5 do
     fmPuz.Canvas.Pixels[x-5-i,y+5+i]:=c;
   fmPuz.Canvas.Pixels[x-7,y+6]:=c;
   fmPuz.Canvas.Pixels[x-8,y+6]:=c;
   fmPuz.Canvas.Pixels[x-6,y+7]:=c;
   fmPuz.Canvas.Pixels[x-6,y+8]:=c;
 end;

 procedure FigurePlanet ;
 var
   npc  : Byte ;
   x,y  : Integer ;
   rad  : Integer ;
   col  : TColor ; // type from Graphics
   clb  : TColor ; // for BruchColor
   clp  : TColor ; // for PenColor
 begin
   npc:=ObjectTopCur.nall; // from UnCotVar
   col:=PlanetColor(npc); // color for planets
   clb:=fmPuz.Canvas.Brush.Color;
   clp:=fmPuz.Canvas.Pen.Color; // the main colors to keep
   fmPuz.Canvas.Brush.Color:=col;
   fmPuz.Canvas.Pen.Color:=col; // new colors for planet
   x:=ObjectTopCur.xpos;
   y:=ObjectTopCur.ypos;
   if  not ( npc in [ NEarth , NMoon , NSun , NVenus , NMercury ] )
     then
       begin
         rad:=ScreenRadiusFromV(ObjectTopCur.Mag);
         If  rad = 0
           Then // only simple point
             fmPuz.Canvas.Pixels[x,y]:=col
           Else // a try to figure ellipse
             fmPuz.Canvas.Ellipse(x-rad,y-rad,x+rad,y+rad);
       end
     else
       SpecialImageFigure(npc,col); // object and color
   fmPuz.Canvas.Brush.Color:=clb;
   fmPuz.Canvas.Pen.Color:=clp; // the main colors to replace
   if  npc < NMoon  then  ToArrowObject(x,y,clSilver);
 end;

 procedure ToArrowSatellite ( x , y  : Integer ;  c  : TColor );
 var
   i  : Byte ;
 begin // simple arrow for to select object
   for i:=1 to 5 do
     fmPuz.Canvas.Pixels[x-1-i,y+1+i]:=c;
   fmPuz.Canvas.Pixels[x-3,y+2]:=c;
   fmPuz.Canvas.Pixels[x-4,y+2]:=c;
   fmPuz.Canvas.Pixels[x-2,y+3]:=c;
   fmPuz.Canvas.Pixels[x-2,y+4]:=c;
 end;

 procedure FigureSatellite ;
 var
   x,y  : Integer ;
   rad  : Integer ;
   col  : TColor ; // type from Graphics
   clb  : TColor ; // for BruchColor
   clp  : TColor ; // for PenColor
 begin
   col:=clAqua; // color for satellite
   clb:=fmPuz.Canvas.Brush.Color;
   clp:=fmPuz.Canvas.Pen.Color; // the main colors to keep
   fmPuz.Canvas.Brush.Color:=col;
   fmPuz.Canvas.Pen.Color:=col; // new colors for planet
   x:=ObjectTopCur.xpos;
   y:=ObjectTopCur.ypos;
   rad:=ScreenRadiusFromV(ObjectTopCur.Mag);
   If  rad = 0
     Then // only simple point
       fmPuz.Canvas.Pixels[x,y]:=col
     Else // a try to figure ellipse
       fmPuz.Canvas.Ellipse(x-rad,y-rad,x+rad,y+rad);
   fmPuz.Canvas.Brush.Color:=clb;
   fmPuz.Canvas.Pen.Color:=clp; // the main colors to replace
   ToArrowSatellite(x,y,clAqua);
 end;

 procedure FigureImageSat ( nsat  : Byte ;  rlon , rlat , rokm  : Extended ) ;
 var // proc is called by SatelliteToMapWorld from UnForSat
   rl,rf  : Extended ; // longitude latitude in degree
   rx,ry  : Extended ; // position in the map
   xc,yc  : Integer ; // position in the screen
   hsize  : Integer ; // half size of the satellite image
   rectc  : TRect ;  // type from Windows rectangular for image
 begin  // RangeSatMap[] UnGloVar const UnConTyp
   rl:=rlon; // longitude in degree from UnForSat
   rf:=rlat; // latitude  in degree from UnForSat RangeSatMap for UnPlaceO
   RangeSatMap[nsat]:=rokm-GeoR0*(1-ParmA*Sqr(Sin(GraRad*rf))); // height
   MapW.FilaPoint(rf,rl,rx,ry); // from the centre
   xc:=MapW.NulW+Round(rx); // x centre position in screen
   yc:=MapW.NulH-Round(ry); // y centre position in screen
   MapSatX[nsat]:=xc; // x position to memory for the proc ToScreenObjectMap
   MapSatY[nsat]:=yc; // y position to memory var declared in UnGloVar
   if  SmallStarSize = 0  then  hsize:=6  else  hsize:=10;
   rectc.Left:=xc-hsize;   // size of rectangular with image
   rectc.Top:=yc-hsize;    // hsize half of size
   rectc.Right:=xc+hsize;  // SmallStarSize from UnGloVar
   rectc.Bottom:=yc+hsize; // array MapSatShadow from UnGloVar
   with  fmPuz.Canvas  do  // method StretchDraw from Graphics
     if  MapSatShadow[nsat] = 0 // satellite is lighted by the Sun
       then // ImageSal of TImage file BlueSat.bmp
         StretchDraw(rectc,fmPuz.ImageSal.Picture.Bitmap)
       else // ImageSat of TImage file FirstSat.bmp
         StretchDraw(rectc,fmPuz.ImageSat.Picture.Bitmap);
 end;

 END.
