
 { for the procedures of the simple kepler problem }

 UNIT UnitElmK ;

 INTERFACE

 Uses
   UnConTyp ; // TypeDim3 TVect3

 Type
   PosVType     = Array [1..6]  of Extended ;

 Procedure AnomVM ( AE , AV   : Extended ;
                    Var  AM   : Extended ) ;

 Function KeplerEquation ( ElemE , MeanAnom  : Extended ) : Extended ;

 Procedure KeplXV ( AB,AE,AI,AO,AU,AM   : Extended ;
                    Var   X             : PosVType ) ;

 Procedure ToObtainOrbitalMatr ( r , v  : TypeDim3 ;
                                 Var OrbMatr  : TypeDim33 ) ;

 Procedure FromCartToKepler ( Pos , Vel  : TVect3 ;
                              Var   PosVar , AngVar : TVect3 ) ;

 Procedure FromKeplerToCart ( PosVar , AngVar  : TVect3 ;
                              Var   Pos , Vel  : TVect3 ) ;

 IMPLEMENTATION

 Uses
   UnForFun ; // simple function DATan2



 Procedure AnomVM ( AE , AV   : Extended ;
                    Var  AM   : Extended ) ;
 Var
   V,CV,SV,E,U,
   R,SE,CE,RS,RC   : Extended;
 Begin
   V:=GRARAD*AV;
   CV:=COS(V);
   SV:=SIN(V);
   R:=1.0e0/(1.0e0+AE*CV);
   SE:=SQRT(1.0e0-AE*AE)*SV*R;
   CE:=(CV+AE)*R;
   RS:=SE*CV-CE*SV;
   RC:=CE*CV+SE*SV;
   E:=V+DATAN2(RS,RC);
   U:=E-AE*SE;
   AM:=RADGRA*U;
   If  AM < 0  Then  AM:=360+AM;
 End ;

 { Kepler equation   M=E-e*sin(E)
   to solve with the use Newton iteration method
   from the russian excellent book
   M.B.Balk. Elements of the space flight dinamics.
   M., Nauka, 1965.
   Algorithm from page  113:
   ...    .
   ., , 1965.
      113:
   If  fi(x)=0  then iteration may be possible
   x_(n+1)=f(x_n)  where  f(x)=x-fi(x)/(d/dx(fi(x))) }

 Function KeplerEquation ( ElemE , MeanAnom  : Extended ) : Extended ;
 Var
   iter  : Byte ;
   EccAnom  : Extended ;
   cura,difa  : Extended ;
 Begin
   EccAnom:=MeanAnom+ElemE*Sin(MeanAnom); { in radian }
   difa:=1.0;
   iter:=0;
   While  ( ( difa > 1.0e-15 ) AND ( iter < 241 ) )  Do
     Begin
       cura:=EccAnom;
       EccAnom:=cura-(cura-ElemE*Sin(cura)-MeanAnom)/(1-ElemE*Cos(cura));
       difa:=Abs(cura-EccAnom);
       iter:=iter+1;
     End;
   KeplerEquation:=EccAnom; { eccentric anomaly in radian }
 End;


 Procedure KeplXV ( AB,AE,AI,AO,AU,AM   : Extended ;
                    Var   X             : PosVType ) ;
 Var
   j                          : Byte ;
   FM,S,C,O,R,P,Q,
   VR,VN,SV,CV,SB,CB,SU,CU,
   V,B,U,W,E,SE,CE,RR,RS,RC   : Extended ;
 Begin
   FM:=3.9860047E5;
   S:=SIN(GRARAD*AI);
   C:=COS(GRARAD*AI);
   O:=GRARAD*AO;
   U:=GRARAD*AU;
   W:=GRARAD*AM;            { mean anomaly in radian }
   E:=KeplerEquation(AE,W); { eccentric anomaly in radian }
   CE:=COS(E);
   SE:=SIN(E);
   RR:=AB*(1.0e0-AE*CE);
   R:=1.0e0/(1.0e0-AE*CE);
   SV:=SQRT(1.0e0-AE*AE)*SE*R;
   CV:=(CE-AE)*R;
   RS:=SV*CE-CV*SE;
   RC:=CV*CE+SV*SE;
   V:=E+DATAN2(RS,RC);
   B:=O+V;
   SB:=SIN(B);
   CB:=COS(B);
   SU:=SIN(U);
   CU:=COS(U);
   X[1]:=RR*(CB*CU-SB*SU*C);
   X[2]:=RR*(CB*SU+SB*CU*C);
   X[3]:=RR*SB*S;
   P:=AB*(1.0e0-AE*AE);
   Q:=SQRT(FM/P);
   VR:=Q*AE*SV/RR;
   VN:=Q*(1.0e0+AE*CV);
   X[4]:=VR*X[1]+VN*(-SB*CU-CB*SU*C);
   X[5]:=VR*X[2]+VN*(-SB*SU+CB*CU*C);
   X[6]:=VR*X[3]+VN*CB*S;
 End ;

 Procedure ToObtainOrbitalMatr ( r , v  : TypeDim3 ;
                                 Var OrbMatr  : TypeDim33 ) ;
 Var
   i            : Byte ;
   c,d          : TypeDim3 ;
   p,q,si,ci    : Extended ;
   cucn,cusn    : Extended ;
   su,cu,sn,cn  : Extended ;
 Begin
   { integrals of the angular momentum }
   c[1]:=r[2]*v[3]-v[2]*r[3]; // +sin(Omega)*sin(i)*c
   c[2]:=r[3]*v[1]-v[3]*r[1]; // -cos(Omega)*sin(i)*c
   c[3]:=r[1]*v[2]-v[1]*r[2]; // +cos(i)*c
   q:=Sqrt(Sqr(r[1])+Sqr(r[2])+Sqr(r[3])); { the modul }
   p:=Sqrt(Sqr(c[1])+Sqr(c[2])+Sqr(c[3])); { the modul }
   For i:=1 To 3 Do
     Begin { to unit vector }
       d[i]:=r[i]/q;
       c[i]:=c[i]/p;
     End;
   ci:=c[3]; { Cos(Inclination) }
   si:=Sqrt(Sqr(c[1])+Sqr(c[2])); { Sin(Inclination) }
   sn:=+c[1]/si; { Sin(the Ascending Node) }
   cn:=-c[2]/si; { Cos(the Ascending Node) }
   su:=+d[3]/si; { Sin(u) or Sin(argument of the latitude) }
   cucn:=d[1]+su*sn*ci; { Cos(u)*Cos(Omega) }
   cusn:=d[2]-su*cn*ci; { Cos(u)*Sin(Omega) }
   If  ( Abs(cn) > 0.1 )
     Then { Cos(u) }
       cu:=cucn/cn
     Else
       cu:=cusn/sn;
   For i:=1 To 3 Do
     Begin
       OrbMatr[1,i]:=d[i];
       OrbMatr[3,i]:=c[i];
     End;
   OrbMatr[2,1]:=-su*cn-cusn*ci;
   OrbMatr[2,2]:=-su*sn+cucn*ci;
   OrbMatr[2,3]:=+cu*si;
 End;

 { to change from
   Descart position and velosity to Kepler osculating elements
   position Pos x,y,z in km 
   velosity Vel vx,vy,vz in km/s
   GeoFM from  UnConTyp in km**3/s**2
   is geocentric gravitationl const for the current model
   RadGra from UnConTyp for angle from radian to degree
   PosVar 1,2,3 : a in km, e,  Inclination in degree
   AngVar 1,2,3 : the ascending node in degree
                : the argument of perigei in degree
                : the mean anomaly in degree }

 Procedure FromCartToKepler ( Pos , Vel  : TVect3 ;
                              Var   PosVar , AngVar : TVect3 ) ;
 Var
   i                  : Byte;
   h,c,l,r,v2,q,
   c1,c2,c3,l1,l2,l3  : Extended ;
   node,argp,
   a,e,p,rincl,
   ArgLat,TrueAnom,
   EccAnom,MeanAnom,
   si,ci,su,cu,sn,cn  : Extended ;
   sp,cp,sv,cv,se,ce  : Extended ;
 Begin
   r:=Sqrt(Sqr(Pos[1])+Sqr(Pos[2])+Sqr(Pos[3])); { in km }
   v2:=Sqr(Vel[1])+Sqr(Vel[2])+Sqr(Vel[3]); { in (km/s)**2 }
   q:=GeoFM/r; { fm in (km**3/s**2)/km from UnL01010 for the Earth }
   h:=v2-2*q; { (km/s)^2 energy integral }
   c1:=Pos[2]*Vel[3]-Pos[3]*Vel[2]; { three momentum integrals }
   c2:=Pos[3]*Vel[1]-Pos[1]*Vel[3];
   c3:=Pos[1]*Vel[2]-Pos[2]*Vel[1];
   c:=Sqrt(c1*c1+c2*c2+c3*c3);
   l1:=-q*Pos[1]+Vel[2]*c3-Vel[3]*c2; { three integrals }
   l2:=-q*Pos[2]+Vel[3]*c1-Vel[1]*c3;
   l3:=-q*Pos[3]+Vel[1]*c2-Vel[2]*c1;
   l:=Sqrt(l1*l1+l2*l2+l3*l3);
   a:=-GeoFM/h; { in km semi-major axis }
   e:=l/GeoFM; { eccentricity }
   p:=c*c/GeoFM;
   ci:=c3/c; si:=Sqrt(Abs(1-ci*ci));
   If  si < 1.0e-11  Then
     Begin
       si:=1.0e-11; rincl:=0; { singularity inclination = 0 }
     End
                     Else  rincl:=DATan2(si,ci); { inclination in radian }
   sn:=c1/(c*si); cn:=-c2/(c*si);
   node:=DATan2(sn,cn); { node in radian }
   sp:=l3/(l*si); cp:=(l1/l)*cn+(l2/l)*sn;
   argp:=DATan2(sp,cp); { argument of perigei in radian }
   q:=1/r; su:=q*Pos[3]/si; cu:=q*Pos[1]*cn+q*Pos[2]*sn;
   sv:=su*cp-cu*sp; cv:=cu*cp+su*sp; { argument of latitude u = v + omega }
   TrueAnom:=DATan2(sv,cv); { true anomaly in radian }
   q:=1/(1+e*cv); se:=Sqrt(1-e*e)*sv*q; ce:=(cv+e)*q;
   EccAnom:=TrueAnom+ArcTan((se*cv-ce*sv)/(ce*cv+se*sv));
   MeanAnom:=EccAnom-e*se; { M = E - e * Sin(E) in radian }
   PosVar[1]:=a; { in km }
   PosVar[2]:=e;
   PosVar[3]:=RadGra*rincl; { in degree }
   AngVar[1]:=RadGra*node; { in degree }
   AngVar[2]:=RadGra*argp; { in degree }
   AngVar[3]:=RadGra*MeanAnom; { in degree }
   For i:=1 To 3 Do
     Begin
       While  AngVar[i] < -180  Do  AngVar[i]:=AngVar[i]+360;
       While  AngVar[i] > +180  Do  AngVar[i]:=AngVar[i]-360;
     End;
 End;

 { to change from
   Kepler osculating elements to Descart position and velosity
   GeoFM from UnConTyp in km**3/s**2
   is geocentric gravitationl const for the current model
   RadGra from UnConTyp for angle from radian to degree
   PosVar 1,2,3 : a in km, e,  Inclination in degree
   AngVar 1,2,3 : the ascending node in degree
                : the argument of perigei in degree
                : the mean anomaly un degree
   position Pos 1,2,3  : x,y,z in km 
   velosity Vel 1,2,3  : vx,vy,vz in km/s }

 Procedure FromKeplerToCart ( PosVar , AngVar  : TVect3 ;
                              Var   pos , vel  : TVect3 ) ;
 Var
   q,vr,vn  : Extended ;
   EccAnom  : Extended ;
   MeanAnom  : Extended ;
   TrueAnom  : Extended ;
   ArgLatitude  : Extended ;
   AscendingNode  : Extended ;
   r,xr,yr,zr,xv,yv,zv  : Extended ;
   ElemA,ElemE,CosInc,SinInc  : Extended ;
   CosE,SinE,CosV,SinV,CosU,SinU,CosNode,SinNode  : Extended ;
 Begin
   ElemA:=PosVar[1];
   ElemE:=PosVar[2];
   CosInc:=Cos(GraRad*PosVar[3]);
   SinInc:=Sin(GraRad*PosVar[3]);
   MeanAnom:=GraRad*AngVar[3]; { to radian }
   EccAnom:=KeplerEquation(ElemE,MeanAnom);
   SinE:=Sin(EccAnom);
   CosE:=Cos(EccAnom);
   SinV:=Sqrt(1-ElemE*ElemE)*SinE/(1-ElemE*CosE);
   CosV:=(CosE-ElemE)/(1-ElemE*CosE);
   TrueAnom:=EccAnom+ArcTan((SinV*CosE-CosV*SinE)/(CosV*CosE+SinV*SinE));
   ArgLatitude:=TrueAnom+GraRad*AngVar[2]; { in radian }
   SinU:=Sin(ArgLatitude);
   CosU:=Cos(ArgLatitude);
   AscendingNode:=GraRad*AngVar[1];
   SinNode:=Sin(AscendingNode);
   CosNode:=Cos(AscendingNode);
   r:=ElemA*(1-ElemE*CosE); { in km }
   xr:=+CosU*CosNode-SinU*SinNode*CosInc;
   yr:=+CosU*SinNode+SinU*CosNode*CosInc;
   zr:=+SinU*SinInc;
   xv:=-SinU*CosNode-CosU*SinNode*CosInc;
   yv:=-SinU*SinNode+CosU*CosNode*CosInc;
   zv:=+CosU*SinInc;
   q:=Sqrt(GeoFM/(ElemA*(1-Sqr(ElemE))));
   vr:=q*ElemE*SinV;
   vn:=q*(1+ElemE*CosV);
   pos[1]:=r*xr;
   pos[2]:=r*yr;
   pos[3]:=r*zr;
   vel[1]:=vr*xr+vn*xv;
   vel[2]:=vr*yr+vn*yv;
   vel[3]:=vr*zr+vn*zv;
 End;

 END.
