unit UnitShad;

interface

uses
  UnConTyp ; // for type TVect3

function ToGetShadowCase ( tc  : Extended ;  xp  : TVect3 ) : Byte ;

function ToMapShadowCase ( xp  , xs  : TVect3 ) : Byte ; // satellite the Sun

function ToGetMoonEclipse ( pe  : TVect3 ) : Byte ;

function ToGetShadow ( np  : Byte ) : Byte ;

implementation

uses
  UnForTim , // proc FromUTCtoTT
  UnGloVar , // var JulianDate
  PUnDE200 , // proc NMoon NSun
  UnForFun , // function VectorModul MatrMultVect
  UnForPos ; // procedure PosGeoPlanet

function ToGetShadowCase ( tc  : Extended ;  xp  : TVect3 ) : Byte ;
var // only for satellite  after call proc ClcThreeRotMatr for RotMatr
  i  : Byte ; // this function is called by SatellitesAmongStars UnForSat
  xs,xa  : TVect3 ;
  ps,pe  : TVect3 ;
  rs,re  : Extended ;
  cb,sb  : Extended ;
  rperp  : Extended ;
  sperp  : Extended ;
  rr,qq  : Extended ;
  ze,zs  : Extended ;
begin // xp[1..3] is satellite position refer to the centre of the Earth
  ToGetShadowCase:=0; // default
  PosGeoPlanet(NSun,tc,xs); // UnForPos
  if  not BooExistPos  then  Exit ; // no ephemeris
  MatrMultVector(RotMatr,xs,xa); // from fixed to true equator UnForFun
  for i:=1 to 3 do
    begin // to satellite
      ps[i]:=xa[i]-xp[i]; // the Sun refer to the satellite
      pe[i]:=-xp[i]; // the centre of the Earth refer to the satellite
    end;
  rs:=VectorModul(ps); // range from satellite to the Sun
  re:=VectorModul(pe); // range from satellite to the centre of the Earth
  cb:=(ps[1]*pe[1]+ps[2]*pe[2]+ps[3]*pe[3])/(rs*re); // cos(beta)
  if  cb < 0.0  then  Exit ; // no shadow ToGetShadowCase = 0
  sb:=Sqrt(1-Sqr(cb)); // sin(beta)
  rperp:=re*sb; // length of perpendicular in km
  sperp:=re*cb; // range from satellite to perpendicular
  rr:=rs-sperp;  // range from the Sun to perpendicular
  qq:=sperp/rr;   // the ratio
  for i:=1 to 3 do  xs[i]:=(xp[i]+qq*xa[i])/(1+qq); // vector perpendicular
  ze:=GeoR0*(1-ParmA*Sqr(xs[3]/rperp)); // const UnConTyp
  zs:=RadiusOfPlanet[NSun]*sperp/rs; // projection the Sun radius
  if  ( rperp < ( ze-zs) )
    then // full shadow
      ToGetShadowCase:=2
    else
      if  ( rperp < ( ze+zs) )
        then // semishadow case
          ToGetShadowCase:=1
        else
          ToGetShadowCase:=0;
end;

function ToMapShadowCase ( xp  , xs  : TVect3 ) : Byte ; // satellite the Sun
var                      // refer to the Earth in rotating system
  i  : Byte ;
  x  : TVect3 ;
  ps,pe  : TVect3 ;
  rs,re  : Extended ;
  cb,sb  : Extended ;
  rperp  : Extended ;
  sperp  : Extended ;
  rr,qq  : Extended ;
  ze,zs  : Extended ;
begin
  ToMapShadowCase:=0; // default no shadow
  for i:=1 to 3 do
    begin // to satellite
      ps[i]:=xs[i]-xp[i]; // the Sun refer to the satellite
      pe[i]:=-xp[i]; // the centre of the Earth refer to the satellite
    end;
  rs:=VectorModul(ps); // range from satellite to the Sun
  re:=VectorModul(pe); // range from satellite to the centre of the Earth
  cb:=(ps[1]*pe[1]+ps[2]*pe[2]+ps[3]*pe[3])/(rs*re);
  if  cb < 0.0  then  Exit ; // no shadow ToGetShadowCase = 0
  sb:=Sqrt(1-Sqr(cb)); // sin(beta)
  rperp:=re*sb; // length of perpendicular in km
  sperp:=re*cb; // range from satellite to perpendicular
  rr:=rs-sperp;  // range from the Sun to perpendicular
  qq:=sperp/rr;   // the ratio
  for i:=1 to 3 do  x[i]:=(xp[i]+qq*xs[i])/(1+qq); // vector perpendicular
  ze:=GeoR0*(1-ParmA*Sqr(x[3]/rperp)); // const UnConTyp
  zs:=RadiusOfPlanet[NSun]*sperp/rs; // projection the Sun radius
  if  ( rperp < ( ze-zs) )
    then // full shadow
      ToMapShadowCase:=2
    else
      if  ( rperp < ( ze+zs) )
        then // semishadow case
          ToMapShadowCase:=1
        else
          ToMapShadowCase:=0;
end;


function ToGetMoonEclipse ( pe  : TVect3 ) : Byte ;
var // pe[..]  the Moon position in true equator refer to the Earth centre
  ps,pa  : TVect3 ;
  r,p,s,q  : Extended ;
  rs,re,cb  : Extended ;
  rf,sf,cf  : Extended ;
  rh,sh,ch  : Extended ;
begin
  ToGetMoonEclipse:=0; // default
  PosGeoPlanet(NSun,JulianDate,pa); // procedure from UnForPos
  if  not BooExistPos  then  Exit ; // no planet ephemeris strange case
  MatrMultVector(RotMatr,pa,ps);    // to true equator UnForFun the Sun
  rs:=VectorModul(ps); // range from the Earth to the Sun
  re:=VectorModul(pe); // range from the Earth to the Moon
  cb:=(ps[1]*pe[1]+ps[2]*pe[2]+ps[3]*pe[3])/(rs*re); // cos(beta)
  if  cb > -0.9988  then  Exit ; // no shadow ToGetMoonEclipse = 0
  r:=RadiusOfPlanet[NEarth]; // more detail analyse of the condition
  p:=RadiusOfPlanet[NMoon];  // const from PUnDE200
  s:=RadiusOfPlanet[NSun];
  q:=rs*r/(s-r);
  rf:=r*(q-re)/q; // radius of the shadow
  sf:=(rf+p)/re;
  cf:=-Sqrt(1-sf*sf); // max cos for shadow
  q:=rs*r/(s+r);
  rh:=r*(q+re)/q; // radius of the semishadow
  sh:=(rh+p)/re;
  ch:=-Sqrt(1-sh*sh); // max cos for semishadow
  if  cb < cf
    then // full shadow
      ToGetMoonEclipse:=2
    else
      if  cb < ch
        then // semi-shadow case
          ToGetMoonEclipse:=1
        else
          ToGetMoonEclipse:=0;
end;

function ToGetShadow ( np  : Byte ) : Byte ;
var
  xp,xe  : TVect3 ; // type UnConTyp
begin // moment of time in global var JulianDate
  ToGetShadow:=0;
  if  np = NMoon
    then // only for the Moon  may be shadow
      begin // xp the Moon refer to the Earth centre
        PosGeoPlanet(np,JulianDate,xe); // UnForPos the Moon fix equator
        if  not BooExistPos  then  Exit ; // no planet ephemeris strange case
        MatrMultVector(RotMatr,xe,xp);  // th Moon to true equator UnForFun
        ToGetShadow:=ToGetMoonEclipse(xp); // function from UnitShad
      end
    else
      ToGetShadow:=0; // no shadow
end;

end.
