
 UNIT UnitSort ; // to order any list by using Shell method
                 // there is ObjectSort of TObjectSortPtr ^[MaxObjectInHeap]
 INTERFACE

 Uses
   UnCotVar ; // for ObjectSort^ type TObjectSortPtr

 Const
   MaxForOrder     = 97 ;
 Type
   TRealNumber     = Array[1..MaxForOrder] of Extended ;
   TIntegerNumber  = Array[1..MaxForOrder] of LongInt ;

 Procedure ShellMethodForOrdering ( NumberOfTerm : Integer ;
                                    Var  RealOrderMassive : TRealNumber ;
                                    Var  IntOrderMassive  : TIntegerNumber ) ;

 Procedure ShellMethodForSatName ( NFirst  : Integer ) ;

 Procedure ShellMethodForElem ( chelm  : Char ;  NFirst : Integer ) ;

 Procedure ShellMethodForParm ( chprm  : Char ;  NFirst : Integer ) ;

 Procedure ShellMethodSatName ; // to order with alphabet using SatElemHeap

 Procedure ToGetMemForOrder ;

 Procedure ToFreeMemForOrder ;

 IMPLEMENTATION

 {  Algorithm of the quick ordering
              has been presented by  B.Mejer and K.Boduin
                  in the book   Methods of programming   .
                     Volume 2 , page 151 , printed in Moscow , MIR , 1982 . }

 Procedure ShellMethodForOrdering ( NumberOfTerm : Integer ;
                                    Var  RealOrderMassive : TRealNumber ;
                                    Var  IntOrderMassive  : TIntegerNumber ) ;
 Var
   IBuf,InCurrent,
   InOne,InComp,NumJ,NumL   : Integer ;
   Buf                      : Extended ;
 Begin
   If  ( NumberOfTerm <= 1 ) Then  Exit;
   InCurrent:=1; InComp:=NumberOfTerm div 9;
   While  ( InCurrent < InComp ) Do  InCurrent:=3*InCurrent+1;
   While  ( InCurrent >= 1 )  Do
     Begin
       InOne:=InCurrent+1;
       For NumJ:=InOne To NumberOfTerm Do
         Begin
           NumL:=NumJ-InCurrent;
           While  ( ( NumL >= 1 )  AND
                    ( RealOrderMassive[NumL] <
                      RealOrderMassive[NumL+InCurrent] ) )  Do
             Begin
                      { to replace the elements }
               Buf:=RealOrderMassive[NumL];
               RealOrderMassive[NumL]:=RealOrderMassive[NumL+InCurrent];
               RealOrderMassive[NumL+InCurrent]:=Buf;
                      { to replace the number of the elements }
               IBuf:=IntOrderMassive[NumL];
               IntOrderMassive[NumL]:=IntOrderMassive[NumL+InCurrent];
               IntOrderMassive[NumL+InCurrent]:=IBuf;
               NumL:=NumL-InCurrent;
             End;
         End;
       InCurrent:=InCurrent div 3;
     End;
 End;

 {  Algorithm of the quick ordering for strings by alphabet }

 Procedure ShellMethodForSatName ( NFirst  : LongInt ) ;
 Var
   IBuf,InCurrent           : LongInt ;
   InOne,InComp,NumJ,NumL   : LongInt ;
   BufTopRec                : TObjectTopRec ; // UnGloVar
 Begin
   If  ( NumObjectInHeap < NFirst ) Then  Exit;
   InCurrent:=1;
   InComp:=NumObjectInHeap div 9; // UnGloVar
   While  ( InCurrent < InComp ) Do  InCurrent:=3*InCurrent+1;
   While  ( InCurrent >= 1 )  Do
     Begin
       InOne:=InCurrent+1;
       If  InOne < NFirst  Then  InOne:=NFirst ;
       For NumJ:=InOne To NumObjectInHeap Do // UnGloVar
         Begin
           NumL:=NumJ-InCurrent;
           While  ( ( NumL >= NFirst )  AND
                    ( ObjectTopHeap^[NumL].name >
                      ObjectTopHeap^[NumL+InCurrent].name ) )  Do
             Begin  { to replace the elements }
               BufTopRec:=ObjectTopHeap^[NumL]; // UnGloVar
               ObjectTopHeap^[NumL]:=ObjectTopHeap^[NumL+InCurrent];
               ObjectTopHeap^[NumL+InCurrent]:=BufTopRec;
                      { to replace the number of the elements }
               IBuf:=ObjectSort^[NumL];
               ObjectSort^[NumL]:=ObjectSort^[NumL+InCurrent];
               ObjectSort^[NumL+InCurrent]:=IBuf;
               NumL:=NumL-InCurrent;
             End;
         End;
       InCurrent:=InCurrent div 3;
     End;
 End;

 Procedure ToPrepareOrderElem ( chelm  : Char ;  NFirst : Integer ) ;
 Var
   i  : LongInt ;
   n  : LongInt ;
   v  : LongInt ;
 Begin
   v:=0;
   For i:=NFirst To NumObjectInHeap Do
     Begin
       n:=ObjectTopHeap^[i].nall; // number in SatElemHeap
       case  chelm  of
         'n' : v:=Round(1.0e5*SatElemHeap^[n].an); // revolution per day
         'e' : v:=Round(1.0e7*SatElemHeap^[n].ae); // eccentricity
         'i' : v:=Round(1.0e5*SatElemHeap^[n].ai); // inclination degree
       end; // case chelm
       IntDimForOrder^[i]:=v;
     End;
 End;

 {  Algorithm of the quick ordering by orbit elements
    all number of objects is NumObjectInHeap
    to order from the number NFirst in the list }

 Procedure ShellMethodForElem ( chelm  : Char ;  NFirst : Integer ) ;
 Var
   IBuf,InCurrent,
   InOne,InComp,NumJ,NumL   : Integer ;
   BufTopRec                : TObjectTopRec ;
 Begin
   If  ( NumObjectInHeap < NFirst ) Then  Exit;
   ToPrepareOrderElem(chelm,NFirst);
   InCurrent:=1;
   InComp:=NumObjectInHeap div 9;
   While  ( InCurrent < InComp ) Do  InCurrent:=3*InCurrent+1;
   While  ( InCurrent >= 1 )  Do
     Begin
       InOne:=InCurrent+1;
       If  InOne < NFirst  Then  InOne:=NFirst ;
       For NumJ:=InOne To NumObjectInHeap Do
         Begin
           NumL:=NumJ-InCurrent;
           While  ( ( NumL >= NFirst )  AND
                    ( IntDimForOrder^[NumL] <
                      IntDimForOrder^[NumL+InCurrent] ) )  Do
             Begin
               { to replace the elements }
               IBuf:=IntDimForOrder^[NumL];
               IntDimForOrder^[NumL]:=IntDimForOrder^[NumL+InCurrent];
               IntDimForOrder^[NumL+InCurrent]:=IBuf;
               { to replace the records }
               BufTopRec:=ObjectTopHeap^[NumL];
               ObjectTopHeap^[NumL]:=ObjectTopHeap^[NumL+InCurrent];
               ObjectTopHeap^[NumL+InCurrent]:=BufTopRec;
               NumL:=NumL-InCurrent;
             End;
         End;
       InCurrent:=InCurrent div 3;
     End;
 End;

 Procedure ToPrepareOrderParm ( chprm  : Char ;  NFirst : Integer ) ;
 Var
   n  : LongInt ;
   v  : LongInt ;
 Begin
   v:=0;
   For n:=NFirst To NumObjectInHeap Do
     Begin
       case  chprm  of
         'r' : v:=Round(1.0e3*ObjectTopHeap^[n].Rot); // range in km
         'h' : v:=Round(1.0e5*ObjectTopHeap^[n].Alt); // altitude degree
       end; // case chelm
       IntDimForOrder^[n]:=v;
     End;
 End;

 {  Algorithm of the quick ordering by orbit elements
    all number of objects is NumObjectInHeap
    to order from the number NFirst in the list }

 Procedure ShellMethodForParm ( chprm  : Char ;  NFirst : Integer ) ;
 Var
   IBuf,InCurrent,
   InOne,InComp,NumJ,NumL   : Integer ;
   BufTopRec                : TObjectTopRec ;
 Begin
   If  ( NumObjectInHeap < NFirst ) Then  Exit;
   ToPrepareOrderParm(chprm,NFirst);
   InCurrent:=1;
   InComp:=NumObjectInHeap div 9;
   While  ( InCurrent < InComp ) Do  InCurrent:=3*InCurrent+1;
   While  ( InCurrent >= 1 )  Do
     Begin
       InOne:=InCurrent+1;
       If  InOne < NFirst  Then  InOne:=NFirst ;
       For NumJ:=InOne To NumObjectInHeap Do
         Begin
           NumL:=NumJ-InCurrent;
           While  ( ( NumL >= NFirst )  AND
                    ( IntDimForOrder^[NumL] <
                      IntDimForOrder^[NumL+InCurrent] ) )  Do
             Begin
               { to replace the elements }
               IBuf:=IntDimForOrder^[NumL];
               IntDimForOrder^[NumL]:=IntDimForOrder^[NumL+InCurrent];
               IntDimForOrder^[NumL+InCurrent]:=IBuf;
               { to replace the records }
               BufTopRec:=ObjectTopHeap^[NumL];
               ObjectTopHeap^[NumL]:=ObjectTopHeap^[NumL+InCurrent];
               ObjectTopHeap^[NumL+InCurrent]:=BufTopRec;
               NumL:=NumL-InCurrent;
             End;
         End;
       InCurrent:=InCurrent div 3;
     End;
 End;

 Procedure ShellMethodSatName ; // to order with alphabet using SatElemHeap
 Var
   IBuf,InCurrent           : LongInt ;
   InOne,InComp,NumJ,NumL   : LongInt ;
   BufElemRec               : TElemRec ; // UnCoTVar
 Begin
   If  ( NumSatElemInHeap < 1 ) Then  Exit;
   InCurrent:=1;
   InComp:=NumSatElemInHeap div 9; // UnCoTVar
   While  ( InCurrent < InComp ) Do  InCurrent:=3*InCurrent+1;
   While  ( InCurrent >= 1 )  Do
     Begin
       InOne:=InCurrent+1;
       For NumJ:=InOne To NumSatElemInHeap Do // UnCoTVar
         Begin
           NumL:=NumJ-InCurrent;
           While  ( ( NumL >= 1 )  AND
                    ( SatElemHeap^[NumL].satname >
                      SatElemHeap^[NumL+InCurrent].satname ) )  Do
             Begin  { to replace the elements }
               BufElemRec:=SatElemHeap^[NumL]; // UnGloVar
               SatElemHeap^[NumL]:=SatElemHeap^[NumL+InCurrent];
               SatElemHeap^[NumL+InCurrent]:=BufElemRec;
                      { to replace the number of the elements }
               IBuf:=ObjectSort^[NumL];
               ObjectSort^[NumL]:=ObjectSort^[NumL+InCurrent];
               ObjectSort^[NumL+InCurrent]:=IBuf;
               NumL:=NumL-InCurrent;
             End;
         End;
       InCurrent:=InCurrent div 3;
     End;
 End;

 Procedure ToGetMemForOrder ;
 Begin
   GetMem(ObjectSort,SizeOf(TObjectSort));
   GetMem(IntDimForOrder,SizeOf(TObjectSort)); // integer array
 End;

 Procedure ToFreeMemForOrder ;
 Begin
   FreeMem(IntDimForOrder,SizeOf(TObjectSort)); // integer array
   FreeMem(ObjectSort,SizeOf(TObjectSort));
 End;

 END.