unit UnitStat;

interface

uses
  UnGloVar ; // for type TPlaceCooRec and variable PlaceCoor

const
  MaxStationL  = 255 ;

var
  NumStationL  : SmallInt ;
  StationList  : Array[1..MaxStationL] of TPlaceCooRec ; // type from UnGloVar

procedure ToAddStationList ( sf  : ShortString ) ; // from new file

procedure StationForView ; // procedure is called by ForButtonP from UnForBut

procedure ToSelectOneStation ;  // proc is called by ForButtonP from UnForBut

procedure StationFromMap ( cn  : Char ;
                           rf , rl  : Extended ;       // proc is called
                           var  pc  : TPlaceCooRec ) ; // from UnClaMap

function ToGetStrForSito ( pc  : TPlaceCooRec ) : ShortString ;

function ToGetStrForSitu ( pc  : TPlaceCooRec ) : ShortString ;

implementation

uses
  Dialogs  , // for ShowMessage
  UnitMain , // fmPuz
  UnSatMap , // fmSatMap.ShowModal it is possible to name point at the map
  UnSelHol , // fmSelHol
  UnSelFel , // fmSelFel
  UnSelFol , // fmSelFol
  UnConTyp , // for const GraRad
  UnForCoo , // GeoCartFromSpher
  UnForFun , // for proc PrHourAngle PrDegrAngle
  UnForStr , // for function FloatToFixStr
  UnGetIni ; // proc TimeVarInit CentreCoorH

procedure ToAddStationList ( sf  : ShortString ) ; // from new file
var
  ft  : TextFile ;
  no,nt  : SmallInt ;
  rf,rl,rh  : Extended ; // altitude longitude height
  rx,ry,rz  : Extended ; // geocentric position in km
  nz,nd,ns  : ShortInt ;  // hour zone , decret hour , summer hour
  st  : String[14]; // name
  ch  : Char ; // index for input parameters
begin
  no:=0; // site number
  nz:=0; // hour zone
  nd:=0; // decret hour
  ns:=0; // summer hour
  AssignFile(ft,sf); // new file
{$I-}
  ReSet(ft);
{$I+}
  if  IOResult = 0
    then  // file exists
      begin
        Read(ft,ch); // the first string with cap information
        if  ch <> 'i'
          then // error in file may be other file
            begin
              CloseFile(ft);
              ShowMessage(' '); // Dialogs
              Exit;
            end;
        ReadLn(ft); // the first string with cap information
        nt:=NumStationL; // there is some stations in list
        while  not EOF(ft)  do
          begin
            Read(ft,ch);
            case  ch  of
             'r'  : begin // x y z position in metre
                      ReadLn(ft,st,no,rx,ry,rz,nz,nd,ns); // no site number
                      GeoSpherFromCart(rx,ry,rz,rf,rl,rh); // UnforCoo degree
                      nt:=nt+1; // the next point
                    end;
             'z'  : begin // decimal point phi lambda degree h in metre
                      ReadLn(ft,st,no,rf,rl,rh,nz,nd,ns); // no site number
                      GeoCartFromSpher(rf,rl,rh,rx,ry,rz); // UnForCoo metre
                      nt:=nt+1; // the next point
                    end
             else  ReadLn(ft);
            end; // case ch - character that input may be r z d h
            if  nt > MaxStationL  then  nt:=MaxStationL;
            if  ch in [ 'r' , 'z' ]
              then
                with  StationList[nt]  do
                  begin
                    num:=no;
                    name:=st;
                    x:=rx; // x position in metre
                    y:=ry; // y position in metre
                    z:=rz; // z position in metre
                    f:=rf; // geodetic latitude in degree
                    l:=rl; // geodetic longitude in degree
                    h:=rh; // geodetic height in meter
                    c:=nz; // hour zone for civil time
                    d:=nd; // decret hour
                    s:=ns; // summer hour
                  end;
          end;
        CloseFile(ft);
        NumStationL:=nt;
      end;
end;

procedure StationForView ;
begin
   TimeVarInit(1) ;  // from UnGetIni
   CentreCoorH ;  // from UnGetIni
   ClcThreeRotMatr(JD2000,JulianDate, // UnConTyp UnGloVar
                   PlaceCoor,RotMatr,RosMatr,TopMatr); // UnForCoo
end;

procedure ShowModalStationList ;
var
  n  : LongInt ;
begin
  fmSelHol.ListBox.Clear; // station list to ListBox1
  for n:=1 to NumStationL do // var from UnitStat
    fmSelHol.ListBox.Items.Add('   '+StationList[n].name);
  fmSelHol.ListBox.ItemIndex:=NumStatValue-1; // station number in list
  fmSelHol.ListBox.Columns:=3;
  fmSelHol.ShowModal; // to select one station from the list
end;

procedure ToSelectOneStation ; // to choose station from the list
begin // but list from the file statfira.txt
  fmSelHol.CapSelHol:=' .'; // form from UnSelHol
  fmSelHol.ButtonAdd.Enabled:=True;
  fmSelHol.ButtonAdd.Visible:=True;
  fmSelFel.BooSelFel:=False;
  fmSelHol.AddSelHol:=1;
  ShowModalStationList; // NumStationL greater or equal 1 default
  fmSelHol.ButtonAdd.Enabled:=False;
  fmSelHol.ButtonAdd.Visible:=False;
  if  fmSelHol.BoaSelHol // ButtonAdd is clicked
    then
      begin
        fmSelFel.FileElemBox.Mask:='*.txt';    // form UnSelFel
        fmSelFel.Caption:=' .  .';
        fmSelFel.stPathFel:=stPathSite; // folder with stations files
        fmSelFel.BinSelFel:=False; // to select text but not binary file
        fmSelFel.ButtonFol.Enabled:=True;
        fmSelFel.ButtonFol.Visible:=True;
        fmSelFel.FileElemBox.ApplyFilePath(fmSelFel.stPathFel);
        if  fmSelFel.FileElemBox.Items.Count > 0
          then // files exist
            fmSelFel.ShowModal  // form UnSelFel to select file
          else // to select other folder
            fmSelFel.BofSelFel:=True;
        fmSelFel.ButtonFol.Enabled:=False;
        fmSelFel.ButtonFol.Visible:=False;
        if  fmSelFel.BofSelFel
          then
            begin
              fmPuz.MenuEphCooClick(fmSelFel.SenderFel);
              if  not fmSelFol.BooSelectFolder  then  Exit ; //  fmSelFol
              fmSelFel.BooSelFel:=False;
              fmSelFel.stPathFel:=stPathSite;
              fmSelFel.FileElemBox.ApplyFilePath(fmSelFel.stPathFel);
              if  fmSelFel.FileElemBox.Items.Count > 0
                then // files exist
                  fmSelFel.ShowModal
                else
                  ShowMessage('  '+stPathSite+'   ');
            end;
        if  fmSelFel.BooSelFel
          then
            begin
              ToAddStationList(fmSelFel.StrSelFel); // from UnitStat
              ShowModalStationList;
            end;
      end;
end;

procedure StationFromMap ( cn  : Char ;
                           rf , rl  : Extended ;       // proc is called
                           var  pc  : TPlaceCooRec ) ; // from UnClaMap
var
  q  : Extended ;
begin  // information to pc of TPlaceCooRec var from UnGloVar
  fmSatMap.SatMapC:=cn;
  if  cn = 'N'
    then
      fmSatMap.ShowModal;
  with  pc  do
    begin
      num:=0;     // default
      name:=fmSatMap.SatMapN;
      f:=rf;      // latitude in degree
      l:=rl;      // longitude in degree
      h:=0.0;     // height in meter
      GeoCartFromSpher(f,l,h,x,y,z); // UnForCoo descart position in meter
      q:=(rl+15.0)/15.0; // longitude in hour
      if  q > 12.0  then  q:=q-24.0;
      c:=Trunc(q);       // hour zone default
      d:=0;       // hour decret default
      s:=0;       // hour summer default
    end;
end;

function ToGetStrForSito ( pc  : TPlaceCooRec ) : ShortString ;
var
  st  : ShortString ;
begin
  st:='  '+IntegerToFixStr(pc.num,4)
     +'    '+FloatToFixStr(pc.x,11,3)
     +'    '+FloatToFixStr(pc.y,11,3)
     +'    '+FloatToFixStr(pc.z,11,3);
  st:=st+'   '+pc.name;
  ToGetStrForSito:=st;
end;

function ToGetStrForSitu ( pc  : TPlaceCooRec ) : ShortString ;
var
  nh,nm  : Integer ;
  kd,km  : Integer ;
  aa,ab  : Extended ;
  sa,st  : ShortString ;
begin
  PrHourAngle(GraRad*pc.l,sa,nh,nm,aa); // geodetic longitude
  PrDegrAngle(GraRad*pc.f,sa,kd,km,ab); // geodetic latitude
  st:='  '+IntegerToFixStr(pc.num,4)
     +'  '+IntegerToFixStr(nh,2)
     +' '+IntegerToFixStr(nm,2)
     +' '+FloatToFixStr(aa,7,4);
  st:=st+'  '
     +sa+IntegerToFixStr(kd,2)
     +' '+IntegerToFixStr(km,2)
     +' '+FloatToFixStr(ab,6,3)
     +'      '+FloatToFixStr(pc.h,9,3);
  st:=st+'   '+pc.name;
  ToGetStrForSitu:=st;
end;

end.
