
 UNIT UnPlaceA ; // initial condition for area to view sky

 INTERFACE

 Uses
   Graphics ; // for type TColor

 Function ToGetSkyColor : TColor ; // day twilight or pure night

 Function ToGetSkyColoo : TColor ; // 'L' our sat may be in shad

 Procedure RectCoordinates ( a , b  : Extended ;  Var  x , y  : Integer ) ;

 Function BooPosInRect ( x , y  : Integer ) : Boolean ;

 Procedure ViewPlac ; // to view area of the sky

 Procedure FiguViewPlac ; // after changing some parameters

 Procedure FiguViewInfo ; // after changing some parameters

 Procedure FiguViewMap ; // to view map of the world

 IMPLEMENTATION

 Uses
   UnForCoo , // for procedure ClcThreeRotMatr
   UnConTyp , // for constant JD2000 type TVect3
   UnGloVar , // for variables JulianDate RotMatr RosMatr TopMatr
   UnForPos , // for function DayOrNightNow
   UnForSys , // for procedure FromTopToFixEqu angle var in degree
   PUnCRead , // for boolean variable BrightStarsOnly  proc ToPutStarToHeap
   UnCotVar , // for pointer arrays Star.. Num Pos Vel Mag Top ..Heap
   UnForSta , // for procedure StarToEquatorOfDate
   UnPlateC , // for procedure IdealCoordinates
   UnForFun , // for procedure MatrMultVector
   UnButIni , // for SkyParm record SkyRect fro example
   UnMouSky , // simle procedure CursorInform
   UnFigIni , // procedure FiguPlac
   UnPlaceO , // for procedure ToClearBoxInform
   UnitFixO , // for procedure ToGetCentrePos
   UnGraAll , // for FigureStar procedure
   UnForRed , // for ToObtainAberrationParm procedure
   UnForSat , // for SatellitesAmongStars procedure
   UnForZem , // for EarthFiguScreen procedure
   PUnDE200 , // for NSun NMoon
   UnClaMap ; // for class TMapW and map world

 // option 'S' sky to look from the station
 Function ToGetSkyColor : TColor ;
 Var
   ch  : Char ;
 Begin // ClcThree.. in UnForCoo where StationPos is in true equator system
   ch:=DayOrNightNow ; // from unit UnForPos
   ToGetSkyColor:=clBlack; // default
   case  ch  of
     'D' : ToGetSkyColor:=clBlue;
     'T' : ToGetSkyColor:=clNavy;
     'N' : ToGetSkyColor:=clBlack; // color from Graphics
   end; // case ch
 End;

 { option 'L' for look from the satellite
   our satellite may be in shadow with index LooShad = 2 }
 Function ToGetSkyColoo : TColor ;
 Begin // ClcThree.. in UnForCoo where StationPos is in true equator system
   ToGetSkyColoo:=clBlack; // default
   case  ShadLookSat  of  // byte function from UnForSat
     0 : begin
           ToGetSkyColoo:=clBlue;
           SpecButt[21]:='    ';
           HintButt[21]:=' ';
         end;
     1 : begin
           ToGetSkyColoo:=clNavy;
           SpecButt[21]:='';
           HintButt[21]:='   ';
         end;
     2 : begin
           ToGetSkyColoo:=clBlack; // color from Graphics
           SpecButt[21]:='    ';
           HintButt[21]:='  ';
         end;
   end; // case ch
 End;

 Procedure RectCoordinates ( a , b  : Extended ;  Var  x , y  : Integer ) ;
 Begin
   x:=-10; // non real current position in SkyRect
   y:=-10; // non real value  RinDegC radius in degree from UnGloVar
   If  ( ( Abs(a) < XinDegC ) and ( Abs(b) < YinDegC ) )
     Then
       Begin // half      reversal                    max range
         x:=SkyParm.xh-SkyParm.rx*Round(a/XinDegC*SkyParm.dx);
         y:=SkyParm.yh-SkyParm.ry*Round(b/YinDegC*SkyParm.dy);
       End;
 End;

 Function BooPosInRect ( x , y  : Integer ) : Boolean ;
 Begin
   With  SkyParm  Do // SkyParm from unit UnButIni
     If  ( ( ( xa < x ) and ( x < xb ) )
         and ( (  ya < y ) and ( y < yb ) ) )
       Then // our point in our rectangle
         BooPosInRect:=True
       Else
         BooPosInRect:=False;
 End;

 Procedure ScreenStarsToHeap ( n  : Word ;
                               x , y  : Integer ;
                               azt , alt  : Extended ) ;
 Begin
   If  NumObjectInHeap < MaxObjectInHeap
     Then // all variables from unit UnCotVar
       Begin
         Inc(NumObjectInHeap);
         ObjectTopHeap^[NumObjectInHeap].nall:=n; // number StarHeap
         ObjectTopHeap^[NumObjectInHeap].chps:='*'; // there is star
         ObjectTopHeap^[NumObjectInHeap].name:=''; // default
         ObjectTopHeap^[NumObjectInHeap].azt:=azt; // current azimuth
         ObjectTopHeap^[NumObjectInHeap].alt:=alt; // current alitude
         ObjectTopHeap^[NumObjectInHeap].rot:=1.0; // range default for star
         ObjectTopHeap^[NumObjectInHeap].Mag:=StarOneCur.Vt; // magnitude
         ObjectTopHeap^[NumObjectInHeap].phase:=1.0; // default for star
         ObjectTopHeap^[NumObjectInHeap].IndexShadow:=0; // default
         ObjectTopHeap^[NumObjectInHeap].boovis:=True; // on screen
         ObjectTopHeap^[NumObjectInHeap].booscreen:=True; // on screen
         ObjectTopHeap^[NumObjectInHeap].xpos:=x; // x on screen
         ObjectTopHeap^[NumObjectInHeap].ypos:=y; // y on screen
       End;
 End;

 { values for AzimutC and AltitudeC must be known
   to use procedure StarHorizonCoor form UnForSta
   to obtain azimuth and altitude of a star
   to find for star being near field of view or no }

 Procedure StarToScreen ; // to put stars to screen
 Var
   n  : Word ;
   azt,alt  : Extended ; // azimuth altitude in degree
   ksi,eta  : Extended ; // ideal coordinates
   xc,yc  : Integer ; // position in the field of view
   bos  : Boolean ; // star may be out of our hemisphere
 Begin                          // all stars from the list
   For n:=1 To NumberOfStars Do // NumberOfStars from StarToMemory
     Begin // StarHorizontCoor with the aberration corr. UnForSta
       StarHorizontCoor(n,azt,alt,bos); // azimuth altitude
       If  bos    // this star may be in our hemisphere
         Then     // this star mey be in our field of view
           Begin  // about this star in record StarOneCur
             IdealCoordinates(azt,alt,ksi,eta); // unit UnPlateC
             RectCoordinates(ksi,eta,xc,yc); // field as plate
           End // position projection to the plate
         Else     // this star is out our hemisphere
           Begin  // this star is out our field if view
             xc:=-30000; // default large minus position
             yc:=-30000; // default large minus position
           End;   // to exlude this star
       If  BooPosInRect(xc,yc)  // to compare current position
            and ( StarOneCur.vt < VMagMax ) // with our rectangle
         Then                 // PUnCRead to limit with magnitude
           Begin                // as field of view
             FigureStar(xc,yc); // UnGraAll star in our rectangle
             Inc(NumStarsInRect); // to add one star for screen
             ScreenStarsToHeap(n,xc,yc,azt,alt); // to heap now
           End; // but other stars are excluded
     End; // after ScreenStarsToHeap always boovis booscreen
 End;

 Procedure ViewPlac ; // to view area of the sky
 Begin
   ClcThreeRotMatr(JD2000,JulianDate,  // UnConTyp UnGloVar
                   PlaceCoor,RotMatr,RosMatr,TopMatr); // UnForCoo
   ToObtainAberrationParm(JulianDate); // UnForRed VelBarEarth of TVect3
   NumStarsInRect:=0; // unit UnGloVar
   NumObjectInHeap:=0; // unit UnGloVar
   ToGetCentrePos ; // UnitFixO for AzimutC AltitudeC values
   StarToScreen ; // to put stars to screen
   PlanetsToScreen ; // from unit UnForPos
   SatellitesAmongStars ; // from unit UnForSat
   if  CharForView = 'L'  // in the case look option
     then                 // the Earth may be in the field of view
       EarthFiguScreen ;  // UnForZem  the Earth the field of view
   HintForStarOnScreen ;  // from UnButIni
   CursorFigu(xpMouse,ypMouse); // proc UnMouSky var UnGloVar
 End;

 { proc StarToMemory from UnForSta
   to obtain fixed equator position
   of the centre of the field of view
   right ascension declination pos[1,2,3]
   from topocentric horizon azimuth altitude }
 Procedure FiguViewPlac ; // after changing some parameters
 Begin                    // 'S'
   If  SkyBooX  // boolean variable from UnButIni
     Then
       ToClearBoxInform ; // procedure from UnPlaceO
   ClcThreeRotMatr(JD2000,JulianDate,  // UnConTyp UnGloVar
                   PlaceCoor,RotMatr,RosMatr,TopMatr); // UnForCoo
   ToObtainAberrationParm(JulianDate); // UnForRed VelBarEarth of TVect3
   HintForButtons ; // UnButIni
   SpecForButtons ; // from unit UnButIni
   FiguPlac ; // unit UnFigIni
   ToGetCentrePos ; // UnitFixO for AzimutC AltitudeC values
   StarToMemory ; // UnForSta  all stars in selected rectangle
   if  CharForView = 'L'  // UnForZem in the case look option
     then                 // the Earth in the field of view
       EarthFiguScrini ;  // for values PosZemelaC AngleZemeL
   ViewPlac ; // from Unit UnPlaceA
 End;

 Procedure FiguViewInfo ; // after changing some parameters
 Begin
   If  SkyBooX  // boolean variable from UnButIni
     Then
       ToClearBoxInform ; // procedure from UnPlaceO
   ClcThreeRotMatr(JD2000,JulianDate,  // UnConTyp UnGloVar
                   PlaceCoor,RotMatr,RosMatr,TopMatr); // UnForCoo
   ToObtainAberrationParm(JulianDate); // UnForRed VelBarEarth of TVect3
   ToGetCentrePos ; // UnitFixO for AzimutC AltitudeC values
   HintForButtons ; // from unit UnButIni
   SpecForButtons ; // from unit UnButIni
   FiguInfo ; // unit UnFigIni
   StarToMemory ; // unit UnForSta all stars in selected rectangle
   if  CharForView = 'L'  // UnForZem in the case look option
     then                 // the Earth in the field of view
       EarthFiguScrini ;  // for values PosZemelaC AngleZemeL
   ViewPlac ; // from Unit UnPlaceA
 End;

 Procedure FiguViewMap ; // to view map of the world
 Var
   fs,vs  : Extended ; // latitude longitude of the Sun in degree
   xs,ys  : Integer ;  // the position of the Sun on the screen
   fm,vm  : Extended ; // latitude longitude of the Moon in degree
   xm,ym  : Integer ;  // the position of the Moon on the screen
 Begin  // the Sun the Moon refer to rotating Earth
   if  CharForView <> 'W'  Then  Exit ; // no action only for WorldMap
   HintForButtons ;
   SpecForButtons ; // from unit UnButIni
   FiguInfo ; // unit UnFigIni
   with  SkyRect  do // rectangular SkyRect from unit UnButIni
     MapW.SimpleMapWorld(x1,y1,x2,y2); // method of class TMapW UnClaMap
   RotatingGeoPos(NSun,MapSatGeoPos[MapSatNumber+1],fs,vs);  // UnForPos
   RotatingGeoPos(NMoon,MapSatGeoPos[MapSatNumber+2],fm,vm); // UnForPos
   MapW.DayNightLine(fs,vs,fm,vm,xs,ys,xm,ym); // UnClaMap with Sun Moon screen
   SatelliteToMapWorld(xs,ys,xm,ym); // procedure from UnForSat
 End;

 END.
