unit UnPrtPos; // form fmPrtPos
               // to print stars position
interface     // equator ecliptic galactic

uses
  Windows, Messages, SysUtils, Classes,
  Graphics, Controls, Forms, Dialogs,
  StdCtrls ,
  UnCoTVar ; // for type TElemRec

type
  TfmPrtPos = class(TForm)
    ButSave: TButton;
    ButEsc: TButton;
    MemoBox: TMemo;
    GroupBox: TGroupBox;
    rbEqu: TRadioButton;
    rbEcl: TRadioButton;
    rbGal: TRadioButton;
    procedure FormShow(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure ButSaveClick(Sender: TObject);
    procedure ButEscClick(Sender: TObject);
    procedure MemoBoxKeyPress(Sender: TObject; var Key: Char);
    procedure rbEquClick(Sender: TObject);
    procedure rbEclClick(Sender: TObject);
    procedure rbGalClick(Sender: TObject);
    procedure MemoBoxKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private declarations }
  public
    { Public declarations }
    numStar  : Integer ; // number of stars in list
    numSatl  : Integer ; // number of satellites in list
    numSolp  : Integer ; // number of planets in list
    elmSatl  : TElemRec ; // type UnCoTVar for elements
    starpom  : Extended ; // moment in julian day
    charact  : Char ;    // 'I' for ICRS 'P' for parameters
    procedure MemoNullo ;
    procedure MemoInsert ( np  : Integer ;
                           nm  : Integer ;
                           sa  : ShortString ) ;
    procedure CondV ; // initial caption
  end;

var
  fmPrtPos: TfmPrtPos;

implementation

uses
  UnitMain , // fmPuz.SaveDialog
  UnGraBmp , // PuzSaveBmp procedure
  UnStandP , // proc StandardPosEqu
  UnVectoS ; // to obtain vector condition initial parameters

{$R *.DFM}

procedure TfmPrtPos.MemoNullo ;
begin
  numStar:=0;
  numSatl:=0;
  numSolp:=0;
  fmPrtPos.MemoBox.Clear;
end;

procedure TfmPrtPos.MemoInsert ( np  : Integer ;
                                 nm  : Integer ;
                                 sa  : ShortString ) ;
var
  st  : ShortString ;
begin
  st:='  '+IntToStr(nm)+'  '+sa;
  fmPrtPos.MemoBox.Lines.Insert(np,st);
end;

procedure TfmPrtPos.CondV ; // initial caption
begin
  fmPrtPos.Caption:='  ';
  fmPrtPos.GroupBox.Caption:=
  '    ';
  fmPrtPos.rbEqu.Caption:=
  ' (, ) ';
  fmPrtPos.rbEcl.Caption:=
  ' (,  J2000.0) ';
  fmPrtPos.rbGal.Caption:=
  '   ( )';
  ToFindCrossEqu(fmPrtPos.starpom,fmPrtPos.elmSatl); // UnVectoS
  SatPosRot(fmPrtPos.elmSatl); // UnVectoS Earth's rotating system
end;

procedure TfmPrtPos.FormShow(Sender: TObject);
begin
  case  fmPrtPos.charact  of
    'I' : StandardPosEqu ; // UnStandP  for rbEqu.Checked = True
    'P' : fmPrtPos.CondV ; // initial caption
  end; // case charact
  if  fmPrtPos.charact = 'I'  then
    fmPrtPos.MemoBox.Lines.Insert(0,'  '+fmPrtPos.rbEqu.Caption);
end;

procedure TfmPrtPos.FormActivate(Sender: TObject);
begin
  fmPrtPos.rbEqu.Checked:=True;  // RadioButton on form fmPrtPos
  fmPrtPos.rbEcl.Checked:=False; // RadioButton on form fmPrtPos
  fmPrtPos.rbGal.Checked:=False; // RadioButton on form fmPrtPos
  fmPrtPos.MemoBox.SetFocus;
end;

procedure TfmPrtPos.ButSaveClick(Sender: TObject);
begin
  if  fmPuz.SaveDialog.Execute
    then
      fmPrtPos.MemoBox.Lines.SaveToFile(fmPuz.SaveDialog.FileName);
end;

procedure TfmPrtPos.ButEscClick(Sender: TObject);
begin
  fmPrtPos.MemoBox.Visible:=False;
  Close;
end;

procedure TfmPrtPos.MemoBoxKeyPress(Sender: TObject; var Key: Char);
begin
  if  Key = #27  then  fmPrtPos.ButEscClick(Sender);  // exit
  if  Key = #13  then  fmPrtPos.ButSaveClick(Sender); // to save
end;

procedure TfmPrtPos.rbEquClick(Sender: TObject);
begin
  fmPrtPos.MemoBox.Visible:=False;
  if  fmPrtPos.rbEqu.Checked
    then
      case  fmPrtPos.charact  of
        'I' : StandardPosEqu; // UnStandP
        'P' : SatPosRot(fmPrtPos.elmSatl); // UnVectoS rotating
      end;
  if  fmPrtPos.charact = 'I'  then
    fmPrtPos.MemoBox.Lines.Insert(0,'  '+fmPrtPos.rbEqu.Caption);
  fmPrtPos.MemoBox.Visible:=True;
  fmPrtPos.MemoBox.SetFocus;
end;

procedure TfmPrtPos.rbEclClick(Sender: TObject);
begin
  fmPrtPos.MemoBox.Visible:=False;
  if  fmPrtPos.rbEcl.Checked
    then
      case  fmPrtPos.charact  of
        'I' : StandardPosEcl ; // UnStandP
        'P' : SatPosFix(fmPrtPos.elmSatl); // UnVectoS fixed system
      end;
  if  fmPrtPos.charact = 'I'  then
    fmPrtPos.MemoBox.Lines.Insert(0,'  '+fmPrtPos.rbEcl.Caption);
  fmPrtPos.MemoBox.Visible:=True;
  fmPrtPos.MemoBox.SetFocus;
end;

procedure TfmPrtPos.rbGalClick(Sender: TObject);
begin
  fmPrtPos.MemoBox.Visible:=False;
  if  fmPrtPos.rbGal.Checked
    then
      case  fmPrtPos.charact  of
        'I' : StandardPosGal ; // UnStandP
        'P' : SatPosElm(fmPrtPos.elmSatl); // UnVectoS kepler elem
      end;
  if  fmPrtPos.charact = 'I'  then
    fmPrtPos.MemoBox.Lines.Insert(0,'  '+fmPrtPos.rbGal.Caption);
  fmPrtPos.MemoBox.Visible:=True;
  fmPrtPos.MemoBox.SetFocus;
end;

procedure TfmPrtPos.MemoBoxKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if  ( Key = Ord('S') ) and ( Shift = [ ssCtrl ] )
    then // to save
      with  fmPrtPos  do
        PuzSaveBmp(ClientWidth,ClientHeight,
                   Width,Height,3,3,Canvas);
end;

end.
