 { procedure to calculate the refraction correction
   has been presented by doctor Konstantin V. Kuimov }
 UNIT UnRefrOl ;
 INTERFACE
 Var
   wlmkm   : Extended ; { length of wave }
   temprc  : Extended ; { temperature in degree Celsius }
   prepas  : Extended ; { pressure in pascal 760 mm = 101325 Pa }
   humrun  : Extended ; { humidity in unit interval [0,1] }
 Procedure SimpleDataForRefraction ;
 Procedure ToGetDataForRefraction ;
 Procedure RefrOlik ( wlmkm  : Extended ;
                      temp , ppas , hum  : Extended ;
                      z : Extended ;
                      Var  ref  : Extended ) ;
 Function ToGetRefraction ( alt  : Extended ) : Extended ;
 IMPLEMENTATION
 Uses
   UnConTyp ; { for GraRad , RadGra and RadSec }
 Procedure SimpleDataForRefraction ;
 Begin
   { initial value }
   wlmkm:=0.5931; { lehgtn of wave in mkm }
   temprc:=0.0; { in degree of Celsius }
   prepas:=99325.158; { in pascal 745 in mm }
   humrun:=0.9; { relative humidity is 90% }
 End;
 Procedure ToGetDataForRefraction ;
 Var
   ft  : Text ;
   tempk,presh,hum  : Extended ;
 Begin
   { initial value }
   wlmkm:=0.5931; { in mkm }
   tempk:=273.15; { in Kelvin }
   presh:=745.0; { in mm }
   hum:=90.0; { in % }
   Assign(ft,'RefrParm.Dat');
 {$I-}
   ReSet(ft);
 {$I+}
   If  IOResult = 0
     Then
       Begin
         If  NOT EOF(ft)  Then  ReadLn(ft,tempk);
         If  NOT EOF(ft)  Then  ReadLn(ft,presh);
         If  NOT EOF(ft)  Then  ReadLn(ft,hum);
         Close(ft);
       End;
   temprc:=tempk-273.15; { to Celsius }
   prepas:=101325.0*(presh/760.0); { pressure in pascal }
   humrun:=hum/100.0; { to unit interval }
 End;
 Procedure RefrOlik ( wlmkm  : Extended ;
                      temp , ppas , hum  : Extended ;
                      z : Extended ;
                      Var  ref  : Extended ) ;
 Const
   R      = 8314.72e0;  {  ﭭ, /( * ) }
   rw     = 8314.72e0/18.0152e0; {  ﭭ/.  }
   g0     = 9.80665e0; { ᪮७ ᨫ 殮, /**2 }
   Dm     = 28.9644e0; { ୠ  宣 , / }
   r0     = 6371000.e0;  { । ࠤ ,  }
   Hpi    = 1.570796326794897e0;    { pi/2 }
   sqpi2  = 0.8862269254527580e0; { sqrt(pi)/2 }
 Var
   tabs,tabs2,tabs3,t,roln,pw,ps,
   ds,dw,s2,s4,s6,alpha0,b0,a,x,y  : Extended ;
 Begin
   Tabs := temp + 273.15e0; { ᮫⭠ ⥬ }
   t := temp;               { ⥬   }
   roln := -5.32917e0 + t * (0.0688825e0 + t * (-2.9815e-4
   + 1.39e-6 * t)); { .  ⭮ ﭮ  (. [2]) }
   pw := Hum * rw * Tabs * exp(roln); {  ﭮ  }
   Ps := (Ppas - pw)/100; { ࠧ    宣  }
   Pw := Pw/100;          { ࠧ    ﭮ  }
   tabs2 := Sqr(Tabs);
   tabs3 := tabs2 * Tabs;
   Ds := Ps/Tabs
         * (1 + Ps * (57.90e-8 - 9.3250e-4/Tabs + 0.25844e0/tabs2));
   Dw := Pw/Tabs * (1 + Pw * (1 + 3.7e-4 * Pw)
         * (-2.37321e-3 + 2.23366e0/Tabs - 710.792e0/tabs2
         + 7.75141e4/tabs3));
   s2 := 1/Sqr(Wlmkm);
   s4 := s2*s2;
   s6 := s2 * s4;
   alpha0 :=((2371.34e0 + 683939.7e0/(130-s2) + 4547.3e0/(38.9e0-s2)) * Ds
            +(6487.31e0+58.058e0*s2-0.71150e0*s4+0.08851e0*s6) * Dw)/1.e8;
   b0 := R * Tabs/(g0 * Dm * r0);           { beta0 }
   a := b0 - alpha0/2;
   x := cos(z)/sqrt(2 * a);
   t := 1.0e0 - 7.5e0/(abs(x) + 3.75e0);   { NAG13 s15adf.for }
   y := (((((((((((((((+3.328130055126039e-10
        * t - 5.718639670776992e-10) * t - 4.066088879757269e-9)
        * t + 7.532536116142436e-9) * t + 3.026547320064576e-8)
        * t - 7.043998994397452e-8) * t - 1.822565715362025e-7)
        * t + 6.575825478226343e-7) * t + 7.478317101785790e-7)
        * t - 6.182369348098529e-6) * t + 3.584014089915968e-6)
        * t + 4.789838226695987e-5) * t - 1.524627476123466e-4)
        * t - 2.553523453642242e-5) * t + 1.802962431316418e-3)
        * t - 8.220621168415435e-3) * t + 2.414322397093253e-2;
   y := (((((y * t - 5.480232669380236e-2) * t + 1.026043120322792e-1)
        * t - 1.635718955239687e-1) * t + 2.260080669166197e-1)
        * t - 2.734219314954260e-1) * t + 1.455897212750385e-1;
   ref := alpha0 * (1 - alpha0/2) * sqrt(2/a) * sin(z) * y * sqpi2;
 End;
 Function ToGetRefraction ( alt  : Extended ) : Extended ;
 Var
   z,zr,ref  : Extended ;
 Begin
   ref:=0;
   z:=90-alt;
   If  z > 90.0  Then  z:=90.0; { if altitude < 0 degree }
   If  z < 0.00  Then  z:=0.00;
   If  ( ( 0.0 <= z ) AND ( z <= 90.0 ) )
     Then
       Begin
         zr:=GraRad*z; { to radian }
         RefrOlik(wlmkm,temprc,prepas,humrun,zr,ref); { ref in radian }
       End;
   ToGetRefraction:=RadGra*ref; { in degree }
 End;
 END.

