unit UnrfIrvs; // to work with IRVS satellite format

interface

function ToFindNumString ( n  : Byte ; st  : ShortString ) : Boolean ;

procedure ToWorkWithIRVSfmt ( FileName  : ShortString ) ;

procedure ToReadPosVelForma ( FileName  : ShortString ) ; 

implementation

uses
  UnConTyp , // for const VelOfRot
  UnCoTVar , // for global var SatElemCur
  UnGloVar , // for TypeFundArg
  UnForDat , // for proc TransDateToJD
  UnForTim , // function ToGetGrMSTime
  PUNutC   , // for ClcNut
  UnrfNord , // for functions ToGetNoradNumber ToGetLaunchNumber
  UnitEilP , // some Eiler elements procedures
  UnitElmK ; // PosVType array[1..6] of extended

var
  nc  : Integer ;
  rc  : Extended ;

 Function TheFirstString ( st  : ShortString ) : Boolean ;
 Var
   bh  : Boolean ;
   sa  : ShortString ;
   ie  : Integer ;
   er  : Integer ;
 Begin
   bh:=False;
   ie:=0;
   sa:=Copy(st,1,5); { year }
   Val(sa,nc,er);
   ie:=ie+Abs(er);
   sa:=Copy(st,6,3); { month }
   Val(sa,nc,er);
   ie:=ie+Abs(er);
   sa:=Copy(st,9,3); { day }
   Val(sa,nc,er);
   ie:=ie+Abs(er);
   sa:=Copy(st,12,3); { hour }
   Val(sa,nc,er);
   ie:=ie+Abs(er);
   sa:=Copy(st,15,3); { min }
   Val(sa,nc,er);
   ie:=ie+Abs(er);
   sa:=Copy(st,18,5); { sec }
   Val(sa,rc,er);
   ie:=ie+Abs(er);
   sa:=Copy(st,23,18); { x }
   Val(sa,rc,er);
   ie:=ie+Abs(er);
   sa:=Copy(st,41,18); { y }
   Val(sa,rc,er);
   ie:=ie+Abs(er);
   sa:=Copy(st,59,18); { z }
   Val(sa,rc,er);
   ie:=ie+Abs(er);
   If  ie = 0  Then  bh:=TRUE;
   TheFirstString:=bh;
 End;

 Function TheSecondString ( st  : ShortString ) : Boolean ;
 Var
   bh  : Boolean ;
   sa  : ShortString ;
   ie  : Integer ;
   er  : Integer ;
 Begin
   bh:=False;
   ie:=0;
   sa:=Copy(st,1,5); { satellite number }
   Val(sa,nc,er);
   ie:=ie+Abs(er);
   sa:=Copy(st,6,6); { current number }
   Val(sa,nc,er);
   ie:=ie+Abs(er);
   sa:=Copy(st,12,6); { current number }
   Val(sa,nc,er);
   ie:=ie+Abs(er);
   sa:=Copy(st,23,18); { vx }
   Val(sa,rc,er);
   ie:=ie+Abs(er);
   sa:=Copy(st,41,18); { vy }
   Val(sa,rc,er);
   ie:=ie+Abs(er);
   sa:=Copy(st,59,18); { vz }
   Val(sa,rc,er);
   ie:=ie+Abs(er);
   If  ie = 0  Then  bh:=TRUE;
   TheSecondString:=bh;
 End;

 Function ControlNumString ( n  : Byte ;  st  : ShortString ) : Boolean ;
 Var
   bh  : Boolean ;
 Begin
   bh:=False;
   If  n = 1  Then  bh:=TheFirstString(st);
   If  n = 2  Then  bh:=TheSecondString(st);
   ControlNumString:=bh;
 End;

 { to find String with the numbers only  for example
 1999 11 22  0  0  0.0   -2716221.771831    4336164.098243   -5360212.500794
 1134   131     0        -6407.683215484   -1907.638349612    1887.572365505 }

 Function ToFindNumString ( n  : Byte ; st  : ShortString ) : Boolean ;
 Var
   ih  : Byte ;
   lh  : Byte ;
   ch  : Char ;
   bh  : Boolean ;
   bf  : Boolean ;
 Begin
   bf:=False;
   lh:=Length(st);
   If lh > 75
     Then
       Begin
         ih:=0;
         bh:=TRUE;
         While  ( bh AND ( ih < lh ) ) Do
           Begin
             ih:=ih+1;
             ch:=st[ih];
             If  NOT ( ch IN [ ' ' , '+' , '-' , '.' , '0'..'9' ] )
               Then { other symbols in the ShortString }
                 bh:=FALSE;
           End;
         If  bh
           Then
             bf:=ControlNumString(n,st);
       End;
   ToFindNumString:=bf;
 End;

procedure FstIrvsString ( st  : ShortString ;
                          var  t , x , y , z  : Extended ) ;
var
  sa  : ShortString ;
  ny,nm,nd  : Integer ;
  kh,km,er  : Integer ;
  ts  : Extended ;
begin
  sa:=Copy(st,1,5); { year }
  Val(sa,ny,er);
  sa:=Copy(st,6,3); { month }
  Val(sa,nm,er);
  sa:=Copy(st,9,3); { day }
  Val(sa,nd,er);
  sa:=Copy(st,12,3); { hour }
  Val(sa,kh,er);
  sa:=Copy(st,15,3); { min }
  Val(sa,km,er);
  sa:=Copy(st,18,5); { sec }
  Val(sa,ts,er);
  sa:=Copy(st,23,18); // x georotating system in meter
  Val(sa,x,er);
  sa:=Copy(st,41,18); // y georotating system
  Val(sa,y,er);
  sa:=Copy(st,59,18); // z georotating system
  Val(sa,z,er);
  TransDateToJD(nd,nm,ny,t); // UnForDat
  t:=t+(kh+(km+ts/60.0)/60.0)/24.0; // date in julian day
end;

procedure ForNoradNumber ( ns  : Integer ) ; // from IRVS number ns
var                                          // with the use file
  ft  : TextFile ;                           // satstdm.pgm
  st  : ShortString ;
  sa  : ShortString ;
  na  : Integer ;  // IRVS number
  er  : Integer ;
  vm  : Single;
begin
  ElemDefault(SatElemCur); // UnitEilP
  AssignFile(ft,stPathDefault+'\satstdm.pgm');
  ReSet(ft);
  ReadLn(ft); // the first string
repeat
  ReadLn(ft,na,st);
  if  na = ns
    then // this satellite IRVS number is in file satstdm.pgm
      begin
        sa:=Copy(st,3,5);
        SatElemCur.satnord:=ToGetNoradNumber(sa);  // UnrfNord
        sa:=Copy(st,11,6);
        SatElemCur.satnuml:=ToGetLaunchNumber(sa); // UnrfNord
        SatElemCur.satname:=Copy(st,19,16);
        sa:=Copy(st,49,5); // for standard magnitude
        Val(sa,vm,er);
        if  er = 0  then  SatElemCur.stdmag:=vm;
      end;
until  EOF(ft) or ( ns = na ) ;
  CloseFile(ft);
end;

procedure SndIrvsString ( st  : ShortString ;
                          var  vx , vy , vz  : Extended ) ;
var
  sa  : ShortString ;
  ns  : Integer ; // IRVS number of satelliteto SatElemCur
  er  : Integer ;
begin
  sa:=Copy(st,1,5);   // IRVS internal satellite number
  Val(sa,ns,er);
  ForNoradNumber(ns); // to SatElemCur  satnord satnuml satname
  sa:=Copy(st,23,18); // vx georotating system meter per second
  Val(sa,vx,er);
  sa:=Copy(st,41,18); // vy georotating system
  Val(sa,vy,er);
  sa:=Copy(st,59,18); // vz georotating system
  Val(sa,vz,er);
end;

procedure TwoIrvsStrings ( sa , sb  : ShortString ) ; // to SatElemCur
var
  t,x,y,z   : Extended ;
  vx,vy,vz  : Extended ;
  gt,gs,gc  : Extended ; // greemwich true sidereal time sinus cosinus
  dp,de,em  : Extended ; // delta(psi) delta(epsilon) epsilon mean
  PosVel    : PosVType ; // type from UnitElmK
begin
  FstIrvsString(sa,t,x,y,z);     // year day month hour min sec to julian day
  SndIrvsString(sb,vx,vy,vz);    // position velosity geo rotation system
  ClcNut(t,dp,de,em);            // PUnNutC in radian delta(psi,eps) eps
  gt:=ToGetGrMSTime(t)+dp*Cos(em+de); // UnForTim true sidereal time
  gs:=Sin(gt);
  gc:=Cos(gt);
  PosVel[1]:=1.0e-3*(x*gc-y*gs);  // from meter to kilometer
  PosVel[2]:=1.0e-3*(x*gs+y*gc);  // from m to km
  PosVel[3]:=1.0e-3*z;
  PosVel[4]:=1.0e-3*(vx*gc-vy*gs)-VelOfRot*PosVel[2]; // const VelOfRot
  PosVel[5]:=1.0e-3*(vx*gs+vy*gc)+VelOfRot*PosVel[1]; // in km/s
  PosVel[6]:=1.0e-3*vz; // from meter per second to kilometer per second
  XVEilR(t,PosVel,SatElemCur); // SatElemCur UnCoTVar t ao au ae ai am an
end;

procedure ElemIrvsToHeap ;
begin
  NumSatElemInHeap:=1; // only one object
  SatElemHeap^[NumSatElemInHeap]:=SatElemCur; // UnCoTVar
end;

procedure ToWorkWithIRVSfmt ( FileName  : ShortString ) ; // if IRVS exists
var
  ft  : TextFile ;
  bt  : Boolean ;
  sa  : ShortString ;
  sb  : ShortString ;
begin
  AssignFile(ft,FileName);
  ReSet(ft);
  while  not EOF(ft)  do
    begin
      ReadLn(ft,sa);             // to check the first string
      bt:=ToFindNumString(1,sa); // function from InrfIrvs
      if  bt     // there is the first string
        then     // it is possible to check
          begin  // the next string
            ReadLn(ft,sb); // a try for the next string
            bt:=ToFindNumString(2,sb); // as second string
          end; // bt true if there are two strings with IRVs data
      if  bt  then  TwoIrvsStrings(sa,sb); // sat elements to SatElemCur
    end;
  ElemIrvsToHeap ;
  CloseFile(ft);
end; // proc is called from UnForBut by ForButtonE

{ position velosity in Relative system 
  Relative system
  25977  Helios 1B           ⭨
  31  01  2006         
  17  00  21.753    ᥬ୮ ६   ᥪ㭤 
  3806.256          x  
  -5948.307         y  
  000.000           z  
  -1.331068         x ᪮  /
  -0.841904         y ᪮  /
  7.439610          z ᪮  / }
procedure ToReadPosVelForma ( FileName  : ShortString ) ; // from UnForBut
var
  ft  : TextFile ;
  nd,nm,ny,kh,km  : Integer ; // day month year hour min
  ts  : Extended ;    // second
  sa  : ShortString ; // for the name
  t,x,y,z   : Extended ;
  vx,vy,vz  : Extended ;
  gt,gs,gc  : Extended ; // greemwich true sidereal time sinus cosinus
  dp,de,em  : Extended ; // delta(psi) delta(epsilon) epsilon mean
  PosVel    : PosVType ; // type from UnitElmK
begin
  AssignFile(ft,FileName);
  ReSet(ft);
  ReadLn(ft); // the first string with record Relative system
  ReadLn(ft,sa);         // satellite name
  ReadLn(ft,nd,nm,ny);   // day month year
  ReadLn(ft,kh,km,ts);   // hour minute second
  ReadLn(ft,x);  // x position in km   refer to the rotating Earth
  ReadLn(ft,y);  // y position in km   refer to the rotating Earth
  ReadLn(ft,z);  // z position in km   refer to the rotating Earth
  ReadLn(ft,vx); // x velosity in km/s refer to the rotating Earth
  ReadLn(ft,vy); // y velosity in km/s refer to the rotating Earth
  ReadLn(ft,vz); // z velosity in km/s refer to the rotating Earth
  CloseFile(ft);
  ElemDefault(SatElemCur); // UnitEilP
  SatElemCur.satname:=sa;
  TransDateToJD(nd,nm,ny,t); // UnForDat
  t:=t+(kh+(km+ts/60.0)/60.0)/24.0; // date in julian day
  ClcNut(t,dp,de,em);        // PUnNutC in radian delta(psi,eps) eps
  gt:=ToGetGrMSTime(t)+dp*Cos(em+de); // UnForTim true sidereal time
  gs:=Sin(gt);           // sinus   true sidereal time
  gc:=Cos(gt);           // cosinus true sidereal time
  PosVel[1]:=x*gc-y*gs;  // x position in kilometer to true equinox
  PosVel[2]:=x*gs+y*gc;  // y position in kilometer to true equinox
  PosVel[3]:=z;          // z position in kilometer to true equinox
  PosVel[4]:=vx*gc-vy*gs-VelOfRot*PosVel[2]; // const VelOfRot UnConTyp
  PosVel[5]:=vx*gs+vy*gc+VelOfRot*PosVel[1]; // y velosity in km/s
  PosVel[6]:=vz; // z velosity kilometer per second
  XVEilR(t,PosVel,SatElemCur); // SatElemCur UnCoTVar t ao au ae ai am an
  ElemIrvsToHeap ; // NumSatElemInHeap = 1 to SatElemHeap^
end;

end.
