unit UnrfKodr;

interface

function  BooKDrFormat ( FileName  : ShortString ) : Boolean ;

procedure ToWorkWithKodrfmt ( FileName  : ShortString ) ;

implementation

uses
  SysUtils , // for function Trim TrimLeft
  UnConTyp , // constant TwoPi GeoFM
  UnCoTVar , // for TElemRec  var NumSatElemInHeap
  UnForDat , // for procedure TransDateToJD
  UnForFun , // for function ATanDegree
  UnitEilP , // for procedure ElemDefault
  UnitElmK ; // AnomVM

type
  TElemKd  = record
               sn  : ShortString ; // name
               md , mm , my  : Integer ; // day month year
               kh , km  : Integer ; // hour minute
               ps  : Extended ; // second
               pr  : Extended ; // period in minute
               dp  : Extended ; // change of period in minite per revolution
               di  : Extended ; // inclination in degree
               du  : Extended ; // ascending node in degree
               dl  : Extended ; // e*Cos(omega);  nonsingular
               dh  : Extended ; // e*Sin(omega);  elements
             end; // type TElemKd

var
  SatElemKd  : TElemKd ;  // variable for this unit

function BooDateKodr ( s  : ShortString ) : Boolean ;
var // to read date from string in form DD MM YY or DD MM YYYY with blanks
  n  : Byte ;
  z  : ShortString ;
  v  : ShortString ;
  d  : Integer ; // day
  m  : Integer ; // month
  y  : Integer ; // year
  e  : Integer ; // for error in Val procedure
begin
  BooDateKodr:=False;
  z:=s;
  n:=Pos(' ',z); // blanks
  while  n = 1  do
    begin
      Delete(z,n,1);
      n:=Pos(' ',z); // to erase blanks forward
    end;
  if  n > 1
    then // may be day
      begin
        v:=Copy(z,1,n-1); // for day
        Delete(z,1,n-1);  // without day
      end
    else
      v:=' ';
  Val(v,d,e); // day
  if  e <> 0  then  Exit ; // error
  n:=Pos(' ',z);
  while  n = 1  do
    begin
      Delete(z,n,1);
      n:=Pos(' ',z); // to erase blanks forward
    end;
  if  n > 1
    then // may be month
      begin
        v:=Copy(z,1,n-1); // for month
        Delete(z,1,n-1); // without month
      end
    else
      v:=' ';
  Val(v,m,e); // month
  if  e <> 0  then  Exit ; // error
  n:=Pos(' ',z);
  while  n > 0  do   // for year
    begin
      Delete(z,n,1);
      n:=Pos(' ',z); // to erase blanks forward backward
    end;
  v:=z;
  Val(v,y,e); // year
  if  e <> 0  then  Exit ; // error
  if  y < 85 // year without thousand
    then
      y:=2000+y
    else
      if  y < 100
        then
          y:=1900+y;
  SatElemKd.md:=d;
  SatElemKd.mm:=m;
  SatElemKd.my:=y;
  BooDateKodr:=True; // this is information about date
end;

function BooTimeKodr ( s  : ShortString ) : Boolean ;
var // to read time from string in form hh mm ss.sss with blanks
  n  : Byte ;
  z  : ShortString ;
  v  : ShortString ;
  h  : Integer ; // hour
  m  : Integer ; // minute
  c  : Extended ; // second
  e  : Integer ; // for error in Val procedure
begin
  BooTimeKodr:=False;
  z:=s;
  n:=Pos(' ',z); // blanks
  while  n = 1  do
    begin
      Delete(z,n,1);
      n:=Pos(' ',z); // to erase blanks forward
    end;
  if  n > 1
    then // may be hour
      begin
        v:=Copy(z,1,n-1); // for hour
        Delete(z,1,n-1);  // without hour
      end
    else
      v:=' ';
  Val(v,h,e); // hour
  if  e <> 0  then  Exit ; // error
  n:=Pos(' ',z);
  while  n = 1  do
    begin
      Delete(z,n,1);
      n:=Pos(' ',z); // to erase blanks forward
    end;
  if  n > 1
    then // may be minute
      begin
        v:=Copy(z,1,n-1); // for minute
        Delete(z,1,n-1); // without minute
      end
    else
      v:=' ';
  Val(v,m,e); // minute
  if  e <> 0  then  Exit ; // error
  n:=Pos(' ',z);
  while  n > 0  do   // for second
    begin
      Delete(z,n,1);
      n:=Pos(' ',z); // to erase blanks forward backward
    end;
  v:=z;
  Val(v,c,e); // second
  if  e <> 0  then  Exit ; // error
  SatElemKd.kh:=h; // hour
  SatElemKd.km:=m; // minute
  SatElemKd.ps:=c; // second
  BooTimeKodr:=True; // this is information about time
end;

function BooElemKodr ( k  : Byte ;  s  : ShortString ) : Boolean ;
var
  n  : Byte ;
  z  : ShortString ;
  v  : Extended ; // any value
  e  : Integer ;  // for error in Val procedure
begin
  BooElemKodr:=False;
  z:=s;
  n:=Pos(' ',z);
  while  n > 0  do   // for second
    begin
      Delete(z,n,1);
      n:=Pos(' ',z); // to erase blanks forward backward
    end;
  Val(z,v,e); // any value
  if  e <> 0  then  Exit ; // error
  case  k  of // always k > 3  k < 10
    4  : SatElemKd.pr:=v; // period in minute
    5  : SatElemKd.dp:=v; // change of period
    6  : SatElemKd.di:=v; // inclination in degree
    7  : SatElemKd.du:=v; // ascending node in degree
    8  : SatElemKd.dl:=v; // e*Cos(omega);
    9  : SatElemKd.dh:=v; // e*Sin(omega);
  end; // case n
  BooElemKodr:=True;
end;

function BooStrKodr ( n  : Byte ;  s  : ShortString ) : Boolean ;
var
  b  : Boolean ;
begin
  case  n  of  // for n > 1
     2  : b:=BooDateKodr(s);
     3  : b:=BooTimeKodr(s)
    else  b:=BooElemKodr(n,s); // elements for n > 3
  end; // case n
  BooStrKodr:=b;
end;

function  BooKDrFormat ( FileName  : ShortString ) : Boolean ;
var
  ft  : TextFile ;
  nt  : Byte ;
  st  : ShortString ;
  sa  : ShortString ;
  ba  : Boolean ;
  bt  : Boolean ;
  vr  : Extended ;  // for variable in Val procedure
  er  : Integer ;   // for error in Val procedure
begin               // to check Kosmos Dracon format may be
  bt:=False;
  AssignFile(ft,FileName);
  ReSet(ft);
  nt:=0;
  ba:=True;
  while  ( not EOF(ft) ) and ( not bt )  do
    begin
      ReadLn(ft,st);  // for the first string
      sa:=Trim(st);   // without blanks forward and backward
      if  ( Length(sa) > 3 ) and  ( nt < 9 ) // for current string
        then     // length greater than three char
          begin
            nt:=nt+1;
            sa:=Trim(Copy(st,1,20));
            if  nt = 1
              then
                SatElemKd.sn:=sa // for name
              else
                begin
                  if  nt = 5  // may be value for inclination
                    then      // or may be change of period
                      begin
                        Val(sa,vr,er);
                        if  er = 0
                          then
                            if  vr > 0.001  // not string with period change
                              then
                                begin
                                  ba:= ba and BooStrKodr(nt,'0.00');
                                  nt:=nt+1;
                                end;
                      end;
                  ba:= ba and BooStrKodr(nt,sa);
                end;
          end;
      if  nt = 9  then  bt:=ba  else  bt:=False;  // only for one string
    end;
  CloseFile(ft);
  BooKDrFormat:=bt;
end;

procedure ElemKodrToHeap ; // to transform kepler elements
var                        // eiler elements and to keep in heap
  a,e,o,s,c,r,u,v,w  : Extended ;
begin
  with  SatElemKd  do
    begin
      SatElemCur.satname:=sn;
      TransDateToJD(md,mm,my,SatElemCur.t); // var SatElemCur from UnCoTVar
      SatElemCur.t:=SatElemCur.t+(kh+(km+ps/60.0)/60.0)/24.0; // julian date
      SatElemCur.ai:=di; // inclination in degree
      SatElemCur.au:=du; // ascending node in degree
      e:=Sqrt(Sqr(dl)+Sqr(dh)); // eccentricity nonsingular elements
      SatElemCur.ae:=e;
      c:=dl/e; // cos(omega) kepler argument
      s:=dh/e; // sin(omega) of perigei
      o:=ATanDegree(s,c); // omega argument of perigei in degree
      v:=-o; // at the equator u = v + omega = 0 v true anomaly in degree
      AnomVM(e,v,w); // w mean anomaly in degree UnitElmK
      SatElemCur.ao:=o; // argument of perigei in degree
      SatElemCur.am:=w; // mean anomaly in degree
      u:=1440.0/pr; // draconic motion in revolution per day
      r:=TwoPi*u/86400.0; // the same but in radian per second
      a:=Exp(1.0/3.0*Ln(GeoFM/Sqr(r))); // const from UnConTyp
      c:=Cos(GraRad*di);
      s:=Sqr(GeoR0/a)*(5*Sqr(c)-1)/(1-Sqr(e));
      r:=0.75*1.082e-3*u*s; // correction to draconic motion in rev./day
      u:=u-r; // mean motion with correction in revolution per day
      c:=-Sqr(u)*dp/pr; // change of mean motion in rev./day per day
      SatElemCur.an:=u; // mean motion in revolution per day
      SatElemCur.dn:=c; // change of mean motion in rev./day per day
    end;
  Inc(NumSatElemInHeap);
  SatElemHeap^[NumSatElemInHeap]:=SatElemCur; // UnCoTVar
end;

procedure ToWorkWithKodrfmt ( FileName  : ShortString ) ;
var
  ft  : TextFile ;
  nt  : Byte ;
  st  : ShortString ;
  sa  : ShortString ;
  ba  : Boolean ;
  vr  : Extended ;  // for variable in Val procedure
  er  : Integer ;   // for error in Val procedure
begin               // to check Kosmos Dracon format may be
  ElemDefault(SatElemCur); // from UnitEilP
  NumSatElemInHeap:=0; // to clear heap  var UnCoTVar
  AssignFile(ft,FileName);
  ReSet(ft);
  nt:=0;
  ba:=True;
  while   not EOF(ft)  do
    begin
      ReadLn(ft,st);  // for the first string
      sa:=Trim(st);   // without blanks forward and backward
      if  ( Length(sa) > 3 ) and  ( nt < 9 ) // for current string
        then     // length greater than three char
          begin
            nt:=nt+1;
            sa:=Trim(Copy(st,1,20));
            if  nt = 1
              then
                SatElemKd.sn:=sa // for name
              else
                begin
                  if  nt = 5  // may be value for inclination
                    then      // or may be change of period
                      begin
                        Val(sa,vr,er);
                        if  er = 0
                          then
                            if  vr > 0.001  // not string with period change
                              then
                                begin
                                  ba:= ba and BooStrKodr(nt,'0.00');
                                  nt:=nt+1;
                                end;
                      end;
                  ba:= ba and BooStrKodr(nt,sa);
                end;
          end;
      if  nt = 9
        then
          begin
            if  ba  then  ElemKodrToHeap;
            nt:=0;
            ba:=True;
          end;
    end;
  CloseFile(ft);
end;

end.
