
 UNIT UnrfNord ; { to read Norad format of the elements }

 INTERFACE

 Function ToGetNoradNumber ( snum       : ShortString ) : LongInt ;

 function ToGetLaunchNumber ( st  : ShortString ) : LongInt ;

 Procedure ToReadNoradFormat ( FileName  : ShortString ) ;

 IMPLEMENTATION

 Uses
   SysUtils , // for boolean function FileExists
   UnCoTVar , // types and constants SatElemCur SatElemHeap
   UnForDat , // for TransDateTiMJD
   UnForStr ; // procedures ToEraseSomeBlanks ToReplaceBlanks

 Function ToGetNoradNumber ( snum       : ShortString ) : LongInt ;
 Var
   i,l     : Byte ;
   err     : Integer ;
   ch      : Char ;
   sa      : ShortString ;
   numl    : LongInt ;
   nums    : LongInt ;
 Begin
   nums:=100000;
   sa:=snum;
   l:=Length(sa);
   If  l < 2
     Then
       Begin
         ToGetNoradNumber:=nums;
         Exit ;
       end;
   For i:=1 To l Do
     Begin
       ch:=sa[i];
       If  ch = ' '  Then  ch:='1';
       If  NOT ( ch IN [ '0'..'9' ] )
         Then
           ch:='0';
       sa[i]:=ch;
     End;
   Val(sa,numl,err);
   If  err = 0  Then  nums:=numl;
   ToGetNoradNumber:=nums;
 End;

 function ToGetPartNumber ( ca  : Char ) : LongInt ;
 begin
   case  ca  of
    'A' : ToGetPartNumber:=1;
    'B' : ToGetPartNumber:=2;
    'C' : ToGetPartNumber:=3;
    'D' : ToGetPartNumber:=4;
    'E' : ToGetPartNumber:=5;
    'F' : ToGetPartNumber:=6;
    'G' : ToGetPartNumber:=7;
    'H' : ToGetPartNumber:=8;
    'I' : ToGetPartNumber:=9
    else  ToGetPartNumber:=10;
   end; // case
 end;

 function ToGetLaunchNumber ( st  : ShortString ) : LongInt ;
 var
   sa  : ShortString ;
   ca  : Char ;
   ns  : LongInt;
 begin
   sa:=Copy(st,1,5);
   ca:=st[6];
   ns:=100*ToGetNoradNumber(sa)+ToGetPartNumber(ca);
   ToGetLaunchNumber:=ns;
 end;

 Procedure ToGetDateFromString ( st       : ShortString ;
                                 Var  el  : TElemRec ;  // type UnCoTVar
                                 Var  b   : Boolean  ) ;
 Var
   ErrVal       : Integer ;
   Day,
   Month,Year   : Integer ;
   PDay,d,v     : Extended ;
   sa,sy,sd,sn  : ShortString ;
 Begin
   b:=FALSE;
   sa:=Copy(st,3,5);  // number of a satellite from norad
   el.satnord:=ToGetNoradNumber(sa);  // NORAD number
   sa:=Copy(st,10,6); // number of satellite as date of launch
   el.satnuml:=ToGetLaunchNumber(sa);
   sy:=Copy(st,19,2);
   sd:=Copy(st,21,12);
   sn:=Copy(st,33,11);
   ToReplaceBlanks(sy); // UnForStr
   Val(sy,Year,ErrVal);
   If  ErrVal <> 0  Then  Exit ;
   If  Year < 51   Then  Year:=2000+Year;
   If  Year < 100  Then  Year:=1900+Year;
   Day:=31;
   Month:=12;
   Year:=Year-1;
   TransDateToJD(Day,Month,Year,d); // UnForDat
   ToReplaceBlanks(sd); // UnForStr
   Val(sd,PDay,ErrVal);
   If  ErrVal <> 0
     Then
       Exit ;
   d:=d+PDay;
   el.t:=d; // julian date of elements
   ToReplaceBlanks(sn);  // UnForStr
   Val(sn,v,ErrVal);
   If  ErrVal <> 0  Then  v:=0.0;
   el.dn:=v; // dn/dt n mean motion rev/day per day
   b:=TRUE;
 End;

 Procedure ToGetElementsFromString ( st       : ShortString ;
                                     Var  el  : TElemRec ; // UnCoTVar
                                     Var  b   : Boolean  ) ;
 Var
   ErrVal  : Integer ;
   sc      : ShortString ;
   ac      : Extended ;
 Begin
   b:=FALSE;
   sc:=Copy(st,9,8); { inclination in degree }
   ToReplaceBlanks(sc);
   Val(sc,ac,ErrVal);
   If  ErrVal <> 0  Then  Exit ;
   if  ac < 0.00025  then  ac:=0.00025; // more than one ArcSec
   el.ai:=ac;
   sc:=Copy(st,18,8); { ascending node in degree }
   ToReplaceBlanks(sc);
   Val(sc,ac,ErrVal);
   If  ErrVal <> 0  Then  Exit ;
   el.au:=ac;
   sc:=Copy(st,27,7); { pack eccentricity }
   ToReplaceBlanks(sc);
   sc:='0.'+sc;
   Val(sc,ac,ErrVal);
   If  ErrVal <> 0  Then  Exit ;
   el.ae:=ac;
   sc:=Copy(st,35,8); { argument of perigei in degree }
   ToReplaceBlanks(sc);
   Val(sc,ac,ErrVal);
   If  ErrVal <> 0  Then  Exit ;
   el.ao:=ac;
   sc:=Copy(st,44,8); { mean anomaly in degree }
   ToReplaceBlanks(sc);
   Val(sc,ac,ErrVal);
   If  ErrVal <> 0  Then  Exit ;
   el.am:=ac;
   sc:=Copy(st,53,11); { mean motion in rev/day }
   ToReplaceBlanks(sc);
   Val(sc,ac,ErrVal);
   If  ErrVal <> 0  Then  Exit ;
   el.an:=ac;
   b:=TRUE;
 End;

 Function MagFromNulloLine ( st  : ShortString ) : Single ;
 Var
   vm  : Single ;
   ls  : Byte ;
   er  : Integer ;
   sa  : String[5];
 Begin
   ls:=Length(st);
   If  ls > 35
     Then { may be magnitude information }
       Begin
         sa:=Copy(st,31,5);
         Val(sa,vm,er);
         If  er <> 0
           Then
             vm:=99; { no real number in string }
        End
     Else
       vm:=99; { string is too short }
   MagFromNulloLine:=vm;
 End;

 function ToCheckNoradFile ( FileName  : ShortString ) : Boolean ;
 Var
   fr        : TextFile ;
   sa,sr,st  : ShortString ;
   l,no,nt   : Byte ;
   boofr     : Boolean ;
 Begin
   boofr:=TRUE;
   AssignFile(fr,FileName);
   ReSet(fr);
   sa:='';
   While  ( ( NOT EOF(fr) ) AND boofr )  Do
     Begin
       ReadLn(fr,sr);
       no:=Pos('1 ',sr);
       If  no = 1
         Then { the first string in norad elements }
           Begin
             l:=Length(sa);
             If  l = 0  Then  boofr:=FALSE;
             sa:='';
             Repeat
               ReadLn(fr,st);
               nt:=Pos('2 ',st); { to find the second string }
             Until  ( ( EOF(fr) ) OR ( nt = 1 ) ) ;
           End
         Else
           sa:=sr;
     End;
   CloseFile(fr);
   ToCheckNoradFile:=boofr;
 End;

 procedure ToReadNoradWithName ( FileName  : ShortString ) ;
 Var
   TmpFile  : TextFile ;
   stwo     : String[2] ;
   StrN     : String[16] ;
   StrTmp   : ShortString ;
   ba,bb    : Boolean ;
 begin
   NumSatElemInHeap:=0;
   AssignFile(TmpFile,FileName);
   ReSet(TmpFile);
   ReadLn(TmpFile,StrTmp);
   While  NOT EOF(TmpFile)  Do
     Begin
       StrN:=Copy(StrTmp,1,16); // name of satellite
       SatElemCur.stdmag:=MagFromNulloLine(StrTmp); // var from UnCoTVar
       ReadLn(TmpFile,StrTmp);
       stwo:=Copy(StrTmp,1,2);
       If  ( ( stwo = '1 ' )
            AND ( NOT EOF(TmpFile) ) )
         Then
           Begin
             SatElemCur.satname:=StrN;
             ToGetDateFromString(StrTmp,SatElemCur,ba);
             ReadLn(TmpFile,StrTmp);
             ToGetElementsFromString(StrTmp,SatElemCur,bb);
             If  ( ba AND bb )
               Then
                 Begin
                   Inc(NumSatElemInHeap); // +1 System function
                   SatElemHeap^[NumSatElemInHeap]:=SatElemCur; // UnCoTVar
                 End;
           End;
     End;
   CloseFile(TmpFile);
 end;

 function ToGetNameFromString ( st  : ShortString ) : ShortString ;
 begin
   ToGetNameFromString:=Copy(st,3,5)+'-'+Copy(st,19,10);
 end;

 procedure ToReadNoradTwoLines ( FileName  : ShortString ) ;
 Var  // file without string number 0 with the name and magnitude
   ftmp  : TextFile ;
   snul  : ShortString ;
   sone  : ShortString ;
   stwo  : ShortString ;
   ba,bb : Boolean ;
 begin // only two lines with indexes 1 and 2 are in file
   NumSatElemInHeap:=0;
   SatElemCur.stdmag:=99.0; // var from UnCoTVar no information
   AssignFile(ftmp,FileName);
   ReSet(ftmp);
   While  NOT EOF(ftmp)  Do
     Begin
       ba:=False;
       bb:=False;
       ReadLn(ftmp,snul);
       if  Pos('1 ',snul) = 1
         then // in string snul there is information of the first line
           begin
             sone:=snul;
             ToGetDateFromString(sone,SatElemCur,ba);
             if  ba  then  SatElemCur.satname:=ToGetNameFromString(sone);
             ReadLn(ftmp,stwo);
             if  Pos('2 ',stwo) = 1
               then
                 ToGetElementsFromString(stwo,SatElemCur,bb);
             If  ( ba AND bb )
               Then
                 Begin
                   Inc(NumSatElemInHeap); // +1 System function
                   SatElemHeap^[NumSatElemInHeap]:=SatElemCur; // UnCoTVar
                 End;
           End;
     End;
   CloseFile(ftmp);
 end;

 // to read text file with the name FileName usually stNoradFile

 Procedure ToReadNoradFormat ( FileName  : ShortString ) ;
 Begin
   if  not FileExists(FileName)  Then  Exit ; // function in SysUtils
   if  ToCheckNoradFile(FileName) // three lines with name of satellite
     then
       ToReadNoradWithName(FileName)
     else
       ToReadNoradTwoLines(FileName);
 End;

 END.
