unit UnSatEph; // form fmSatEph  to print ephemeris for the satellite  'P'
               // information about object looking from the satellite  'O'
interface      // information about object in small selected rectangle 'R'

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls;

type
  TfmSatEph = class(TForm)
    MemoEph: TMemo;
    ButEsc: TButton;
    ButSave: TButton;
    ButGraph: TButton;
    ButOther: TButton;
    PaintBox: TPaintBox;
    lbSite: TLabel;
    lbSatl: TLabel;
    procedure FormActivate(Sender: TObject);
    procedure ButGraphClick(Sender: TObject);
    procedure ButEscClick(Sender: TObject);
    procedure ButOtherClick(Sender: TObject);
    procedure ButSaveClick(Sender: TObject);
    procedure MemoEphKeyPress(Sender: TObject; var Key: Char);
    procedure FormPaint(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormClick(Sender: TObject);
    procedure MemoEphKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private declarations }
    BooGraph  : Boolean ;
    BooPaint  : Boolean ;
    mmCursor  : TCursor ;  // Controls
    BmpPaint  : TBitMap ;  // to keep part of the form fmPuz
    procedure AfterDrawBmp ;
    procedure ForDiagram ;
    procedure ForLabelCap ;
    procedure TitleToMemo ;
    procedure WriteAppEnd ( sf  : ShortString ) ;
    procedure ToSaveGraph ;
  public
    { Public declarations }
    BooOther  : Boolean ;
    BinStream : TMemoryStream ; // Classes
    BooPoint  : Boolean ;
    NumPoint  : LongInt ; // number of point
    procedure ToGetBmp ;
    procedure PassToMemo ;
  end;

var
  fmSatEph: TfmSatEph;

implementation

uses
  ClipBrd  , // for the ClipBoard functions
  UnitMain , // component SaveDialog fmPuz form
  UnGloVar , // for CharForEphem 'P' or 'O'
  UnCoTVar , // for PlaceCoor
  UnSelEph , // for public var from fmSelEph
  UnitStat , // proc ToGetStrForSito ToGetStrForSitu
  UnSatPas , // to find satellite passages
  UnSatVas , // for procedure ToFindEvents body from sat
  UnSatVaz , // 'L' look option information to MemoEph
  UnSatRaf , // for diagram
  UnForDat , // ToGetStrForDate function
  UnMuDraw , // for procedures AfterDraw Sky and Look
  UnGraBmp , // for PuzSaveBmp procedure
  UnSatVau ; // for procedure ToLimbCourse

{$R *.DFM}

{ part of form fmPuz to BmpPaint of TBitMap and to keep }
procedure TfmSatEph.ToGetBmp ; { proc is called from UnMuDraw }
var
  x1,x2,y1,y2,xw,yh  : Integer ; { for our rectangle }
  dst,sr  : TRect ;   // from Windows simple rectangles
begin
  BmpPaint:=TBitMap.Create; // only for this case
  xw:=PaintBox.Width;       // to place to PaintBox width
  yh:=PaintBox.Height;      // to place to PaintBox height
  x1:=xcdraw-xw div 2;      // rectangle in form fmPuz
  x2:=xcdraw+xw div 2;      // this rectangle
  y1:=ycdraw-yh div 2;      // for copying
  y2:=ycdraw+yh div 2;      // to our BitMap
  BmpPaint.Width:=xw;   // size of our BitMap
  BmpPaint.Height:=yh;  // it is limited by PaintBox component
  sr.Left:=x1;          // rectangle
  sr.Top:=y1;           // of the source
  sr.Right:=x2;         // for por BitMap
  sr.Bottom:=y2;        // at the form fmPuz
  dst.Left:=0;                   // rectangle
  dst.Top:=0;                    // for the Canvas
  dst.Right:=BmpPaint.Width;     // on our BitMap
  dst.Bottom:=BmpPaint.Height;   // selectad rectangle to BitMap
  BmpPaint.Canvas.CopyRect(dst,fmPuz.Canvas,sr); { Graphic method }
end;
{ simple image to fmSatEph.PaintBox }
procedure TfmSatEph.AfterDrawBmp ;
begin { this rectangle was in the form fmPuz }
  fmSatEph.PaintBox.Canvas.Draw(0,0,BmpPaint);
end; { try with bitmap it is selected by proc ToGetBmp }

procedure TfmSatEph.ForDiagram ; // graphical presentation
begin // 'P' satellite from station 'O' body from satellite
  case  CharForEphem  of // UnGloVar
    'P' : GraS.CulmFigure ; // class from UnSatRaf
    'O' : Gras.EventFigur ; // class from UnSatRaf
    'R' , 'Q' : AfterDrawBmp ;    // image to PaintBox
  end; // case CharForEphem kind of ephemeris
end;

procedure TfmSatEph.ForLabelCap ;
begin // 'P' satellite from station 'O' body from satellite
  case  CharForEphem  of // UnGloVar
    'P' : begin { a look from the station to the satellite }
            fmSatEph.Caption:='   ';
            fmSatEph.lbSite.Caption:='   - '
                                    +fmSelEph.SiteCooEph.name+' ';
            fmSatEph.lbSatl.Caption:='   - '
                                    +fmSelEph.SatElemEph.satname+' ';
          end;
    'O' : begin { a look from the satellite to the body }
            fmSatEph.Caption:='  ';
            fmSatEph.lbSite.Caption:='   - '
                                    +fmSelEph.SatElemEph.satname;
            fmSatEph.lbSatl.Caption:='   - '
                                    +fmSelEph.SiteCooEph.name+' ';
          end;
    'R' : begin { some information about objects in rectangle }
            fmSatEph.Caption:='    ';
            fmSatEph.lbSite.Caption:='   - '
                                    +PlaceCoor.name+' '; { UnCoTVar }
            fmSatEph.lbSatl.Caption:='   - '
                                    +', ,  ';
          end;
    'Q' : begin { some information about objects in rectangle }
            fmSatEph.Caption:='    ';
            fmSatEph.lbSite.Caption:='    - '
                                    +SatElemLook.satname+' '; { UnCoTVar }
            fmSatEph.lbSatl.Caption:='   - '
                                    +', ,  ';
          end;
    'L' : begin { about objects in rectangle around limb }
            fmSatEph.Caption:='    ';
            fmSatEph.lbSite.Caption:='    - '
                                    +SatElemLook.satname+' '; { UnCoTVar }
            fmSatEph.lbSatl.Caption:='   - '
                                    +', ,  ';
          end;
  end; // case CharForEphem 'P' or 'O' or 'R' or 'Q' or 'L'
end;

procedure TfmSatEph.TitleToMemo ;
begin
case  CharForEphem  of
  'P' , 'O' : AboutSatellite(fmSelEph.SatElemEph); // UnSatPas
  'Q' , 'L' : AboutSatellite(SatElemLook); // from UnSatPas
end; // CharForEPhem
case  CharForEphem  of
  'P' , 'R' :
        with  fmSatEph.MemoEph.Lines  do
          begin
            if  CharForEphem = 'R'
              then
                Add(ToGetStrForDate(JulianDate)); // UnForDat
            Add(ToGetStrForSito(PlaceCoor)); { global var }
            Add(ToGetStrForSitu(PlaceCoor)); { UnCoTVar }
          end;
  'O' : with  fmSatEph.MemoEph.Lines  do
          begin // for 'O' may be 'S' satellite 'P' planet
            Add('      ');
            Add('     '
            +'      ');
            case  CharBodEph  of // UnGloVar 'S' or 'P'
              'P' : Add('           '+fmSelEph.SiteCooEph.name);
              'S' : Add('           '+fmSelEph.SiteCooEph.name);
            end; // CharBodEph
          end;
  'Q' : with  fmSatEph.MemoEph.Lines  do
          begin
            Add(ToGetStrForDate(JulianDate)); // UnForDat
            MoreAboutSat(JulianDate,SatElemLook);  // UnSatVaz
          end;
end; // case 'P' 'O' 'R' 'Q'
  fmSatEph.MemoEph.Lines.Add('  ');    // ToGetStr... from UnitStat
end;

procedure TfmSatEph.PassToMemo ;
begin
  fmSatEph.BinStream:=TMemoryStream.Create; // Classes
  mmCursor:=fmPuz.Cursor;
  fmPuz.Cursor:=crHourGlass;
  fmSatEph.MemoEph.Visible:=False;
  fmSatEph.MemoEph.Clear;
  fmSatEph.TitleToMemo;
  case  CharForEphem  of  // UnGloVar
    'P' : ToFindPassage ; // UnSatPas
    'O' : ToFindEvents  ; // UnSatVas
    'R' : AfterDrawSky  ; // UnMuDraw
    'Q' : AfterDrawLook ; // UnMuDraw
    'L' : ToLimbCourse  ; // UnSatVau
  end; // case 'P' 'O' 'R' 'Q' 'L'
  if  fmSatEph.NumPoint = 0
    then
      begin
        fmSatEph.MemoEph.Lines.Add('  ');
        case  CharForEphem  of  // UnGloVar
          'P' : fmSatEph.MemoEph.Lines.Add('   ');
          'O' : fmSatEph.MemoEph.Lines.Add('   ');
        end; // case 'P' or 'O'
      end;
  fmSatEph.MemoEph.Lines.Insert(0,'  ');
  fmPuz.Cursor:=mmCursor;
end;

procedure TfmSatEph.FormActivate(Sender: TObject);
begin
  fmSatEph.ForLabelCap;
  fmSatEph.BooGraph:=False; // no diagram but table
  fmSatEph.BooOther:=False;
  fmSatEph.ButGraph.Caption:='';
  if  CharForEphem = 'L' // course to point of limb
    then
      begin
        fmSatEph.ButGraph.Visible:=False;
        fmSatEph.ButOther.Visible:=False;
      end
    else
      begin
        fmSatEph.ButGraph.Visible:=True;
        fmSatEph.ButOther.Visible:=True;
      end;
  fmSatEph.MemoEph.Visible:=True;
  fmSatEph.MemoEph.SetFocus;
end;

procedure TfmSatEph.ButGraphClick(Sender: TObject);
begin
  fmSatEph.BooGraph:= not fmSatEph.BooGraph ;
  if  fmSatEph.BooGraph
    then
      begin
        fmSatEph.MemoEph.Visible:=False;
        fmSatEph.MemoEph.Enabled:=False;
        fmSatEph.ButGraph.Caption:='';
        fmSatEph.ButGraph.Hint:=' ';
        fmSatEph.ForDiagram ;
        fmSatEph.BooPaint:=False;
      end
    else
      begin
        fmSatEph.MemoEph.Enabled:=True;
        fmSatEph.MemoEph.Visible:=True;
        fmSatEph.ButGraph.Caption:='';
        fmSatEph.ButGraph.Hint:=' ';
      end;
end;

procedure TfmSatEph.ButEscClick(Sender: TObject);
begin { to close form }
  fmSatEph.Close;
end;

procedure TfmSatEph.ButOtherClick(Sender: TObject);
begin { to close form }
  fmSatEph.BooOther:=True;
  fmSatEph.Close ;
end;

{ if new file to write or append
  the contents of MemoEph.Lines  sf name of file }
procedure TfmSatEph.WriteAppEnd ( sf  : ShortString ) ;
var
  ft  : TextFile ;   // our file with text
  ns  : Integer ;
begin
  AssignFile(ft,sf); // our text file
  if  FileExists(sf) // function SysUtils
    then             // file exists already
      AppEnd(ft)     // to write to the end of file
    else
      ReWrite(ft);   // to open new file
  for ns:=0 to fmSatEph.MemoEph.Lines.Count-1 do
    WriteLn(ft,fmSatEph.MemoEph.Lines[ns]);
  CloseFile(ft);
end;

procedure TfmSatEph.ToSaveGraph ;
var
  BitMap  : TBitMap ; // from Graphics
  dst,sr  : TRect ;   // from Windows
begin
  BitMap:=TBitMap.Create;
  BitMap.Width:=fmSatEph.ClientWidth-4;
  BitMap.Height:=fmSatEph.ButGraph.Top-5;
  sr.Left:=2;
  sr.Top:=2;
  sr.Right:=fmSatEph.ClientWidth-2;
  sr.Bottom:=fmSatEph.ButGraph.Top-3;
  dst.Left:=0;
  dst.Top:=0;
  dst.Right:=BitMap.Width;
  dst.Bottom:=BitMap.Height;
  BitMap.Canvas.CopyRect(dst,fmSatEph.Canvas,sr);
  if  fmPuz.SavePictureDialog.Execute
    then { standard dialog }
      BitMap.SaveToFile(fmPuz.SavePictureDialog.FileName);
  BitMap.Free;
end;

procedure TfmSatEph.ButSaveClick(Sender: TObject);
begin
  if  not BooGraph
    then // for table
      if  fmPuz.SaveDialog.Execute
        then // fmPuz UnitMain
          fmSatEph.WriteAppEnd(fmPuz.SaveDialog.FileName)
        else
    else // for diagram
      fmSatEph.ToSaveGraph;
end;

procedure TfmSatEph.MemoEphKeyPress(Sender: TObject; var Key: Char);
var
  ch  : Char ;
begin
  ch:=UpCase(Key);
  if  ch in [ 'Q' , #27 ]  then  fmSatEph.ButEscClick(Sender);  // exit Esc
  if  ch in [ 'S' , #13 ]  then  fmSatEph.ButSaveClick(Sender); // save Enter
  if  ( ch = 'D' ) and ( CharForEphem <> 'L' )
    then  fmSatEph.ButGraphClick(Sender);
  Key:=#0;
end;

procedure TfmSatEph.FormPaint(Sender: TObject);
begin
  if  BooGraph and BooPaint  then  fmSatEph.ForDiagram;
  BooPaint:=True ;
end;

procedure TfmSatEph.FormKeyPress(Sender: TObject; var Key: Char);
var
  ch  : Char ;
begin
  ch:=UpCase(Key);
  if  ch in [ 'Q' , #27 ]  then  fmSatEph.ButEscClick(Sender);  // exit Esc
  if  ch in [ 'S' , #13 ]  then  fmSatEph.ButSaveClick(Sender); // save Enter
  if  ( ch = 'D' ) and ( CharForEphem <> 'L' )
    then  fmSatEph.ButGraphClick(Sender);
end;

procedure TfmSatEph.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  fmSatEph.BinStream.Free;
  if  not fmSatEph.MemoEph.Enabled
    then { this situation may be in diagram case }
      fmSatEph.MemoEph.Enabled:=True; { always }
  if  CharForEphem in [ 'R' , 'Q' ]
    then
      BmpPaint.Free; { BitMap is created only for select rectangle }
end;

procedure TfmSatEph.FormClick(Sender: TObject);
begin
  with  fmSatEph  do  // procedure from UnGraBmp
     PuzSaveBmp(ClientWidth,ClientHeight,
                Width,Height,3,3,Canvas);
end;

procedure TfmSatEph.MemoEphKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if  fmSatEph.BooGraph or ( Key = Ord('D') )
    then  Exit ; // only for MemoEph component
  if  ( Key = Ord('A') ) and ( Shift = [ ssCtrl ] ) // to select
    then                                            // all in Memo
      fmSatEph.MemoEph.SelectAll ;
  if  ( Key = Ord('C') ) and ( Shift = [ ssCtrl ] ) // to
    then                                            // ClipBoard
      fmSatEph.MemoEph.CopyToClipboard ;
  if  ( Key = Ord('X') ) and ( Shift = [ ssCtrl ] ) // to
    then                                            // Clipboard
      fmSatEph.MemoEph.CutToClipboard ;
  if  ( Key = Ord('V') ) and ( Shift = [ ssCtrl ] ) // from
    then                                            // ClipBoard
      fmSatEph.MemoEph.PasteFromClipboard ;
end;

end.
