unit UnSatMas; // form fmSatMas for four diagrams
               // in case 'W" map world
interface      // to compare visibility conditions

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, ExtCtrls ,
  UnGloVar ; // for TPlaceCooRec MapSatNumber SatElemMap[..] of TElemRec

type
  TfmSatMas = class(TForm)
    mnuSatMas: TMainMenu;
    pbDia: TPaintBox;
    mnuSatMasD: TMenuItem;
    mnuSatMasDs: TMenuItem;
    mnuSatMasDl: TMenuItem;
    mnuSatMasDe: TMenuItem;
    procedure FormActivate(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure mnuSatMasDsClick(Sender: TObject);
    procedure mnuSatMasDeClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private declarations }
    NumBin  : array [1..4] of LongInt ;  // numbers in SatEph.BinStream
    EphOne  : array [1..4] of Extended ; // start  moment
    EphTwo  : array [1..4] of Extended ; // finish moment
    xt,yt   : Integer ;     // for text
    xd,dh   : Integer ;     // for diagram
    btSan   : array [1..4] of Byte ;
    stPan   : array [1..4] of ShortString ;      // for station name
    ptCoo   : array [1..4] of TPlaceCooRec ;     // for each point
    stSan   : array [1..4,1..8] of ShortString ; // for satellite names
    numc    : Byte ;    // current ephemeris
    munx    : Integer ; // unit in pixel x axis
    muny    : Integer ; // unit in pixel y axis
    bood    : Boolean ;  // day or night
    polf    : Array[1..4] of TPoint ; // type from Windows
    tbeg    : Extended ;
    xbeg    : Integer ;
    ybeg    : Integer ;
    xend    : Integer ;
    yend    : Integer ;
    procedure TextPointS ;  // point satellites to canvas
    procedure DiaPlaceGr ;  // place for each diagram
    procedure DiaXAxis ;    // x axis time in day
    procedure DiaYAxis ;    // y axis altitude in degree
    procedure DiaDates ;    // start and finish dates
    function  DiaTwiLi ( t  : Extended ) : Boolean ;
    procedure DiaLimit ( tc  : Extended ;  var  t1 , t2 : Extended ) ;
    procedure DiaNAxis ( t1 , t2 : Extended ) ;
    procedure DiaNight ;    // day twiligth night band
    procedure DiaSatel ;    // for satellites altitude
    procedure AllDiagram ;  // for NumEph diagrams
  public
    { Public declarations }
    NumEph  : Byte ;         // number of ephemeris case  nullo in UnForBut
    NumPnt  : LongInt ;      // count for number of point
    procedure NulloNumbers ; // from UnForBut
    procedure PassForPoint ; // from UnForBut
  end;

var
  fmSatMas: TfmSatMas;

implementation

{$R *.DFM}

uses
  UnitMain , // for fmPuz
  UnConTyp , // for constants
  UnCoTVar , // for type TElemRec
  UnForCoo , // for proc ClcThreeRotMatr
  UnForDat , // for function StringFromDate
  UnForFun , // function MatrMultVector
  UnForPos , // function chDayNight
  UnGraBmp , // for PuzSaveBmp procedure
  UnitEilP , // for proc ElemX
  UnitShad , // for function ToGetShadowCase
  UnSatEph , // for fmSatEph.BinStream of TMemoryStream
  UnSatPas ; // for record EphAll of type TEphAll

procedure TfmSatMas.NulloNumbers ;
var
  i  : Byte ;
begin
  NumEph:=0;
  for i:=1 to 4 do  NumBin[i]:=0; // initial numbers in BinStream
end;

procedure TfmSatMas.PassForPoint ;
var
  cr  : TCursor ; // Controls
  nm  : Byte ;
  tn  : Extended ;
  tk  : Extended ;
  dt  : Extended ;
  pc  : TPlaceCooRec ; // UnGloVar
  ps  : TVect3 ;       // type from UnConTyp
  el  : TElemRec ;     // UnCoTVar
  xe  : TVect3 ;       // position of the satellite in km in true equator
  xs  : TVect3 ;       // satellite refer to the station  in true equator
  xp  : TVect3 ;       // satellite topocentric horizont position
begin // EphAll  tc az al ro
  cr:=fmPuz.Cursor;
  fmPuz.Cursor:=crHourGlass;
  tn:=Int(JulianDate+0.5)-0.5; // JulianDate UnGloVar
  tk:=tn+3.0;
  dt:=1.0/1440.0;   // step 1 minute
  pc:=PlaceCoor;    // of TPlaceCooRec from UnGloVar assigned in UnitStat
  EphAll.tc:=tn-dt; // record from UnSatPas
  NumPnt:=0;        // count for number of point of visibility
repeat
  EphAll.tc:=EphAll.tc+dt;
  ClcThreeRotMatr(JD2000,EphAll.tc, // UnConTyp UnGloVar for StationPos true
                  pc,RotMatr,RosMatr,TopMatr); // UnForCoo  equator
  ps:=StationPos; // StationPos of TVect3 UnGloVar in true equator position
  for nm:=1 to MapSatNumber do  // UnGloVar
    begin // current moment for all satellites
      el:=SatElemMap[nm];     // current system of elements
      ElemX(EphAll.tc,el,xe); // unit UnitEilP refer to the Earth
      xs[1]:=xe[1]-ps[1]; // in true equator system
      xs[2]:=xe[2]-ps[2]; // topocentric satellite position
      xs[3]:=xe[3]-ps[3]; // in true equator system
      MatrMultVector(TopMatr,xs,xp); // to topocentric horizon UnForFun
      ClcTopCoorInDegree(xp,         // UnForCoo with refraction corr.
                         EphAll.az,EphAll.al,EphAll.ro);
      if  EphAll.al > 0.0
        then     // for altitude in degree
          begin  // this satellite above horizon
            EphAll.shadow:=ToGetShadowCase(EphAll.tc,  // from UnitShad
                                           xe);        // for geocentric pos
            fmSatEph.BinStream.Write(EphAll,41);       // current passage
            NumPnt:=NumPnt+1;                          // next visibility
          end;
    end;
until  EphAll.tc > tk;
  if  NumPnt > 0
    then
      begin                     // this is current ephemeris
        NumEph:=NumEph+1;       // number of epheneris
        NumBin[NumEph]:=NumPnt; // number of records in BinStream
        EphOne[NumEph]:=tn-1;     // start  moment
        EphTwo[NumEph]:=tk+1;     // finish moment
        stPan[NumEph]:=pc.name;       // name of station in this case
        ptCoo[NumEph]:=pc;            // station position in this case
        btSan[NumEph]:=MapSatNumber;  // number fo satellite in this case
        for nm:=1 to btSan[NumEph] do
          stSan[NumEph,nm]:=SatElemMap[nm].satname;
      end;
  fmPuz.Cursor:=cr;
end;

procedure TfmSatMas.TextPointS ;
var
  m  : Byte ;
  y  : Integer ;
begin
  y:=(dh-8) div 9 ;
  with  pbDia  do
    begin
      Canvas.Brush.Color:=clLime;
      Canvas.Font.Color:=clRed;
      Canvas.TextOut(xt,yt,stPan[numc]);
      Canvas.Font.Color:=clBlack;
      for m:=1 to btSan[numc] do
        Canvas.TextOut(xt,yt+m*y,stSan[numc,m]);
    end;
end;

procedure TfmSatMas.DiaPlaceGr ;
begin
  with  pbDia  do
    begin
      Canvas.Pen.Color:=clWhite;
      Canvas.Brush.Color:=clAqua;
      Canvas.Rectangle(xd,yt,Width-4,yt+(dh-8));
    end;
end;

procedure TfmSatMas.DiaXAxis ;
var
  nx,kx,ky  : Integer ;
begin
  nx:=Round(EphTwo[numc]-EphOne[numc]);   // integer for big interval
  munx:=((pbDia.Width-4)-(xd+15)) div nx; // unit for one day
  xbeg:=xd+10;
  xend:=(pbDia.Width-4)-5;
  ybeg:=yt+(dh-8)-8;
  with  pbDia  do
    begin
      Canvas.Brush.Color:=clAqua;
      Canvas.Pen.Color:=clBlack;
      Canvas.MoveTo(xbeg,ybeg);
      Canvas.LineTo(xend,ybeg);
      kx:=xbeg;
      ky:=ybeg;
      while  kx < xend  do
        begin
          Canvas.MoveTo(kx,ky);
          Canvas.LineTo(kx,ky+5);
          kx:=kx+munx;
        end;
    end;
end;

procedure TfmSatMas.DiaYAxis ;
var
  kx,ky  : Integer ;
begin
  xbeg:=xd+10;
  ybeg:=yt+(dh-8)-8;
  yend:=yt+15;
  muny:=(ybeg-yend) div 9;
  yend:=ybeg-9*muny-1;
  with  pbDia  do
    begin
      Canvas.Brush.Color:=clAqua;
      Canvas.Pen.Color:=clBlack;
      Canvas.MoveTo(xbeg,ybeg);
      Canvas.LineTo(xbeg,yend);
      kx:=xbeg;
      ky:=ybeg;
      while  ky > yend  do
        begin
          Canvas.MoveTo(kx,ky);
          Canvas.LineTo(kx-5,ky);
          ky:=ky-muny;
        end;
    end;
end;

procedure TfmSatMas.DiaDates ;
var
  kx,ky  : Integer ;
  sa,sb  : ShortString ;
begin
  kx:=xbeg+munx;
  ky:=yt+4;
  sa:=Copy(StringFromDate(EphOne[numc]+1),2,10); // function form UnForDat
  sb:=Copy(StringFromDate(EphTwo[numc]-1),2,10);
  with  pbDia  do
    begin
      Canvas.Brush.Color:=clAqua;
      Canvas.Font.Color:=clNavy;
      Canvas.TextOut(kx,ky,sa);
      Canvas.TextOut(kx+3*munx,ky,sb);
    end;
end;

function TfmSatMas.DiaTwiLi ( t  : Extended ) : Boolean ;
var
  c  : Char ; // 'N' night 'T' twilight 'D' day
begin // ClcThreeRotMatr from UnForCoo  RotMatr from UnGloVar
  ClcThreeRotMatr(JD2000,t,ptCoo[numc],RotMatr,RosMatr,TopMatr);
  c:=chDayOrNight(t); // UnForCoo and UnForPos
  if  ( ( not bood ) and ( c in [ 'T' , 'N' ] ) ) // twilight
       or ( bood and ( c = 'N' ) )                   // night
    then
      DiaTwiLi:=True
    else
      DiaTwiLi:=False; // false for day
end;

procedure TfmSatMas.DiaLimit ( tc  : Extended ;  var  t1 , t2 : Extended ) ;
var
  t      : Extended ;
  dt,tm  : Extended ;
  b1,b2  : Boolean ;
begin
  t:=tc; // initial moment in midday on this station
  tm:=tc+1;
  t1:=t+1.0e-3;  // for very simple case
  t2:=t;         // allways day
  dt:=0.1/24.0; // step in day
  b2:=DiaTwiLi(t); // allways false for day
  if  b2 // twilight or night without rise or set of the Sun
    then
      begin
        t1:=t+1.0e-3;  // for very simple case
        t2:=tm-1.0e-3; // always night or twilight
        Exit; // always night or twilight during this date
      end;
  while  t < tm  do
    begin // may be rise or set
      t:=t+dt;
      b1:=DiaTwiLi(t);  // false for day
      if  b1 <> b2
        then
          begin
            if  b2 = False
              then
                t1:=t
              else
                t2:=t;
            b2:=b1;
          end;
    end;
end;

procedure TfmSatMas.DiaNAxis ( t1 , t2 : Extended ) ;
var
  x1,x2  : Integer ;
begin
  x1:=xbeg+Round(munx*(t1-tbeg));
  x2:=xbeg+Round(munx*(t2-tbeg));
  if  x1 = x2  then  Exit; // always the day
  polf[1].x:=x1;
  polf[1].y:=yend+2;
  polf[2].x:=x1;
  polf[2].y:=ybeg-2;
  polf[3].x:=x2;
  polf[3].y:=ybeg-2;
  polf[4].x:=x2;
  polf[4].y:=yend+2;
  if  x2 < xend-2
    then
      pbDia.Canvas.Polygon(polf);
end;

procedure TfmSatMas.DiaNight ; // band for night twiligth
var
  t1,t2  : Extended ;
  tcur   : Extended ;
  cold   : TColor ;   // Graphics
begin
  tbeg:=EphOne[numc];
  bood:=False; // for twilight
repeat
  if  bood
    then { night }
      cold:=clBlack
    else { twilight }
      cold:=clBlue;
  pbDia.Canvas.Brush.Color:=cold;
  pbDia.Canvas.Pen.Color:=cold;
  tcur:=tbeg+(12.0-ptCoo[numc].l/15.0)/24.0; // midday for point
  if  tcur < EphOne[numc]  then  tcur:=tcur+1;
  while  tcur < EphTwo[numc]  do
    begin
      DiaLimit(tcur,t1,t2);
      DiaNAxis(t1,t2);
      tcur:=tcur+1; { one day forward }
    end;
  bood:=not bood;
until  not bood ;
end;

procedure TfmSatMas.DiaSatel ; // for satellites altitude
var
  n  : LongInt ;
  x  : Integer ;
  y  : Integer ;
  r  : Integer ;
begin
  for n:=1 to NumBin[numc] do
    begin
      fmSatEph.BinStream.Read(EphAll,41);
      with  EphAll  do
        begin
          x:=xbeg+Round(munx*(tc-tbeg));
          y:=ybeg-Round(0.1*muny*al);
          if  ( 20.0 < al) and ( al <= 30.0 ) // altitude greater than 20
            then
              r:=1
            else
              if  al > 30.0 // high altitude gt 30 in degree
                then
                  r:=2
                else
                  r:=1; // for low passage less than 20 in degree
          if  shadow = 0
            then // satellite is lightning
              begin
                pbDia.Canvas.Brush.Color:=clYellow;
                pbDia.Canvas.Pen.Color:=clYellow;
              end
            else // shadow or semishadow
              begin
                pbDia.Canvas.Brush.Color:=clRed;
                pbDia.Canvas.Pen.Color:=clRed;
              end;
          pbDia.Canvas.Ellipse(x-r,y-r,x+r,y+r);
        end; // with EphhAll
    end; // for current ephemeris
end;

procedure TfmSatMas.AllDiagram ;
var
  n  : Byte ;
begin
  fmSatEph.BinStream.Position:=0; // to the begining
  for n:=1 to NumEph do
    begin // current ephemeris
      numc:=n;
      TextPointS ;  // name points satellites to canvas
      DiaPlaceGr ;  // place for current diagram
      DiaXAxis ;    // scale for x axis time
      DiaYAxis ;    // scale for y axis altitude
      DiaDates ;    // start finish dates to canvas
      DiaNight ;    // for day twilight night rectangle
      DiaSatel ;    // for satellite points with altitude
      yt:=yt+dh;
    end;
end;

procedure TfmSatMas.FormActivate(Sender: TObject);
begin             { boomaif global var from UnGloVar }
  boomaif:=False; { Form with the help is active but no main form }
  fmSatMas.Left:=10;
  fmSatMas.Top:=10;
  fmSatMas.ClientWidth:= Screen.Width div 2;
  fmSatMas.ClientHeight:= NumEph * (( Screen.Height - 100 ) div 4 ) ;
  pbDia.Left:=4;
  pbDia.Top:=4;
  pbDia.Width:=fmSatMas.ClientWidth-8;
  pbDia.Height:=fmSatMas.ClientHeight-8;
  dh:=pbDia.Height div NumEph; // step in height for one ephemeris
  xt:=4; // in PaintBox
  yt:=4; // in PaintBox
  xd:=pbDia.Width div 4;  // in PaintBox
end;

procedure TfmSatMas.FormPaint(Sender: TObject);
begin
  with  pbDia  do
    begin
      Canvas.Brush.Color:=clLime;
      Canvas.Pen.Color:=clYellow;
      Canvas.Rectangle(0,0,Width,Height);
    end;
  AllDiagram ;
end;

procedure TfmSatMas.mnuSatMasDsClick(Sender: TObject);
begin
with  fmSatMas  do  // procedure from UnGraBmp
  PuzSaveBmp(ClientWidth,ClientHeight,Width,Height,3,3,Canvas);
end;

procedure TfmSatMas.mnuSatMasDeClick(Sender: TObject);
begin
  Close;
end;

procedure TfmSatMas.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if  ( Key = Ord('S') ) and ( Shift = [ ssCtrl ] )
    then  mnuSatMasDsClick(Sender);
  if  ( Key = Ord('X') ) and ( Shift = [ ssAlt ] )
    then  mnuSatMasDeClick(Sender);
end;

end.
