unit UnSatPas ;

interface

uses
  UnCoTVar , // type TElemRec
  UnGloVar ; // type TPlaceCooRec

type
  TRisCul  = record   // for moment rise culmination set of satellite
               tr,tc,ts  : Extended ;
             end;     // size 3 * 10 = 30 byte
  TEphAll  = record   // for any satellite that are visible from point
               tc        : Extended ; // moment in julian day
               az,al,ro  : Extended ; // azimuth altitude in degree
               shadow    : Byte ;     // range in km shadow case 2 1 0
             end;     // size 4 * 10 + 1 = 41 byte

var
  RisCul   : TRisCul ; // for moment rise culmination set of satellite
  EphAll   : TEphAll ; // for any satellite that are visible from point

procedure SaTopSpher ( tc  : Extended ; // moment in julian date
                       el  : TElemRec ; // record for elements UnCoTVar
                       pc  : TPlaceCooRec ; // record for site UnGloVar
                       var  shadow  : Byte ;  // shadow case 2 1 0
                       var  az , al , ro  : Extended ) ; // topocentric

procedure AboutSatellite ( el  : TElemRec ) ;

procedure ToFindPassage ; // CharForEphem = 'P'  sat from stat

implementation

uses
  SysUtils , // function IntToStr
  Classes  , // for proc WriteBuffer
  UnSelEph , // for variables from class fmSelEph ( BooVisEph )
  UnSatEph , // for variables from class fmSatEph  component MemoEph
  UnConTyp , // constant and type TVect3
  UnForStr , // function FloatToFixStr
  UnForDat , // function StringFromDate
  UnForSat , // for proc PosTopSat
  UnForPos , // for function strDayOrNight chDayOrNight
  UnForCoo , // for proc ClcTopCoorInDegree with refraction correction
  UnitShad , // function ToGetShadowCase
  UnForFun ; // for function SquareInterPol

procedure SaTopSpher ( tc  : Extended ; // moment in julian date
                       el  : TElemRec ; // record for elements UnCoTVar
                       pc  : TPlaceCooRec ; // record for site UnGloVar
                       var  shadow  : Byte ; // shadow case
                       var  az , al , ro  : Extended ) ; // topocentric
var // azimut altitude in degree range in kilimeter with refraction
  xe  : TVect3 ; //  geocentric satellite position true equator
  xs  : TVect3 ; // topocentric satellite position type from UnConTyp
begin // in PosTopSat ClcThreeRotMatr is called with value StationPos
  GeoTopSat(tc,fmSelEph.SatElemEph,fmSelEph.SiteCooEph,xe,xs); // UnForSat
  shadow:=ToGetShadowCase(tc,xe);  // UnitShad for geocentric pos
  ClcTopCoorInDegree(xs,az,al,ro); // UnForCoo with refraction correction
end; // byte 2 shadow 1 semishadow 0 lightning

procedure AboutSatellite ( el  : TElemRec ) ;
begin
with  fmSatEph.MemoEph.Lines  do
begin
  Add('  '+IntToStr(el.satnuml)+'  '+el.satname+'  ');
  Add('  '); // functions from SysUtils UnForStr UnForDate
  Add('  '+FloatToFixStr(el.t-DifEpoch,14,8)+'  '+StringFromDate(el.t));
  Add('     '+FloatToFixStr(el.an,11,8)+'   ');
  Add('      '+FloatToFixStr(el.ae,9,7)+'   ');
  Add('    '+FloatToFixStr(el.ai,9,5)+'      ');  // UnForStr
  Add('  ');
end;
end;

function strDayNight ( ch  : Char ) : ShortString ;
begin
case  ch  of
  'D' : strDayNight:='  ';
  'T' : strDayNight:='  ';
  'N' : strDayNight:='  ';
end; // case character
end;

function strShadowCase ( b  : Byte ) : ShortString ;
begin
case  b  of
   2  :  strShadowCase:='  ' ;
   1  :  strShadowCase:='  ' ;
   0  :  strShadowCase:='  ';
end; // case byte
end;

procedure PasCulMemo ( t  : Extended ;  s  : ShortString ) ;
var
  c  : Char ;         // character for day or night
  b  : Byte ;         // 2 shadow 1 semishadow 0 lightning
  z,h,r  : Extended ; // topocentric satellite position
begin
  SaTopSpher(t,fmSelEph.SatElemEph,fmSelEph.SiteCooEph,b,z,h,r);
  c:=chDayOrNight(t); // this may be called after ClcThreeRotMatr proc
  with  fmSatEph.MemoEph.Lines  do
    Add(ToGetSimpleDate(t)           // date
       +'    '+FloatToFixStr(z,7,3)   // azimuth degree
       +'    '+FloatToFixStr(h,6,3)   // altitude degree
       +strDayNight(c)+s  // day or night rise culmination or set
       +strShadowCase(b));    // shadow case
end;

procedure AboutPasCulm ( tr , ts , tc  : Extended ) ;
begin
  Inc(fmSatEph.NumPoint);
  fmSatEph.MemoEph.Lines.Add('  ');
  fmSatEph.MemoEph.Lines.Add
  ('   '
          +IntToStr(fmSatEph.NumPoint)  // SysUtils
          +'     '
          +FloatToFixStr(1440.0*(ts-tr),8,3)+' ');
  PasCulMemo(tr,' ');
  PasCulMemo(tc,' ');
  PasCulMemo(ts,' ');
end;

procedure AboutPassage ( tt  : Extended ;
                         bh  : Byte ;
                         az , al , ro  : Extended ) ;
var
  ch  : Char ;
  sr  : ShortString ; // string for range
begin
  ch:=chDayOrNight(tt);
  if  ( not fmSelEph.BooDayEph ) and ( ch = 'D' )  then  Exit ; // this is day
  Str(ro:10:3,sr); // range with simple format to string
  with  fmSatEph.MemoEph.Lines  do
    Add(ToGetSimpleDate(tt)            // UnForDat
        +'     '+FloatToFixStr(az,8,4) // UnForStr
        +'    '+FloatToFixStr(al,7,4)  // azimuth altitude
        +'   '+sr                      // range in km
        +strDayNight(ch)               // day night twilight
        +strShadowCase(bh));           // may be shadow UnForPos
  Inc(fmSatEph.NumPoint);              // UnSatEph
end;

procedure StepForStart ( var  tc  : Extended ; // start to find passage
                         var  az , al , ro  : Extended ) ;
var
  b  : Byte ;
begin
  SaTopSpher(tc,fmSelEph.SatElemEph,fmSelEph.SiteCooEph,b,az,al,ro); // here
  if  al > 0.0 // altitude in degree
    then // initial moment but satellite is above horizon
      begin
        tc:=tc-0.4*fmSelEph.SatPeriod; // one step backward
        SaTopSpher(tc,fmSelEph.SatElemEph,fmSelEph.SiteCooEph,b,az,al,ro);
      end;
end;

procedure ToCorrectMoment ( dt  : Extended ;  var  tt  : Extended ) ;
var
  md  : LongInt ;
  mc  : LongInt ;
begin // for correction moment with stepprint
  mc:=Round(86400.0*dt); // stepprint to integer second
  md:=mc*Round(Frac(tt)/dt); // in second with stepprint
  tt:=Int(tt)+md/86400.0;   // new moment
end;

function ToFindCulm ( tr , ts  : Extended ) : Extended ;
var // interpolation by using square polinom
  b      : Byte ;        // for shadow case 2 shadow 1 0
  kt,kc  : Byte ;        // counts for try
  tc,dt  : Extended ;    // date and half of interval for interpolation
  az,ro  : Extended ;    // current azimuth range
  t0,t1,t2  : Extended ; // three date for interpolation
  h0,h1,h2  : Extended ; // three value of altitude for interpolation
begin
  tc:=0.5*(tr+ts); // approximately moment of culmination
  dt:=0.2*(ts-tr); // initial half of interval around approx value
  t0:=tc;
  kt:=1; // a first try with approximately value as half of interval
  kc:=0;
repeat
  t1:=t0-dt;
  t2:=t0+dt;
  if  t1 < tr  then  t1:=tr;
  if  t2 > ts  then  t2:=ts;
  SaTopSpher(t1,fmSelEph.SatElemEph,fmSelEph.SiteCooEph,b,az,h1,ro);
  SaTopSpher(t0,fmSelEph.SatElemEph,fmSelEph.SiteCooEph,b,az,h0,ro);
  SaTopSpher(t2,fmSelEph.SatElemEph,fmSelEph.SiteCooEph,b,az,h2,ro);
  if  ( h1 > h0 ) or ( h2 > h0 )
    then // to prevent strange case in interpolation
      begin
        if  ( h1 > h0 )
          then
            begin
              t0:=t1;
              if  kc < 2  then  kc:=kc+1  else  kc:=0;
            end;
        if  h2 > h0
          then
            begin
              t0:=t2;
              if  kc < 2  then  kc:=kc+1  else  kc:=0;
            end
      end
    else
      kc:=0;
  if  kc = 0
    then
      begin
        tc:=SquareInterPol(t1,t0,t2,h1,h0,h2); // simple square polinom
        t0:=tc; // current value for culmination moment
        kt:=kt+1;   // the next try to find culmination
        dt:=0.1*dt; // to compress interval
      end;
until  kt = 4 ;
  ToFindCulm:=tc;
end;

procedure ForEphCapl ;
begin
with  fmSatEph.MemoEph.Lines  do
  begin
    Add('   ');
    Add('  ');
    Add('       I4');
    Add('     I3');
    Add('      I3');
    Add('      I4');
    Add('    I3');
    Add('   F7.3');
    Add('  ()  F11.3');
    Add('  ()  F10.3');
  end;
end;

procedure ForPasCulm ; // to obtain parameters of culmination
begin
  ForEphCapl ;
  while  fmSatEph.BinStream.Position < fmSatEph.BinStream.Size  do
    begin
      fmSatEph.BinStream.Read(RisCul,30); // rise in passage
      with  RisCul  do  AboutPasCulm(tr,ts,tc); // information to MemoEph
    end;
end;

procedure ForEphCapa ;
begin
with  fmSatEph.MemoEph.Lines  do
  begin
    Add('   ');
    Add('  ');
    Add('       I4');
    Add('     I3');
    Add('      I3');
    Add('      I4');
    Add('    I3');
    Add('   F7.3');
    Add('  ()  F13.4');
    Add('  ()  F11.4');
    Add('  ()   F13.3');
    Add('  ');
  end;
end;

procedure ForEphSatl ;   // general calculations
var                      // topocentric spheric positions
  bs        : Byte ;     // for shadow case
  tt        : Extended ; // current moment
  az,al,ro  : Extended ; // topocentric satellite position
begin
  ForEphCapa ;
  while  fmSatEph.BinStream.Position < fmSatEph.BinStream.Size  do
    begin
      fmSatEph.BinStream.Read(RisCul,30); // rise culm set in passage
      tt:=RisCul.tr; // from rise
      ToCorrectMoment(fmSelEph.StepPrint,tt); // to equal with step
      repeat
        SaTopSpher(tt,fmSelEph.SatElemEph,fmSelEph.SiteCooEph,bs,az,al,ro);
        if  al > fmSelEph.LatMinVal
          then
            AboutPassage(tt,bs,az,al,ro); // information to MemoEph
        tt:=tt+fmSelEph.StepPrint;
      until  tt > RisCul.ts ; // terminal moment in julian day set moment
    end;
end;

procedure ForEphCapr ; // format for rectangular positions
begin
with  fmSatEph.MemoEph.Lines  do
  begin
    Add('  ');
    Add('   ');
    Add('  ');
    Add('       I4');
    Add('     I3');
    Add('      I3');
    Add('      I4');
    Add('    I3');
    Add('   F7.3');
    Add(' x ()   F13.3');
    Add(' y ()   F13.3');
    Add(' z ()   F13.3');
    Add('  ');
  end;
end;

procedure AboutTopRect ( tt  : Extended ;
                         bh  : Byte ;
                         xs  : TVect3 ) ;
var
  ch  : Char ;
  sx  : ShortString ;
  st  : ShortString ;
begin
  ch:=chDayOrNight(tt);  // char function from UnForPos
  Str(xs[1]:11:3,sx);
  st:='  '+sx;
  Str(xs[2]:11:3,sx);
  st:=st+'  '+sx;
  Str(xs[3]:11:3,sx);
  st:=st+'  '+sx;
  if  ( not fmSelEph.BooDayEph ) and ( ch = 'D' )  then  Exit ; // this is day
  with  fmSatEph.MemoEph.Lines  do  // string functions UnForPos
    Add(ToGetSimpleDate(tt)+st+strDayNight(ch)+strShadowCase(bh));
  Inc(fmSatEph.NumPoint);  // date to string UnForDat   UnSatEph
end;

procedure ForEphSatr ;   // rectangular positions
var
  bs  : Byte ;     // for shadow case
  tt  : Extended ; // current moment
  xe  : TVect3 ; // geocentric  satellite positions
  xs  : TVect3 ; // topocentric satellite positions
begin
  ForEphCapr ;
  fmSatEph.BinStream.Seek(0,soFromBeginning); // Classes
  fmSatEph.NumPoint:=0; // number of ephemeris points
  while  fmSatEph.BinStream.Position < fmSatEph.BinStream.Size  do
    begin
      fmSatEph.BinStream.Read(RisCul,30); // rise culm set in passage
      tt:=RisCul.tr; // from rise
      ToCorrectMoment(fmSelEph.StepPrint,tt); // to equal with step
      repeat
        GeoTopSat(tt,fmSelEph.SatElemEph,
                     fmSelEph.SiteCooEph,xe,xs); // UnForSat
        bs:=ToGetShadowCase(tt,xe);  // UnitShad for geocentric pos
        AboutTopRect(tt,bs,xs); // information to MemoEph
        tt:=tt+fmSelEph.StepPrint;
      until  tt > RisCul.ts ; // terminal moment in julian day set moment
    end;
end;

function rsCase ( a0 , t1 , a1 , t2 , a2  : Extended ) : Extended ;
var
  bh  : Byte ; // shadow case
  d1,d2,b1,b2,db,dd,az,ro  : Extended ;
begin // to evaluate rise or set moment of the satellite
  d1:=t1; // it is possible to find
  b1:=a1; // a solution between t1 and t2 in julian day
  d2:=t2; // a0 is minimum altitude in degree to find rise and set
  b2:=a2;
  dd:=d2;
  db:=b2-b1;
  while  ( Abs(db) > 1.0e-3 )  do
    begin
      dd:=d1+((a0-b1)/db)*(d2-d1); // linear interpolation for moment
      d1:=d2;
      b1:=b2;
      d2:=dd;
      SaTopSpher(d2,fmSelEph.SatElemEph,fmSelEph.SiteCooEph,bh,az,b2,ro);
      db:=b2-b1;
    end;
  rsCase:=dd;
end;

procedure ForStacSat ; // special case for geosynchronous satellite
var // only simulation
  tt,dt  : Extended ; // current moment and step in day
begin // tr rise tc culm ts set simulation
  tt:=fmSelEph.StartDate;  // initial moment in julian day
  dt:=0.05; // step in day for geosynchronous and strange satellites
repeat
  RisCul.tr:=tt;
  RisCul.tc:=RisCul.tr+dt;
  RisCul.ts:=RisCul.tc+dt;
  fmSatEph.BinStream.Write(RisCul,30); // to simulate rise culm set event
  tt:=RisCul.ts;
until  tt > fmSelEph.FinishDate;
end;

procedure ForRiseSet ;
var
  tt,dt  : Extended ; // current moment and step in day
  az,ro  : Extended ; // topocentric satellite position
  t1,t2  : Extended ;
  a1,a2,a0  : Extended ; // current altitude and minimum altitude
  bh  : Byte ; // 2 shadow 1 semishadow 0
  bo  : Boolean ;
begin
  a0:=fmSelEph.LatMinVal; // minimum altitude in degree
  dt:=0.01*fmSelEph.SatPeriod ; // period of revolituon in day step in day
  tt:=fmSelEph.StartDate;  // initial moment in julian day
  StepForStart(tt,az,a2,ro); // now altitude a2 < 0 at the moment t2
  t2:=tt; // initial altitude a2 for t2
  bo:=True; // to find rise
repeat
  tt:=tt+dt;
  t1:=t2;
  a1:=a2;
  t2:=tt;
  SaTopSpher(t2,fmSelEph.SatElemEph,fmSelEph.SiteCooEph,bh,az,a2,ro);
  if  bo
    then // to find rise event
      if  ( a1 < a0 ) and ( a0 < a2 )
        then // case of rise
          begin
            RisCul.tr:=rsCase(a0,t1,a1,t2,a2);  // rise moment
            bo:= not bo ; // from rise to set
          end
        else
    else // to find set event
      if  ( a1 > a0 ) and ( a0 > a2 )
        then
          begin
            RisCul.ts:=rsCase(a0,t1,a1,t2,a2);  // set moment
            RisCul.tc:=ToFindCulm(RisCul.tr,RisCul.ts); // culm
            fmSatEph.BinStream.Write(RisCul,30); // rise culm set event
            bo:= not bo ; // from set to rise
            tt:=tt+0.5*fmSelEph.SatPeriod; // big step forward
            t2:=tt;
            SaTopSpher(t2,fmSelEph.SatElemEph,fmSelEph.SiteCooEph,bh,az,a2,ro);
          end;
until  bo and ( tt > fmSelEph.FinishDate ) ; // terminal moment in julian day
end;

procedure ForDetailEph ;
begin
  fmSatEph.BooPoint:=
  ( fmSelEph.SatPeriod > 0.7 )  // period of revolition in day
    or ( ( fmSelEph.SatPeriod > 0.3 )
           and ( fmSelEph.SatElemEph.ae > 0.2 ) ) ; // eccentricity
  if  fmSatEph.BooPoint
    then           // to fill BinStream of TMemoryStream
      ForStacSat   // for geosynchronous or high ellipticity orbit
    else
      ForRiseSet ; // to write moment of rise and set to BinStream
end ;

procedure ForPointsEph ;
begin
  fmSatEph.BinStream.Seek(0,soFromBeginning); // Classes
  fmSatEph.NumPoint:=0; // number of ephemeris points
  if  fmSatEph.BooPoint or ( not fmSelEph.BooVisEph )
    then
      begin
        ForEphSatl ; // topocentric spherical positions
        if  ( not fmSatEph.BooPoint )
          then
            ForEphSatr ; // rectangular positions
      end
    else
      ForPasCulm ; // may be only ForPasCulm three point to MemoEph
end;

 // CharForEphem = 'P'  sat from stat
procedure ToFindPassage ;   // called by fmSatEph.FormActivate from UnSatEph
begin                       // information to fmSatEph.MemoEph
  fmSatEph.BinStream.Clear; // Classes
  ForDetailEph;
  ForPointsEph;
end;

end.
