unit UnSatRaf; // methods from this class are called by fmSatEph.ForDiagram

interface

uses
  Graphics , // Canvas
  Windows  , // for TPoint
  Classes  ; // soFromBeginning for TSream

type
  TGraS = class

    private

      xbeg,xend  : Integer ;
      ybeg,yend  : Integer ;
      maxx,maxy  : Integer ;
      munx,muny  : Integer ;
      maxt,numd  : Integer ;
      numx,numy  : Integer ;
      day,month  : Integer ;
      year       : Integer ;
      numh,numz  : Integer ;
      tbeg,tend  : Extended ;
      tcur,secp  : Extended ;
      bood       : Boolean ;
      bodh       : Boolean ;
      npof       : Byte ;
      polf       : Array[1..4] of TPoint ; // type from Windows
      potf       : TPoint ;
      procedure CulmPlace;
      procedure CulmXAxis;
      procedure CulmYAxis;
      procedure DegrYAxis;
      function  BooForTwiLight ( t  : Extended ) : Boolean ;
      procedure ToObtainTwoLimit ( var  t1 , t2 : Extended ) ;
      procedure ForOneLimit ( t  : Extended ) ;
      procedure FixTwoLimit ( t1 , t2  : Extended ) ;
      procedure ScreenTwoLimit ( t1 , t2  : Extended ) ;
      procedure ForTwoLimit ( t1 , t2  : Extended ;
                              var  x1 , x2  : Integer ) ;
      procedure XAxisNight ( t1 , t2  : Extended ) ;
      procedure YAxisNight ( t1 , t2  : Extended ) ;
      function  ToGetAltiRad ( al  : Extended ) : Integer ; // size of circle
      procedure PointSatDegr ( tt , al  : Extended ; // in degree scale
                               var  x , y , r  : Integer ) ;
      procedure PointSatHour ( tt , al  : Extended ; // in hour scale long
                               var  x , y , r  : Integer ) ;
      procedure PointForSatl ; // to view from point
      procedure DegrYLats ;    // from -90 to +90 degree
      procedure PointSatLats ( tt , al  : Extended ; // in degree scale
                               var  x , y  : Integer ) ; // short time
      procedure PointForBody ; // to view from satellite
      procedure LimitForNight;

    public

      procedure CulmFigure ; // CharForEphem = 'O' from point
      procedure EventFigur ; // CharForEphem = 'O' from satl

          end; // class TGraS

var
  GraS  : TGraS ;

implementation

uses
  UnConTyp , // for const StrMonth name of month
  UnGloVar , // RotMatr CharForEphem
  UnSatEph , // for fmSatEph.PaintBox
  UnSelEph , // for fmSelEph.StartDate FinishDate in julian day
  UnSatPas , // for RisCul record and some procedures
  UnSatVas , // for procedure ToGetPerpH CharForEphem = 'O'
  UnForDat , // for proc TransJDtoDate
  UnForStr , // for proc IntegerToFixStr
  UnForFun , // for proc MatrMultVector
  UnForCoo , // for proc ClcThreeRotMatr ClcTrueRotMatr
  UnForPos , // for proc chDayOrNight
  UnitEilP ; // for satellite position ElemX proc

procedure TGraS.CulmPlace;
begin
  tbeg:=fmSelEph.StartDate-1.0;
  tend:=fmSelEph.FinishDate+1.0;
  numz:=Round(tend-tbeg); // number of day
  if  numz < 9
    then
      begin
        bodh:=True;
        numh:=9;   // for degree  y axis
        if  CharForEphem = 'O'  // UnGloVar to view from satellite
          then  numh:=18 ;  // from -90 to +90 degree
      end
    else
      begin
        bodh:=False;
        numh:=12 ; // for hour    y axis
        if  CharForEphem = 'O'  // UnGloVar to view from satellite
          then  numh:=18 ;  // from -90 to +90 degree
      end;
  munx:=(fmSatEph.PaintBox.Width-35) div numz; // unit for one day
  xbeg:=25;
  xend:=fmSatEph.PaintBox.Width-10;
  numd:=(numz div 13)+1;
  maxt:=numz div numd;
  maxx:=xend-xbeg;   // length of x axis
  ybeg:=10;
  muny:=(fmSatEph.PaintBox.Height-60) div numh; // unit y axis 24 hour time
  yend:=ybeg+numh*muny;                         // or 90 degree altitude
  maxy:=yend-ybeg;   // length of y axis
  with  fmSatEph.PaintBox  do
    begin
      Canvas.Pen.Color:=clYellow;
      Canvas.Brush.Color:=clLime;
      Canvas.Rectangle(0,0,Width,Height);
      Canvas.Pen.Color:=clGray;
      Canvas.Brush.Color:=clAqua;
      Canvas.Rectangle(xbeg-1,ybeg-1,xend+1,yend+1);
    end;
end;

procedure TGraS.CulmXAxis;
var
  m  : Integer ;
  y  : Integer ;
begin
  m:=0;
  y:=0;
  tcur:=Round(tbeg);
  with  fmSatEph.PaintBox.Canvas  do // UnSatEph
    begin
      Brush.Color:=clLime;
      Pen.Color:=clBlack;
      MoveTo(xbeg,yend);
      LineTo(xend,yend);
      numx:=xbeg;
      while  numx < xend  do
        begin
          TransJDtoDate(tcur,day,month,year,secp); // UnForDat
          MoveTo(numx,yend);
          if  ( ( day mod 5 ) = 0 ) or ( day = 1 )
            then
              LineTo(numx,yend+8)
            else
              LineTo(numx,yend+5);
          TextOut(numx-6,yend+10,IntegerToFixStr(day,2));
          if  month <> m
            then
              begin
                TextOut(numx-10,yend+22,StrMonth[month]); // Graphics UnConTyp
                m:=month;
              end;
          if  year <> y
            then
              begin
                TextOut(numx-6,yend+34,IntegerToFixStr(year,4));
                y:=year;
              end;
          numx:=numx+munx*numd;
          tcur:=tcur+numd;
        end;
    end;
end;

procedure TGraS.DegrYAxis;
var
  n,m,x,y  : Integer ;
begin
  with  fmSatEph.PaintBox.Canvas  do
    begin
      Brush.Color:=clLime;
      Pen.Color:=clBlack;
      MoveTo(xbeg,ybeg);
      LineTo(xbeg,yend);
      x:=xbeg-22;
      for n:=0 to 9 do // 90 degree altitude
        begin
          numy:=yend-muny*n;
          MoveTo(xbeg,numy);
          m:=10*n;
          LineTo(xbeg-5,numy);
          y:=numy-6;
          TextOut(x,y,IntegerToFixStr(m,2)+'');
        end;
    end;
end;

procedure TGraS.CulmYAxis; // time scale on y axis
var
  n,m  : Integer ;
  x,y  : Integer ;
begin
  with  fmSatEph.PaintBox.Canvas  do
    begin
      Brush.Color:=clLime;
      Pen.Color:=clBlack;
      MoveTo(xbeg,ybeg);
      LineTo(xbeg,yend);
      x:=xbeg-22;
      for n:=0 to 12 do // 24 hour one day
        begin
          numy:=yend-muny*n;
          MoveTo(xbeg,numy);
          m:=2*n-numh-Trunc(fmSelEph.SiteCooEph.l/15);
          while  m <= 0  do  m:=24+m;
          if  ( ( m mod 6 ) = 0 )
            then
              LineTo(xbeg-8,numy)
            else
              LineTo(xbeg-5,numy);
          y:=numy-6;
          TextOut(x,y,IntegerToFixStr(m,2));
        end;
    end;
end;

function TGraS.BooForTwiLight ( t  : Extended ) : Boolean ;
var
  c  : Char ; // 'N' night 'T' twilight 'D' day
begin // ClcThreeRotMatr from UnForCoo  RotMatr from UnGloVar
  ClcThreeRotMatr(JD2000,t,fmSelEph.SiteCooEph,RotMatr,RosMatr,TopMatr);
  c:=chDayOrNight(t); // UnForCoo and UnForPos
  if  ( ( not bood ) and ( c in [ 'T' , 'N' ] ) ) // twilight
       or ( bood and ( c = 'N' ) )                   // night
    then
      BooForTwiLight:=True
    else
      BooForTwiLight:=False; // false for day
end;

procedure TGras.ToObtainTwoLimit ( var  t1 , t2 : Extended ) ;
var
  t      : Extended ;
  dt,tm  : Extended ;
  b1,b2  : Boolean ;
begin
  t:=tcur; // initial moment in midday on this station
  tm:=tcur+1;
  t1:=t+1.0e-3;  // for very simple case
  t2:=t;         // allways day
  dt:=0.1/24.0; // step in day
  b2:=BooForTwiLight(t); // allways false for day
  if  b2 // twilight or night without rise or set of the Sun
    then
      begin
        t1:=t+1.0e-3;  // for very simple case
        t2:=tm-1.0e-3; // always night or twilight
        Exit; // always night or twilight during this date
      end;
  while  t < tm  do
    begin // may be rise or set
      t:=t+dt;
      b1:=BooForTwiLight(t);  // false for day
      if  b1 <> b2
        then
          begin
            if  b2 = False
              then
                t1:=t
              else
                t2:=t;
            b2:=b1;
          end;
    end;
end;

procedure TGraS.ForOneLimit ( t  : Extended ) ;
var
  mt  : Integer ;
  dt  : Extended ;
begin
  dt:=24.0*Frac(t-DifEpoch+(12.0+fmSelEph.SiteCooEph.l/15.0)/24.0);
  while  dt > 24.0  do  dt:=dt-24.0;
  mt:=Trunc(dt/2.0);
  numy:=yend-muny*mt-Round(muny*(Frac(dt/2.0)));
  potf.x:=numx;
  potf.y:=numy;
end;

procedure TGraS.ForTwoLimit ( t1 , t2  : Extended ;
                              var  x1 , x2  : Integer ) ;
begin
  x1:=xbeg+Round(munx*(t1-tbeg));
  x2:=xbeg+Round(munx*(t2-tbeg));
end;

procedure TGras.FixTwoLimit ( t1 , t2  : Extended ) ;
begin
  GraS.ForOneLimit(t1); // to obtain potf of TPoint
  npof:=npof+1;
  polf[npof]:=potf;
  GraS.ForOneLimit(t2);
  npof:=npof+1;
  polf[npof]:=potf;
end;

procedure TGras.ScreenTwoLimit ( t1 , t2  : Extended ) ;
begin
  GraS.ForOneLimit(t2); // to obtain potf of TPoint
  npof:=npof+1;
  polf[npof]:=potf;
  GraS.ForOneLimit(t1);
  npof:=npof+1;
  polf[npof]:=potf;
end;

procedure TGraS.XAxisNight ( t1 , t2  : Extended ) ;
begin
  if  npof = 0
    then
      FixTwoLimit(t1,t2)
    else
      begin
        ScreenTwoLimit(t1,t2);
        fmSatEph.PaintBox.Canvas.Polygon(polf);
        polf[1]:=polf[4];
        polf[2]:=polf[3];
        npof:=2;
      end;
end;

procedure TGraS.YAxisNight ( t1 , t2  : Extended ) ;
var
  x1,x2  : Integer ;
begin
  ForTwoLimit(t1,t2,x1,x2);
  if  x1 = x2  then  Exit; // always the day
  polf[1].x:=x1;
  polf[1].y:=yend-2;
  polf[2].x:=x1;
  polf[2].y:=ybeg+2;
  polf[3].x:=x2;
  polf[3].y:=ybeg+2;
  polf[4].x:=x2;
  polf[4].y:=yend-2;
  if  x2 < xend-2
    then
      fmSatEph.PaintBox.Canvas.Polygon(polf);
end;

procedure TGras.LimitForNight ;
var
  t1,t2  : Extended ;
  cold   : TColor ;   // Graphics
begin
  bood:=False; // for twilight
repeat
  if  bood
    then { night }
      cold:=clBlack
    else { twilight }
      cold:=clBlue;
  fmSatEph.PaintBox.Canvas.Brush.Color:=cold;
  fmSatEph.PaintBox.Canvas.Pen.Color:=cold;
  npof:=0; { for polypoints }
  tcur:=tbeg+(12.0-fmSelEph.SiteCooEph.l/15.0)/24.0; // midday for point
  if  tcur < tbeg  then  tcur:=tcur+1;
  numx:=xbeg-munx div 2;
  while  tcur < tend  do
    begin
      numx:=numx+munx;
      ToObtainTwoLimit(t1,t2);
      if  bodh  // degree scale y axis
        then
          YAxisNight(t1,t2)
        else
          XAxisNight(t1,t2);
      tcur:=tcur+1; { one day forward }
    end;
  bood:=not bood;
until  not bood ;
end;

function TGraS.ToGetAltiRad ( al  : Extended ) : Integer ;
var
  r  : Integer ;
begin
  if  ( 20.0 < al) and ( al <= 30.0 ) // altitude greater than 20 in degree
    then
      r:=2
    else
      if  al > 30.0 // high altitude gt 30 in degree
        then
          r:=3
        else
          r:=1; // for low passage less than 20 in degree
  ToGetAltiRad:=r;
end;

procedure TGraS.PointSatDegr ( tt , al  : Extended ; // in degree scale
                               var  x , y , r  : Integer ) ; // short time
begin
  x:=xbeg+Round(munx*(tt-tbeg));
  y:=yend-Round(0.1*muny*al);
  r:=ToGetAltiRad(al);
end;

procedure TGraS.PointSatHour ( tt , al  : Extended ; // in hour scale
                               var  x , y , r  : Integer ) ; // long time
var
  mt  : Integer ;
  dt  : Extended ;
begin
  x:=xbeg+Round(munx*(tt-tbeg)); // time x - axis
  dt:=24.0*Frac(tt-DifEpoch+(12.0+fmSelEph.SiteCooEph.l/15.0)/24.0);
  while  dt > 24.0  do  dt:=dt-24.0;
  mt:=Trunc(dt/2.0);
  y:=yend-muny*mt-Round(muny*(Frac(dt/2.0)));
  r:=ToGetAltiRad(al);
end;

procedure TGraS.PointForSatl ;
var
  bh  : Byte ; // for shadow case
  x,y,r  : Integer ; // center and radius of the circle as satellite
  tt,az,al,ro  : Extended ; // moment azimuth altitude range
begin
  fmSatEph.BinStream.Seek(0,soFromBeginning); // Classes
  while  fmSatEph.BinStream.Position < fmSatEph.BinStream.Size  do
    begin
      fmSatEph.BinStream.Read(RisCul,30); // rise culm set in passage
      tt:=RisCul.tc; // for culmination  SaTopSpher from UnSatPas
      SaTopSpher(tt,fmSelEph.SatElemEph,fmSelEph.SiteCooEph,bh,az,al,ro);
      if  bh = 0
        then // satellite is lightning
          begin
            fmSatEph.PaintBox.Canvas.Brush.Color:=clYellow;
            fmSatEph.PaintBox.Canvas.Pen.Color:=clYellow;
          end
        else // shadow or semishadow
          begin
            fmSatEph.PaintBox.Canvas.Brush.Color:=clRed;
            fmSatEph.PaintBox.Canvas.Pen.Color:=clRed;
          end;
      if  bodh
        then  PointSatDegr(tt,al,x,y,r)  // in degree scale  for xculm yculm
        else  PointSatHour(tt,al,x,y,r); // in hour scale    for xculm yculm
      if  al > fmSelEph.LatMinVal
        then
          fmSatEph.PaintBox.Canvas.Ellipse(x-r,y-r,x+r,y+r);
    end;
end;

procedure TGraS.CulmFigure ; // CharForEphem = 'P' from point
begin
  CulmPlace;
  CulmXAxis;
  if  bodh
    then
      DegrYAxis
    else
      CulmYAxis;
  LimitForNight;
  PointForSatl;
end;

procedure TGraS.DegrYLats ; // from -90 to +90 degree
var
  n,m,x,y  : Integer ;
begin
  with  fmSatEph.PaintBox.Canvas  do
    begin
      Brush.Color:=clLime;
      Pen.Color:=clBlack;
      MoveTo(xbeg,ybeg);
      LineTo(xbeg,yend);
      x:=xbeg-22;
      for n:=0 to 18 do // 90 degree altitude
        begin
          numy:=yend-muny*n;
          MoveTo(xbeg,numy);
          m:=10*(n-9);
          LineTo(xbeg-5,numy);
          y:=numy-6;
          TextOut(x,y,IntegerToFixStr(m,2)+'');
        end;
    end;
end;

procedure TGraS.PointSatLats ( tt , al  : Extended ; // in degree scale
                               var  x , y  : Integer ) ; // short time
begin
  x:=xbeg+Round(munx*(tt-tbeg));
  y:=yend-Round(0.1*muny*(90.0+al)); // from -90 to +90 degree
end;

procedure TGraS.PointForBody ; // to view from satellite
var
  zp,rr  : TVect3 ;  // vectors for satellite true equator and terra
  rm,rs  : TMatr33 ; // matrix to change to rotating system UnForCoo
  x,y,r  : Integer ; // center and radius of the circle as satellite
  al,ap,zr  : Extended ; // longitude atitude range
begin
  fmSatEph.BinStream.Seek(0,soFromBeginning); // Classes
  while  fmSatEph.BinStream.Position < fmSatEph.BinStream.Size  do
    begin
      fmSatEph.BinStream.Read(SerBop,40); // set rise event moment
      { set event on moment SerBop.ta }
      fmSatEph.PaintBox.Canvas.Brush.Color:=clRed; // for set event
      fmSatEph.PaintBox.Canvas.Pen.Color:=clRed;   // for set event
      ElemX(SerBop.ta,fmSelEph.SatElemEph,zp); // satellit UnitEilP
      ClcTrueRotMatr(JD2000,SerBop.ta,rm,rs); // UnForCoo
      MatrMultVector(rs,zp,rr); // to rotating system UnForFun
      ClcSpherCoorInDegree(rr,al,ap,zr);  // satellite spheric
      PointSatLats(SerBop.ta,ap,x,y); // in degree xset yset
      r:=1; // radius of point
      fmSatEph.PaintBox.Canvas.Ellipse(x-r,y-r,x+r,y+r); // set
      PointSatLats(SerBop.ta,SerBop.fa,x,y); // perpendicular
      r:=3; // radius of point                  latitude
      fmSatEph.PaintBox.Canvas.Ellipse(x-r,y-r,x+r,y+r); // set
      { rise event on moment SerBop.tb }
      fmSatEph.PaintBox.Canvas.Brush.Color:=clYellow; // for rise
      fmSatEph.PaintBox.Canvas.Pen.Color:=clYellow;   // for rise
      ElemX(SerBop.tb,fmSelEph.SatElemEph,zp); // satellit UnitEilP
      ClcTrueRotMatr(JD2000,SerBop.tb,rm,rs); // UnForCoo
      MatrMultVector(rs,zp,rr); // to rotating system UnForFun
      ClcSpherCoorInDegree(rr,al,ap,zr);  // satellite spheric
      PointSatLats(SerBop.tb,ap,x,y); // in degree scale xset yset
      r:=1; // radius of point
      fmSatEph.PaintBox.Canvas.Ellipse(x-r,y-r,x+r,y+r); // rise
      PointSatLats(SerBop.tb,SerBop.fb,x,y); // perpendicular
      r:=3; // radius of point                  latitude
      fmSatEph.PaintBox.Canvas.Ellipse(x-r,y-r,x+r,y+r); // rise
    end;
end;

procedure TGras.EventFigur ; // CharForEphem = 'O' from satl
begin
  CulmPlace;
  CulmXAxis;
  DegrYLats;
  PointForBody;
end;

end.
