
unit UnSatVas ;

interface

type
  TSerBop  = record
               ta , tb  : Extended ; // two moment set rise
               fa , fb  : Extended ; // latitude of perpend
             end;

var
  SerBop   : TSerBop ; // to BinStream as result

{ a view from the satellite
    to find events
    when a body may be eclipsed by the Earth }

procedure ToFindEvents ;  // CharForEphem = 'O' body from  sat

implementation

uses
  SysUtils , // function IntToStr
  Classes  , // for proc WriteBuffer
  UnConTyp , // constant and type TVect3
  UnGloVar , // global variables CharBodEph NumberBody
  UnSelEph , // for variables from class fmSelEph ( BooVisEph )
  UnSatEph , // for variables from class fmSatEph  component MemoEph
  UnitEilP , // for procedure ElemX EilXV
  UnitElmK , // for procedure ToObtainOrbitalMatr
  PUnDE200 , // for RadiusOfPlanet const
  UnForPos , // for procedure PosGeoPlanet
  UnForSat , // for function ToGetSatPhase
  UnForCoo , // for procedure ClcTrueRotMatr ClcSpherCoorInDegree
  UnForFun , // for procedure MatrMultVector
  UnForStr , // function FloatToFixStr
  UnForDat ; // function StringFromDate

var
  boevent  : Boolean ;  // body yes or no beyond the Earth
  planetd  : Extended ; // for the Sun the Moon projection range
  sinphip  : Extended ; // sinus of latitude for perpenficular
  perpvec  : TVect3 ;   // vector of perpendicular true equa
  posbods  : TVect3 ;   // body position refer to satellite
  rotmatu  : TMatr33 ;  // to change from fixed to true equator
  rosmatu  : TMatr33 ;  // to change from true to rotating equa

procedure ToClcBodyP ( t  : Extended ;  var  r  : TVect3 ) ;
var
  np  : Byte ;
  re  : TVect3 ;
begin
case  CharBodEph  of  // CharBodEph UnGloVar 'P' or 'S'
  'P' : begin
          np:=NumberBody; // number of selected planet
          PosGeoPlanet(np,t,re); // UnForPos fixed equator
          MatrMultVector(rotmatu,re,r); // from fixed to true equa
        end;
  'S' : ElemX(t,fmSelEph.SaoElemEph,r); // UnitEilP UnSelEph
end; // 'P' or 'S'
end;

{ the elements of the satellite
   in fmSelEph.SatElemEph of TElemRec }
procedure ToClcSatlP ( t  : Extended ;  var  r  : TVect3 ) ;
begin
  ElemX(t,fmSelEph.SatElemEph,r); // from UnitEilP true equator
end;

{ for the perpendicular
  from the centre of the Earth
    to the satellite - body direction
    to obtain height from the surface
    cos(beta) is angle between two directions
    from the satellite to the centre of the Earth
    from the satellite to the body (planet or another satellite)
    GeoR0 radius of the Earth ParmA oblateness const from UnConTyp }

function ToGetPerpH ( tt  : Extended ) : Extended ; // height above
var
  i  : Byte ;         // count
  rs,rb  : TVect3 ;   // satellite body refer to the Earth's centre
  ps,pe  : TVect3 ;   // position refer to satellite
  ze,zp  : Extended ; // range from satellite to the Earth body
  sc,cb  : Extended ; // scalar multiplication cos(beta)
  zr,zq  : Extended ; // range from object to perpendicular ratio
  sp,sf  : Extended ; // range from sat to perpendicular sin(phi)
begin
  ClcTrueRotMatr(JD2000,tt,rotmatu,rosmatu); // UnForCoo  equator
  ToClcBodyP(tt,rb); // body position       refer to the centre
  ToClcSatlP(tt,rs); // satellite position  of the Earth
  for i:=1 to 3 do // to the satellite as a centre
    begin                 // two vectors \vec B \vec E
      ps[i]:=rb[i]-rs[i]; // the selected body or satellite
      pe[i]:=-rs[i];      // the centre of the Earth \vec E
      posbods[i]:=ps[i];  // body position refer to satellite
    end;
  sc:=ScalarMult(pe,ps); // UnForFun scalar multiplication
  ze:=VectorModul(pe);   // UnForFun range satellite - Earth
  zp:=VectorModul(ps);   // UnForFun range satellite - body
  cb:=sc/(ze*zp); // cos(beta) is angle between two directions
  ToGetPerpH:=ze; // for the next case when cos(beta) < 0
  if  ( cb < 0.0 ) or ( zp < ze ) // refer to the satellite
    then // a body may be satellite and may be very closed
      Exit; // the Earth and our body are in different directions
 { length of perpendicular in kilometer from the surface }
  sp:=ze*cb; { range from satellite to perpendicular }
  zr:=zp-sp; { range from object to perpendicular }
  zq:=sp/zr; { the ratio }
  zr:=1.0/(1.0+zq); // for the new vector
  for i:=1 to 3 do // vector perpendicular  true equator system
    perpvec[i]:=zr*(rs[i]+zq*rb[i]); // refer to the centre
  if  CharBodEph = 'P' // NumberBody for the planet
    then // to obtain projection of diameter to perpendicular
      planetd:=2*RadiusOfPlanet[NumberBody]*sp/zp; // projection
  sc:=ze*Sqrt(1-Sqr(cb)); // range of perpendicular from centre
  sf:=perpvec[3]/sc;     // sin(latitude) refer to true equator
  sinphip:=sf;           // sinus of latitude for perpenficular
  ToGetPerpH:=sc-GeoR0*(1-ParmA*Sqr(sf)); // UnConTyp
end;

{ to find any moment when the body is visible }

function FirstMoment : Extended ;
var
  nc  : Byte ; // count satellite body true equator
  tc,dt  : Extended ; // current moment
begin // fmSelEph.StartDate is initial moment in julian day
  tc:=fmSelEph.StartDate;      // the same moment UnSelEph
  dt:=0.01*fmSelEph.SatPeriod; // UnSelEph period in day
  nc:=0;
  while  ( ToGetPerpH(tc) < 10.0 ) and ( nc < 50 )  do
    begin // body may be beyond the Earth
      nc:=nc+1;  // simple count to prevent
      tc:=tc-dt; // one step backward
    end;
  FirstMoment:=tc; // close to fmSelEph.StartDate
end;

{ set event exists
  it is possible to find accurate set moment
  in interval tn < ta < tk }

function ToGetSetM ( tk , dt  : Extended ) : Extended ;
var
  tc,st  : Extended ;
  ha,hb  : Extended ;
begin
  tc:=tk-dt;  // for start to find
  st:=0.01*dt; // new step in day more small
  hb:=ToGetPerpH(tc); // current height
  ha:=hb;             // the same
  while  ( hb > 0.0 )  do // it is condition for set event
    begin
      ha:=hb;    // previous height for interpolation
      tc:=tc+st; // one step forward
      hb:=ToGetPerpH(tc); // new value of height
    end;
  if  ( Abs(hb-ha) > 0.0 )
    then
      ToGetSetM:=tc-hb*st/(hb-ha) // linear intepolation
    else
      ToGetSetM:=tc;
end;

{ rise event exists
  it is possible to find accurate set moment
  in interval tn < tb < tk }

function ToGetRisM ( tk , dt  : Extended ) : Extended ;
var
  tc,st  : Extended ;
  ha,hb  : Extended ;
begin
  tc:=tk-dt;   // for start to find
  st:=0.01*dt; // new step in day more small
  hb:=ToGetPerpH(tc); // current height
  ha:=hb;             // the same
  while  ( hb < 0.0 )  do  // it is condition for rise event
    begin
      ha:=hb;    // previous height for interpolation
      tc:=tc+st; // one step forward
      hb:=ToGetPerpH(tc); // new value of height
    end;
  if  ( Abs(hb-ha) > 0.0 )
    then
      ToGetRisM:=tc-hb*st/(hb-ha) // linear intepolation
    else
      ToGetRisM:=tc;
end;

{ to find two moments if it is possible
  ta or set tb for rise when body beyond the Earth
  RadGra const from UnConTyp  DATan2 function from UnForFun }

procedure CurEvent ( tt  : Extended ) ; // result to SerBop
var
  tc,dt  : Extended ; // current moment step
  bs,br  : Boolean ;  // set and rise event
begin
  boevent:=False ; // default
  tc:=tt;    // current moment the first to find set event
  dt:=0.01*fmSelEph.SatPeriod; // UnSelEph period in day
  bs:=( ToGetPerpH(tc) < 0.0 ); // case for set
  while  ( not bs ) and ( tc < fmSelEph.FinishDate)  do
    begin
      tc:=tc+dt;
      bs:=( ToGetPerpH(tc) < 0.0 ); // case for set
    end;
  if  ( not bs )  then  Exit ; // no events in this case
  { to find moment for set event between two moments }
  SerBop.ta:=ToGetSetM(tc,dt); // there is set event ta moment
  { for pair satellite satellite may be strange case }
  if  ( Abs(ToGetPerpH(SerBop.ta)) > 1.0 )  then  Exit ; // this is
  SerBop.fa:=RadGra*DATan2(sinphip,Sqrt(1-Sqr(sinphip))); // latit.
  tc:=SerBop.ta+dt;              // to continue to find rise
  br:=( ToGetPerpH(tc) > 0.0 ); // case for rise
  while  ( not br ) and ( tc < fmSelEph.FinishDate)  do
    begin
      tc:=tc+dt;
      br:=( ToGetPerpH(tc) > 0.0 ); // case for rise
    end;
  if  ( not br )  then  Exit ; // no events in this case
  { to find moment for rise event between two moments }
  SerBop.tb:=ToGetRisM(tc,dt); // there is rise event tb moment
  { for pair satellite satellite may be strange case }
  if  ( Abs(ToGetPerpH(SerBop.tb)) > 1.0 )  then  Exit ; // this is
  SerBop.fb:=RadGra*DATan2(sinphip,Sqrt(1-Sqr(sinphip))); // latit.
  boevent:=True;  // there are set and rise
end;

{ a try to find some events
  when selected body is beyond the Earth }

procedure PosEvent ;
var
  tt  : Extended ; // current moment
begin
  tt:=FirstMoment; // any moment close to fmSelEph.StartDate
repeat                              // set and rise in day
  CurEvent(tt); // to SerBop two moments ta tb two latitude fa fb
  if  boevent
    then  // result is in record SerBop
      fmSatEph.BinStream.Write(SerBop,40); //
  case  CharBodEph  of  // 'P' or 'S'    the next moment to find
    'P' : tt:=SerBop.ta+0.90*fmSelEph.SatPeriod; // planet case
    'S' : tt:=SerBop.tb+0.01*fmSelEph.SatPeriod; // satellite case
  end; // case CharBodEph 'P' or 'S'
until  ( not boevent ) or ( tt > fmSelEph.FinishDate ) ;
end;

procedure CapEphBody ; // some usefull words about strings
begin
with  fmSatEph.MemoEph.Lines  do
  begin
    Add('     ');
    Add('  ,     ');
    Add('  P -      ');
    Add('  ');
    Add('       I4');
    Add('     I3');
    Add('      I3');
    Add('   ,   F8.1');
    Add('   P      :');
    Add('     P      () F12.1');
    Add('        (.)  F9.3');
    Add('       (.)  F9.3');
    Add('       :');
    Add('          ()   F11.3');
    Add('    .    (.)  F9.3');
    Add('   .    (.)  F9.3');
    Add('       :');
    Add('                    (.)  F9.3');
    Add('             (.)  F9.3');
    Add('  ,      :');
    Add('        ()    F8.3');
    Add('  ,      :');
    Add(' ,      (..)   F7.3');
    Add('    ');
    Add('            F5.1');
    Add('  ');
  end;
end;

{ long of event from ta set to tb rise in second of time }

procedure SetRisL ( ta , tb  : Extended ) ;
var
  dt  : Extended ;    // difference in second of time
  st  : ShortString ; // string for difference
begin
  dt:=86400.0*(tb-ta); // difference in second of time
  Str(dt:18:1,st);
  with  fmSatEph.MemoEph.Lines  do
    begin
      Add('  ');
      Add(st+'     ');
    end;
end;

{ phase magnitude information if it is possible
  to use procedure ClcPlanetMagnitude from UnForPos
     and procedure ToGetSatPhase      from UnForSat }

function StrPhaM ( tc  : Extended ;  zp  : TVect3 ) : ShortString ;
var  // tc current moment  zp satellite position refer to centre
  i   : Byte ;
  np  : Byte ;        // number of the planet
  rs  : TVect3 ;      // the Sun position fixed equator
  ps  : TVect3 ;      // the Sun position true  equator
  vm  : Single ;      // magnitude
  pv  : Single ;      // phase
  ph  : Extended ;    // phase
  sp  : ShortString ; // our string for phase information
  sv  : ShortString ; // our string for magnitude
begin // posbods body position refer to the satellite
  StrPhaM:='';                   // default no information
  PosGeoPlanet(NSun,tc,rs);      // the Sun UnForPos fixed equator
  MatrMultVector(rotmatu,rs,ps); // from fixed to true equator
  for i:=1 to 3 do               // the Sun position true equa
    ps[i]:=ps[i]-zp[i];          // refer to the satellite
  pv:=1.;// default
  vm:=99; // default
  sv:=''; // default
case  CharBodEph  of // 'P' or 'S'
  'P' : begin
          np:=NumberBody; // planet number
          if  ( np in [ NEarth , NSun ] )  then  Exit ; // no
          ClcPlanetMagnitude(np,posbods,ps,vm,ph); // UnForPos
          pv:=ph; // phase
        end;      // function ToGetSatPhase from UnForSat
  'S' : pv:=ToGetSatPhase(posbods,ps); // body Sun refer to satellite
end; // case CharBodEph 'P' or 'S'
  Str(pv:7:3,sp); // phase to string
  if  ( vm < 25.0 )  then  Str(vm:5:1,sv);
  StrPhaM:=sp+sv;
end;

{ to add information to fmSatEph.MemoEph
  procedure ToObtainOritalMatr from UnitElmK }

procedure SetRisO ( nc  : Integer ;  tc , hp  : Extended ) ;
var
  rr  : TVect3 ;      // vector  rectangular
  al  : Extended ;    // longitude in degree
  ap  : Extended ;    // latitude  in degree
  zr  : Extended ;    // range  in kilometre
  zp  : TVect3 ;      // satellite position
  zv  : TVect3 ;      // satellite velosity
  om  : TMatr33 ;     // satellite orbital matrix
  sh  : ShortString ; // string for height
  st  : ShortString ; // string for print
  sp  : ShortString ; // string for phase
begin      // to prevent strange result for satellite - satellite
  if  ( Abs(hp) > fmSelEph.LatMinVal )  then  Exit ; // no inform
  st:=ToGetDateSecP(tc); // UnForDat date with part of day in sec
  Str(1.0e3*hp:12:1,sh);  // height to string in metr
  st:=st+sh;             // our string date and height
  MatrMultVector(rosmatu,perpvec,rr); // to rotating system
  ClcSpherCoorInDegree(rr,al,ap,zr);  // perpendicular spheric
  Str(ap:9:3,sh); // latitude of perpendicular in degree
  st:=st+sh; // latitude of perpendicular in degree to string
  Str(al:9:3,sh); // longitude of perpendicular in degree
  st:=st+sh; // longitude of perpendicular in degree to string
  EilXV(tc,fmSelEph.SatElemEph,zp,zv); // satellite to centre
  MatrMultVector(rosmatu,zp,rr); // to rotating system
  ClcSpherCoorInDegree(rr,al,ap,zr);  // satellite spheric
  Str(zr:11:3,sh); // satellite range from the Earth's centre
  st:=st+sh; // satellite range from the Earth's centre to string
  Str(ap:9:3,sh); // latitude of satellite in degree
  st:=st+sh; // latitude of sateliite in degree to string
  Str(al:9:3,sh); // longitude of satellite in degree
  st:=st+sh; // longitude of satellite in degree to string
  ToObtainOrbitalMatr(zp,zv,om); // for sat orbital matrix UnitEilP
  MatrMultVector(om,posbods,rr); // body to orbital system UnForFun
  ClcOrbCoorInDegree(rr,al,ap,zr); // azimuth zenith distance range
  Str(ap:9:3,sh); // body zenith distance in degree orbital system
  st:=st+sh; // body zenith distance in degree to string
  Str(al:9:3,sh); // body azimuth in degree orbital system
  st:=st+sh; // body azimuth in degree to string
  if  CharBodEph = 'P' // NumberBody for the planet
    then // to print projection of diameter to perpendicular
      begin                  // only for planet
        Str(planetd:8:3,sh); // projection in km
        st:=st+sh;           // to our common string
      end;
  sp:=StrPhaM(tc,zp); // phase magnitude information to string
  st:=st+sp; // plus phase magnitude information
  if  nc > 0
    then
      fmSatEph.MemoEph.Lines.Insert(nc,st) // result to memo
    else
      fmSatEph.MemoEph.Lines.Add(st); // result to memo
end;

{ information about some points around set  event }

procedure SetInfo ( ta  : Extended ) ; // set  event
var
  ic  : Byte ;
  nc  : Integer ;     // count for MemoEph.Lines
  tc  : Extended ;    // current moment
  hp  : Extended ;    // current height in km above surface
begin
  fmSatEph.MemoEph.Lines.Add('  ');
  nc:=fmSatEph.MemoEph.Lines.Count; // number of lines
  ic:=0;                     // simple count for print
  tc:=ta;                    // start moment set moment
repeat                       // backward from start moment
  hp:=ToGetPerpH(tc);        // height of perpendicular in km
  SetRisO(nc,tc,hp); // to add information to fmSatEph.MemoEph
  if  fmSelEph.BooVisEph  then  Exit ; // one moment only
  ic:=ic+1;
  tc:=tc-fmSelEph.StepPrint; // the next step backward
until  ( ic = 20 ) or ( hp > fmSelEph.LatMinVal ) ;
  { some points for heght less than nullo }
  ic:=0;                     // simple count for print
  tc:=ta+fmSelEph.StepPrint; // the next step forward
repeat                       // backward from start moment
  hp:=ToGetPerpH(tc);        // height of perpendicular in km
  SetRisO(0,tc,hp); // to add information to fmSatEph.MemoEph
  ic:=ic+1;
  tc:=tc+fmSelEph.StepPrint; // the next step backward
until  ( ic = 10 ) ;
end;

{ information about some points around rise event }

procedure RisInfo ( tb  : Extended ) ; // rise event
var
  ic  : Byte ;
  nc  : Integer ;     // count for MemoEph.Lines
  tc  : Extended ;    // current moment
  hp  : Extended ;    // current height in km above surface
begin
  nc:=fmSatEph.MemoEph.Lines.Count; // number of lines
  ic:=0;                     // simple count for print
  tc:=tb;                    // start moment set moment
repeat                       // backward from start moment
  hp:=ToGetPerpH(tc);        // height of perpendicular in km
  SetRisO(nc,tc,hp); // to add information to fmSatEph.MemoEph
  if  fmSelEph.BooVisEph  then  Exit ; // one moment only
  ic:=ic+1;
  tc:=tc-fmSelEph.StepPrint; // the next step backward
until  ( ic = 10 ) ;
  { some points for heght less than nullo }
  ic:=0;                     // simple count for print
  tc:=tb+fmSelEph.StepPrint; // the next step forward            
repeat                       // backward from start moment
  hp:=ToGetPerpH(tc);        // height of perpendicular in km
  SetRisO(0,tc,hp); // to add information to fmSatEph.MemoEph
  ic:=ic+1;
  tc:=tc+fmSelEph.StepPrint; // the next step backward
until  ( ic = 20 ) or ( hp > fmSelEph.LatMinVal ) ;
end;

procedure PrtEvent ; // result ro memo
begin
  fmSatEph.BinStream.Seek(0,soFromBeginning);
  fmSatEph.NumPoint:=0; // number of ephemeris points
  CapEphBody ; // some usefull words about strings
  while  fmSatEph.BinStream.Position < fmSatEph.BinStream.Size  do
    begin
      fmSatEph.BinStream.Read(SerBop,40); // set and rise event
      Inc(fmSatEph.NumPoint);
      with  SerBop  do
        begin
          SetRisL(ta,tb); // long of eclipse in second
          SetInfo(ta); // some points around set  event
          RisInfo(tb); // some points around rise event
        end;
    end;
end;

procedure CapSunCase ; // some usefull words about strings
begin
with  fmSatEph.MemoEph.Lines  do
  begin
    Add('  ');
    Add('     ');
    Add('       ');
    Add('  ');
    Add('       I4');
    Add('     I3');
    Add('      I3');
    Add('   ,   F8.1');
    Add('   P      :');
    Add('     P      () F12.1');
    Add('        (.)  F9.3');
    Add('       (.)  F9.3');
    Add('       :');
    Add('          ()   F11.3');
    Add('    .    (.)  F9.3');
    Add('   .    (.)  F9.3');
    Add('       :');
    Add('                 (.)  F9.3');
    Add('          (.)  F9.3');
    Add('  ');
  end;
end;

{ CharForEphem = 'O' body from satellite
  proc is called from UnSatEph proc PassToMemo }

procedure ToFindEvents ;    // called by fmSatEph.FormActivate
begin                       // information to fmSatEph.MemoEph
  fmSatEph.BinStream.Clear; // Classes UnSatEph
  PosEvent ; // when selected body is beyond the Earth
  PrtEvent ; // to print and graph result
end;

end.
