
unit UnSatVau ; // automatic information

interface

{ a view from the satellite  to the limb of the Earth }

procedure ToLimbCourse ;  // CharForEphem = 'L' from  sat to limb of course

implementation

uses
  SysUtils , // function IntToStr
  Classes  , // for proc WriteBuffer
  UnConTyp , // constant and type TVect3
  UnCoTVar , // for all Heap's
  UnGloVar , // global variables CharBodEph NumberBody
  UnSelEph , // for variables from class fmSelEph ( BooVisEph )
  UnSatEph , // for variables from class fmSatEph  component MemoEph
  UnitEilP , // for procedure ElemX EilXV
  UnParamO , // for procedure ParamLimbuP
  UnForSta , // for procedure StarToFixEquator
  PUnDE200 , // for RadiusOfPlanet and some const
  UnForPos , // for procedure PosGeoPlanet
  UnForCoo , // for procedure ClcTrueRotMatr ClcSpherCoorInDegree
  UnForRed , // for procedure ToObtainAberrationParm
  UnForFun , // for procedure MatrMultVector
  UnForStr , // function FloatToFixStr
  UnForDat ; // function StringFromDate

var                    // refer to the satellite
  nulsta  : Integer ;
  nulpla  : Integer ;
  nulsat  : Integer ;
  azlimb  : Extended ; // azimuth  limb orbital system
  ahlimb  : Extended ; // altutude limb orbital system
  rolimb  : Extended ; // range point of limb from sat
  satloo  : TVect3 ;   // true equa satellite position
  zemloo  : TVect3 ;   // the Earth to satl in orbital
  rotmao  : TMatr33 ;  // from fixed to true equator
  rosmao  : TMatr33 ;  // Earth's rotating matrix
  topmao  : TMatr33 ;  // from true equator to orbital system

procedure LimbOutFormat ;  // information about output values
begin
with  fmSatEph.MemoEph.Lines  do
  begin
    Insert(Count-1,'        ');
    Add('      ');
    Add(' 1x,i4,1x,i2,1x,i2,2x,i2,1x,i2,1x,f6.3');
    Add('      ');
    Add('    ');
    Add(' 1x,f10.6,2x,f10.6,2x,f10.3,2x,f10.3');
    Add('   ()   ()');
    Add('     ');
    Add(' 1x,f7.3,2x,f7.3,2x,f13.3');
    Add('    ()  ()');
    Add('      ');
    Add(' 1x,f7.3,2x,f7.3,2x,f13.3');
    Add('   ()  ()');
    Add(' ');
    Add(' 1x,f7.3');
    Add('   ()');
    Add('     ');
    Add(' 1x,f10.6,2x,f10.6,2x,f10.3');
    Add('    ()  ()');
    Add('     ');
    Add(' 1x,f10.6,2x,f10.6');
    Add('   ()');
    Add(' 1x,i3   ');
    Add('    ');
    Add(' 1x,f6.3,2x,f7.3,2x,f7.3,2x,f7.3,2x,f8.3,2x,i4,2x,i8,2x,f6.3');
    Add('      ()');
    Add('    ');
    Add('   ()  ()');
    Add('    5   ');
    Add(' 1x,i3    (, , )');
    Add('    ');
    Add(' 1x,f6.3,2x,f7.3,2x,f7.3,2x,f7.3,2x,f8.3,2x,f15.3');
    Add('      ()');
    Add('    ');
    Add('   ()   ()');
    Add(' 1x,i3   ');
    Add('    ');
    Add(' 1x,f6.3,2x,f7.3,2x,f7.3,2x,f7.3,2x,f8.3,2x,f10.3');
    Add('      ()');
    Add('    ');
    Add('   ()   ()');
  end;
end;

{ satellite position in different systems
  the centre of the Earth to the satellite
      zemloo in orbital system
  by using satloo of TVect3 satellite in true equator
           rosmao of TMatr33 matrix to change to rotating system
           topmao of TMatr33 matrix to change to orbital  system }
procedure SatPosToMemo ; // satellite pos true equa
                         // rotation matrix
var
  ic  : Byte ;
  rx  : TVect3 ; // satellite refer to greenwich meridian and equa
  sl  : Extended ; // geographic longitude of the satellite degree
  sf  : Extended ; // geographic latitude  of the satellite degree
  sr  : Extended ; // range from the centre of the Earth kilometer
  sh  : Extended ; // height of the satellite from the surface
begin
  for ic:=1 to 3 do    // the centre of the Earth refer
    rx[ic]:=-satloo[ic]; // to the satellite in true equator
  MatrMultVector(topmao,rx,zemloo); // the same in orbital
  { position to Earth's rotating system
    from true equinox to greenwich meridian rotating system }
  MatrMultVector(rosmao,satloo,rx); // UnForFun to rotating system
  ClcSpherCoorInDegree(rx,sl,sf,sr); // UnForCoo geographic spheric
  sh:=sr-GeoR0*(1-ParmA*Sqr(Sin(GraRad*sf))); // UnConTyp
  fmSatEph.MemoEph.Lines.Add(' '
  +FloatToFixStr(sl,10,6)+'  '
  +FloatToFixStr(sf,10,6)+'  '
  +FloatToFixStr(sr,10,3)+'  '
  +FloatToFixStr(sh,10,3)
  +'   . . . .');
end;

{ information about the Sun }
procedure SunPosToMemo ( tt  : Extended ) ;
var
  ic  : Byte ;     // simple count
  xp  : TVect3 ;   // position vector
  yp  : TVect3 ;   // the same
  zp  : Extended ; // orbital azimuth  in degree spheric
  hp  : Extended ; // orbital altitude in degree spheric
  rp  : Extended ; // range from the satellite to the body
  am  : Extended ; // geographic longitude of the perpendicular
  ap  : Extended ; // geographic latitude  of the perpendicular
  ar  : Extended ; // perp. range from the surface of the Earth
  ud  : Extended ; // angle of dissipation only for the Sun
begin                       // special for the Sun UnForPos
  PosGeoPlanet(NSun,tt,xp); // fixed equator refer to Earth
  MatrMultVector(rotmao,xp,yp); // UnForFun  true equator
  for ic:=1 to 3 do  // to change centre from
    yp[ic]:=yp[ic]-satloo[ic]; // the Sun to our satellite
  MatrMultVector(topmao,yp,xp); // to orbital system
  ClcTopCoorInDegree(xp,zp,hp,rp); // UnForCoo spheric orbital
  fmSatEph.MemoEph.Lines.Add(' '
  +FloatToFixStr(zp,7,3)+'  '
  +FloatToFixStr((90.0-hp),7,3)+'  '
  +FloatToFixStr(rp,13,3)
  +'    . . .');
  { proc from UnParamO for perpendicular
    input
      xp      the Sun refer to the satellite in orbital system
      zemloo  the centre of the Earth refer to satl in orbital
      topmao rosmao matrix to change
    output  for the perpendicular
      am ap ar geographic longitude latitude height }
  ParamLimbuP(NSun,xp,zemloo,topmao,rosmao,am,ap,ar,ud); // UnParamO
  fmSatEph.MemoEph.Lines.Add(' '
  +FloatToFixStr(am,7,3)+'  '
  +FloatToFixStr(ap,7,3)+'  '
  +FloatToFixStr(ar,13,3)
  +'   .'
  +'  . . . .');
  fmSatEph.MemoEph.Lines.Add(' '
  +FloatToFixStr(ud,7,3)
  +'    ');
end;

procedure LimbuP ( tt  : Extended ) ;
var
  p,q,r  : Extended ;
begin                     // function UnForFun
  r:=VectorModul(satloo); // range from the Earth centre to the sat
  q:=Sqrt(Sqr(r)-Sqr(GeoR0)); // UnConTyp range from sat to limb
  p:=RadGra*DATan2(GeoR0,q); // angle in degree from centre to limb
  azlimb:=0.0; // orbital azimuth of the Earth's limb point degree
  ahlimb:=p-90.0; // orbital altitude of the Earth's limb point
  rolimb:=q; // range in km from satellite to the limb point
  fmSatEph.MemoEph.Lines.Add(' '
  +FloatToFixStr(azlimb,10,6)+'  '
  +FloatToFixStr((90.0-ahlimb),10,6)+'  '
  +FloatToFixSTr(rolimb,10,3)
  +'     . . .');
end;

{ information to memo about limb point coordinates }
procedure AboutLimbuP ;
var
  i  : Byte ;
  rx,ry  : TVect3 ; // descart position in different systems
  sl,sf,sr  : Extended ; // longitude latitude in degree range
begin  // proc from UnForCoo limb point refer to the satellite
  DescFromHorizCoor(azlimb,ahlimb,rolimb,rx); // limb point orbital
  UMatrMultVect(topmao,rx,ry); // limb to true equa refer to satel.
  for i:=1 to 3 do // to change centre from satellite to the Earth
    ry[i]:=satloo[i]+ry[i]; // sat refer to Earth plus limb to sat
  { position to Earth's rotating system
    from true equinox to greenwich meridian rotating system }
  MatrMultVector(rosmao,ry,rx); // UnForFun to rotating system
  ClcSpherCoorInDegree(rx,sl,sf,sr); // UnForCoo geographic spheric
  fmSatEph.MemoEph.Lines.Add(' '
  +FloatToFixStr(sl,10,6)+'  '
  +FloatToFixStr(sf,10,6)
  +'      ');
  nulsta:=fmSatEph.MemoEph.Lines.Count;
end;

{ all bright stars NumBrStarsInHeap UnCoTVar
  StarAllHeap^[..] record StarOneCur fixed equator
  from list to check near limb point }
procedure StarsAroundLimP ( tt  : Extended ) ;
var
  i  : Word ;
  n  : LongInt ; // count for stars in StarAllHeap^
  p  : TVect3 ;  // star position with proper motion to date
  q  : TVect3 ;  // star position in true equator
  o  : TVect3 ;  // star position in orbital system
  h  : Extended ; // altitude in degree
  z  : Extended ; // azimuth  in degree
  r  : Extended ; // unit range for star
  s  : ShortString ; // for the MemoEph
  am,ap,ar,ud  : Extended ; // for the perpendicular
begin
  i:=0;
  for n:=1 to NumBrStarsInHeap do
    begin // to check only brigth stars variables from UnCoTVar
      StarOneCur:=StarAllHeap^[n]; // current record for star
      StarToFixEquator(p); // UnForSta with proper motion to date
      ForAberrationCorrection(p); // UnForRed in fixed equator
      MatrMultVector(rotmao,p,q); // UnForFun to true equator
      MatrMultVector(topmao,q,o); // UnForFun to orbital system
      ClcTopCoorInDegree(o,z,h,r); // UnForCoo spheric orbital
      if  z > 180.0  then  z:=z-360.0; // around nullo azimuth
      { to compare with simple area around limb point }
      if  ( StarOneCur.Vt < VMagMax ) // to truncate with magnitude
           and ( Abs(z-azlimb) < fmSelEph.LatMinVal )  // azimuth
           and ( ( 0.0 < ( h-ahlimb ) )            // altitude
                and ( ( h-ahlimb ) < fmSelEph.LatMinVal ) )
        then // it is our star near limb point
          begin
            Inc(i);
            ParamLimbuP(0,o,zemloo, // vector in orbital system
                        topmao,rosmao,am,ap,ar,ud); // UnParamO
            s:=' '+FloatToFixStr(z,6,3)+'  '
              +FloatToFixStr((90.0-h),7,3)+'  '
              +FloatToFixStr(am,7,3)+'  '  // parametr longitude
              +FloatToFixStr(ap,7,3)+'  '  // parametr latitude
              +FloatToFixStr(ar,8,3)+'  '  // parametr height
              +WordToFixStr(StarOneCur.FK5Number,4)+'  '
              +IntegerToFixStr(StarOneCur.HipNumber,6)+'  '
              +FloatToFixStr(StarOneCur.Vt,6,3);
            fmSatEph.MemoEph.Lines.Add(s);
          end;
    end;
  fmSatEph.MemoEph.Lines.Insert
  (nulsta,' '+WordToFixStr(i,3)+'   '
             +'  . . , . , . .');
  nulpla:=fmSatEph.MemoEph.Lines.Count;
end;

{ the Moon the Sun the planets to check near limb point }
procedure PlanetsAroundLimP ( tt  : Extended ) ;
var
  ic  : Byte ;
  ip  : Byte ;
  np  : Byte ; // count for planet
  xp  : TVect3 ; // position fixed equator and others
  yp  : TVect3 ; // position true  equator and others
  az  : Extended ; // azimuth
  ah  : Extended ; // altitude
  rp  : Extended ; // range
  st  : ShortString ; // for information
  am,ap,ar,ud  : Extended ; // for the perpendicular
begin
  ip:=0;
  for np:=NMercury to NSun do // PUnDE200 const
    if  np <> NEarth
      then
        begin                     // proc PosGeoPos from UnForPos
          PosGeoPlanet(np,tt,xp); // fixed equator refer to Earth
          MatrMultVector(rotmao,xp,yp); // UnForFun  true equator
          for ic:=1 to 3 do  // to change centre from
            yp[ic]:=yp[ic]-satloo[ic]; // the Earth to our satellite
          MatrMultVector(topmao,yp,xp); // to orbital system
          ClcTopCoorInDegree(xp,az,ah,rp); // UnForCoo  spher orb
          if  az > 180.0  then  az:=az-360.0; // around nullo azm
         { to compare with simple area around limb point }
          if  ( Abs(az-azlimb) < fmSelEph.LatMinVal )  // azimuth
              and ( ( 0.0 < ( ah-ahlimb ) )           // altitude
                  and ( ( ah-ahlimb ) < fmSelEph.LatMinVal ) )
            then // it is our planet near limb point
              begin
                Inc(ip);
                ParamLimbuP(0,xp,zemloo, // vector orbital system
                            topmao,rosmao,am,ap,ar,ud); // UnParamO
                st:=' '
                +FloatToFixStr(az,6,3)+'  '
                +FloatToFixStr((90.0-ah),7,3)+'  '
                +FloatToFixStr(am,7,3)+'  '  // parametr longitude
                +FloatToFixStr(ap,7,3)+'  '  // parametr latitude
                +FloatToFixStr(ar,8,3)+'  '  // parametr height
                +FloatToFixStr(rp,15,3)+'  '
                +PlanetNameR[np];
                fmSatEph.MemoEph.Lines.Add(st);
             end;
        end;
  fmSatEph.MemoEph.Lines.Insert
  (nulpla,' '+WordToFixStr(ip,3)+'   '
             +'  . . , . , .');
  nulsat:=fmSatEph.MemoEph.Lines.Count;
end;

{ all satellites   NumSatElemInHeap
  SatElemHeap^[..] record SatElemCur
  from list to check near limb point }
procedure SatellitesAroundLimP ( tt  : Extended ) ;
var
  jc  : Byte ; // simple count
  ks  : Byte ; // count for satellites in small area
  ns  : Word ; // count for satellites in SatElemHeap
  xs  : TVect3 ; // current satellite position true equator
  xo  : TVect3 ; // current satellite position orbital system
  az  : Extended ; // azimuth
  ah  : Extended ; // altitude
  rp  : Extended ; // range
  st  : ShortString ; // for information to Memo
  am,ap,ar,ud  : Extended ; // for the perpendicular
begin
  ks:=0;
  for ns:=1 to NumSatElemInHeap do  // for all satellites in heap
    begin                           // count UnCoTVar variables
      SatElemCur:=SatElemHeap^[ns]; // the next satellite
      ElemX(tt,SatElemCur,xs);      // the Earth true equator
      for jc:=1 to 3 do // to change from the centre of the Earth
        xs[jc]:=xs[jc]-satloo[jc]; // to the our satellite truequ
      if  VectorModul(xs) > 10.0   // may be the same object
        then                       // or too close object
          begin                    // without this case
            MatrMultVector(topmao,xs,xo); // to orbital system
            ClcTopCoorInDegree(xo,az,ah,rp); // UnForCoo  spher orb
            if  az > 180.0  then  az:=az-360.0; // around nullo azm
           { to compare with simple area around limb point }
            if  ( Abs(az-azlimb) < fmSelEph.LatMinVal )  // azimuth
                and ( ( 0.0 < ( ah-ahlimb ) )           // altitude
                    and ( ( ah-ahlimb ) < fmSelEph.LatMinVal ) )
              then // it is one satellite near limb point
                begin
                  Inc(ks);                 // proc from UnParamO
                  ParamLimbuP(0,xo,zemloo, // vector orbital system
                              topmao,rosmao,am,ap,ar,ud); // param
                  st:=' '
                  +FloatToFixStr(az,6,3)+'  '  // orbital azimuth
                  +FloatToFixStr((90.0-ah),7,3)+'  ' // zenith d.
                  +FloatToFixStr(am,7,3)+'  '  // param longitude
                  +FloatToFixStr(ap,7,3)+'  '  // param latitude
                  +FloatToFixStr(ar,8,3)+'  '  // param height
                  +FloatToFixStr(rp,10,3)+'  ' // distance
                  +SatElemCur.satname;
                  fmSatEph.MemoEph.Lines.Add(st);
               end;
          end;
    end;
  fmSatEph.MemoEph.Lines.Insert
  (nulsat,' '+WordToFixStr(ks,3)+'   '
             +'  . . , . , .');
end;

procedure Limbus ( tt  : Extended ) ;
begin
  LimbuP(tt); // orbital spheric coordinates of the limb point
  AboutLimbuP ; // information about limb point coordinates
  { some objects may occur near the Earth's limb point }
  StarsAroundLimP(tt);      // bright stars from list
  PlanetsAroundLimP(tt);    // the Moon the Sun the planets
  SatellitesAroundLimP(tt); // all satellites from list
end;

{ CharForEphem = 'L' from  sat to limb of course
  proc is called from UnSatEph proc PassToMemo }

procedure ToLimbCourse ;    // called by fmSatEph.FormActivate
var
  kk  : Byte ;     // count for moment
  tt  : Extended ; // current moment
begin
  LimbOutFormat ;  // information about output values
  tt:=JulianDate;  // the first moment
  kk:=0;
repeat               // step by step objects in small area
  fmSatEph.MemoEph.Lines.Add('');
  fmSatEph.MemoEph.Lines.Add
  (' '+ToGetSimpleDate(tt)+'       ');
  ClcThreeRotMatr(JD2000,tt, // UnConTyp UnGloVar UnForCoo 'L' option
                  PlaceCoor, // without this variable
                  rotmao,rosmao,topmao); // to orbital system
  ToObtainAberrationParm(tt); // UnForRed   StationPos satellite pos
  satloo:=StationPos; // value is assigned satellite true equator
  SatPosToMemo ; // satellite position by using satloo rosmao
  SunPosToMemo(tt); // information about the Sun
  Limbus(tt); // information for limbus point
  tt:=tt+fmSelEph.StepPrint; // the next moment
  kk:=kk+1;
until  kk = 25 ;
end;

end.
