unit UnStandP; // for PopupMenus items ICRS

interface

procedure StandardPosEqu ; // equator

procedure StandardPosEcl ; // ecliptic

procedure StandardPosGal ; // galactic

procedure SmallRecPosEqu ( xa , xb , ya , yb  : Integer ) ; // small rectangle

procedure SmallRecPosLoo ( xa , xb , ya , yb  : Integer ) ; // rect

implementation

uses
  SysUtils , // function IntToStr
  UnPrtPos , // form fmPrtPos with components
  UnSatEph , // form fmSatEph for fmSatEph.MemoEph component
  UnConTyp , // type TVect3  const GraRad
  UnCoTVar , // for NumObjectInHeap ObjectTopHeap^[..]
  UnGloVar , // var StationPos YinDegC XinDegC
  UnButIni , // for record SkyParm dx dy half in pixels
  PUnCRead , // for proc ToGetStarFromHeap
  PUnDE200 , // for proc ClcPosVel planet ephemeris
  UnitEilP , // for proc ElemX
  UnitFixO , // for var chFixObject
  UnitStat , // for function ToGetStrForSite
  UnForDat , // function ToGetStrForDate
  UnForSta , // for proc StarToFixEquator
  UnForSat , // for proc SatPosWithLightDelay
  UnForCoo , // for proc ClcSpherCoorInDegree
  UnForSys , // for proc FromFixEquToEcl
  UnForTim , // for proc ToGetTEph
  UnForFun , // for proc PrHourAngle PrDegreeAngle
  UnForStr , // function IntegerToFixStr FloatToFixStr
  UW_cns   , // for proc Identify for constellation
  UW_cnst  ; // for const MaxProperName RusStarName[].ProperName

{ angles alpha delta in radian }
procedure AnglesToCivilString ( a , d  : Extended ;  var  s  : ShortString ) ;
var
  z  : ShortString ;
  h  : Integer ;
  m  : Integer ;
  p  : Extended ;
begin
  PrHourAngle(a,z,h,m,p); // right ascension in radian
  s:=s+IntegerToFixStr(h,2)+' '+IntegerToFixStr(m,2)+' '
      +FloatToFixStr(p,5,2)+'  ';
  PrDegrAngle(d,z,h,m,p); // declination in radian
  s:=s+z+IntegerToFixStr(h,2)+' '+IntegerToFixStr(m,2)+' '
        +FloatToFixStr(p,4,1);
end;

{ angles a d in radian }
procedure AnglesDegreeString ( a , d  : Extended ;  var  s  : ShortString ) ;
var
  z  : ShortString ;
  h  : Integer ;
  m  : Integer ;
  p  : Extended ;
begin
  PrDegrAngle(a,z,h,m,p); // longitude ascension in radian
  s:=s+IntegerToFixStr(h,3)+' '+IntegerToFixStr(m,2)+' '
      +FloatToFixStr(p,4,1)+'  ';
  PrDegrAngle(d,z,h,m,p); // latitude in radian
  s:=s+z+IntegerToFixStr(h,2)+' '+IntegerToFixStr(m,2)+' '
        +FloatToFixStr(p,4,1);
end;

function stNumbersToString : ShortString ; // to use global var StarOneCur
var
  kt  : LongInt ;
begin
  kt:=StarOneCur.TycNumber; // UnCoTVar
  if  kt =0  then  kt:=StarOneCur.TrcNumber;
  with  StarOneCur  do // function from UnForStr
    stNumbersToString:='  '+IntegerToFixStr(kt,7)
                      +'  '+IntegerToFixStr(HipNumber,6)
                      +'  '+IntegerToFixStr(FK5Number,4);
end;

function stTwoNumbToString : ShortString ; // to use global var StarOneCur
begin
  with  StarOneCur  do // function from UnForStr
    stTwoNumbToString:='  '+IntegerToFixStr(HipNumber,6)
                      +'  '+IntegerToFixStr(FK5Number,4);
end;

procedure StarFromHeap ( n  : LongInt ) ;
var
  nb  : Word ;
  ps  : TVect3 ;
  aa,da,ra  : Extended ;
  sc,sh,st  : ShortString ;
begin
  ToGetStarFromHeap(n); // procedure from unit PUnCRead to StarOneCur
  StarToFixEquator(ps); // proc from UnForSta pos with proper motions
  ClcSpherCoor(ps,aa,da,ra); // UnForCoo angle in radian
  st:='  ';
  AnglesToCivilString(aa,da,st); // here above
  Identify(2000.0e0,aa,da,sh); // procedure from unit UW_Cns constellation
  for nb:=1 To 88 do // to compare with short latin name of constellation
    if  DimConStel[nb].LatN = sh // const DimConStel from unit UW_CnsT
      then // sh short name sc full name of constellation
        sc:=DimConStel[nb].RusN; // russian name from UW_CnsT
  st:=st+stNumbersToString; // here above to use global var StarOneCur
  st:=st+'  '+FloatToFixStr(StarOneCur.Vt,6,3)+'  * '+sc+' ';
  if  StarOneCur.FK5Number > 0
    then
      for nb:=1 to MaxProperName do
        if  StarOneCur.FK5Number = RusStarName[nb].FK5Number // unit UW_cnst
          then
            st:=st+RusStarName[nb].ProperName;
  fmPrtPos.MemoBox.Lines.Add(st);
  Inc(fmPrtPos.numStar);
end;

{ for fmPrtPos }
procedure SatlFromHeap ( ns  : LongInt ;  ks  : Byte ) ;
var // ns number fo satellite in SatElemHeap^[ns]  ks index for shadow
  xe,xs,ps  : TVect3 ;    // xs[] topocentric true equator
  aa,da,ra  : Extended ;
  st  : ShortString ;
begin // SatElem Cur Heap^[] from UnCoTVar
  SatElemCur:=SatElemHeap^[ns]; // heap is filled in UnitSatF ToFind...
  SatPosWithLightDelay(JulianDate,SatElemCur, // xs[] topocentric true equator
                       StationPos,xe,xs);     // UnForSat xe geocentric
  UMatrMultVect(RotMatr,xs,ps); // from true equator to fixed equator
  ClcSpherCoor(ps,aa,da,ra); // UnForCoo angle in radian
  st:='  ';
  AnglesToCivilString(aa,da,st); // here above
  st:=st+'  '+FloatToFixStr(ra,10,3);
  with  SatElemCur  do
    st:=st+'  '+IntegerToFixStr(satnuml,7)+'  '+satname;
  case  ks  of
    1 : st:=st+' ';
    2 : st:=st+' ';
  end; // case for shadow
  if  BooFixObject and ( chFixObject = 'S')
     then // chFixObject from UnitFixO
       if  SatElemCur.satnuml = SatElemFix.satnuml
         then
           st:=st+'   ';
  fmPrtPos.MemoBox.Lines.Insert(3,st);
  Inc(fmPrtPos.numSatl);
end;

procedure StandardPosEqu ; // star and satellite that are on screen
var                        // topocentric position
  n  : LongInt ;           // refer to fixed equator
begin // to view and print ICRS topocentric position all screen objects
  fmPrtPos.MemoNullo; // fmPrtPos with MemoBox from UnPrtPos
  fmPrtPos.MemoBox.Lines.Add(ToGetStrForDate(JulianDate)); // UnForDat
  fmPrtPos.MemoBox.Lines.Add(ToGetStrForSito(PlaceCoor));  // UnitStat
  fmPrtPos.MemoBox.Lines.Add(ToGetStrForSitu(PlaceCoor));  // UnitStat
  n:=0;
repeat
  if  n < NumObjectInHeap  // it is possible to go to the next n
    then  n:=n+1;          // var NumObjectInHeap from UnCoTVar
  while ( not ObjectTopHeap^[n].booscreen ) // object out of screen
          and ( n < NumObjectInHeap )       // limit for number n
    do  n:=n+1; // the next number to find our object in screen
  case  ObjectTopHeap^[n].chps  of
    '*' : StarFromHeap(ObjectTopHeap^[n].nall); // proc here above for stars
    'S' : if  ObjectTopHeap^[n].booscreen // this is at the screen
            then // IndexShadow  0 , 1 - semishadow
              with  ObjectTopHeap^[n] do // 2 - shadow
                SatlFromHeap(nall,IndexShadow); // for satellite
  end; // case chps
until  n = NumObjectInHeap ;
  fmPrtPos.MemoInsert(3,fmPrtPos.NumStar,' ');
  fmPrtPos.MemoInsert(3,fmPrtPos.NumSatl,' ');
end;

procedure StarEcliPos ( n  : LongInt ) ; // ecliptic positions for stars
var
  nb  : Word ;
  ps,pe  : TVect3 ;
  aa,da  : Extended ;
  al,ab,ra  : Extended ;
  sh,sc,st  : ShortString ;
begin
  ToGetStarFromHeap(n); // procedure from unit PUnCRead to StarOneCur
  if  StarOneCur.Vt > 5.5  Then  Exit ; // too faint star
  StarToFixEquator(ps); // proc from UnForSta pos with proper motions
  ClcSpherCoor(ps,aa,da,ra); // UnForCoo angle in radian right ascension decl.
  Identify(2000.0e0,aa,da,sh); // procedure from unit UW_Cns constellation
  for nb:=1 To 88 do // to compare with short latin name of constellation
    if  DimConStel[nb].LatN = sh // const DimConStel from unit UW_CnsT
      then // sh short name sc full name of constellation
        sc:=DimConStel[nb].RusN; // russian name from UW_CnsT
  FromFixEquToEcl(ps,pe); // proc from UnForSys
  ClcSpherCoor(pe,al,ab,ra); // UnForCoo angle in radian longitude latitude
  st:='  ';
  AnglesToCivilString(aa,da,st); // here above
  st:=st+'  ';
  AnglesDegreeString(al,ab,st); // here above
  st:=st+stTwoNumbToString; // here above to use global var StarOneCur
  st:=st+'  '+FloatToFixStr(StarOneCur.Vt,6,3)+'  * '+sc+' ';
  if  StarOneCur.FK5Number > 0
    then
      for nb:=1 to MaxProperName do
        if  StarOneCur.FK5Number = RusStarName[nb].FK5Number // unit UW_cnst
          then
            st:=st+RusStarName[nb].ProperName;
  fmPrtPos.MemoBox.Lines.Add(st);
  Inc(fmPrtPos.numStar);
end;

procedure EarthMoonEclip ; // ecliptic position for the Moon the Earth
var
  ip  : Byte ;
  tt,tb,dt  : Extended ;
  xs,xm,xe,xp,vp  : TVect3 ;
  al,ab,ra  : Extended ;
  st  : ShortString ;
begin  // for heliocentric position
  FromUTCtoTT(JulianDate,tt,dt);  // unit UnForTim
  tb:=ToGetTEph(tt); // function from unit UnForTim
  ClcPosVel(NSun,tb,xs,vp); // for the Sun refer to Solar system barycenter
  ClcEphEarth(tb,xp,vp);  // the Earth PUnDE200 to barycenter
  ClcPosVel(NMoon,tb,xm,vp); // for the Moon refer to the Earth
  for ip:=1 to 3 do
    begin
      xp[ip]:=xp[ip]-xs[ip];   // heliocentric for the Earth
      xm[ip]:=xp[ip]+xm[ip]; // heliocentric for the Moon
    end;
  FromFixEquToEcl(xp,xe); // proc from UnForSys for the Earth
  ClcSpherCoor(xe,al,ab,ra); // UnForCoo angle in radian longitude latitude
  st:='  ';
  AnglesDegreeString(al,ab,st); // here above
  st:=st+'  '+FloatToFixStr(ra/AstrUnit,8,5);
  st:=st+'  '+PlanetNameR[NEarth];
  fmPrtPos.MemoBox.Lines.Insert(1,st);
  FromFixEquToEcl(xm,xe); // proc from UnForSys for the Moon
  ClcSpherCoor(xe,al,ab,ra); // UnForCoo angle in radian longitude latitude
  st:='  ';
  AnglesDegreeString(al,ab,st); // here above
  st:=st+'  '+FloatToFixStr(ra/AstrUnit,8,5);
  st:=st+'  '+PlanetNameR[NMoon];
  fmPrtPos.MemoBox.Lines.Insert(1,st);
end;

procedure AllPlanetEclip ( n : LongInt ) ; // for all planets
var
  ip,np  : Byte ;
  tt,tb,dt  : Extended ;
  xs,xe,xp,vp  : TVect3 ;
  al,ab,ra  : Extended ;
  st  : ShortString ;
begin
  np:=ObjectTopHeap^[n].nall; // number of planet
  if  np in [ NEarth , NSun ]  then  Exit ; // heliocentric position
  if  np = NMoon
    then
      begin
        EarthMoonEclip ;
        Exit;
      end;
  FromUTCtoTT(JulianDate,tt,dt);  // unit UnForTim
  tb:=ToGetTEph(tt); // function from unit UnForTim
  ClcPosVel(NSun,tb,xs,vp); // for the Sun refer to Solar system barycenter
  ClcPosVel(np,tb,xp,vp); // xp planet fixed equator position
  for ip:=1 to 3 do  xp[ip]:=xp[ip]-xs[ip]; // heliocentric position
  FromFixEquToEcl(xp,xe); // proc from UnForSys
  ClcSpherCoor(xe,al,ab,ra); // UnForCoo angle in radian longitude latitude
  st:='  ';
  AnglesDegreeString(al,ab,st); // here above
  st:=st+'  '+FloatToFixStr(ra/AstrUnit,8,5);
  st:=st+'  '+FloatToFixStr(ObjectTopHeap^[n].Mag,6,3);
  st:=st+'  '+PlanetNameR[np];
  fmPrtPos.MemoBox.Lines.Insert(1,st);
end;

procedure StandardPosEcl ; // all planets and bright stars refer to ecliptic
var
  n  : LongInt ;
begin // to print standard ecliptic positions for all planets
  fmPrtPos.MemoNullo; // fmPrtPos with MemoBox from UnPrtPos
  fmPrtPos.MemoBox.Lines.Add(ToGetStrForDate(JulianDate)); // UnForDat
  n:=0;
repeat
  if  n < NumObjectInHeap  then  n:=n+1; // var NumObjectInHeap from UnCoTVar
  while ( not ObjectTopHeap^[n].booscreen )     // this object
          and ( ObjectTopHeap^[n].chps <> 'P' ) // may be out
          and ( n < NumObjectInHeap )           // of the field
     do  n:=n+1;  // to the next number to find object among screen
  case  ObjectTopHeap^[n].chps  of
    '*' : StarEcliPos(ObjectTopHeap^[n].nall); // proc here above for stars
    'P' : AllPlanetEclip(n);
  end; // case chps
until  n = NumObjectInHeap ;
  fmPrtPos.MemoInsert(1,fmPrtPos.NumStar,' ');
end;

procedure StarGalactos ( n  : LongInt ) ; // galactic positions for stars
var
  nb  : Word ;
  ps,pe  : TVect3 ;
  aa,da  : Extended ;
  al,ab,ra  : Extended ;
  sh,sc,st  : ShortString ;
begin
  ToGetStarFromHeap(n); // procedure from unit PUnCRead to StarOneCur
  StarToFixEquator(ps); // proc from UnForSta pos with proper motions
  ClcSpherCoor(ps,aa,da,ra); // UnForCoo angle in radian right ascension decl.
  Identify(2000.0e0,aa,da,sh); // procedure from unit UW_Cns constellation
  for nb:=1 To 88 do // to compare with short latin name of constellation
    if  DimConStel[nb].LatN = sh // const DimConStel from unit UW_CnsT
      then // sh short name sc full name of constellation
        sc:=DimConStel[nb].RusN; // russian name from UW_CnsT
  FromFixEquToGal(ps,pe); // proc from UnForSys
  ClcSpherCoor(pe,al,ab,ra); // UnForCoo angle in radian longitude latitude
  st:='  ';
  AnglesToCivilString(aa,da,st); // here above
  st:=st+'  ';
  AnglesDegreeString(al,ab,st); // here above
  st:=st+stTwoNumbToString; // here above to use global var StarOneCur
  st:=st+'  '+FloatToFixStr(StarOneCur.Vt,6,3)+'  * '+sc+' ';
  if  StarOneCur.FK5Number > 0
    then
      for nb:=1 to MaxProperName do
        if  StarOneCur.FK5Number = RusStarName[nb].FK5Number // unit UW_cnst
          then
            st:=st+RusStarName[nb].ProperName;
  fmPrtPos.MemoBox.Lines.Add(st);
  Inc(fmPrtPos.numStar);
end;

procedure StandardPosGal ; // for galactic coordinates of stars
var
  n  : LongInt ;
begin // to view and print ICRS position all screen objects
  fmPrtPos.MemoNullo; // fmPrtPos with MemoBox from UnPrtPos
  fmPrtPos.MemoBox.Lines.Add(ToGetStrForDate(JulianDate)); // UnForDat
  n:=0;
repeat
  if  n < NumObjectInHeap  then  n:=n+1; // var NumObjectInHeap from UnCoTVar
  while ( not ObjectTopHeap^[n].booscreen ) // out of our
          and ( n < NumObjectInHeap )       // field of view
     do  n:=n+1;  // to the next number to find object among screen
  if  ObjectTopHeap^[n].chps = '*'
    then
      StarGalactos(ObjectTopHeap^[n].nall); // proc here above for stars
until  n = NumObjectInHeap ;
  fmPrtPos.MemoInsert(1,fmPrtPos.NumStar,' ');
end;

{ for fmSatEph form
  o is record type UnCoTVar with the variables
  nall chps name azt alt boovis xpos ypos booscreen IndexShadow }
procedure StanFromHeap ( l  : Integer ;  o  : TObjectTopRec ) ;
var                   // l number of position in MemoEph to insert
  nb  : Word ;
  ps  : TVect3 ;
  aa,da,ra  : Extended ;
  sc,sh,st  : ShortString ;
begin
  ToGetStarFromHeap(o.nall); // procedure from unit PUnCRead to StarOneCur
  StarToFixEquator(ps); // proc from UnForSta pos with proper motions
  ClcSpherCoor(ps,aa,da,ra); // UnForCoo angle in radian
  st:='  ';
  AnglesDegreeString(GraRad*o.azt,GraRad*o.alt,st); // azimuth altitude
  Identify(2000.0e0,aa,da,sh); // procedure from unit UW_Cns constellation
  for nb:=1 To 88 do // to compare with short latin name of constellation
    if  DimConStel[nb].LatN = sh // const DimConStel from unit UW_CnsT
      then // sh short name sc full name of constellation
        sc:=DimConStel[nb].RusN; // russian name from UW_CnsT
  st:=st+stNumbersToString; // here above to use global var StarOneCur
  st:=st+'  '+FloatToFixStr(StarOneCur.Vt,6,3)+'  * '+sc+' ';
  if  StarOneCur.FK5Number > 0 // global variable UnCoTVar
    then
      for nb:=1 to MaxProperName do // unit UW_cnst
        if  StarOneCur.FK5Number = RusStarName[nb].FK5Number
          then
            st:=st+RusStarName[nb].ProperName;
  fmSatEph.MemoEph.Lines.Insert(l,st);
end;

{ for fmSatEph form  planets in rectangle
  o is record type UnCoTVar with the variables
  nall chps name azt alt boovis xpos ypos booscreen IndexShadow }
procedure SpanFromHeap ( l  : Integer ;  o  : TObjectTopRec ) ; { type UnCoTVar }
var // ns number fo satellite in SatElemHeap^[ns]  ks index for shadow
  st  : ShortString ; // l number of position in MemoEph to insert
begin // SatElem Cur Heap^[] from UnCoTVar
  st:='  '; { in o.nall is contained the number of the planet }
  AnglesDegreeString(GraRad*o.azt,GraRad*o.alt,st); // here above
  if  o.nall = NMoon // PUnDE200   azimuth    altitude to radian
    then // range for the Moon in km  function fron UnForStr
     st:=st+'  '+FloatToFixStr(o.rot,10,3) // GraRad UnConTyp
    else // range for the Sun and planets in AstrUnit
     st:=st+'  '+FloatToFixStr(o.rot/AstrUnit,10,6); // UnConTyp
  st:=st+'  '+o.name;
  case  o.IndexShadow  of
    1 : st:=st+' ';
    2 : st:=st+' ';
  end; // case for shadow
  fmSatEph.MemoEph.Lines.Insert(l,st);
end;

{ for fmSatEph form
  o is record type UnCoTVar with the variables
  nall chps name azt alt boovis xpos ypos booscreen IndexShadow }
procedure SatnFromHeap ( l  : Integer ;  o  : TObjectTopRec ) ; { type UnCoTVar }
var // ns number fo satellite in SatElemHeap^[ns]  ks index for shadow
  st  : ShortString ; // l number of position in MemoEph to insert
begin // SatElem Cur Heap^[] from UnCoTVar
  SatElemCur:=SatElemHeap^[o.nall]; // heap is filled in UnitSatF ToFind...
  st:='  ';
  AnglesDegreeString(GraRad*o.azt,GraRad*o.alt,st); // azimuth altitude
  st:=st+'  '+FloatToFixStr(o.rot,10,3); // range GraRad UnConTyp
  with  SatElemCur  do
    st:=st+'  '+IntegerToFixStr(satnuml,7)+'  '+satname;
  case  o.IndexShadow  of
    1 : st:=st+' ';
    2 : st:=st+' ';
  end; // case for shadow
  if  BooFixObject and ( chFixObject = 'S')
     then // chFixObject from UnitFixO
       if  SatElemCur.satnuml = SatElemFix.satnuml
         then
           st:=st+'   ';
  fmSatEph.MemoEph.Lines.Insert(l,st);
end;

{ simple information about size of small rectangle }
procedure AboutSmallRect ( lp , xa , xb , ya , yb  : Integer ) ;
var                   // lp numer line in MemoEph to print
  dx,dy  : Extended ; // size in pixel  x and y
  rx,ry  : Extended ; // size in degree x and y
  st  : ShortString ; // string with information
begin
  dx:=xb-xa; // x size in pixels
  dy:=yb-ya; // y size in pixels
  rx:=(dx/SkyParm.dx)*XinDegC; // x in degree UnButIni UnGloVar
  ry:=(dy/SkyParm.dy)*YinDegC; // y in degree UnButIni UnGloVar
  st:='  ';
  AnglesDegreeString(GraRad*rx,GraRad*ry,st); // size to string
  st:=st+'    ';
  fmSatEph.MemoEph.Lines.Insert(lp,st);
end;

{ proc is called from UnMuDraw by procedure AfterDrawSky
  to find objects in small rectangle area
  from global arrays ObjectTopHeap^ and others }
procedure SmallRecPosEqu ( xa , xb , ya , yb  : Integer ) ; // small rectangle
var               // stars , planets and satellites in the small rectangle
  n  : LongInt ;  // horizontal topocentric position
  i  : Word ;  { number of stars }
  j  : Word ;  { number of planets }
  k  : Word ;  { number of satellites }
  o  : TObjectTopRec ; { type UnCoTVar as global var ObjectTopCur }
begin // to view and print topocentric position all screen objects
  n:=0;  { initial number all object }
  i:=0;  { number of stars }
  j:=0;  { number of planets }
  k:=0;  { number of satellites }
repeat
  if  n < NumObjectInHeap  then  n:=n+1; // var NumObjectInHeap
  while ( not ObjectTopHeap^[n].booscreen ) // to check object is
          and ( n < NumObjectInHeap )   // on the screen UnCotVar
      do  n:=n+1; // the record for visible object
  { current record from heap to simple record }
  o:=ObjectTopHeap^[n]; { record UnCoTVar with the variables }
 { nall chps name azt alt boovis xpos ypos booscreen IndexShadow }
  if  o.booscreen // if  booscreen then it is at the screen
      and ( xa < o.xpos ) and ( o.xpos < xb ) // width  rectangle
      and ( ya < o.ypos ) and ( o.ypos < yb ) // height rectangle
    then                                   // small  rectangle
      case  o.chps  of  // char may be 'S' 'P' '*'
       '*' :            // nall number of star in StarHeap^
        begin
          StanFromHeap(3,o); // stars ObjectTopHeap^[n].nall
          Inc(i); // the next star in small rectangle
        end;
       'P' :               // planet case
        begin
          SpanFromHeap(3,o); // planet ObjectTopHeap^[n].name
          Inc(j); // the next planet in small rectangle
        end;
       'S' :  // IndexShadow  0 , 1 - semishadow
        begin // for satellite nall number in SatHeap^
          SatnFromHeap(3,o); // information from TObjectTopRec
          Inc(k); // the next satellite
        end; { for 'S' satellite }
      end; // case chps
until  n = NumObjectInHeap ;
  fmSatEph.MemoEph.Lines.Insert  //  to fmSatEph.MemoEph
          (3,'  '+IntToStr(i)+'   ');
  fmSatEph.MemoEph.Lines.Insert
          (3,'  '+IntToStr(j)+'   ');
  fmSatEph.MemoEph.Lines.Insert
          (3,'  '+IntToStr(k)+'   ');
  AboutSmallRect(3,xa,xb,ya,yb);   // simple information
end;

{ proc is called from UnMuDraw by procedure AfterDrawLook
  to find objects in small rectangle area
  from global arrays ObjectTopHeap^ and others }
procedure SmallRecPosLoo ( xa , xb , ya , yb  : Integer ) ; // rect
var      // stars , planets and satellites in the small rectangle
  n  : LongInt ;  // orbital position for all object
  l  : Integer ;  // to keep current position in MemoEph
  i  : Word ;  { number of stars }
  j  : Word ;  { number of planets }
  k  : Word ;  { number of satellites }
  o  : TObjectTopRec ; { type UnCoTVar as global var ObjectTopCur }
begin // to view and print orbital system position screen objects
  l:=8; // current position in MemoEph
  n:=0;  { initial number all object }
  i:=0;  { number of stars }
  j:=0;  { number of planets }
  k:=0;  { number of satellites }
repeat
  if  n < NumObjectInHeap  then  n:=n+1; // var NumObjectInHeap
  while ( not ObjectTopHeap^[n].booscreen ) // to check object is
          and ( n < NumObjectInHeap )  // on the sceen UnCotVar
      do  n:=n+1; // the record for visible object
  { current record from heap to simple record }
  o:=ObjectTopHeap^[n]; { record UnCoTVar with the variables }
 { nall chps name azt alt boovis xpos ypos booscreen IndexShadow }
  if  o.booscreen // if  booscreen then it is at the screen
      and ( xa < o.xpos ) and ( o.xpos < xb ) // width  rectangle
      and ( ya < o.ypos ) and ( o.ypos < yb ) // height rectangle
    then                                   // small  rectangle
      case  o.chps  of  // char may be 'S' 'P' '*'
       '*' :            // nall number of star in StarHeap^
        begin
          StanFromHeap(l,o); // stars ObjectTopHeap^[n].nall
          Inc(i); // the next star in small rectangle
        end;
       'P' :               // planet case
        begin
          SpanFromHeap(l,o); // planet ObjectTopHeap^[n].name
          Inc(j); // the next planet in small rectangle
        end;
       'S' :  // IndexShadow  0 , 1 - semishadow
        begin // for satellite nall number in SatHeap^
          SatnFromHeap(l,o); // information from TObjectTopRec
          Inc(k); // the next satellite
        end; { for 'S' satellite }
      end; // case chps
until  n = NumObjectInHeap ;
  fmSatEph.MemoEph.Lines.Insert  //  to fmSatEph.MemoEph
          (l,'  '+IntToStr(i)+'   ');
  fmSatEph.MemoEph.Lines.Insert
          (l,'  '+IntToStr(j)+'   ');
  fmSatEph.MemoEph.Lines.Insert
          (l,'  '+IntToStr(k)+'   ');
  AboutSmallRect(l,xa,xb,ya,yb); // simple information l pos
end;

end.
