unit UnVectoS ; // set of conditional vectors for current satellite
interface       // for the moment equator crossing
uses            // in ascending node of the orbit
  UnConTyp , // for type TVect3
  UnCoTVar ; // record type TElemRec
var
  ncross  : Byte ;          // count for crossing moments
  tcross  : array [1..25] of Extended ; // set of moments
  rcross  : array [1..25] of TVect3 ;   // set of positions
  vcross  : array [1..25] of TVect3 ;   // set of velosities
{ tt moment in julian day
  el elements of the satellite
  these procedures are called from UnPrtPos }
procedure ToFindCrossEqu ( tt  : Extended ;  el  : TElemRec ) ;
procedure SatPosRot ( el  : TElemRec ) ; // Earth's rotating system
procedure SatPosFix ( el  : TElemRec ) ; // fixed equator system
procedure SatPosElm ( el  : TElemRec ) ; // kepler elements
implementation // well a try is not a cry
uses
  SysUtils , // function IntToStr
  UnPrtPos , // for method of the form fmPrtPos
  UnGloVar , // global variables
  UnitEilP , // proc ElemX EilXV
  UnitElmK , // proc FromCartToKepler
  UnForDat , // function ToGetStrForDate
  UnForCoo , // proc ClcTrueRotMatr
  UnForFun , // proc MatrMultVector
  UnForStr ; // function FloatToFixStr
{ more detail operatiom to find cross moment
  elem of TElemRec elements of the satellite }
function CrossMom ( tbeg , tend  : Extended ;
                    elem : TElemRec ) : Extended ; // UnCoTVar
var
  ta,tb,dt  : Extended ;
  bx,ax  : TVect3 ;
begin
  dt:=0.01*(tend-tbeg); { simple step to find cross moment }
  tb:=tbeg; { the first moment for find }
  ElemX(tb,elem,bx); { true equa position UnitEilP }
repeat
  ta:=tb;            // to keep moment
  ax:=bx;            // to keep position vector
  tb:=tb+dt;         // the next moment
  ElemX(tb,elem,bx); { true equa position }
until  ( tb > (tend-dt-1.0e-8) ) or ( bx[3] > 0.0 ) ;
  CrossMom:=ta+(-ax[3]/(bx[3]-ax[3]))*dt;
end;
{ a try to find our event in small time interval
  if z(ta) < 0 and z(tb) > 0 then our crossing case }
procedure CrossFind ( ta , tb  : Extended ;   // interval in day
                      el  : TElemRec ;        // record elements
                      var  bc  : Boolean ;    // yes or no cross
                      var  tc  : Extended ) ; // moment if cross
var
  xa, xb  : TVect3 ; // temporary satellite positions
begin
  bc:=False; // default
  tc:=ta;    // default
  { to compare with nullo value of z-coordinate for equator }
  ElemX(ta,el,xa); // UnitEilP satellite true equator moment ta
  if  xa[3] > 0.0  then  Exit ; // no crossing case above equator
  ElemX(tb,el,xb); // UnitEilP satellite true equator moment tb
  if  xb[3] < 0.0  then  Exit ; // no crossing case lower equator
  if  ( (xb[3]-xa[3]) > 0.0001 )  { in kilometer }
    then { linear  interpolation }
      tc:=CrossMom(ta,tb,el) // more detail to find crossing mom.
    else { wothout interpolation }
      tc:=ta;
  bc:=True; { it is our crossing case }
end;
{ tt moment in julian day  el elements of the satellite }
procedure ToFindCrossEqu ( tt  : Extended ;  el  : TElemRec ) ;
var
  ta,tb  : Extended ; // start and finish moment in julian day
  pp,dt  : Extended ; // period of revolution and step in day
  tc,dp  : Extended ; // crossing moment and step close to period
  rc,vc  : TVect3 ;   // position velosity in crossing case
  boocr  : Boolean ;  // yes or no crossing case
begin
  ncross:=0; // initial nullo crossing events
  { el of TElemRec  el.an mean motion revolution per day }
  pp:=1.00/el.an; // period of revolution in day
  ta:=tt-2.00*pp; // start  moment to find crossing case
  tb:=ta+21.0*pp; // finish moment to find crossing case
  dt:=0.05*pp;    // step in day to find crossing case
  dp:=0.9*pp;     // step in day close to period
  while  ( ta < tb ) and ( ncross < 25 )  do // step by step
    begin            // to find all crossing events
      CrossFind(ta,ta+dt,el,boocr,tc); // small interval elements
      if  boocr   // crossing moment
        then      // is found
          begin   // this moment is tc in julian day
            EilXV(tc,el,rc,vc); // UnitEilP position velosity
            ncross:=ncross+1;   // the next crossing case
            tcross[ncross]:=tc; // the next crossing moment
            rcross[ncross]:=rc; // the next crossing position
            vcross[ncross]:=vc; // the next crossing velosity
            ta:=ta+dp;
          end;
      ta:=ta+dt;
    end;
end;
{ position velosity in rotating system }
procedure PosVelRot ( n  : Byte ) ;
var
  rx,rv  : TVect3 ;  // position velosity greenwich meridian
  wt,ws  : TMatr33 ; // matrixes to change
begin
  ClcTrueRotMatr(JD2000,tcross[n],wt,ws); // UnForCoo matrix
  { position to Earth's rotating system
    RosMatr UnGloVar to change
    from true equinox to greenwich meridian rotating system }
  MatrMultVector(ws,rcross[n],rx); // UnForFun
  { part of velosity in Earth's rotating system
    without velosity of the rotation of the Earth }
  MatrMultVector(ws,vcross[n],rv); // only part for velosity
  { the next part for velosity in rotating system only for x y }
  rv[1]:=rv[1]+VelOfRot*rx[2]; // to take rotation into account
  rv[2]:=rv[2]-VelOfRot*rx[1]; // to take rotation into account
  fmPrtPos.MemoBox.Lines.Add         // UnPrtPos
  ('  '+FloatToFixStr(rx[1],7,3)+'      x  ');
  fmPrtPos.MemoBox.Lines.Add         // UnPrtPos
  ('  '+FloatToFixStr(rx[2],7,3)+'      y  ');
  fmPrtPos.MemoBox.Lines.Add         // UnPrtPos
  ('  '+FloatToFixStr(rx[3],7,3)+'      z  ');
  fmPrtPos.MemoBox.Lines.Add         // UnPrtPos
  ('  '+FloatToFixStr(rv[1],7,6)+'      x   /');
  fmPrtPos.MemoBox.Lines.Add         // UnPrtPos
  ('  '+FloatToFixStr(rv[2],7,6)+'      y   /');
  fmPrtPos.MemoBox.Lines.Add         // UnPrtPos
  ('  '+FloatToFixStr(rv[3],7,6)+'      z   /');
end;
{ period in minute and change of period in minute per revolution }
procedure PeriodOrb ( n  : Byte ) ; // period and change
var
  pa,pb  : Extended ; // periods in minute
begin
  pa:=1440.0*(tcross[n]-tcross[n-1]); // the first  period minute
  pb:=1440.0*(tcross[n+1]-tcross[n]); // the second period minute
  fmPrtPos.MemoBox.Lines.Add         // UnPrtPos
  ('  '+FloatToFixStr(pb,10,6)+'       ');
  fmPrtPos.MemoBox.Lines.Add         // UnPrtPos
  ('  '+FloatToFixStr((pb-pa),10,8)
       +'        ');
end;
{ format of the condition vector Earth's rotating system }
procedure SatPosRot ( el  : TElemRec ) ; // Earth's rotating system
var
  n  : Byte ;
begin
  fmPrtPos.MemoNullo ; // to clear MemoBox
  fmPrtPos.MemoBox.Lines.Add(' '+fmPrtPos.Caption);
  fmPrtPos.MemoBox.Lines.Add('  '+fmPrtPos.GroupBox.Caption);
  fmPrtPos.MemoBox.Lines.Add('  '+fmPrtPos.rbEqu.Caption);
  for n:=2 to ncross-1 do
    begin
      fmPrtPos.MemoBox.Lines.Add('  ');
      fmPrtPos.MemoBox.Lines.Add
      ('  '+IntToStr(el.satnord)+'  '+el.satname+'    ');
      fmPrtPos.MemoBox.Lines.Add         // UnPrtPos
      ('  '+ToGetOnlyDate(tcross[n]+0.125)  // UnForDat
           +'         ');
      fmPrtPos.MemoBox.Lines.Add         // UnPrtPos
      ('  '+ToGetOnlyTime(tcross[n]+0.125)  // UnForDat
           +'       ');
      PosVelRot(n); // position velosity in rotating system
      PeriodOrb(n); // period and change of period in minute
    end;
end;
{ position velosity  fixed system epoch J2000.0 }
procedure PosVelFix ( n  : Byte ) ;
var
  rx,rv  : TVect3 ;  // position velosity greenwich meridian
  wt,ws  : TMatr33 ; // matrixes to change
begin
  ClcTrueRotMatr(JD2000,tcross[n],wt,ws); // UnForCoo matrix
  { position to fixed system
    wt is matrix to change
    from true equinox to fixed equinox }
  UMatrMultVect(wt,rcross[n],rx); // UnForFun
  { velosity }
  UMatrMultVect(wt,vcross[n],rv); // fixed equator
  fmPrtPos.MemoBox.Lines.Add         // UnPrtPos
  ('  '+FloatToFixStr(rx[1],7,3)+'      x  ');
  fmPrtPos.MemoBox.Lines.Add         // UnPrtPos
  ('  '+FloatToFixStr(rx[2],7,3)+'      y  ');
  fmPrtPos.MemoBox.Lines.Add         // UnPrtPos
  ('  '+FloatToFixStr(rx[3],7,3)+'      z  ');
  fmPrtPos.MemoBox.Lines.Add         // UnPrtPos
  ('  '+FloatToFixStr(rv[1],7,6)+'      x   /');
  fmPrtPos.MemoBox.Lines.Add          // UnPrtPos
  ('  '+FloatToFixStr(rv[2],7,6)+'      y   /');
  fmPrtPos.MemoBox.Lines.Add         // UnPrtPos
  ('  '+FloatToFixStr(rv[3],7,6)+'      z   /');
end;
{ format of the condition vector
  standard epoch J2000.0 fixed system }
procedure SatPosFix ( el  : TElemRec ) ; // fixed equator system J2000.0
var
  n  : Byte ;
begin
  fmPrtPos.MemoNullo ; // to clear MemoBox
  fmPrtPos.MemoBox.Lines.Add(' '+fmPrtPos.Caption);
  fmPrtPos.MemoBox.Lines.Add('  '+fmPrtPos.GroupBox.Caption);
  fmPrtPos.MemoBox.Lines.Add('  '+fmPrtPos.rbEcl.Caption);
  for n:=2 to ncross-1 do
    begin
      fmPrtPos.MemoBox.Lines.Add('  ');
      fmPrtPos.MemoBox.Lines.Add
      ('  '+IntToStr(el.satnord)+'  '+el.satname+'  ');
      fmPrtPos.MemoBox.Lines.Add         // UnPrtPos
      ('  '+ToGetOnlyDate(tcross[n]+0.125)  // UnForDat
           +'         ');
      fmPrtPos.MemoBox.Lines.Add         // UnPrtPos
      ('  '+ToGetOnlyTime(tcross[n]+0.125)  // UnForDat
           +'       ');
      PosVelFix(n); // position velosity fixed system
      PeriodOrb(n); // period and change of period in minute
    end;
end;
{ from condition vector to kepler elements }
procedure PosVelElm ( n  : Byte ) ; // to kepler elements
var
  a,u  : TVect3 ; // kepler elements UnitElmK
begin // a( a e i degree )  u( Omega omega M degree )
  FromCartToKepler(rcross[n],vcross[n],a,u); // UnitElmK
  fmPrtPos.MemoBox.Lines.Add           // UnPrtPos
  ('  '+FloatToFixStr(a[1],8,3)        // semimajor axis
       +'       ');      // semimajor axis km
  fmPrtPos.MemoBox.Lines.Add           // UnPrtPos
  ('  '+FloatToFixStr(a[2],8,6)        // eccentricity
       +'    ');           // eccentricity
  fmPrtPos.MemoBox.Lines.Add           // UnPrtPos
  ('  '+FloatToFixStr(a[3],8,5)        // inclination degree
       +'       '); // inclination degree
  fmPrtPos.MemoBox.Lines.Add           // UnPrtPos
  ('  '+FloatToFixStr(u[1],8,5)  // ascending node degree
       +'        '); // ascending node
  fmPrtPos.MemoBox.Lines.Add            // UnPrtPos
  ('  '+FloatToFixStr(u[2],8,5)         // argument perigei
       +'       '); // argument perigei degree
  fmPrtPos.MemoBox.Lines.Add            // UnPrtPos
  ('  '+FloatToFixStr(u[3],8,5)         // mean anomaly degree
       +'       '); // mean anomaly degree
end;
{ format of the condition vector kepler elements }
procedure SatPosElm ( el  : TElemRec ) ; // kepler elements
var
  n  : Byte ;
begin
  fmPrtPos.MemoNullo ; // to clear MemoBox
  fmPrtPos.MemoBox.Lines.Add(' '+fmPrtPos.Caption);
  fmPrtPos.MemoBox.Lines.Add('  '+fmPrtPos.GroupBox.Caption);
  fmPrtPos.MemoBox.Lines.Add('  '+fmPrtPos.rbGal.Caption);
  for n:=2 to ncross-1 do
    begin
      fmPrtPos.MemoBox.Lines.Add('  ');
      fmPrtPos.MemoBox.Lines.Add
      ('  '+IntToStr(el.satnord)+'  '+el.satname+'  ');
      fmPrtPos.MemoBox.Lines.Add         // UnPrtPos
      ('  '+ToGetOnlyDate(tcross[n]+0.125)  // UnForDat
           +'         ');
      fmPrtPos.MemoBox.Lines.Add         // UnPrtPos
      ('  '+ToGetOnlyTime(tcross[n]+0.125)  // UnForDat
           +'       ');
      PosVelElm(n); // from position velosity to kepler elements
      PeriodOrb(n); // period and change of period in minute
    end;
end;
end.